#include <tree_sitter/parser.h>

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#define LANGUAGE_VERSION 14
#define STATE_COUNT 1569
#define LARGE_STATE_COUNT 88
#define SYMBOL_COUNT 293
#define ALIAS_COUNT 0
#define TOKEN_COUNT 106
#define EXTERNAL_TOKEN_COUNT 105
#define FIELD_COUNT 2
#define MAX_ALIAS_SEQUENCE_LENGTH 5
#define PRODUCTION_ID_COUNT 13

enum {
  sym__eof = 1,
  sym__s_dir_yml_bgn = 2,
  sym__r_dir_yml_ver = 3,
  sym__s_dir_tag_bgn = 4,
  sym__r_dir_tag_hdl = 5,
  sym__r_dir_tag_pfx = 6,
  sym__s_dir_rsv_bgn = 7,
  sym__r_dir_rsv_prm = 8,
  sym__s_drs_end = 9,
  sym__s_doc_end = 10,
  sym__r_blk_seq_bgn = 11,
  sym__br_blk_seq_bgn = 12,
  sym__b_blk_seq_bgn = 13,
  sym__r_blk_key_bgn = 14,
  sym__br_blk_key_bgn = 15,
  sym__b_blk_key_bgn = 16,
  sym__r_blk_val_bgn = 17,
  sym__br_blk_val_bgn = 18,
  sym__b_blk_val_bgn = 19,
  sym__r_blk_imp_bgn = 20,
  sym__r_blk_lit_bgn = 21,
  sym__br_blk_lit_bgn = 22,
  sym__r_blk_fld_bgn = 23,
  sym__br_blk_fld_bgn = 24,
  sym__br_blk_str_ctn = 25,
  sym__r_flw_seq_bgn = 26,
  sym__br_flw_seq_bgn = 27,
  sym__b_flw_seq_bgn = 28,
  sym__r_flw_seq_end = 29,
  sym__br_flw_seq_end = 30,
  sym__r_flw_map_bgn = 31,
  sym__br_flw_map_bgn = 32,
  sym__b_flw_map_bgn = 33,
  sym__r_flw_map_end = 34,
  sym__br_flw_map_end = 35,
  sym__r_flw_sep_bgn = 36,
  sym__br_flw_sep_bgn = 37,
  sym__r_flw_key_bgn = 38,
  sym__br_flw_key_bgn = 39,
  sym__r_flw_jsv_bgn = 40,
  sym__br_flw_jsv_bgn = 41,
  sym__r_flw_njv_bgn = 42,
  sym__br_flw_njv_bgn = 43,
  sym__r_dqt_str_bgn = 44,
  sym__br_dqt_str_bgn = 45,
  sym__b_dqt_str_bgn = 46,
  sym__r_dqt_str_ctn = 47,
  sym__br_dqt_str_ctn = 48,
  sym__r_dqt_esc_nwl = 49,
  sym__br_dqt_esc_nwl = 50,
  sym__r_dqt_esc_seq = 51,
  sym__br_dqt_esc_seq = 52,
  sym__r_dqt_str_end = 53,
  sym__br_dqt_str_end = 54,
  sym__r_sqt_str_bgn = 55,
  sym__br_sqt_str_bgn = 56,
  sym__b_sqt_str_bgn = 57,
  sym__r_sqt_str_ctn = 58,
  sym__br_sqt_str_ctn = 59,
  sym__r_sqt_esc_sqt = 60,
  sym__br_sqt_esc_sqt = 61,
  sym__r_sqt_str_end = 62,
  sym__br_sqt_str_end = 63,
  sym__r_sgl_pln_nul_blk = 64,
  sym__br_sgl_pln_nul_blk = 65,
  sym__b_sgl_pln_nul_blk = 66,
  sym__r_sgl_pln_nul_flw = 67,
  sym__br_sgl_pln_nul_flw = 68,
  sym__r_sgl_pln_bol_blk = 69,
  sym__br_sgl_pln_bol_blk = 70,
  sym__b_sgl_pln_bol_blk = 71,
  sym__r_sgl_pln_bol_flw = 72,
  sym__br_sgl_pln_bol_flw = 73,
  sym__r_sgl_pln_int_blk = 74,
  sym__br_sgl_pln_int_blk = 75,
  sym__b_sgl_pln_int_blk = 76,
  sym__r_sgl_pln_int_flw = 77,
  sym__br_sgl_pln_int_flw = 78,
  sym__r_sgl_pln_flt_blk = 79,
  sym__br_sgl_pln_flt_blk = 80,
  sym__b_sgl_pln_flt_blk = 81,
  sym__r_sgl_pln_flt_flw = 82,
  sym__br_sgl_pln_flt_flw = 83,
  sym__r_sgl_pln_str_blk = 84,
  sym__br_sgl_pln_str_blk = 85,
  sym__b_sgl_pln_str_blk = 86,
  sym__r_sgl_pln_str_flw = 87,
  sym__br_sgl_pln_str_flw = 88,
  sym__r_mtl_pln_str_blk = 89,
  sym__br_mtl_pln_str_blk = 90,
  sym__r_mtl_pln_str_flw = 91,
  sym__br_mtl_pln_str_flw = 92,
  sym__r_tag = 93,
  sym__br_tag = 94,
  sym__b_tag = 95,
  sym__r_acr_bgn = 96,
  sym__br_acr_bgn = 97,
  sym__b_acr_bgn = 98,
  sym__r_acr_ctn = 99,
  sym__r_als_bgn = 100,
  sym__br_als_bgn = 101,
  sym__b_als_bgn = 102,
  sym__r_als_ctn = 103,
  sym__bl = 104,
  sym_comment = 105,
  sym_stream = 106,
  sym__doc_w_bgn_w_end_seq = 107,
  sym__doc_w_bgn_wo_end_seq = 108,
  sym__doc_wo_bgn_w_end_seq = 109,
  sym__doc_wo_bgn_wo_end_seq = 110,
  sym__doc_w_bgn_w_end = 111,
  sym__doc_w_bgn_wo_end = 112,
  sym__doc_wo_bgn_w_end = 113,
  sym__doc_wo_bgn_wo_end = 114,
  sym__bgn_imp_doc = 115,
  sym__drs_doc = 116,
  sym__exp_doc = 117,
  sym__imp_doc = 118,
  sym__drs_doc_end = 119,
  sym__exp_doc_end = 120,
  sym__imp_doc_end = 121,
  sym__bgn_imp_doc_end = 122,
  sym__doc_end = 123,
  sym__exp_doc_tal = 124,
  sym__s_dir = 125,
  sym__s_dir_yml = 126,
  sym__s_dir_tag = 127,
  sym__s_dir_rsv = 128,
  sym__r_prp_val = 129,
  sym__br_prp_val = 130,
  sym__r_sgl_prp_val = 131,
  sym__br_sgl_prp_val = 132,
  sym__b_sgl_prp_val = 133,
  sym__r_prp = 134,
  sym__br_prp = 135,
  sym__r_sgl_prp = 136,
  sym__br_sgl_prp = 137,
  sym__b_sgl_prp = 138,
  sym__r_blk_seq_r_val = 139,
  sym__r_blk_seq_br_val = 140,
  sym__br_blk_seq_val = 141,
  sym__r_blk_seq_spc_val = 142,
  sym__br_blk_seq_spc_val = 143,
  sym__b_blk_seq_spc_val = 144,
  sym__r_blk_seq = 145,
  sym__br_blk_seq = 146,
  sym__b_blk_seq_spc = 147,
  sym__r_blk_seq_itm = 148,
  sym__br_blk_seq_itm = 149,
  sym__b_blk_seq_itm = 150,
  sym__blk_seq_itm_tal = 151,
  sym__r_blk_map_r_val = 152,
  sym__r_blk_map_br_val = 153,
  sym__br_blk_map_val = 154,
  sym__r_blk_map = 155,
  sym__br_blk_map = 156,
  sym__r_blk_map_itm = 157,
  sym__br_blk_map_itm = 158,
  sym__b_blk_map_itm = 159,
  sym__r_blk_exp_itm = 160,
  sym__br_blk_exp_itm = 161,
  sym__b_blk_exp_itm = 162,
  sym__r_blk_key_itm = 163,
  sym__br_blk_key_itm = 164,
  sym__b_blk_key_itm = 165,
  sym__r_blk_val_itm = 166,
  sym__br_blk_val_itm = 167,
  sym__b_blk_val_itm = 168,
  sym__r_blk_imp_itm_key_only = 169,
  sym__br_blk_imp_itm_key_only = 170,
  sym__b_blk_imp_itm_key_only = 171,
  sym__blk_exp_itm_tal = 172,
  sym__blk_imp_itm_tal = 173,
  sym__r_blk_str_val = 174,
  sym__br_blk_str_val = 175,
  sym__r_blk_str = 176,
  sym__br_blk_str = 177,
  sym__r_flw_seq_val = 178,
  sym__br_flw_seq_val = 179,
  sym__r_sgl_flw_seq_val = 180,
  sym__br_sgl_flw_seq_val = 181,
  sym__b_sgl_flw_seq_val = 182,
  sym__r_flw_seq = 183,
  sym__br_flw_seq = 184,
  sym__r_sgl_flw_seq = 185,
  sym__br_sgl_flw_seq = 186,
  sym__b_sgl_flw_seq = 187,
  sym__flw_seq_tal = 188,
  sym__sgl_flw_seq_tal = 189,
  sym__r_flw_map_val = 190,
  sym__br_flw_map_val = 191,
  sym__r_sgl_flw_map_val = 192,
  sym__br_sgl_flw_map_val = 193,
  sym__b_sgl_flw_map_val = 194,
  sym__r_flw_map = 195,
  sym__br_flw_map = 196,
  sym__r_sgl_flw_map = 197,
  sym__br_sgl_flw_map = 198,
  sym__b_sgl_flw_map = 199,
  sym__flw_map_tal = 200,
  sym__sgl_flw_map_tal = 201,
  sym__r_flw_seq_dat = 202,
  sym__br_flw_seq_dat = 203,
  sym__r_flw_map_dat = 204,
  sym__br_flw_map_dat = 205,
  sym__r_sgl_flw_col_dat = 206,
  sym__flw_seq_dat_rpt = 207,
  sym__flw_map_dat_rpt = 208,
  sym__sgl_flw_col_dat_rpt = 209,
  sym__r_flw_seq_itm = 210,
  sym__br_flw_seq_itm = 211,
  sym__r_flw_map_itm = 212,
  sym__br_flw_map_itm = 213,
  sym__r_sgl_flw_col_itm = 214,
  sym__r_flw_exp_par = 215,
  sym__br_flw_exp_par = 216,
  sym__r_sgl_flw_exp_par = 217,
  sym__r_flw_imp_r_par = 218,
  sym__r_flw_imp_br_par = 219,
  sym__br_flw_imp_r_par = 220,
  sym__br_flw_imp_br_par = 221,
  sym__r_sgl_flw_imp_par = 222,
  sym__r_flw_jsl_ann_par = 223,
  sym__br_flw_jsl_ann_par = 224,
  sym__r_sgl_flw_jsl_ann_par = 225,
  sym__r_flw_njl_ann_par = 226,
  sym__br_flw_njl_ann_par = 227,
  sym__r_sgl_flw_njl_ann_par = 228,
  sym__flw_ann_par_tal = 229,
  sym__sgl_flw_ann_par_tal = 230,
  sym__r_dqt_str_val = 231,
  sym__br_dqt_str_val = 232,
  sym__r_sgl_dqt_str_val = 233,
  sym__br_sgl_dqt_str_val = 234,
  sym__b_sgl_dqt_str_val = 235,
  sym__r_dqt_str = 236,
  sym__br_dqt_str = 237,
  sym__r_sgl_dqt_str = 238,
  sym__br_sgl_dqt_str = 239,
  sym__b_sgl_dqt_str = 240,
  aux_sym__r_sgl_dqt_ctn = 241,
  sym__br_mtl_dqt_ctn = 242,
  sym__r_sqt_str_val = 243,
  sym__br_sqt_str_val = 244,
  sym__r_sgl_sqt_str_val = 245,
  sym__br_sgl_sqt_str_val = 246,
  sym__b_sgl_sqt_str_val = 247,
  sym__r_sqt_str = 248,
  sym__br_sqt_str = 249,
  sym__r_sgl_sqt_str = 250,
  sym__br_sgl_sqt_str = 251,
  sym__b_sgl_sqt_str = 252,
  aux_sym__r_sgl_sqt_ctn = 253,
  sym__br_mtl_sqt_ctn = 254,
  sym__r_pln_blk_val = 255,
  sym__br_pln_blk_val = 256,
  sym__r_sgl_pln_blk_val = 257,
  sym__br_sgl_pln_blk_val = 258,
  sym__b_sgl_pln_blk_val = 259,
  sym__r_pln_flw_val = 260,
  sym__br_pln_flw_val = 261,
  sym__r_sgl_pln_flw_val = 262,
  sym__r_sgl_pln_blk = 263,
  sym__br_sgl_pln_blk = 264,
  sym__b_sgl_pln_blk = 265,
  sym__r_sgl_pln_flw = 266,
  sym__br_sgl_pln_flw = 267,
  sym__r_mtl_pln_blk = 268,
  sym__br_mtl_pln_blk = 269,
  sym__r_mtl_pln_flw = 270,
  sym__br_mtl_pln_flw = 271,
  sym__r_als_val = 272,
  sym__br_als_val = 273,
  sym__b_als_val = 274,
  sym__r_als = 275,
  sym__br_als = 276,
  sym__b_als = 277,
  sym__r_acr = 278,
  sym__br_acr = 279,
  sym__b_acr = 280,
  aux_sym__drs_doc_repeat1 = 281,
  aux_sym__s_dir_rsv_repeat1 = 282,
  aux_sym__r_blk_seq_repeat1 = 283,
  aux_sym__r_blk_map_repeat1 = 284,
  aux_sym__r_blk_str_repeat1 = 285,
  aux_sym__r_flw_seq_dat_repeat1 = 286,
  aux_sym__r_flw_map_dat_repeat1 = 287,
  aux_sym__r_sgl_flw_col_dat_repeat1 = 288,
  aux_sym__r_dqt_str_repeat1 = 289,
  aux_sym__br_mtl_dqt_ctn_repeat1 = 290,
  aux_sym__r_sqt_str_repeat1 = 291,
  aux_sym__br_mtl_sqt_ctn_repeat1 = 292,
};

static const char * const ts_symbol_names[] = {
  [ts_builtin_sym_end] = "end",
  [sym__eof] = "_eof",
  [sym__s_dir_yml_bgn] = "_s_dir_yml_bgn",
  [sym__r_dir_yml_ver] = "yaml_version",
  [sym__s_dir_tag_bgn] = "_s_dir_tag_bgn",
  [sym__r_dir_tag_hdl] = "tag_handle",
  [sym__r_dir_tag_pfx] = "tag_prefix",
  [sym__s_dir_rsv_bgn] = "directive_name",
  [sym__r_dir_rsv_prm] = "directive_parameter",
  [sym__s_drs_end] = "---",
  [sym__s_doc_end] = "...",
  [sym__r_blk_seq_bgn] = "-",
  [sym__br_blk_seq_bgn] = "-",
  [sym__b_blk_seq_bgn] = "-",
  [sym__r_blk_key_bgn] = "\?",
  [sym__br_blk_key_bgn] = "\?",
  [sym__b_blk_key_bgn] = "\?",
  [sym__r_blk_val_bgn] = ":",
  [sym__br_blk_val_bgn] = ":",
  [sym__b_blk_val_bgn] = ":",
  [sym__r_blk_imp_bgn] = ":",
  [sym__r_blk_lit_bgn] = "|",
  [sym__br_blk_lit_bgn] = "|",
  [sym__r_blk_fld_bgn] = ">",
  [sym__br_blk_fld_bgn] = ">",
  [sym__br_blk_str_ctn] = "_br_blk_str_ctn",
  [sym__r_flw_seq_bgn] = "[",
  [sym__br_flw_seq_bgn] = "[",
  [sym__b_flw_seq_bgn] = "[",
  [sym__r_flw_seq_end] = "]",
  [sym__br_flw_seq_end] = "]",
  [sym__r_flw_map_bgn] = "{",
  [sym__br_flw_map_bgn] = "{",
  [sym__b_flw_map_bgn] = "{",
  [sym__r_flw_map_end] = "}",
  [sym__br_flw_map_end] = "}",
  [sym__r_flw_sep_bgn] = ",",
  [sym__br_flw_sep_bgn] = ",",
  [sym__r_flw_key_bgn] = "\?",
  [sym__br_flw_key_bgn] = "\?",
  [sym__r_flw_jsv_bgn] = ":",
  [sym__br_flw_jsv_bgn] = ":",
  [sym__r_flw_njv_bgn] = ":",
  [sym__br_flw_njv_bgn] = ":",
  [sym__r_dqt_str_bgn] = "\"",
  [sym__br_dqt_str_bgn] = "\"",
  [sym__b_dqt_str_bgn] = "\"",
  [sym__r_dqt_str_ctn] = "_r_dqt_str_ctn",
  [sym__br_dqt_str_ctn] = "_br_dqt_str_ctn",
  [sym__r_dqt_esc_nwl] = "escape_sequence",
  [sym__br_dqt_esc_nwl] = "escape_sequence",
  [sym__r_dqt_esc_seq] = "escape_sequence",
  [sym__br_dqt_esc_seq] = "escape_sequence",
  [sym__r_dqt_str_end] = "\"",
  [sym__br_dqt_str_end] = "\"",
  [sym__r_sqt_str_bgn] = "'",
  [sym__br_sqt_str_bgn] = "'",
  [sym__b_sqt_str_bgn] = "'",
  [sym__r_sqt_str_ctn] = "_r_sqt_str_ctn",
  [sym__br_sqt_str_ctn] = "_br_sqt_str_ctn",
  [sym__r_sqt_esc_sqt] = "escape_sequence",
  [sym__br_sqt_esc_sqt] = "escape_sequence",
  [sym__r_sqt_str_end] = "'",
  [sym__br_sqt_str_end] = "'",
  [sym__r_sgl_pln_nul_blk] = "null_scalar",
  [sym__br_sgl_pln_nul_blk] = "null_scalar",
  [sym__b_sgl_pln_nul_blk] = "null_scalar",
  [sym__r_sgl_pln_nul_flw] = "null_scalar",
  [sym__br_sgl_pln_nul_flw] = "null_scalar",
  [sym__r_sgl_pln_bol_blk] = "boolean_scalar",
  [sym__br_sgl_pln_bol_blk] = "boolean_scalar",
  [sym__b_sgl_pln_bol_blk] = "boolean_scalar",
  [sym__r_sgl_pln_bol_flw] = "boolean_scalar",
  [sym__br_sgl_pln_bol_flw] = "boolean_scalar",
  [sym__r_sgl_pln_int_blk] = "integer_scalar",
  [sym__br_sgl_pln_int_blk] = "integer_scalar",
  [sym__b_sgl_pln_int_blk] = "integer_scalar",
  [sym__r_sgl_pln_int_flw] = "integer_scalar",
  [sym__br_sgl_pln_int_flw] = "integer_scalar",
  [sym__r_sgl_pln_flt_blk] = "float_scalar",
  [sym__br_sgl_pln_flt_blk] = "float_scalar",
  [sym__b_sgl_pln_flt_blk] = "float_scalar",
  [sym__r_sgl_pln_flt_flw] = "float_scalar",
  [sym__br_sgl_pln_flt_flw] = "float_scalar",
  [sym__r_sgl_pln_str_blk] = "string_scalar",
  [sym__br_sgl_pln_str_blk] = "string_scalar",
  [sym__b_sgl_pln_str_blk] = "string_scalar",
  [sym__r_sgl_pln_str_flw] = "string_scalar",
  [sym__br_sgl_pln_str_flw] = "string_scalar",
  [sym__r_mtl_pln_str_blk] = "string_scalar",
  [sym__br_mtl_pln_str_blk] = "string_scalar",
  [sym__r_mtl_pln_str_flw] = "string_scalar",
  [sym__br_mtl_pln_str_flw] = "string_scalar",
  [sym__r_tag] = "tag",
  [sym__br_tag] = "tag",
  [sym__b_tag] = "tag",
  [sym__r_acr_bgn] = "&",
  [sym__br_acr_bgn] = "&",
  [sym__b_acr_bgn] = "&",
  [sym__r_acr_ctn] = "anchor_name",
  [sym__r_als_bgn] = "*",
  [sym__br_als_bgn] = "*",
  [sym__b_als_bgn] = "*",
  [sym__r_als_ctn] = "alias_name",
  [sym__bl] = "_bl",
  [sym_comment] = "comment",
  [sym_stream] = "stream",
  [sym__doc_w_bgn_w_end_seq] = "_doc_w_bgn_w_end_seq",
  [sym__doc_w_bgn_wo_end_seq] = "_doc_w_bgn_wo_end_seq",
  [sym__doc_wo_bgn_w_end_seq] = "_doc_wo_bgn_w_end_seq",
  [sym__doc_wo_bgn_wo_end_seq] = "_doc_wo_bgn_wo_end_seq",
  [sym__doc_w_bgn_w_end] = "_doc_w_bgn_w_end",
  [sym__doc_w_bgn_wo_end] = "_doc_w_bgn_wo_end",
  [sym__doc_wo_bgn_w_end] = "_doc_wo_bgn_w_end",
  [sym__doc_wo_bgn_wo_end] = "_doc_wo_bgn_wo_end",
  [sym__bgn_imp_doc] = "_bgn_imp_doc",
  [sym__drs_doc] = "_drs_doc",
  [sym__exp_doc] = "_exp_doc",
  [sym__imp_doc] = "_imp_doc",
  [sym__drs_doc_end] = "document",
  [sym__exp_doc_end] = "document",
  [sym__imp_doc_end] = "document",
  [sym__bgn_imp_doc_end] = "document",
  [sym__doc_end] = "document",
  [sym__exp_doc_tal] = "_exp_doc_tal",
  [sym__s_dir] = "_s_dir",
  [sym__s_dir_yml] = "yaml_directive",
  [sym__s_dir_tag] = "tag_directive",
  [sym__s_dir_rsv] = "reserved_directive",
  [sym__r_prp_val] = "flow_node",
  [sym__br_prp_val] = "flow_node",
  [sym__r_sgl_prp_val] = "flow_node",
  [sym__br_sgl_prp_val] = "flow_node",
  [sym__b_sgl_prp_val] = "flow_node",
  [sym__r_prp] = "_r_prp",
  [sym__br_prp] = "_br_prp",
  [sym__r_sgl_prp] = "_r_sgl_prp",
  [sym__br_sgl_prp] = "_br_sgl_prp",
  [sym__b_sgl_prp] = "_b_sgl_prp",
  [sym__r_blk_seq_r_val] = "block_node",
  [sym__r_blk_seq_br_val] = "block_node",
  [sym__br_blk_seq_val] = "block_node",
  [sym__r_blk_seq_spc_val] = "block_node",
  [sym__br_blk_seq_spc_val] = "block_node",
  [sym__b_blk_seq_spc_val] = "block_node",
  [sym__r_blk_seq] = "block_sequence",
  [sym__br_blk_seq] = "block_sequence",
  [sym__b_blk_seq_spc] = "block_sequence",
  [sym__r_blk_seq_itm] = "block_sequence_item",
  [sym__br_blk_seq_itm] = "block_sequence_item",
  [sym__b_blk_seq_itm] = "block_sequence_item",
  [sym__blk_seq_itm_tal] = "_blk_seq_itm_tal",
  [sym__r_blk_map_r_val] = "block_node",
  [sym__r_blk_map_br_val] = "block_node",
  [sym__br_blk_map_val] = "block_node",
  [sym__r_blk_map] = "block_mapping",
  [sym__br_blk_map] = "block_mapping",
  [sym__r_blk_map_itm] = "_r_blk_map_itm",
  [sym__br_blk_map_itm] = "_br_blk_map_itm",
  [sym__b_blk_map_itm] = "_b_blk_map_itm",
  [sym__r_blk_exp_itm] = "block_mapping_pair",
  [sym__br_blk_exp_itm] = "block_mapping_pair",
  [sym__b_blk_exp_itm] = "block_mapping_pair",
  [sym__r_blk_key_itm] = "_r_blk_key_itm",
  [sym__br_blk_key_itm] = "_br_blk_key_itm",
  [sym__b_blk_key_itm] = "_b_blk_key_itm",
  [sym__r_blk_val_itm] = "_r_blk_val_itm",
  [sym__br_blk_val_itm] = "_br_blk_val_itm",
  [sym__b_blk_val_itm] = "_b_blk_val_itm",
  [sym__r_blk_imp_itm_key_only] = "block_mapping_pair",
  [sym__br_blk_imp_itm_key_only] = "block_mapping_pair",
  [sym__b_blk_imp_itm_key_only] = "block_mapping_pair",
  [sym__blk_exp_itm_tal] = "_blk_exp_itm_tal",
  [sym__blk_imp_itm_tal] = "_blk_imp_itm_tal",
  [sym__r_blk_str_val] = "block_node",
  [sym__br_blk_str_val] = "block_node",
  [sym__r_blk_str] = "block_scalar",
  [sym__br_blk_str] = "block_scalar",
  [sym__r_flw_seq_val] = "flow_node",
  [sym__br_flw_seq_val] = "flow_node",
  [sym__r_sgl_flw_seq_val] = "flow_node",
  [sym__br_sgl_flw_seq_val] = "flow_node",
  [sym__b_sgl_flw_seq_val] = "flow_node",
  [sym__r_flw_seq] = "flow_sequence",
  [sym__br_flw_seq] = "flow_sequence",
  [sym__r_sgl_flw_seq] = "flow_sequence",
  [sym__br_sgl_flw_seq] = "flow_sequence",
  [sym__b_sgl_flw_seq] = "flow_sequence",
  [sym__flw_seq_tal] = "_flw_seq_tal",
  [sym__sgl_flw_seq_tal] = "_sgl_flw_seq_tal",
  [sym__r_flw_map_val] = "flow_node",
  [sym__br_flw_map_val] = "flow_node",
  [sym__r_sgl_flw_map_val] = "flow_node",
  [sym__br_sgl_flw_map_val] = "flow_node",
  [sym__b_sgl_flw_map_val] = "flow_node",
  [sym__r_flw_map] = "flow_mapping",
  [sym__br_flw_map] = "flow_mapping",
  [sym__r_sgl_flw_map] = "flow_mapping",
  [sym__br_sgl_flw_map] = "flow_mapping",
  [sym__b_sgl_flw_map] = "flow_mapping",
  [sym__flw_map_tal] = "_flw_map_tal",
  [sym__sgl_flw_map_tal] = "_sgl_flw_map_tal",
  [sym__r_flw_seq_dat] = "_r_flw_seq_dat",
  [sym__br_flw_seq_dat] = "_br_flw_seq_dat",
  [sym__r_flw_map_dat] = "_r_flw_map_dat",
  [sym__br_flw_map_dat] = "_br_flw_map_dat",
  [sym__r_sgl_flw_col_dat] = "_r_sgl_flw_col_dat",
  [sym__flw_seq_dat_rpt] = "_flw_seq_dat_rpt",
  [sym__flw_map_dat_rpt] = "_flw_map_dat_rpt",
  [sym__sgl_flw_col_dat_rpt] = "_sgl_flw_col_dat_rpt",
  [sym__r_flw_seq_itm] = "_r_flw_seq_itm",
  [sym__br_flw_seq_itm] = "_br_flw_seq_itm",
  [sym__r_flw_map_itm] = "_r_flw_map_itm",
  [sym__br_flw_map_itm] = "_br_flw_map_itm",
  [sym__r_sgl_flw_col_itm] = "_r_sgl_flw_col_itm",
  [sym__r_flw_exp_par] = "flow_pair",
  [sym__br_flw_exp_par] = "flow_pair",
  [sym__r_sgl_flw_exp_par] = "flow_pair",
  [sym__r_flw_imp_r_par] = "_r_flw_imp_r_par",
  [sym__r_flw_imp_br_par] = "_r_flw_imp_br_par",
  [sym__br_flw_imp_r_par] = "_br_flw_imp_r_par",
  [sym__br_flw_imp_br_par] = "_br_flw_imp_br_par",
  [sym__r_sgl_flw_imp_par] = "_r_sgl_flw_imp_par",
  [sym__r_flw_jsl_ann_par] = "_r_flw_jsl_ann_par",
  [sym__br_flw_jsl_ann_par] = "_br_flw_jsl_ann_par",
  [sym__r_sgl_flw_jsl_ann_par] = "_r_sgl_flw_jsl_ann_par",
  [sym__r_flw_njl_ann_par] = "_r_flw_njl_ann_par",
  [sym__br_flw_njl_ann_par] = "_br_flw_njl_ann_par",
  [sym__r_sgl_flw_njl_ann_par] = "_r_sgl_flw_njl_ann_par",
  [sym__flw_ann_par_tal] = "_flw_ann_par_tal",
  [sym__sgl_flw_ann_par_tal] = "_sgl_flw_ann_par_tal",
  [sym__r_dqt_str_val] = "flow_node",
  [sym__br_dqt_str_val] = "flow_node",
  [sym__r_sgl_dqt_str_val] = "flow_node",
  [sym__br_sgl_dqt_str_val] = "flow_node",
  [sym__b_sgl_dqt_str_val] = "flow_node",
  [sym__r_dqt_str] = "double_quote_scalar",
  [sym__br_dqt_str] = "double_quote_scalar",
  [sym__r_sgl_dqt_str] = "double_quote_scalar",
  [sym__br_sgl_dqt_str] = "double_quote_scalar",
  [sym__b_sgl_dqt_str] = "double_quote_scalar",
  [aux_sym__r_sgl_dqt_ctn] = "_r_sgl_dqt_ctn",
  [sym__br_mtl_dqt_ctn] = "_br_mtl_dqt_ctn",
  [sym__r_sqt_str_val] = "flow_node",
  [sym__br_sqt_str_val] = "flow_node",
  [sym__r_sgl_sqt_str_val] = "flow_node",
  [sym__br_sgl_sqt_str_val] = "flow_node",
  [sym__b_sgl_sqt_str_val] = "flow_node",
  [sym__r_sqt_str] = "single_quote_scalar",
  [sym__br_sqt_str] = "single_quote_scalar",
  [sym__r_sgl_sqt_str] = "single_quote_scalar",
  [sym__br_sgl_sqt_str] = "single_quote_scalar",
  [sym__b_sgl_sqt_str] = "single_quote_scalar",
  [aux_sym__r_sgl_sqt_ctn] = "_r_sgl_sqt_ctn",
  [sym__br_mtl_sqt_ctn] = "_br_mtl_sqt_ctn",
  [sym__r_pln_blk_val] = "flow_node",
  [sym__br_pln_blk_val] = "flow_node",
  [sym__r_sgl_pln_blk_val] = "flow_node",
  [sym__br_sgl_pln_blk_val] = "flow_node",
  [sym__b_sgl_pln_blk_val] = "flow_node",
  [sym__r_pln_flw_val] = "flow_node",
  [sym__br_pln_flw_val] = "flow_node",
  [sym__r_sgl_pln_flw_val] = "flow_node",
  [sym__r_sgl_pln_blk] = "plain_scalar",
  [sym__br_sgl_pln_blk] = "plain_scalar",
  [sym__b_sgl_pln_blk] = "plain_scalar",
  [sym__r_sgl_pln_flw] = "plain_scalar",
  [sym__br_sgl_pln_flw] = "plain_scalar",
  [sym__r_mtl_pln_blk] = "plain_scalar",
  [sym__br_mtl_pln_blk] = "plain_scalar",
  [sym__r_mtl_pln_flw] = "plain_scalar",
  [sym__br_mtl_pln_flw] = "plain_scalar",
  [sym__r_als_val] = "flow_node",
  [sym__br_als_val] = "flow_node",
  [sym__b_als_val] = "flow_node",
  [sym__r_als] = "alias",
  [sym__br_als] = "alias",
  [sym__b_als] = "alias",
  [sym__r_acr] = "anchor",
  [sym__br_acr] = "anchor",
  [sym__b_acr] = "anchor",
  [aux_sym__drs_doc_repeat1] = "_drs_doc_repeat1",
  [aux_sym__s_dir_rsv_repeat1] = "_s_dir_rsv_repeat1",
  [aux_sym__r_blk_seq_repeat1] = "_r_blk_seq_repeat1",
  [aux_sym__r_blk_map_repeat1] = "_r_blk_map_repeat1",
  [aux_sym__r_blk_str_repeat1] = "_r_blk_str_repeat1",
  [aux_sym__r_flw_seq_dat_repeat1] = "_r_flw_seq_dat_repeat1",
  [aux_sym__r_flw_map_dat_repeat1] = "_r_flw_map_dat_repeat1",
  [aux_sym__r_sgl_flw_col_dat_repeat1] = "_r_sgl_flw_col_dat_repeat1",
  [aux_sym__r_dqt_str_repeat1] = "_r_dqt_str_repeat1",
  [aux_sym__br_mtl_dqt_ctn_repeat1] = "_br_mtl_dqt_ctn_repeat1",
  [aux_sym__r_sqt_str_repeat1] = "_r_sqt_str_repeat1",
  [aux_sym__br_mtl_sqt_ctn_repeat1] = "_br_mtl_sqt_ctn_repeat1",
};

static const TSSymbol ts_symbol_map[] = {
  [ts_builtin_sym_end] = ts_builtin_sym_end,
  [sym__eof] = sym__eof,
  [sym__s_dir_yml_bgn] = sym__s_dir_yml_bgn,
  [sym__r_dir_yml_ver] = sym__r_dir_yml_ver,
  [sym__s_dir_tag_bgn] = sym__s_dir_tag_bgn,
  [sym__r_dir_tag_hdl] = sym__r_dir_tag_hdl,
  [sym__r_dir_tag_pfx] = sym__r_dir_tag_pfx,
  [sym__s_dir_rsv_bgn] = sym__s_dir_rsv_bgn,
  [sym__r_dir_rsv_prm] = sym__r_dir_rsv_prm,
  [sym__s_drs_end] = sym__s_drs_end,
  [sym__s_doc_end] = sym__s_doc_end,
  [sym__r_blk_seq_bgn] = sym__r_blk_seq_bgn,
  [sym__br_blk_seq_bgn] = sym__r_blk_seq_bgn,
  [sym__b_blk_seq_bgn] = sym__r_blk_seq_bgn,
  [sym__r_blk_key_bgn] = sym__r_blk_key_bgn,
  [sym__br_blk_key_bgn] = sym__r_blk_key_bgn,
  [sym__b_blk_key_bgn] = sym__r_blk_key_bgn,
  [sym__r_blk_val_bgn] = sym__r_blk_val_bgn,
  [sym__br_blk_val_bgn] = sym__r_blk_val_bgn,
  [sym__b_blk_val_bgn] = sym__r_blk_val_bgn,
  [sym__r_blk_imp_bgn] = sym__r_blk_val_bgn,
  [sym__r_blk_lit_bgn] = sym__r_blk_lit_bgn,
  [sym__br_blk_lit_bgn] = sym__r_blk_lit_bgn,
  [sym__r_blk_fld_bgn] = sym__r_blk_fld_bgn,
  [sym__br_blk_fld_bgn] = sym__r_blk_fld_bgn,
  [sym__br_blk_str_ctn] = sym__br_blk_str_ctn,
  [sym__r_flw_seq_bgn] = sym__r_flw_seq_bgn,
  [sym__br_flw_seq_bgn] = sym__r_flw_seq_bgn,
  [sym__b_flw_seq_bgn] = sym__r_flw_seq_bgn,
  [sym__r_flw_seq_end] = sym__r_flw_seq_end,
  [sym__br_flw_seq_end] = sym__r_flw_seq_end,
  [sym__r_flw_map_bgn] = sym__r_flw_map_bgn,
  [sym__br_flw_map_bgn] = sym__r_flw_map_bgn,
  [sym__b_flw_map_bgn] = sym__r_flw_map_bgn,
  [sym__r_flw_map_end] = sym__r_flw_map_end,
  [sym__br_flw_map_end] = sym__r_flw_map_end,
  [sym__r_flw_sep_bgn] = sym__r_flw_sep_bgn,
  [sym__br_flw_sep_bgn] = sym__r_flw_sep_bgn,
  [sym__r_flw_key_bgn] = sym__r_blk_key_bgn,
  [sym__br_flw_key_bgn] = sym__r_blk_key_bgn,
  [sym__r_flw_jsv_bgn] = sym__r_blk_val_bgn,
  [sym__br_flw_jsv_bgn] = sym__r_blk_val_bgn,
  [sym__r_flw_njv_bgn] = sym__r_blk_val_bgn,
  [sym__br_flw_njv_bgn] = sym__r_blk_val_bgn,
  [sym__r_dqt_str_bgn] = sym__r_dqt_str_bgn,
  [sym__br_dqt_str_bgn] = sym__r_dqt_str_bgn,
  [sym__b_dqt_str_bgn] = sym__r_dqt_str_bgn,
  [sym__r_dqt_str_ctn] = sym__r_dqt_str_ctn,
  [sym__br_dqt_str_ctn] = sym__br_dqt_str_ctn,
  [sym__r_dqt_esc_nwl] = sym__r_dqt_esc_nwl,
  [sym__br_dqt_esc_nwl] = sym__r_dqt_esc_nwl,
  [sym__r_dqt_esc_seq] = sym__r_dqt_esc_nwl,
  [sym__br_dqt_esc_seq] = sym__r_dqt_esc_nwl,
  [sym__r_dqt_str_end] = sym__r_dqt_str_bgn,
  [sym__br_dqt_str_end] = sym__r_dqt_str_bgn,
  [sym__r_sqt_str_bgn] = sym__r_sqt_str_bgn,
  [sym__br_sqt_str_bgn] = sym__r_sqt_str_bgn,
  [sym__b_sqt_str_bgn] = sym__r_sqt_str_bgn,
  [sym__r_sqt_str_ctn] = sym__r_sqt_str_ctn,
  [sym__br_sqt_str_ctn] = sym__br_sqt_str_ctn,
  [sym__r_sqt_esc_sqt] = sym__r_dqt_esc_nwl,
  [sym__br_sqt_esc_sqt] = sym__r_dqt_esc_nwl,
  [sym__r_sqt_str_end] = sym__r_sqt_str_bgn,
  [sym__br_sqt_str_end] = sym__r_sqt_str_bgn,
  [sym__r_sgl_pln_nul_blk] = sym__r_sgl_pln_nul_blk,
  [sym__br_sgl_pln_nul_blk] = sym__r_sgl_pln_nul_blk,
  [sym__b_sgl_pln_nul_blk] = sym__r_sgl_pln_nul_blk,
  [sym__r_sgl_pln_nul_flw] = sym__r_sgl_pln_nul_blk,
  [sym__br_sgl_pln_nul_flw] = sym__r_sgl_pln_nul_blk,
  [sym__r_sgl_pln_bol_blk] = sym__r_sgl_pln_bol_blk,
  [sym__br_sgl_pln_bol_blk] = sym__r_sgl_pln_bol_blk,
  [sym__b_sgl_pln_bol_blk] = sym__r_sgl_pln_bol_blk,
  [sym__r_sgl_pln_bol_flw] = sym__r_sgl_pln_bol_blk,
  [sym__br_sgl_pln_bol_flw] = sym__r_sgl_pln_bol_blk,
  [sym__r_sgl_pln_int_blk] = sym__r_sgl_pln_int_blk,
  [sym__br_sgl_pln_int_blk] = sym__r_sgl_pln_int_blk,
  [sym__b_sgl_pln_int_blk] = sym__r_sgl_pln_int_blk,
  [sym__r_sgl_pln_int_flw] = sym__r_sgl_pln_int_blk,
  [sym__br_sgl_pln_int_flw] = sym__r_sgl_pln_int_blk,
  [sym__r_sgl_pln_flt_blk] = sym__r_sgl_pln_flt_blk,
  [sym__br_sgl_pln_flt_blk] = sym__r_sgl_pln_flt_blk,
  [sym__b_sgl_pln_flt_blk] = sym__r_sgl_pln_flt_blk,
  [sym__r_sgl_pln_flt_flw] = sym__r_sgl_pln_flt_blk,
  [sym__br_sgl_pln_flt_flw] = sym__r_sgl_pln_flt_blk,
  [sym__r_sgl_pln_str_blk] = sym__r_sgl_pln_str_blk,
  [sym__br_sgl_pln_str_blk] = sym__r_sgl_pln_str_blk,
  [sym__b_sgl_pln_str_blk] = sym__r_sgl_pln_str_blk,
  [sym__r_sgl_pln_str_flw] = sym__r_sgl_pln_str_blk,
  [sym__br_sgl_pln_str_flw] = sym__r_sgl_pln_str_blk,
  [sym__r_mtl_pln_str_blk] = sym__r_sgl_pln_str_blk,
  [sym__br_mtl_pln_str_blk] = sym__r_sgl_pln_str_blk,
  [sym__r_mtl_pln_str_flw] = sym__r_sgl_pln_str_blk,
  [sym__br_mtl_pln_str_flw] = sym__r_sgl_pln_str_blk,
  [sym__r_tag] = sym__r_tag,
  [sym__br_tag] = sym__r_tag,
  [sym__b_tag] = sym__r_tag,
  [sym__r_acr_bgn] = sym__r_acr_bgn,
  [sym__br_acr_bgn] = sym__r_acr_bgn,
  [sym__b_acr_bgn] = sym__r_acr_bgn,
  [sym__r_acr_ctn] = sym__r_acr_ctn,
  [sym__r_als_bgn] = sym__r_als_bgn,
  [sym__br_als_bgn] = sym__r_als_bgn,
  [sym__b_als_bgn] = sym__r_als_bgn,
  [sym__r_als_ctn] = sym__r_als_ctn,
  [sym__bl] = sym__bl,
  [sym_comment] = sym_comment,
  [sym_stream] = sym_stream,
  [sym__doc_w_bgn_w_end_seq] = sym__doc_w_bgn_w_end_seq,
  [sym__doc_w_bgn_wo_end_seq] = sym__doc_w_bgn_wo_end_seq,
  [sym__doc_wo_bgn_w_end_seq] = sym__doc_wo_bgn_w_end_seq,
  [sym__doc_wo_bgn_wo_end_seq] = sym__doc_wo_bgn_wo_end_seq,
  [sym__doc_w_bgn_w_end] = sym__doc_w_bgn_w_end,
  [sym__doc_w_bgn_wo_end] = sym__doc_w_bgn_wo_end,
  [sym__doc_wo_bgn_w_end] = sym__doc_wo_bgn_w_end,
  [sym__doc_wo_bgn_wo_end] = sym__doc_wo_bgn_wo_end,
  [sym__bgn_imp_doc] = sym__bgn_imp_doc,
  [sym__drs_doc] = sym__drs_doc,
  [sym__exp_doc] = sym__exp_doc,
  [sym__imp_doc] = sym__imp_doc,
  [sym__drs_doc_end] = sym__drs_doc_end,
  [sym__exp_doc_end] = sym__drs_doc_end,
  [sym__imp_doc_end] = sym__drs_doc_end,
  [sym__bgn_imp_doc_end] = sym__drs_doc_end,
  [sym__doc_end] = sym__drs_doc_end,
  [sym__exp_doc_tal] = sym__exp_doc_tal,
  [sym__s_dir] = sym__s_dir,
  [sym__s_dir_yml] = sym__s_dir_yml,
  [sym__s_dir_tag] = sym__s_dir_tag,
  [sym__s_dir_rsv] = sym__s_dir_rsv,
  [sym__r_prp_val] = sym__r_prp_val,
  [sym__br_prp_val] = sym__r_prp_val,
  [sym__r_sgl_prp_val] = sym__r_prp_val,
  [sym__br_sgl_prp_val] = sym__r_prp_val,
  [sym__b_sgl_prp_val] = sym__r_prp_val,
  [sym__r_prp] = sym__r_prp,
  [sym__br_prp] = sym__br_prp,
  [sym__r_sgl_prp] = sym__r_sgl_prp,
  [sym__br_sgl_prp] = sym__br_sgl_prp,
  [sym__b_sgl_prp] = sym__b_sgl_prp,
  [sym__r_blk_seq_r_val] = sym__r_blk_seq_r_val,
  [sym__r_blk_seq_br_val] = sym__r_blk_seq_r_val,
  [sym__br_blk_seq_val] = sym__r_blk_seq_r_val,
  [sym__r_blk_seq_spc_val] = sym__r_blk_seq_r_val,
  [sym__br_blk_seq_spc_val] = sym__r_blk_seq_r_val,
  [sym__b_blk_seq_spc_val] = sym__r_blk_seq_r_val,
  [sym__r_blk_seq] = sym__r_blk_seq,
  [sym__br_blk_seq] = sym__r_blk_seq,
  [sym__b_blk_seq_spc] = sym__r_blk_seq,
  [sym__r_blk_seq_itm] = sym__r_blk_seq_itm,
  [sym__br_blk_seq_itm] = sym__r_blk_seq_itm,
  [sym__b_blk_seq_itm] = sym__r_blk_seq_itm,
  [sym__blk_seq_itm_tal] = sym__blk_seq_itm_tal,
  [sym__r_blk_map_r_val] = sym__r_blk_seq_r_val,
  [sym__r_blk_map_br_val] = sym__r_blk_seq_r_val,
  [sym__br_blk_map_val] = sym__r_blk_seq_r_val,
  [sym__r_blk_map] = sym__r_blk_map,
  [sym__br_blk_map] = sym__r_blk_map,
  [sym__r_blk_map_itm] = sym__r_blk_map_itm,
  [sym__br_blk_map_itm] = sym__br_blk_map_itm,
  [sym__b_blk_map_itm] = sym__b_blk_map_itm,
  [sym__r_blk_exp_itm] = sym__r_blk_exp_itm,
  [sym__br_blk_exp_itm] = sym__r_blk_exp_itm,
  [sym__b_blk_exp_itm] = sym__r_blk_exp_itm,
  [sym__r_blk_key_itm] = sym__r_blk_key_itm,
  [sym__br_blk_key_itm] = sym__br_blk_key_itm,
  [sym__b_blk_key_itm] = sym__b_blk_key_itm,
  [sym__r_blk_val_itm] = sym__r_blk_val_itm,
  [sym__br_blk_val_itm] = sym__br_blk_val_itm,
  [sym__b_blk_val_itm] = sym__b_blk_val_itm,
  [sym__r_blk_imp_itm_key_only] = sym__r_blk_exp_itm,
  [sym__br_blk_imp_itm_key_only] = sym__r_blk_exp_itm,
  [sym__b_blk_imp_itm_key_only] = sym__r_blk_exp_itm,
  [sym__blk_exp_itm_tal] = sym__blk_exp_itm_tal,
  [sym__blk_imp_itm_tal] = sym__blk_imp_itm_tal,
  [sym__r_blk_str_val] = sym__r_blk_seq_r_val,
  [sym__br_blk_str_val] = sym__r_blk_seq_r_val,
  [sym__r_blk_str] = sym__r_blk_str,
  [sym__br_blk_str] = sym__r_blk_str,
  [sym__r_flw_seq_val] = sym__r_prp_val,
  [sym__br_flw_seq_val] = sym__r_prp_val,
  [sym__r_sgl_flw_seq_val] = sym__r_prp_val,
  [sym__br_sgl_flw_seq_val] = sym__r_prp_val,
  [sym__b_sgl_flw_seq_val] = sym__r_prp_val,
  [sym__r_flw_seq] = sym__r_flw_seq,
  [sym__br_flw_seq] = sym__r_flw_seq,
  [sym__r_sgl_flw_seq] = sym__r_flw_seq,
  [sym__br_sgl_flw_seq] = sym__r_flw_seq,
  [sym__b_sgl_flw_seq] = sym__r_flw_seq,
  [sym__flw_seq_tal] = sym__flw_seq_tal,
  [sym__sgl_flw_seq_tal] = sym__sgl_flw_seq_tal,
  [sym__r_flw_map_val] = sym__r_prp_val,
  [sym__br_flw_map_val] = sym__r_prp_val,
  [sym__r_sgl_flw_map_val] = sym__r_prp_val,
  [sym__br_sgl_flw_map_val] = sym__r_prp_val,
  [sym__b_sgl_flw_map_val] = sym__r_prp_val,
  [sym__r_flw_map] = sym__r_flw_map,
  [sym__br_flw_map] = sym__r_flw_map,
  [sym__r_sgl_flw_map] = sym__r_flw_map,
  [sym__br_sgl_flw_map] = sym__r_flw_map,
  [sym__b_sgl_flw_map] = sym__r_flw_map,
  [sym__flw_map_tal] = sym__flw_map_tal,
  [sym__sgl_flw_map_tal] = sym__sgl_flw_map_tal,
  [sym__r_flw_seq_dat] = sym__r_flw_seq_dat,
  [sym__br_flw_seq_dat] = sym__br_flw_seq_dat,
  [sym__r_flw_map_dat] = sym__r_flw_map_dat,
  [sym__br_flw_map_dat] = sym__br_flw_map_dat,
  [sym__r_sgl_flw_col_dat] = sym__r_sgl_flw_col_dat,
  [sym__flw_seq_dat_rpt] = sym__flw_seq_dat_rpt,
  [sym__flw_map_dat_rpt] = sym__flw_map_dat_rpt,
  [sym__sgl_flw_col_dat_rpt] = sym__sgl_flw_col_dat_rpt,
  [sym__r_flw_seq_itm] = sym__r_flw_seq_itm,
  [sym__br_flw_seq_itm] = sym__br_flw_seq_itm,
  [sym__r_flw_map_itm] = sym__r_flw_map_itm,
  [sym__br_flw_map_itm] = sym__br_flw_map_itm,
  [sym__r_sgl_flw_col_itm] = sym__r_sgl_flw_col_itm,
  [sym__r_flw_exp_par] = sym__r_flw_exp_par,
  [sym__br_flw_exp_par] = sym__r_flw_exp_par,
  [sym__r_sgl_flw_exp_par] = sym__r_flw_exp_par,
  [sym__r_flw_imp_r_par] = sym__r_flw_imp_r_par,
  [sym__r_flw_imp_br_par] = sym__r_flw_imp_br_par,
  [sym__br_flw_imp_r_par] = sym__br_flw_imp_r_par,
  [sym__br_flw_imp_br_par] = sym__br_flw_imp_br_par,
  [sym__r_sgl_flw_imp_par] = sym__r_sgl_flw_imp_par,
  [sym__r_flw_jsl_ann_par] = sym__r_flw_jsl_ann_par,
  [sym__br_flw_jsl_ann_par] = sym__br_flw_jsl_ann_par,
  [sym__r_sgl_flw_jsl_ann_par] = sym__r_sgl_flw_jsl_ann_par,
  [sym__r_flw_njl_ann_par] = sym__r_flw_njl_ann_par,
  [sym__br_flw_njl_ann_par] = sym__br_flw_njl_ann_par,
  [sym__r_sgl_flw_njl_ann_par] = sym__r_sgl_flw_njl_ann_par,
  [sym__flw_ann_par_tal] = sym__flw_ann_par_tal,
  [sym__sgl_flw_ann_par_tal] = sym__sgl_flw_ann_par_tal,
  [sym__r_dqt_str_val] = sym__r_prp_val,
  [sym__br_dqt_str_val] = sym__r_prp_val,
  [sym__r_sgl_dqt_str_val] = sym__r_prp_val,
  [sym__br_sgl_dqt_str_val] = sym__r_prp_val,
  [sym__b_sgl_dqt_str_val] = sym__r_prp_val,
  [sym__r_dqt_str] = sym__r_dqt_str,
  [sym__br_dqt_str] = sym__r_dqt_str,
  [sym__r_sgl_dqt_str] = sym__r_dqt_str,
  [sym__br_sgl_dqt_str] = sym__r_dqt_str,
  [sym__b_sgl_dqt_str] = sym__r_dqt_str,
  [aux_sym__r_sgl_dqt_ctn] = aux_sym__r_sgl_dqt_ctn,
  [sym__br_mtl_dqt_ctn] = sym__br_mtl_dqt_ctn,
  [sym__r_sqt_str_val] = sym__r_prp_val,
  [sym__br_sqt_str_val] = sym__r_prp_val,
  [sym__r_sgl_sqt_str_val] = sym__r_prp_val,
  [sym__br_sgl_sqt_str_val] = sym__r_prp_val,
  [sym__b_sgl_sqt_str_val] = sym__r_prp_val,
  [sym__r_sqt_str] = sym__r_sqt_str,
  [sym__br_sqt_str] = sym__r_sqt_str,
  [sym__r_sgl_sqt_str] = sym__r_sqt_str,
  [sym__br_sgl_sqt_str] = sym__r_sqt_str,
  [sym__b_sgl_sqt_str] = sym__r_sqt_str,
  [aux_sym__r_sgl_sqt_ctn] = aux_sym__r_sgl_sqt_ctn,
  [sym__br_mtl_sqt_ctn] = sym__br_mtl_sqt_ctn,
  [sym__r_pln_blk_val] = sym__r_prp_val,
  [sym__br_pln_blk_val] = sym__r_prp_val,
  [sym__r_sgl_pln_blk_val] = sym__r_prp_val,
  [sym__br_sgl_pln_blk_val] = sym__r_prp_val,
  [sym__b_sgl_pln_blk_val] = sym__r_prp_val,
  [sym__r_pln_flw_val] = sym__r_prp_val,
  [sym__br_pln_flw_val] = sym__r_prp_val,
  [sym__r_sgl_pln_flw_val] = sym__r_prp_val,
  [sym__r_sgl_pln_blk] = sym__r_sgl_pln_blk,
  [sym__br_sgl_pln_blk] = sym__r_sgl_pln_blk,
  [sym__b_sgl_pln_blk] = sym__r_sgl_pln_blk,
  [sym__r_sgl_pln_flw] = sym__r_sgl_pln_blk,
  [sym__br_sgl_pln_flw] = sym__r_sgl_pln_blk,
  [sym__r_mtl_pln_blk] = sym__r_sgl_pln_blk,
  [sym__br_mtl_pln_blk] = sym__r_sgl_pln_blk,
  [sym__r_mtl_pln_flw] = sym__r_sgl_pln_blk,
  [sym__br_mtl_pln_flw] = sym__r_sgl_pln_blk,
  [sym__r_als_val] = sym__r_prp_val,
  [sym__br_als_val] = sym__r_prp_val,
  [sym__b_als_val] = sym__r_prp_val,
  [sym__r_als] = sym__r_als,
  [sym__br_als] = sym__r_als,
  [sym__b_als] = sym__r_als,
  [sym__r_acr] = sym__r_acr,
  [sym__br_acr] = sym__r_acr,
  [sym__b_acr] = sym__r_acr,
  [aux_sym__drs_doc_repeat1] = aux_sym__drs_doc_repeat1,
  [aux_sym__s_dir_rsv_repeat1] = aux_sym__s_dir_rsv_repeat1,
  [aux_sym__r_blk_seq_repeat1] = aux_sym__r_blk_seq_repeat1,
  [aux_sym__r_blk_map_repeat1] = aux_sym__r_blk_map_repeat1,
  [aux_sym__r_blk_str_repeat1] = aux_sym__r_blk_str_repeat1,
  [aux_sym__r_flw_seq_dat_repeat1] = aux_sym__r_flw_seq_dat_repeat1,
  [aux_sym__r_flw_map_dat_repeat1] = aux_sym__r_flw_map_dat_repeat1,
  [aux_sym__r_sgl_flw_col_dat_repeat1] = aux_sym__r_sgl_flw_col_dat_repeat1,
  [aux_sym__r_dqt_str_repeat1] = aux_sym__r_dqt_str_repeat1,
  [aux_sym__br_mtl_dqt_ctn_repeat1] = aux_sym__br_mtl_dqt_ctn_repeat1,
  [aux_sym__r_sqt_str_repeat1] = aux_sym__r_sqt_str_repeat1,
  [aux_sym__br_mtl_sqt_ctn_repeat1] = aux_sym__br_mtl_sqt_ctn_repeat1,
};

static const TSSymbolMetadata ts_symbol_metadata[] = {
  [ts_builtin_sym_end] = {
    .visible = false,
    .named = true,
  },
  [sym__eof] = {
    .visible = false,
    .named = true,
  },
  [sym__s_dir_yml_bgn] = {
    .visible = false,
    .named = true,
  },
  [sym__r_dir_yml_ver] = {
    .visible = true,
    .named = true,
  },
  [sym__s_dir_tag_bgn] = {
    .visible = false,
    .named = true,
  },
  [sym__r_dir_tag_hdl] = {
    .visible = true,
    .named = true,
  },
  [sym__r_dir_tag_pfx] = {
    .visible = true,
    .named = true,
  },
  [sym__s_dir_rsv_bgn] = {
    .visible = true,
    .named = true,
  },
  [sym__r_dir_rsv_prm] = {
    .visible = true,
    .named = true,
  },
  [sym__s_drs_end] = {
    .visible = true,
    .named = false,
  },
  [sym__s_doc_end] = {
    .visible = true,
    .named = false,
  },
  [sym__r_blk_seq_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_blk_seq_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__b_blk_seq_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_blk_key_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_blk_key_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__b_blk_key_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_blk_val_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_blk_val_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__b_blk_val_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_blk_imp_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_blk_lit_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_blk_lit_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_blk_fld_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_blk_fld_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_blk_str_ctn] = {
    .visible = false,
    .named = true,
  },
  [sym__r_flw_seq_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_flw_seq_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__b_flw_seq_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_flw_seq_end] = {
    .visible = true,
    .named = false,
  },
  [sym__br_flw_seq_end] = {
    .visible = true,
    .named = false,
  },
  [sym__r_flw_map_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_flw_map_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__b_flw_map_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_flw_map_end] = {
    .visible = true,
    .named = false,
  },
  [sym__br_flw_map_end] = {
    .visible = true,
    .named = false,
  },
  [sym__r_flw_sep_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_flw_sep_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_flw_key_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_flw_key_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_flw_jsv_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_flw_jsv_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_flw_njv_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_flw_njv_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_dqt_str_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_dqt_str_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__b_dqt_str_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_dqt_str_ctn] = {
    .visible = false,
    .named = true,
  },
  [sym__br_dqt_str_ctn] = {
    .visible = false,
    .named = true,
  },
  [sym__r_dqt_esc_nwl] = {
    .visible = true,
    .named = true,
  },
  [sym__br_dqt_esc_nwl] = {
    .visible = true,
    .named = true,
  },
  [sym__r_dqt_esc_seq] = {
    .visible = true,
    .named = true,
  },
  [sym__br_dqt_esc_seq] = {
    .visible = true,
    .named = true,
  },
  [sym__r_dqt_str_end] = {
    .visible = true,
    .named = false,
  },
  [sym__br_dqt_str_end] = {
    .visible = true,
    .named = false,
  },
  [sym__r_sqt_str_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_sqt_str_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__b_sqt_str_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_sqt_str_ctn] = {
    .visible = false,
    .named = true,
  },
  [sym__br_sqt_str_ctn] = {
    .visible = false,
    .named = true,
  },
  [sym__r_sqt_esc_sqt] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sqt_esc_sqt] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sqt_str_end] = {
    .visible = true,
    .named = false,
  },
  [sym__br_sqt_str_end] = {
    .visible = true,
    .named = false,
  },
  [sym__r_sgl_pln_nul_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_pln_nul_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_pln_nul_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_pln_nul_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_pln_nul_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_pln_bol_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_pln_bol_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_pln_bol_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_pln_bol_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_pln_bol_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_pln_int_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_pln_int_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_pln_int_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_pln_int_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_pln_int_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_pln_flt_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_pln_flt_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_pln_flt_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_pln_flt_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_pln_flt_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_pln_str_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_pln_str_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_pln_str_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_pln_str_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_pln_str_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__r_mtl_pln_str_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__br_mtl_pln_str_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__r_mtl_pln_str_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__br_mtl_pln_str_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__r_tag] = {
    .visible = true,
    .named = true,
  },
  [sym__br_tag] = {
    .visible = true,
    .named = true,
  },
  [sym__b_tag] = {
    .visible = true,
    .named = true,
  },
  [sym__r_acr_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_acr_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__b_acr_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_acr_ctn] = {
    .visible = true,
    .named = true,
  },
  [sym__r_als_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__br_als_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__b_als_bgn] = {
    .visible = true,
    .named = false,
  },
  [sym__r_als_ctn] = {
    .visible = true,
    .named = true,
  },
  [sym__bl] = {
    .visible = false,
    .named = true,
  },
  [sym_comment] = {
    .visible = true,
    .named = true,
  },
  [sym_stream] = {
    .visible = true,
    .named = true,
  },
  [sym__doc_w_bgn_w_end_seq] = {
    .visible = false,
    .named = true,
  },
  [sym__doc_w_bgn_wo_end_seq] = {
    .visible = false,
    .named = true,
  },
  [sym__doc_wo_bgn_w_end_seq] = {
    .visible = false,
    .named = true,
  },
  [sym__doc_wo_bgn_wo_end_seq] = {
    .visible = false,
    .named = true,
  },
  [sym__doc_w_bgn_w_end] = {
    .visible = false,
    .named = true,
  },
  [sym__doc_w_bgn_wo_end] = {
    .visible = false,
    .named = true,
  },
  [sym__doc_wo_bgn_w_end] = {
    .visible = false,
    .named = true,
  },
  [sym__doc_wo_bgn_wo_end] = {
    .visible = false,
    .named = true,
  },
  [sym__bgn_imp_doc] = {
    .visible = false,
    .named = true,
  },
  [sym__drs_doc] = {
    .visible = false,
    .named = true,
  },
  [sym__exp_doc] = {
    .visible = false,
    .named = true,
  },
  [sym__imp_doc] = {
    .visible = false,
    .named = true,
  },
  [sym__drs_doc_end] = {
    .visible = true,
    .named = true,
  },
  [sym__exp_doc_end] = {
    .visible = true,
    .named = true,
  },
  [sym__imp_doc_end] = {
    .visible = true,
    .named = true,
  },
  [sym__bgn_imp_doc_end] = {
    .visible = true,
    .named = true,
  },
  [sym__doc_end] = {
    .visible = true,
    .named = true,
  },
  [sym__exp_doc_tal] = {
    .visible = false,
    .named = true,
  },
  [sym__s_dir] = {
    .visible = false,
    .named = true,
  },
  [sym__s_dir_yml] = {
    .visible = true,
    .named = true,
  },
  [sym__s_dir_tag] = {
    .visible = true,
    .named = true,
  },
  [sym__s_dir_rsv] = {
    .visible = true,
    .named = true,
  },
  [sym__r_prp_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_prp_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_prp_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_prp_val] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_prp_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_prp] = {
    .visible = false,
    .named = true,
  },
  [sym__br_prp] = {
    .visible = false,
    .named = true,
  },
  [sym__r_sgl_prp] = {
    .visible = false,
    .named = true,
  },
  [sym__br_sgl_prp] = {
    .visible = false,
    .named = true,
  },
  [sym__b_sgl_prp] = {
    .visible = false,
    .named = true,
  },
  [sym__r_blk_seq_r_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_blk_seq_br_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_blk_seq_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_blk_seq_spc_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_blk_seq_spc_val] = {
    .visible = true,
    .named = true,
  },
  [sym__b_blk_seq_spc_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_blk_seq] = {
    .visible = true,
    .named = true,
  },
  [sym__br_blk_seq] = {
    .visible = true,
    .named = true,
  },
  [sym__b_blk_seq_spc] = {
    .visible = true,
    .named = true,
  },
  [sym__r_blk_seq_itm] = {
    .visible = true,
    .named = true,
  },
  [sym__br_blk_seq_itm] = {
    .visible = true,
    .named = true,
  },
  [sym__b_blk_seq_itm] = {
    .visible = true,
    .named = true,
  },
  [sym__blk_seq_itm_tal] = {
    .visible = false,
    .named = true,
  },
  [sym__r_blk_map_r_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_blk_map_br_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_blk_map_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_blk_map] = {
    .visible = true,
    .named = true,
  },
  [sym__br_blk_map] = {
    .visible = true,
    .named = true,
  },
  [sym__r_blk_map_itm] = {
    .visible = false,
    .named = true,
  },
  [sym__br_blk_map_itm] = {
    .visible = false,
    .named = true,
  },
  [sym__b_blk_map_itm] = {
    .visible = false,
    .named = true,
  },
  [sym__r_blk_exp_itm] = {
    .visible = true,
    .named = true,
  },
  [sym__br_blk_exp_itm] = {
    .visible = true,
    .named = true,
  },
  [sym__b_blk_exp_itm] = {
    .visible = true,
    .named = true,
  },
  [sym__r_blk_key_itm] = {
    .visible = false,
    .named = true,
  },
  [sym__br_blk_key_itm] = {
    .visible = false,
    .named = true,
  },
  [sym__b_blk_key_itm] = {
    .visible = false,
    .named = true,
  },
  [sym__r_blk_val_itm] = {
    .visible = false,
    .named = true,
  },
  [sym__br_blk_val_itm] = {
    .visible = false,
    .named = true,
  },
  [sym__b_blk_val_itm] = {
    .visible = false,
    .named = true,
  },
  [sym__r_blk_imp_itm_key_only] = {
    .visible = true,
    .named = true,
  },
  [sym__br_blk_imp_itm_key_only] = {
    .visible = true,
    .named = true,
  },
  [sym__b_blk_imp_itm_key_only] = {
    .visible = true,
    .named = true,
  },
  [sym__blk_exp_itm_tal] = {
    .visible = false,
    .named = true,
  },
  [sym__blk_imp_itm_tal] = {
    .visible = false,
    .named = true,
  },
  [sym__r_blk_str_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_blk_str_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_blk_str] = {
    .visible = true,
    .named = true,
  },
  [sym__br_blk_str] = {
    .visible = true,
    .named = true,
  },
  [sym__r_flw_seq_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_flw_seq_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_flw_seq_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_flw_seq_val] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_flw_seq_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_flw_seq] = {
    .visible = true,
    .named = true,
  },
  [sym__br_flw_seq] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_flw_seq] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_flw_seq] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_flw_seq] = {
    .visible = true,
    .named = true,
  },
  [sym__flw_seq_tal] = {
    .visible = false,
    .named = true,
  },
  [sym__sgl_flw_seq_tal] = {
    .visible = false,
    .named = true,
  },
  [sym__r_flw_map_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_flw_map_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_flw_map_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_flw_map_val] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_flw_map_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_flw_map] = {
    .visible = true,
    .named = true,
  },
  [sym__br_flw_map] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_flw_map] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_flw_map] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_flw_map] = {
    .visible = true,
    .named = true,
  },
  [sym__flw_map_tal] = {
    .visible = false,
    .named = true,
  },
  [sym__sgl_flw_map_tal] = {
    .visible = false,
    .named = true,
  },
  [sym__r_flw_seq_dat] = {
    .visible = false,
    .named = true,
  },
  [sym__br_flw_seq_dat] = {
    .visible = false,
    .named = true,
  },
  [sym__r_flw_map_dat] = {
    .visible = false,
    .named = true,
  },
  [sym__br_flw_map_dat] = {
    .visible = false,
    .named = true,
  },
  [sym__r_sgl_flw_col_dat] = {
    .visible = false,
    .named = true,
  },
  [sym__flw_seq_dat_rpt] = {
    .visible = false,
    .named = true,
  },
  [sym__flw_map_dat_rpt] = {
    .visible = false,
    .named = true,
  },
  [sym__sgl_flw_col_dat_rpt] = {
    .visible = false,
    .named = true,
  },
  [sym__r_flw_seq_itm] = {
    .visible = false,
    .named = true,
  },
  [sym__br_flw_seq_itm] = {
    .visible = false,
    .named = true,
  },
  [sym__r_flw_map_itm] = {
    .visible = false,
    .named = true,
  },
  [sym__br_flw_map_itm] = {
    .visible = false,
    .named = true,
  },
  [sym__r_sgl_flw_col_itm] = {
    .visible = false,
    .named = true,
  },
  [sym__r_flw_exp_par] = {
    .visible = true,
    .named = true,
  },
  [sym__br_flw_exp_par] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_flw_exp_par] = {
    .visible = true,
    .named = true,
  },
  [sym__r_flw_imp_r_par] = {
    .visible = false,
    .named = true,
  },
  [sym__r_flw_imp_br_par] = {
    .visible = false,
    .named = true,
  },
  [sym__br_flw_imp_r_par] = {
    .visible = false,
    .named = true,
  },
  [sym__br_flw_imp_br_par] = {
    .visible = false,
    .named = true,
  },
  [sym__r_sgl_flw_imp_par] = {
    .visible = false,
    .named = true,
  },
  [sym__r_flw_jsl_ann_par] = {
    .visible = false,
    .named = true,
  },
  [sym__br_flw_jsl_ann_par] = {
    .visible = false,
    .named = true,
  },
  [sym__r_sgl_flw_jsl_ann_par] = {
    .visible = false,
    .named = true,
  },
  [sym__r_flw_njl_ann_par] = {
    .visible = false,
    .named = true,
  },
  [sym__br_flw_njl_ann_par] = {
    .visible = false,
    .named = true,
  },
  [sym__r_sgl_flw_njl_ann_par] = {
    .visible = false,
    .named = true,
  },
  [sym__flw_ann_par_tal] = {
    .visible = false,
    .named = true,
  },
  [sym__sgl_flw_ann_par_tal] = {
    .visible = false,
    .named = true,
  },
  [sym__r_dqt_str_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_dqt_str_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_dqt_str_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_dqt_str_val] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_dqt_str_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_dqt_str] = {
    .visible = true,
    .named = true,
  },
  [sym__br_dqt_str] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_dqt_str] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_dqt_str] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_dqt_str] = {
    .visible = true,
    .named = true,
  },
  [aux_sym__r_sgl_dqt_ctn] = {
    .visible = false,
    .named = false,
  },
  [sym__br_mtl_dqt_ctn] = {
    .visible = false,
    .named = true,
  },
  [sym__r_sqt_str_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sqt_str_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_sqt_str_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_sqt_str_val] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_sqt_str_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sqt_str] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sqt_str] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_sqt_str] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_sqt_str] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_sqt_str] = {
    .visible = true,
    .named = true,
  },
  [aux_sym__r_sgl_sqt_ctn] = {
    .visible = false,
    .named = false,
  },
  [sym__br_mtl_sqt_ctn] = {
    .visible = false,
    .named = true,
  },
  [sym__r_pln_blk_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_pln_blk_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_pln_blk_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_pln_blk_val] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_pln_blk_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_pln_flw_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_pln_flw_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_pln_flw_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_pln_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_pln_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__b_sgl_pln_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__r_sgl_pln_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__br_sgl_pln_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__r_mtl_pln_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__br_mtl_pln_blk] = {
    .visible = true,
    .named = true,
  },
  [sym__r_mtl_pln_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__br_mtl_pln_flw] = {
    .visible = true,
    .named = true,
  },
  [sym__r_als_val] = {
    .visible = true,
    .named = true,
  },
  [sym__br_als_val] = {
    .visible = true,
    .named = true,
  },
  [sym__b_als_val] = {
    .visible = true,
    .named = true,
  },
  [sym__r_als] = {
    .visible = true,
    .named = true,
  },
  [sym__br_als] = {
    .visible = true,
    .named = true,
  },
  [sym__b_als] = {
    .visible = true,
    .named = true,
  },
  [sym__r_acr] = {
    .visible = true,
    .named = true,
  },
  [sym__br_acr] = {
    .visible = true,
    .named = true,
  },
  [sym__b_acr] = {
    .visible = true,
    .named = true,
  },
  [aux_sym__drs_doc_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__s_dir_rsv_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__r_blk_seq_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__r_blk_map_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__r_blk_str_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__r_flw_seq_dat_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__r_flw_map_dat_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__r_sgl_flw_col_dat_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__r_dqt_str_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__br_mtl_dqt_ctn_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__r_sqt_str_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__br_mtl_sqt_ctn_repeat1] = {
    .visible = false,
    .named = false,
  },
};

enum {
  field_key = 1,
  field_value = 2,
};

static const char * const ts_field_names[] = {
  [0] = NULL,
  [field_key] = "key",
  [field_value] = "value",
};

static const TSFieldMapSlice ts_field_map_slices[PRODUCTION_ID_COUNT] = {
  [1] = {.index = 0, .length = 1},
  [2] = {.index = 1, .length = 2},
  [3] = {.index = 3, .length = 1},
  [4] = {.index = 4, .length = 1},
  [5] = {.index = 5, .length = 1},
  [6] = {.index = 6, .length = 1},
  [7] = {.index = 1, .length = 2},
  [8] = {.index = 4, .length = 1},
  [10] = {.index = 7, .length = 2},
  [11] = {.index = 9, .length = 2},
  [12] = {.index = 11, .length = 2},
};

static const TSFieldMapEntry ts_field_map_entries[] = {
  [0] =
    {field_key, 0},
  [1] =
    {field_key, 0, .inherited = true},
    {field_value, 0, .inherited = true},
  [3] =
    {field_key, 0, .inherited = true},
  [4] =
    {field_value, 0, .inherited = true},
  [5] =
    {field_key, 1},
  [6] =
    {field_value, 1},
  [7] =
    {field_key, 0},
    {field_value, 1, .inherited = true},
  [9] =
    {field_key, 0, .inherited = true},
    {field_value, 1, .inherited = true},
  [11] =
    {field_key, 1, .inherited = true},
    {field_value, 1, .inherited = true},
};

static const TSSymbol ts_alias_sequences[PRODUCTION_ID_COUNT][MAX_ALIAS_SEQUENCE_LENGTH] = {
  [0] = {0},
  [7] = {
    [0] = sym__r_flw_exp_par,
  },
  [8] = {
    [0] = sym__r_flw_exp_par,
  },
  [9] = {
    [0] = sym__drs_doc_end,
  },
};

static const uint16_t ts_non_terminal_alias_map[] = {
  sym__bgn_imp_doc, 2,
    sym__bgn_imp_doc,
    sym__drs_doc_end,
  sym__drs_doc, 2,
    sym__drs_doc,
    sym__drs_doc_end,
  sym__exp_doc, 2,
    sym__exp_doc,
    sym__drs_doc_end,
  sym__imp_doc, 2,
    sym__imp_doc,
    sym__drs_doc_end,
  sym__r_flw_imp_r_par, 2,
    sym__r_flw_imp_r_par,
    sym__r_flw_exp_par,
  sym__r_flw_imp_br_par, 2,
    sym__r_flw_imp_br_par,
    sym__r_flw_exp_par,
  sym__br_flw_imp_r_par, 2,
    sym__br_flw_imp_r_par,
    sym__r_flw_exp_par,
  sym__br_flw_imp_br_par, 2,
    sym__br_flw_imp_br_par,
    sym__r_flw_exp_par,
  sym__r_sgl_flw_imp_par, 2,
    sym__r_sgl_flw_imp_par,
    sym__r_flw_exp_par,
  sym__r_flw_njl_ann_par, 2,
    sym__r_flw_njl_ann_par,
    sym__r_flw_exp_par,
  sym__br_flw_njl_ann_par, 2,
    sym__br_flw_njl_ann_par,
    sym__r_flw_exp_par,
  sym__r_sgl_flw_njl_ann_par, 2,
    sym__r_sgl_flw_njl_ann_par,
    sym__r_flw_exp_par,
  0,
};

static const TSStateId ts_primary_state_ids[STATE_COUNT] = {
  [0] = 0,
  [1] = 1,
  [2] = 2,
  [3] = 3,
  [4] = 4,
  [5] = 5,
  [6] = 6,
  [7] = 7,
  [8] = 8,
  [9] = 9,
  [10] = 10,
  [11] = 11,
  [12] = 12,
  [13] = 13,
  [14] = 13,
  [15] = 13,
  [16] = 13,
  [17] = 13,
  [18] = 13,
  [19] = 19,
  [20] = 19,
  [21] = 13,
  [22] = 19,
  [23] = 13,
  [24] = 24,
  [25] = 24,
  [26] = 24,
  [27] = 24,
  [28] = 24,
  [29] = 24,
  [30] = 30,
  [31] = 30,
  [32] = 24,
  [33] = 24,
  [34] = 30,
  [35] = 35,
  [36] = 36,
  [37] = 37,
  [38] = 38,
  [39] = 39,
  [40] = 40,
  [41] = 40,
  [42] = 42,
  [43] = 42,
  [44] = 42,
  [45] = 42,
  [46] = 46,
  [47] = 46,
  [48] = 42,
  [49] = 42,
  [50] = 46,
  [51] = 42,
  [52] = 46,
  [53] = 46,
  [54] = 42,
  [55] = 55,
  [56] = 55,
  [57] = 55,
  [58] = 58,
  [59] = 59,
  [60] = 55,
  [61] = 55,
  [62] = 59,
  [63] = 58,
  [64] = 58,
  [65] = 55,
  [66] = 58,
  [67] = 58,
  [68] = 55,
  [69] = 55,
  [70] = 70,
  [71] = 71,
  [72] = 72,
  [73] = 73,
  [74] = 74,
  [75] = 75,
  [76] = 76,
  [77] = 77,
  [78] = 78,
  [79] = 79,
  [80] = 80,
  [81] = 81,
  [82] = 80,
  [83] = 81,
  [84] = 84,
  [85] = 85,
  [86] = 84,
  [87] = 85,
  [88] = 88,
  [89] = 89,
  [90] = 90,
  [91] = 89,
  [92] = 92,
  [93] = 88,
  [94] = 90,
  [95] = 92,
  [96] = 96,
  [97] = 97,
  [98] = 98,
  [99] = 99,
  [100] = 100,
  [101] = 96,
  [102] = 96,
  [103] = 97,
  [104] = 97,
  [105] = 105,
  [106] = 106,
  [107] = 107,
  [108] = 98,
  [109] = 109,
  [110] = 98,
  [111] = 105,
  [112] = 99,
  [113] = 99,
  [114] = 105,
  [115] = 100,
  [116] = 109,
  [117] = 106,
  [118] = 106,
  [119] = 109,
  [120] = 120,
  [121] = 98,
  [122] = 98,
  [123] = 98,
  [124] = 124,
  [125] = 125,
  [126] = 100,
  [127] = 127,
  [128] = 128,
  [129] = 128,
  [130] = 127,
  [131] = 131,
  [132] = 132,
  [133] = 133,
  [134] = 134,
  [135] = 133,
  [136] = 136,
  [137] = 136,
  [138] = 138,
  [139] = 139,
  [140] = 133,
  [141] = 136,
  [142] = 139,
  [143] = 133,
  [144] = 128,
  [145] = 131,
  [146] = 132,
  [147] = 127,
  [148] = 136,
  [149] = 149,
  [150] = 107,
  [151] = 149,
  [152] = 127,
  [153] = 136,
  [154] = 131,
  [155] = 133,
  [156] = 136,
  [157] = 149,
  [158] = 133,
  [159] = 132,
  [160] = 128,
  [161] = 161,
  [162] = 139,
  [163] = 127,
  [164] = 128,
  [165] = 98,
  [166] = 166,
  [167] = 167,
  [168] = 120,
  [169] = 166,
  [170] = 167,
  [171] = 98,
  [172] = 172,
  [173] = 125,
  [174] = 124,
  [175] = 172,
  [176] = 100,
  [177] = 124,
  [178] = 100,
  [179] = 125,
  [180] = 180,
  [181] = 181,
  [182] = 182,
  [183] = 183,
  [184] = 184,
  [185] = 107,
  [186] = 181,
  [187] = 97,
  [188] = 188,
  [189] = 188,
  [190] = 180,
  [191] = 191,
  [192] = 107,
  [193] = 184,
  [194] = 97,
  [195] = 107,
  [196] = 97,
  [197] = 120,
  [198] = 97,
  [199] = 98,
  [200] = 100,
  [201] = 184,
  [202] = 202,
  [203] = 184,
  [204] = 105,
  [205] = 107,
  [206] = 98,
  [207] = 120,
  [208] = 208,
  [209] = 209,
  [210] = 202,
  [211] = 105,
  [212] = 100,
  [213] = 120,
  [214] = 109,
  [215] = 184,
  [216] = 100,
  [217] = 98,
  [218] = 218,
  [219] = 219,
  [220] = 100,
  [221] = 98,
  [222] = 107,
  [223] = 100,
  [224] = 105,
  [225] = 98,
  [226] = 202,
  [227] = 100,
  [228] = 105,
  [229] = 97,
  [230] = 125,
  [231] = 231,
  [232] = 120,
  [233] = 124,
  [234] = 202,
  [235] = 98,
  [236] = 236,
  [237] = 125,
  [238] = 124,
  [239] = 109,
  [240] = 100,
  [241] = 241,
  [242] = 98,
  [243] = 124,
  [244] = 109,
  [245] = 245,
  [246] = 202,
  [247] = 98,
  [248] = 98,
  [249] = 249,
  [250] = 100,
  [251] = 105,
  [252] = 252,
  [253] = 253,
  [254] = 109,
  [255] = 100,
  [256] = 125,
  [257] = 98,
  [258] = 258,
  [259] = 124,
  [260] = 125,
  [261] = 120,
  [262] = 262,
  [263] = 263,
  [264] = 100,
  [265] = 100,
  [266] = 109,
  [267] = 124,
  [268] = 125,
  [269] = 98,
  [270] = 270,
  [271] = 271,
  [272] = 272,
  [273] = 273,
  [274] = 274,
  [275] = 275,
  [276] = 276,
  [277] = 277,
  [278] = 278,
  [279] = 279,
  [280] = 280,
  [281] = 281,
  [282] = 282,
  [283] = 283,
  [284] = 284,
  [285] = 285,
  [286] = 286,
  [287] = 287,
  [288] = 288,
  [289] = 289,
  [290] = 290,
  [291] = 291,
  [292] = 292,
  [293] = 293,
  [294] = 294,
  [295] = 284,
  [296] = 275,
  [297] = 277,
  [298] = 278,
  [299] = 280,
  [300] = 281,
  [301] = 301,
  [302] = 302,
  [303] = 276,
  [304] = 304,
  [305] = 305,
  [306] = 306,
  [307] = 285,
  [308] = 287,
  [309] = 304,
  [310] = 290,
  [311] = 289,
  [312] = 288,
  [313] = 286,
  [314] = 294,
  [315] = 293,
  [316] = 279,
  [317] = 292,
  [318] = 291,
  [319] = 274,
  [320] = 273,
  [321] = 283,
  [322] = 282,
  [323] = 323,
  [324] = 324,
  [325] = 325,
  [326] = 326,
  [327] = 327,
  [328] = 328,
  [329] = 286,
  [330] = 330,
  [331] = 331,
  [332] = 275,
  [333] = 333,
  [334] = 282,
  [335] = 283,
  [336] = 336,
  [337] = 337,
  [338] = 338,
  [339] = 274,
  [340] = 340,
  [341] = 288,
  [342] = 289,
  [343] = 273,
  [344] = 344,
  [345] = 304,
  [346] = 346,
  [347] = 290,
  [348] = 281,
  [349] = 280,
  [350] = 350,
  [351] = 351,
  [352] = 278,
  [353] = 277,
  [354] = 284,
  [355] = 355,
  [356] = 356,
  [357] = 357,
  [358] = 358,
  [359] = 359,
  [360] = 360,
  [361] = 361,
  [362] = 362,
  [363] = 363,
  [364] = 276,
  [365] = 292,
  [366] = 366,
  [367] = 367,
  [368] = 368,
  [369] = 304,
  [370] = 370,
  [371] = 285,
  [372] = 372,
  [373] = 373,
  [374] = 374,
  [375] = 375,
  [376] = 376,
  [377] = 377,
  [378] = 378,
  [379] = 287,
  [380] = 380,
  [381] = 381,
  [382] = 382,
  [383] = 383,
  [384] = 384,
  [385] = 279,
  [386] = 291,
  [387] = 387,
  [388] = 388,
  [389] = 389,
  [390] = 390,
  [391] = 391,
  [392] = 392,
  [393] = 393,
  [394] = 394,
  [395] = 395,
  [396] = 396,
  [397] = 397,
  [398] = 398,
  [399] = 276,
  [400] = 287,
  [401] = 401,
  [402] = 402,
  [403] = 403,
  [404] = 404,
  [405] = 405,
  [406] = 406,
  [407] = 407,
  [408] = 408,
  [409] = 409,
  [410] = 410,
  [411] = 411,
  [412] = 412,
  [413] = 413,
  [414] = 414,
  [415] = 415,
  [416] = 416,
  [417] = 417,
  [418] = 418,
  [419] = 419,
  [420] = 420,
  [421] = 421,
  [422] = 422,
  [423] = 423,
  [424] = 284,
  [425] = 425,
  [426] = 426,
  [427] = 427,
  [428] = 428,
  [429] = 429,
  [430] = 430,
  [431] = 431,
  [432] = 432,
  [433] = 433,
  [434] = 434,
  [435] = 435,
  [436] = 436,
  [437] = 437,
  [438] = 438,
  [439] = 439,
  [440] = 440,
  [441] = 441,
  [442] = 442,
  [443] = 443,
  [444] = 304,
  [445] = 445,
  [446] = 446,
  [447] = 447,
  [448] = 448,
  [449] = 449,
  [450] = 450,
  [451] = 451,
  [452] = 452,
  [453] = 453,
  [454] = 454,
  [455] = 455,
  [456] = 456,
  [457] = 457,
  [458] = 458,
  [459] = 459,
  [460] = 460,
  [461] = 461,
  [462] = 462,
  [463] = 463,
  [464] = 464,
  [465] = 465,
  [466] = 466,
  [467] = 467,
  [468] = 468,
  [469] = 469,
  [470] = 470,
  [471] = 471,
  [472] = 472,
  [473] = 473,
  [474] = 474,
  [475] = 475,
  [476] = 476,
  [477] = 477,
  [478] = 478,
  [479] = 479,
  [480] = 480,
  [481] = 481,
  [482] = 482,
  [483] = 483,
  [484] = 484,
  [485] = 485,
  [486] = 486,
  [487] = 487,
  [488] = 488,
  [489] = 489,
  [490] = 490,
  [491] = 491,
  [492] = 492,
  [493] = 493,
  [494] = 493,
  [495] = 495,
  [496] = 493,
  [497] = 98,
  [498] = 495,
  [499] = 98,
  [500] = 493,
  [501] = 501,
  [502] = 495,
  [503] = 501,
  [504] = 98,
  [505] = 98,
  [506] = 98,
  [507] = 98,
  [508] = 493,
  [509] = 495,
  [510] = 510,
  [511] = 511,
  [512] = 512,
  [513] = 513,
  [514] = 514,
  [515] = 511,
  [516] = 516,
  [517] = 510,
  [518] = 510,
  [519] = 516,
  [520] = 501,
  [521] = 516,
  [522] = 522,
  [523] = 513,
  [524] = 513,
  [525] = 511,
  [526] = 510,
  [527] = 514,
  [528] = 522,
  [529] = 511,
  [530] = 510,
  [531] = 516,
  [532] = 532,
  [533] = 510,
  [534] = 514,
  [535] = 501,
  [536] = 98,
  [537] = 511,
  [538] = 532,
  [539] = 532,
  [540] = 532,
  [541] = 98,
  [542] = 98,
  [543] = 511,
  [544] = 532,
  [545] = 516,
  [546] = 513,
  [547] = 512,
  [548] = 516,
  [549] = 522,
  [550] = 511,
  [551] = 532,
  [552] = 495,
  [553] = 511,
  [554] = 510,
  [555] = 512,
  [556] = 532,
  [557] = 516,
  [558] = 514,
  [559] = 516,
  [560] = 510,
  [561] = 514,
  [562] = 513,
  [563] = 532,
  [564] = 501,
  [565] = 565,
  [566] = 566,
  [567] = 567,
  [568] = 568,
  [569] = 569,
  [570] = 98,
  [571] = 571,
  [572] = 572,
  [573] = 573,
  [574] = 574,
  [575] = 575,
  [576] = 576,
  [577] = 577,
  [578] = 572,
  [579] = 579,
  [580] = 572,
  [581] = 581,
  [582] = 572,
  [583] = 579,
  [584] = 584,
  [585] = 585,
  [586] = 586,
  [587] = 577,
  [588] = 577,
  [589] = 589,
  [590] = 590,
  [591] = 577,
  [592] = 590,
  [593] = 577,
  [594] = 572,
  [595] = 579,
  [596] = 574,
  [597] = 573,
  [598] = 576,
  [599] = 579,
  [600] = 600,
  [601] = 579,
  [602] = 579,
  [603] = 574,
  [604] = 579,
  [605] = 572,
  [606] = 573,
  [607] = 574,
  [608] = 576,
  [609] = 574,
  [610] = 573,
  [611] = 572,
  [612] = 577,
  [613] = 590,
  [614] = 584,
  [615] = 574,
  [616] = 584,
  [617] = 574,
  [618] = 584,
  [619] = 577,
  [620] = 572,
  [621] = 577,
  [622] = 574,
  [623] = 584,
  [624] = 579,
  [625] = 573,
  [626] = 626,
  [627] = 627,
  [628] = 628,
  [629] = 629,
  [630] = 630,
  [631] = 631,
  [632] = 632,
  [633] = 633,
  [634] = 634,
  [635] = 635,
  [636] = 636,
  [637] = 637,
  [638] = 636,
  [639] = 637,
  [640] = 632,
  [641] = 635,
  [642] = 642,
  [643] = 636,
  [644] = 637,
  [645] = 645,
  [646] = 632,
  [647] = 635,
  [648] = 648,
  [649] = 630,
  [650] = 631,
  [651] = 651,
  [652] = 651,
  [653] = 631,
  [654] = 648,
  [655] = 630,
  [656] = 630,
  [657] = 631,
  [658] = 634,
  [659] = 651,
  [660] = 634,
  [661] = 648,
  [662] = 662,
  [663] = 662,
  [664] = 662,
  [665] = 662,
  [666] = 631,
  [667] = 630,
  [668] = 634,
  [669] = 648,
  [670] = 670,
  [671] = 635,
  [672] = 632,
  [673] = 636,
  [674] = 637,
  [675] = 636,
  [676] = 637,
  [677] = 632,
  [678] = 634,
  [679] = 662,
  [680] = 635,
  [681] = 648,
  [682] = 630,
  [683] = 683,
  [684] = 631,
  [685] = 651,
  [686] = 631,
  [687] = 651,
  [688] = 648,
  [689] = 662,
  [690] = 634,
  [691] = 636,
  [692] = 651,
  [693] = 632,
  [694] = 634,
  [695] = 662,
  [696] = 630,
  [697] = 635,
  [698] = 648,
  [699] = 662,
  [700] = 634,
  [701] = 651,
  [702] = 648,
  [703] = 631,
  [704] = 637,
  [705] = 651,
  [706] = 632,
  [707] = 707,
  [708] = 636,
  [709] = 637,
  [710] = 632,
  [711] = 635,
  [712] = 637,
  [713] = 636,
  [714] = 630,
  [715] = 635,
  [716] = 716,
  [717] = 716,
  [718] = 718,
  [719] = 719,
  [720] = 418,
  [721] = 417,
  [722] = 722,
  [723] = 723,
  [724] = 287,
  [725] = 716,
  [726] = 726,
  [727] = 726,
  [728] = 726,
  [729] = 716,
  [730] = 730,
  [731] = 731,
  [732] = 716,
  [733] = 733,
  [734] = 734,
  [735] = 735,
  [736] = 736,
  [737] = 737,
  [738] = 738,
  [739] = 726,
  [740] = 736,
  [741] = 723,
  [742] = 726,
  [743] = 716,
  [744] = 723,
  [745] = 736,
  [746] = 276,
  [747] = 736,
  [748] = 723,
  [749] = 749,
  [750] = 750,
  [751] = 751,
  [752] = 472,
  [753] = 753,
  [754] = 473,
  [755] = 420,
  [756] = 736,
  [757] = 426,
  [758] = 475,
  [759] = 759,
  [760] = 760,
  [761] = 761,
  [762] = 476,
  [763] = 479,
  [764] = 480,
  [765] = 492,
  [766] = 491,
  [767] = 278,
  [768] = 726,
  [769] = 769,
  [770] = 490,
  [771] = 281,
  [772] = 489,
  [773] = 488,
  [774] = 487,
  [775] = 775,
  [776] = 428,
  [777] = 777,
  [778] = 778,
  [779] = 470,
  [780] = 780,
  [781] = 469,
  [782] = 723,
  [783] = 468,
  [784] = 467,
  [785] = 461,
  [786] = 460,
  [787] = 723,
  [788] = 736,
  [789] = 459,
  [790] = 458,
  [791] = 288,
  [792] = 430,
  [793] = 793,
  [794] = 434,
  [795] = 795,
  [796] = 454,
  [797] = 419,
  [798] = 798,
  [799] = 453,
  [800] = 716,
  [801] = 726,
  [802] = 273,
  [803] = 451,
  [804] = 736,
  [805] = 449,
  [806] = 282,
  [807] = 448,
  [808] = 736,
  [809] = 447,
  [810] = 275,
  [811] = 723,
  [812] = 726,
  [813] = 446,
  [814] = 723,
  [815] = 815,
  [816] = 445,
  [817] = 286,
  [818] = 818,
  [819] = 716,
  [820] = 279,
  [821] = 417,
  [822] = 286,
  [823] = 445,
  [824] = 434,
  [825] = 419,
  [826] = 426,
  [827] = 279,
  [828] = 828,
  [829] = 446,
  [830] = 275,
  [831] = 447,
  [832] = 428,
  [833] = 448,
  [834] = 282,
  [835] = 449,
  [836] = 451,
  [837] = 273,
  [838] = 453,
  [839] = 454,
  [840] = 420,
  [841] = 276,
  [842] = 430,
  [843] = 753,
  [844] = 418,
  [845] = 845,
  [846] = 288,
  [847] = 458,
  [848] = 459,
  [849] = 460,
  [850] = 461,
  [851] = 467,
  [852] = 468,
  [853] = 469,
  [854] = 470,
  [855] = 287,
  [856] = 798,
  [857] = 769,
  [858] = 858,
  [859] = 487,
  [860] = 488,
  [861] = 489,
  [862] = 281,
  [863] = 490,
  [864] = 278,
  [865] = 491,
  [866] = 492,
  [867] = 845,
  [868] = 761,
  [869] = 760,
  [870] = 480,
  [871] = 479,
  [872] = 476,
  [873] = 475,
  [874] = 759,
  [875] = 473,
  [876] = 472,
  [877] = 749,
  [878] = 750,
  [879] = 751,
  [880] = 760,
  [881] = 761,
  [882] = 882,
  [883] = 883,
  [884] = 884,
  [885] = 798,
  [886] = 886,
  [887] = 887,
  [888] = 888,
  [889] = 769,
  [890] = 683,
  [891] = 891,
  [892] = 892,
  [893] = 893,
  [894] = 894,
  [895] = 893,
  [896] = 883,
  [897] = 891,
  [898] = 898,
  [899] = 899,
  [900] = 900,
  [901] = 901,
  [902] = 902,
  [903] = 903,
  [904] = 904,
  [905] = 749,
  [906] = 906,
  [907] = 760,
  [908] = 759,
  [909] = 753,
  [910] = 910,
  [911] = 911,
  [912] = 912,
  [913] = 913,
  [914] = 750,
  [915] = 751,
  [916] = 751,
  [917] = 750,
  [918] = 749,
  [919] = 919,
  [920] = 920,
  [921] = 921,
  [922] = 753,
  [923] = 923,
  [924] = 759,
  [925] = 761,
  [926] = 926,
  [927] = 927,
  [928] = 928,
  [929] = 929,
  [930] = 930,
  [931] = 931,
  [932] = 932,
  [933] = 933,
  [934] = 906,
  [935] = 935,
  [936] = 886,
  [937] = 937,
  [938] = 938,
  [939] = 900,
  [940] = 940,
  [941] = 941,
  [942] = 942,
  [943] = 943,
  [944] = 288,
  [945] = 945,
  [946] = 946,
  [947] = 947,
  [948] = 948,
  [949] = 949,
  [950] = 769,
  [951] = 798,
  [952] = 906,
  [953] = 930,
  [954] = 954,
  [955] = 902,
  [956] = 956,
  [957] = 904,
  [958] = 956,
  [959] = 920,
  [960] = 954,
  [961] = 921,
  [962] = 888,
  [963] = 963,
  [964] = 964,
  [965] = 946,
  [966] = 966,
  [967] = 947,
  [968] = 926,
  [969] = 927,
  [970] = 928,
  [971] = 892,
  [972] = 929,
  [973] = 882,
  [974] = 974,
  [975] = 935,
  [976] = 976,
  [977] = 954,
  [978] = 978,
  [979] = 966,
  [980] = 887,
  [981] = 956,
  [982] = 982,
  [983] = 891,
  [984] = 984,
  [985] = 887,
  [986] = 937,
  [987] = 903,
  [988] = 906,
  [989] = 930,
  [990] = 930,
  [991] = 884,
  [992] = 938,
  [993] = 963,
  [994] = 940,
  [995] = 941,
  [996] = 891,
  [997] = 899,
  [998] = 887,
  [999] = 884,
  [1000] = 901,
  [1001] = 888,
  [1002] = 974,
  [1003] = 898,
  [1004] = 942,
  [1005] = 943,
  [1006] = 976,
  [1007] = 906,
  [1008] = 287,
  [1009] = 276,
  [1010] = 472,
  [1011] = 1011,
  [1012] = 930,
  [1013] = 589,
  [1014] = 473,
  [1015] = 1015,
  [1016] = 1016,
  [1017] = 475,
  [1018] = 476,
  [1019] = 479,
  [1020] = 480,
  [1021] = 492,
  [1022] = 491,
  [1023] = 278,
  [1024] = 490,
  [1025] = 281,
  [1026] = 489,
  [1027] = 488,
  [1028] = 487,
  [1029] = 470,
  [1030] = 469,
  [1031] = 468,
  [1032] = 467,
  [1033] = 461,
  [1034] = 460,
  [1035] = 459,
  [1036] = 458,
  [1037] = 891,
  [1038] = 417,
  [1039] = 418,
  [1040] = 419,
  [1041] = 420,
  [1042] = 426,
  [1043] = 428,
  [1044] = 430,
  [1045] = 434,
  [1046] = 279,
  [1047] = 930,
  [1048] = 906,
  [1049] = 286,
  [1050] = 445,
  [1051] = 446,
  [1052] = 275,
  [1053] = 447,
  [1054] = 448,
  [1055] = 282,
  [1056] = 449,
  [1057] = 451,
  [1058] = 273,
  [1059] = 453,
  [1060] = 454,
  [1061] = 1061,
  [1062] = 1062,
  [1063] = 1063,
  [1064] = 1064,
  [1065] = 288,
  [1066] = 458,
  [1067] = 459,
  [1068] = 460,
  [1069] = 461,
  [1070] = 467,
  [1071] = 468,
  [1072] = 469,
  [1073] = 470,
  [1074] = 487,
  [1075] = 488,
  [1076] = 489,
  [1077] = 281,
  [1078] = 490,
  [1079] = 278,
  [1080] = 491,
  [1081] = 492,
  [1082] = 480,
  [1083] = 479,
  [1084] = 476,
  [1085] = 475,
  [1086] = 473,
  [1087] = 472,
  [1088] = 417,
  [1089] = 418,
  [1090] = 419,
  [1091] = 420,
  [1092] = 426,
  [1093] = 428,
  [1094] = 430,
  [1095] = 434,
  [1096] = 276,
  [1097] = 279,
  [1098] = 1098,
  [1099] = 286,
  [1100] = 445,
  [1101] = 446,
  [1102] = 275,
  [1103] = 447,
  [1104] = 448,
  [1105] = 282,
  [1106] = 449,
  [1107] = 451,
  [1108] = 273,
  [1109] = 453,
  [1110] = 454,
  [1111] = 887,
  [1112] = 1015,
  [1113] = 887,
  [1114] = 891,
  [1115] = 287,
  [1116] = 409,
  [1117] = 451,
  [1118] = 491,
  [1119] = 492,
  [1120] = 1120,
  [1121] = 761,
  [1122] = 484,
  [1123] = 483,
  [1124] = 1124,
  [1125] = 490,
  [1126] = 489,
  [1127] = 395,
  [1128] = 482,
  [1129] = 488,
  [1130] = 480,
  [1131] = 479,
  [1132] = 476,
  [1133] = 475,
  [1134] = 473,
  [1135] = 1135,
  [1136] = 472,
  [1137] = 284,
  [1138] = 487,
  [1139] = 337,
  [1140] = 333,
  [1141] = 1141,
  [1142] = 326,
  [1143] = 358,
  [1144] = 276,
  [1145] = 392,
  [1146] = 388,
  [1147] = 405,
  [1148] = 486,
  [1149] = 1149,
  [1150] = 287,
  [1151] = 406,
  [1152] = 485,
  [1153] = 328,
  [1154] = 323,
  [1155] = 357,
  [1156] = 383,
  [1157] = 1124,
  [1158] = 1158,
  [1159] = 446,
  [1160] = 351,
  [1161] = 408,
  [1162] = 474,
  [1163] = 445,
  [1164] = 366,
  [1165] = 1165,
  [1166] = 1120,
  [1167] = 393,
  [1168] = 394,
  [1169] = 361,
  [1170] = 276,
  [1171] = 360,
  [1172] = 481,
  [1173] = 287,
  [1174] = 276,
  [1175] = 478,
  [1176] = 410,
  [1177] = 287,
  [1178] = 412,
  [1179] = 437,
  [1180] = 337,
  [1181] = 333,
  [1182] = 326,
  [1183] = 358,
  [1184] = 477,
  [1185] = 471,
  [1186] = 470,
  [1187] = 392,
  [1188] = 447,
  [1189] = 469,
  [1190] = 468,
  [1191] = 1191,
  [1192] = 388,
  [1193] = 443,
  [1194] = 1194,
  [1195] = 467,
  [1196] = 442,
  [1197] = 466,
  [1198] = 465,
  [1199] = 464,
  [1200] = 462,
  [1201] = 461,
  [1202] = 460,
  [1203] = 459,
  [1204] = 413,
  [1205] = 1135,
  [1206] = 1141,
  [1207] = 416,
  [1208] = 458,
  [1209] = 457,
  [1210] = 456,
  [1211] = 455,
  [1212] = 328,
  [1213] = 323,
  [1214] = 357,
  [1215] = 383,
  [1216] = 351,
  [1217] = 417,
  [1218] = 418,
  [1219] = 1191,
  [1220] = 1194,
  [1221] = 366,
  [1222] = 1222,
  [1223] = 419,
  [1224] = 1224,
  [1225] = 420,
  [1226] = 426,
  [1227] = 1194,
  [1228] = 1191,
  [1229] = 428,
  [1230] = 1230,
  [1231] = 1231,
  [1232] = 393,
  [1233] = 394,
  [1234] = 430,
  [1235] = 434,
  [1236] = 436,
  [1237] = 1237,
  [1238] = 761,
  [1239] = 448,
  [1240] = 361,
  [1241] = 1141,
  [1242] = 1135,
  [1243] = 360,
  [1244] = 454,
  [1245] = 453,
  [1246] = 449,
  [1247] = 446,
  [1248] = 472,
  [1249] = 287,
  [1250] = 1120,
  [1251] = 278,
  [1252] = 491,
  [1253] = 1124,
  [1254] = 492,
  [1255] = 393,
  [1256] = 1256,
  [1257] = 1257,
  [1258] = 394,
  [1259] = 480,
  [1260] = 1260,
  [1261] = 479,
  [1262] = 1262,
  [1263] = 1263,
  [1264] = 358,
  [1265] = 476,
  [1266] = 475,
  [1267] = 1267,
  [1268] = 1268,
  [1269] = 276,
  [1270] = 361,
  [1271] = 360,
  [1272] = 284,
  [1273] = 337,
  [1274] = 333,
  [1275] = 326,
  [1276] = 458,
  [1277] = 1277,
  [1278] = 1267,
  [1279] = 473,
  [1280] = 479,
  [1281] = 1268,
  [1282] = 1120,
  [1283] = 1283,
  [1284] = 328,
  [1285] = 405,
  [1286] = 406,
  [1287] = 388,
  [1288] = 328,
  [1289] = 323,
  [1290] = 357,
  [1291] = 383,
  [1292] = 1124,
  [1293] = 1293,
  [1294] = 1294,
  [1295] = 351,
  [1296] = 366,
  [1297] = 490,
  [1298] = 1298,
  [1299] = 408,
  [1300] = 409,
  [1301] = 1268,
  [1302] = 410,
  [1303] = 1267,
  [1304] = 412,
  [1305] = 798,
  [1306] = 769,
  [1307] = 413,
  [1308] = 416,
  [1309] = 393,
  [1310] = 394,
  [1311] = 361,
  [1312] = 360,
  [1313] = 417,
  [1314] = 417,
  [1315] = 418,
  [1316] = 337,
  [1317] = 419,
  [1318] = 420,
  [1319] = 333,
  [1320] = 426,
  [1321] = 428,
  [1322] = 326,
  [1323] = 430,
  [1324] = 434,
  [1325] = 358,
  [1326] = 279,
  [1327] = 1267,
  [1328] = 749,
  [1329] = 750,
  [1330] = 751,
  [1331] = 418,
  [1332] = 286,
  [1333] = 281,
  [1334] = 753,
  [1335] = 759,
  [1336] = 760,
  [1337] = 761,
  [1338] = 287,
  [1339] = 419,
  [1340] = 420,
  [1341] = 489,
  [1342] = 1267,
  [1343] = 1268,
  [1344] = 426,
  [1345] = 445,
  [1346] = 392,
  [1347] = 276,
  [1348] = 287,
  [1349] = 446,
  [1350] = 276,
  [1351] = 275,
  [1352] = 472,
  [1353] = 1268,
  [1354] = 428,
  [1355] = 1267,
  [1356] = 366,
  [1357] = 473,
  [1358] = 447,
  [1359] = 388,
  [1360] = 448,
  [1361] = 475,
  [1362] = 476,
  [1363] = 478,
  [1364] = 480,
  [1365] = 1263,
  [1366] = 1262,
  [1367] = 482,
  [1368] = 395,
  [1369] = 483,
  [1370] = 484,
  [1371] = 282,
  [1372] = 449,
  [1373] = 451,
  [1374] = 492,
  [1375] = 491,
  [1376] = 761,
  [1377] = 490,
  [1378] = 489,
  [1379] = 273,
  [1380] = 488,
  [1381] = 453,
  [1382] = 1277,
  [1383] = 488,
  [1384] = 1268,
  [1385] = 761,
  [1386] = 487,
  [1387] = 430,
  [1388] = 434,
  [1389] = 351,
  [1390] = 436,
  [1391] = 454,
  [1392] = 1298,
  [1393] = 288,
  [1394] = 1394,
  [1395] = 1395,
  [1396] = 1396,
  [1397] = 1397,
  [1398] = 486,
  [1399] = 1268,
  [1400] = 437,
  [1401] = 1267,
  [1402] = 487,
  [1403] = 485,
  [1404] = 481,
  [1405] = 1256,
  [1406] = 477,
  [1407] = 474,
  [1408] = 1394,
  [1409] = 459,
  [1410] = 460,
  [1411] = 442,
  [1412] = 443,
  [1413] = 461,
  [1414] = 471,
  [1415] = 470,
  [1416] = 469,
  [1417] = 468,
  [1418] = 467,
  [1419] = 466,
  [1420] = 465,
  [1421] = 464,
  [1422] = 462,
  [1423] = 461,
  [1424] = 460,
  [1425] = 459,
  [1426] = 458,
  [1427] = 457,
  [1428] = 456,
  [1429] = 1396,
  [1430] = 1397,
  [1431] = 455,
  [1432] = 467,
  [1433] = 445,
  [1434] = 383,
  [1435] = 1283,
  [1436] = 1436,
  [1437] = 1436,
  [1438] = 468,
  [1439] = 469,
  [1440] = 470,
  [1441] = 1268,
  [1442] = 447,
  [1443] = 1267,
  [1444] = 357,
  [1445] = 392,
  [1446] = 1257,
  [1447] = 454,
  [1448] = 453,
  [1449] = 1260,
  [1450] = 451,
  [1451] = 449,
  [1452] = 448,
  [1453] = 323,
  [1454] = 1454,
  [1455] = 393,
  [1456] = 1456,
  [1457] = 1457,
  [1458] = 1458,
  [1459] = 1459,
  [1460] = 1460,
  [1461] = 1456,
  [1462] = 1460,
  [1463] = 1458,
  [1464] = 1457,
  [1465] = 1465,
  [1466] = 1466,
  [1467] = 1467,
  [1468] = 1468,
  [1469] = 1457,
  [1470] = 1458,
  [1471] = 1460,
  [1472] = 1456,
  [1473] = 1456,
  [1474] = 1460,
  [1475] = 1458,
  [1476] = 1457,
  [1477] = 1477,
  [1478] = 1467,
  [1479] = 1468,
  [1480] = 1457,
  [1481] = 1458,
  [1482] = 1460,
  [1483] = 1456,
  [1484] = 1484,
  [1485] = 1485,
  [1486] = 276,
  [1487] = 287,
  [1488] = 1467,
  [1489] = 1456,
  [1490] = 1460,
  [1491] = 1467,
  [1492] = 1468,
  [1493] = 1124,
  [1494] = 1458,
  [1495] = 1457,
  [1496] = 1120,
  [1497] = 1457,
  [1498] = 1458,
  [1499] = 1460,
  [1500] = 1456,
  [1501] = 1468,
  [1502] = 1468,
  [1503] = 1467,
  [1504] = 1504,
  [1505] = 1456,
  [1506] = 1460,
  [1507] = 1458,
  [1508] = 1457,
  [1509] = 1509,
  [1510] = 1468,
  [1511] = 1467,
  [1512] = 1458,
  [1513] = 326,
  [1514] = 358,
  [1515] = 1515,
  [1516] = 360,
  [1517] = 392,
  [1518] = 388,
  [1519] = 394,
  [1520] = 361,
  [1521] = 328,
  [1522] = 323,
  [1523] = 357,
  [1524] = 1524,
  [1525] = 383,
  [1526] = 351,
  [1527] = 1527,
  [1528] = 1528,
  [1529] = 366,
  [1530] = 337,
  [1531] = 761,
  [1532] = 1532,
  [1533] = 1456,
  [1534] = 1460,
  [1535] = 1458,
  [1536] = 1456,
  [1537] = 1460,
  [1538] = 1458,
  [1539] = 1456,
  [1540] = 1460,
  [1541] = 1458,
  [1542] = 1456,
  [1543] = 1460,
  [1544] = 1458,
  [1545] = 1456,
  [1546] = 1460,
  [1547] = 1458,
  [1548] = 1456,
  [1549] = 1460,
  [1550] = 1458,
  [1551] = 1456,
  [1552] = 1460,
  [1553] = 333,
  [1554] = 1456,
  [1555] = 1460,
  [1556] = 1456,
  [1557] = 1456,
  [1558] = 1456,
  [1559] = 1456,
  [1560] = 1456,
  [1561] = 1456,
  [1562] = 1456,
  [1563] = 1456,
  [1564] = 1456,
  [1565] = 1456,
  [1566] = 1456,
  [1567] = 1456,
  [1568] = 1456,
};

static bool ts_lex(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      if (eof) ADVANCE(1);
      END_STATE();
    case 1:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      END_STATE();
    default:
      return false;
  }
}

static const TSLexMode ts_lex_modes[STATE_COUNT] = {
  [0] = {.lex_state = 0, .external_lex_state = 1},
  [1] = {.lex_state = 0, .external_lex_state = 2},
  [2] = {.lex_state = 0, .external_lex_state = 3},
  [3] = {.lex_state = 0, .external_lex_state = 3},
  [4] = {.lex_state = 0, .external_lex_state = 3},
  [5] = {.lex_state = 0, .external_lex_state = 3},
  [6] = {.lex_state = 0, .external_lex_state = 3},
  [7] = {.lex_state = 0, .external_lex_state = 3},
  [8] = {.lex_state = 0, .external_lex_state = 4},
  [9] = {.lex_state = 0, .external_lex_state = 4},
  [10] = {.lex_state = 0, .external_lex_state = 4},
  [11] = {.lex_state = 0, .external_lex_state = 5},
  [12] = {.lex_state = 0, .external_lex_state = 6},
  [13] = {.lex_state = 0, .external_lex_state = 7},
  [14] = {.lex_state = 0, .external_lex_state = 7},
  [15] = {.lex_state = 0, .external_lex_state = 7},
  [16] = {.lex_state = 0, .external_lex_state = 7},
  [17] = {.lex_state = 0, .external_lex_state = 7},
  [18] = {.lex_state = 0, .external_lex_state = 7},
  [19] = {.lex_state = 0, .external_lex_state = 7},
  [20] = {.lex_state = 0, .external_lex_state = 7},
  [21] = {.lex_state = 0, .external_lex_state = 7},
  [22] = {.lex_state = 0, .external_lex_state = 7},
  [23] = {.lex_state = 0, .external_lex_state = 7},
  [24] = {.lex_state = 0, .external_lex_state = 8},
  [25] = {.lex_state = 0, .external_lex_state = 8},
  [26] = {.lex_state = 0, .external_lex_state = 8},
  [27] = {.lex_state = 0, .external_lex_state = 8},
  [28] = {.lex_state = 0, .external_lex_state = 8},
  [29] = {.lex_state = 0, .external_lex_state = 8},
  [30] = {.lex_state = 0, .external_lex_state = 8},
  [31] = {.lex_state = 0, .external_lex_state = 8},
  [32] = {.lex_state = 0, .external_lex_state = 8},
  [33] = {.lex_state = 0, .external_lex_state = 8},
  [34] = {.lex_state = 0, .external_lex_state = 8},
  [35] = {.lex_state = 0, .external_lex_state = 9},
  [36] = {.lex_state = 0, .external_lex_state = 9},
  [37] = {.lex_state = 0, .external_lex_state = 10},
  [38] = {.lex_state = 0, .external_lex_state = 10},
  [39] = {.lex_state = 0, .external_lex_state = 10},
  [40] = {.lex_state = 0, .external_lex_state = 11},
  [41] = {.lex_state = 0, .external_lex_state = 12},
  [42] = {.lex_state = 0, .external_lex_state = 7},
  [43] = {.lex_state = 0, .external_lex_state = 7},
  [44] = {.lex_state = 0, .external_lex_state = 7},
  [45] = {.lex_state = 0, .external_lex_state = 7},
  [46] = {.lex_state = 0, .external_lex_state = 7},
  [47] = {.lex_state = 0, .external_lex_state = 7},
  [48] = {.lex_state = 0, .external_lex_state = 7},
  [49] = {.lex_state = 0, .external_lex_state = 7},
  [50] = {.lex_state = 0, .external_lex_state = 7},
  [51] = {.lex_state = 0, .external_lex_state = 7},
  [52] = {.lex_state = 0, .external_lex_state = 7},
  [53] = {.lex_state = 0, .external_lex_state = 7},
  [54] = {.lex_state = 0, .external_lex_state = 7},
  [55] = {.lex_state = 0, .external_lex_state = 8},
  [56] = {.lex_state = 0, .external_lex_state = 8},
  [57] = {.lex_state = 0, .external_lex_state = 8},
  [58] = {.lex_state = 0, .external_lex_state = 8},
  [59] = {.lex_state = 0, .external_lex_state = 11},
  [60] = {.lex_state = 0, .external_lex_state = 8},
  [61] = {.lex_state = 0, .external_lex_state = 8},
  [62] = {.lex_state = 0, .external_lex_state = 12},
  [63] = {.lex_state = 0, .external_lex_state = 8},
  [64] = {.lex_state = 0, .external_lex_state = 8},
  [65] = {.lex_state = 0, .external_lex_state = 8},
  [66] = {.lex_state = 0, .external_lex_state = 8},
  [67] = {.lex_state = 0, .external_lex_state = 8},
  [68] = {.lex_state = 0, .external_lex_state = 8},
  [69] = {.lex_state = 0, .external_lex_state = 8},
  [70] = {.lex_state = 0, .external_lex_state = 7},
  [71] = {.lex_state = 0, .external_lex_state = 7},
  [72] = {.lex_state = 0, .external_lex_state = 7},
  [73] = {.lex_state = 0, .external_lex_state = 7},
  [74] = {.lex_state = 0, .external_lex_state = 8},
  [75] = {.lex_state = 0, .external_lex_state = 8},
  [76] = {.lex_state = 0, .external_lex_state = 8},
  [77] = {.lex_state = 0, .external_lex_state = 8},
  [78] = {.lex_state = 0, .external_lex_state = 13},
  [79] = {.lex_state = 0, .external_lex_state = 13},
  [80] = {.lex_state = 0, .external_lex_state = 14},
  [81] = {.lex_state = 0, .external_lex_state = 14},
  [82] = {.lex_state = 0, .external_lex_state = 15},
  [83] = {.lex_state = 0, .external_lex_state = 15},
  [84] = {.lex_state = 0, .external_lex_state = 11},
  [85] = {.lex_state = 0, .external_lex_state = 12},
  [86] = {.lex_state = 0, .external_lex_state = 12},
  [87] = {.lex_state = 0, .external_lex_state = 11},
  [88] = {.lex_state = 0, .external_lex_state = 12},
  [89] = {.lex_state = 0, .external_lex_state = 12},
  [90] = {.lex_state = 0, .external_lex_state = 12},
  [91] = {.lex_state = 0, .external_lex_state = 11},
  [92] = {.lex_state = 0, .external_lex_state = 11},
  [93] = {.lex_state = 0, .external_lex_state = 11},
  [94] = {.lex_state = 0, .external_lex_state = 11},
  [95] = {.lex_state = 0, .external_lex_state = 12},
  [96] = {.lex_state = 0, .external_lex_state = 16},
  [97] = {.lex_state = 0, .external_lex_state = 16},
  [98] = {.lex_state = 0, .external_lex_state = 17},
  [99] = {.lex_state = 0, .external_lex_state = 17},
  [100] = {.lex_state = 0, .external_lex_state = 17},
  [101] = {.lex_state = 0, .external_lex_state = 18},
  [102] = {.lex_state = 0, .external_lex_state = 18},
  [103] = {.lex_state = 0, .external_lex_state = 18},
  [104] = {.lex_state = 0, .external_lex_state = 18},
  [105] = {.lex_state = 0, .external_lex_state = 17},
  [106] = {.lex_state = 0, .external_lex_state = 19},
  [107] = {.lex_state = 0, .external_lex_state = 20},
  [108] = {.lex_state = 0, .external_lex_state = 19},
  [109] = {.lex_state = 0, .external_lex_state = 19},
  [110] = {.lex_state = 0, .external_lex_state = 21},
  [111] = {.lex_state = 0, .external_lex_state = 21},
  [112] = {.lex_state = 0, .external_lex_state = 21},
  [113] = {.lex_state = 0, .external_lex_state = 21},
  [114] = {.lex_state = 0, .external_lex_state = 21},
  [115] = {.lex_state = 0, .external_lex_state = 21},
  [116] = {.lex_state = 0, .external_lex_state = 22},
  [117] = {.lex_state = 0, .external_lex_state = 22},
  [118] = {.lex_state = 0, .external_lex_state = 22},
  [119] = {.lex_state = 0, .external_lex_state = 22},
  [120] = {.lex_state = 0, .external_lex_state = 23},
  [121] = {.lex_state = 0, .external_lex_state = 23},
  [122] = {.lex_state = 0, .external_lex_state = 22},
  [123] = {.lex_state = 0, .external_lex_state = 9},
  [124] = {.lex_state = 0, .external_lex_state = 9},
  [125] = {.lex_state = 0, .external_lex_state = 9},
  [126] = {.lex_state = 0, .external_lex_state = 9},
  [127] = {.lex_state = 0, .external_lex_state = 24},
  [128] = {.lex_state = 0, .external_lex_state = 24},
  [129] = {.lex_state = 0, .external_lex_state = 25},
  [130] = {.lex_state = 0, .external_lex_state = 25},
  [131] = {.lex_state = 0, .external_lex_state = 26},
  [132] = {.lex_state = 0, .external_lex_state = 26},
  [133] = {.lex_state = 0, .external_lex_state = 27},
  [134] = {.lex_state = 0, .external_lex_state = 27},
  [135] = {.lex_state = 0, .external_lex_state = 27},
  [136] = {.lex_state = 0, .external_lex_state = 28},
  [137] = {.lex_state = 0, .external_lex_state = 28},
  [138] = {.lex_state = 0, .external_lex_state = 28},
  [139] = {.lex_state = 0, .external_lex_state = 26},
  [140] = {.lex_state = 0, .external_lex_state = 27},
  [141] = {.lex_state = 0, .external_lex_state = 28},
  [142] = {.lex_state = 0, .external_lex_state = 26},
  [143] = {.lex_state = 0, .external_lex_state = 27},
  [144] = {.lex_state = 0, .external_lex_state = 29},
  [145] = {.lex_state = 0, .external_lex_state = 26},
  [146] = {.lex_state = 0, .external_lex_state = 26},
  [147] = {.lex_state = 0, .external_lex_state = 29},
  [148] = {.lex_state = 0, .external_lex_state = 28},
  [149] = {.lex_state = 0, .external_lex_state = 26},
  [150] = {.lex_state = 0, .external_lex_state = 30},
  [151] = {.lex_state = 0, .external_lex_state = 26},
  [152] = {.lex_state = 0, .external_lex_state = 31},
  [153] = {.lex_state = 0, .external_lex_state = 28},
  [154] = {.lex_state = 0, .external_lex_state = 26},
  [155] = {.lex_state = 0, .external_lex_state = 27},
  [156] = {.lex_state = 0, .external_lex_state = 28},
  [157] = {.lex_state = 0, .external_lex_state = 26},
  [158] = {.lex_state = 0, .external_lex_state = 27},
  [159] = {.lex_state = 0, .external_lex_state = 26},
  [160] = {.lex_state = 0, .external_lex_state = 31},
  [161] = {.lex_state = 0, .external_lex_state = 26},
  [162] = {.lex_state = 0, .external_lex_state = 26},
  [163] = {.lex_state = 0, .external_lex_state = 32},
  [164] = {.lex_state = 0, .external_lex_state = 32},
  [165] = {.lex_state = 0, .external_lex_state = 33},
  [166] = {.lex_state = 0, .external_lex_state = 28},
  [167] = {.lex_state = 0, .external_lex_state = 27},
  [168] = {.lex_state = 0, .external_lex_state = 33},
  [169] = {.lex_state = 0, .external_lex_state = 27},
  [170] = {.lex_state = 0, .external_lex_state = 28},
  [171] = {.lex_state = 0, .external_lex_state = 14},
  [172] = {.lex_state = 0, .external_lex_state = 34},
  [173] = {.lex_state = 0, .external_lex_state = 14},
  [174] = {.lex_state = 0, .external_lex_state = 14},
  [175] = {.lex_state = 0, .external_lex_state = 34},
  [176] = {.lex_state = 0, .external_lex_state = 14},
  [177] = {.lex_state = 0, .external_lex_state = 15},
  [178] = {.lex_state = 0, .external_lex_state = 15},
  [179] = {.lex_state = 0, .external_lex_state = 15},
  [180] = {.lex_state = 0, .external_lex_state = 35},
  [181] = {.lex_state = 0, .external_lex_state = 35},
  [182] = {.lex_state = 0, .external_lex_state = 36},
  [183] = {.lex_state = 0, .external_lex_state = 36},
  [184] = {.lex_state = 0, .external_lex_state = 37},
  [185] = {.lex_state = 0, .external_lex_state = 37},
  [186] = {.lex_state = 0, .external_lex_state = 38},
  [187] = {.lex_state = 0, .external_lex_state = 37},
  [188] = {.lex_state = 0, .external_lex_state = 38},
  [189] = {.lex_state = 0, .external_lex_state = 35},
  [190] = {.lex_state = 0, .external_lex_state = 38},
  [191] = {.lex_state = 0, .external_lex_state = 36},
  [192] = {.lex_state = 0, .external_lex_state = 39},
  [193] = {.lex_state = 0, .external_lex_state = 39},
  [194] = {.lex_state = 0, .external_lex_state = 39},
  [195] = {.lex_state = 0, .external_lex_state = 40},
  [196] = {.lex_state = 0, .external_lex_state = 41},
  [197] = {.lex_state = 0, .external_lex_state = 42},
  [198] = {.lex_state = 0, .external_lex_state = 40},
  [199] = {.lex_state = 0, .external_lex_state = 42},
  [200] = {.lex_state = 0, .external_lex_state = 42},
  [201] = {.lex_state = 0, .external_lex_state = 41},
  [202] = {.lex_state = 0, .external_lex_state = 42},
  [203] = {.lex_state = 0, .external_lex_state = 40},
  [204] = {.lex_state = 0, .external_lex_state = 42},
  [205] = {.lex_state = 0, .external_lex_state = 41},
  [206] = {.lex_state = 0, .external_lex_state = 43},
  [207] = {.lex_state = 0, .external_lex_state = 43},
  [208] = {.lex_state = 0, .external_lex_state = 44},
  [209] = {.lex_state = 0, .external_lex_state = 44},
  [210] = {.lex_state = 0, .external_lex_state = 43},
  [211] = {.lex_state = 0, .external_lex_state = 43},
  [212] = {.lex_state = 0, .external_lex_state = 43},
  [213] = {.lex_state = 0, .external_lex_state = 45},
  [214] = {.lex_state = 0, .external_lex_state = 24},
  [215] = {.lex_state = 0, .external_lex_state = 46},
  [216] = {.lex_state = 0, .external_lex_state = 47},
  [217] = {.lex_state = 0, .external_lex_state = 47},
  [218] = {.lex_state = 0, .external_lex_state = 48},
  [219] = {.lex_state = 0, .external_lex_state = 48},
  [220] = {.lex_state = 0, .external_lex_state = 45},
  [221] = {.lex_state = 0, .external_lex_state = 24},
  [222] = {.lex_state = 0, .external_lex_state = 46},
  [223] = {.lex_state = 0, .external_lex_state = 48},
  [224] = {.lex_state = 0, .external_lex_state = 45},
  [225] = {.lex_state = 0, .external_lex_state = 45},
  [226] = {.lex_state = 0, .external_lex_state = 47},
  [227] = {.lex_state = 0, .external_lex_state = 24},
  [228] = {.lex_state = 0, .external_lex_state = 47},
  [229] = {.lex_state = 0, .external_lex_state = 46},
  [230] = {.lex_state = 0, .external_lex_state = 24},
  [231] = {.lex_state = 0, .external_lex_state = 48},
  [232] = {.lex_state = 0, .external_lex_state = 47},
  [233] = {.lex_state = 0, .external_lex_state = 24},
  [234] = {.lex_state = 0, .external_lex_state = 45},
  [235] = {.lex_state = 0, .external_lex_state = 36},
  [236] = {.lex_state = 0, .external_lex_state = 36},
  [237] = {.lex_state = 0, .external_lex_state = 25},
  [238] = {.lex_state = 0, .external_lex_state = 25},
  [239] = {.lex_state = 0, .external_lex_state = 25},
  [240] = {.lex_state = 0, .external_lex_state = 25},
  [241] = {.lex_state = 0, .external_lex_state = 36},
  [242] = {.lex_state = 0, .external_lex_state = 25},
  [243] = {.lex_state = 0, .external_lex_state = 29},
  [244] = {.lex_state = 0, .external_lex_state = 31},
  [245] = {.lex_state = 0, .external_lex_state = 10},
  [246] = {.lex_state = 0, .external_lex_state = 49},
  [247] = {.lex_state = 0, .external_lex_state = 29},
  [248] = {.lex_state = 0, .external_lex_state = 31},
  [249] = {.lex_state = 0, .external_lex_state = 10},
  [250] = {.lex_state = 0, .external_lex_state = 49},
  [251] = {.lex_state = 0, .external_lex_state = 49},
  [252] = {.lex_state = 0, .external_lex_state = 10},
  [253] = {.lex_state = 0, .external_lex_state = 10},
  [254] = {.lex_state = 0, .external_lex_state = 29},
  [255] = {.lex_state = 0, .external_lex_state = 31},
  [256] = {.lex_state = 0, .external_lex_state = 29},
  [257] = {.lex_state = 0, .external_lex_state = 49},
  [258] = {.lex_state = 0, .external_lex_state = 10},
  [259] = {.lex_state = 0, .external_lex_state = 31},
  [260] = {.lex_state = 0, .external_lex_state = 31},
  [261] = {.lex_state = 0, .external_lex_state = 49},
  [262] = {.lex_state = 0, .external_lex_state = 10},
  [263] = {.lex_state = 0, .external_lex_state = 10},
  [264] = {.lex_state = 0, .external_lex_state = 29},
  [265] = {.lex_state = 0, .external_lex_state = 32},
  [266] = {.lex_state = 0, .external_lex_state = 32},
  [267] = {.lex_state = 0, .external_lex_state = 32},
  [268] = {.lex_state = 0, .external_lex_state = 32},
  [269] = {.lex_state = 0, .external_lex_state = 32},
  [270] = {.lex_state = 0, .external_lex_state = 50},
  [271] = {.lex_state = 0, .external_lex_state = 50},
  [272] = {.lex_state = 0, .external_lex_state = 50},
  [273] = {.lex_state = 0, .external_lex_state = 50},
  [274] = {.lex_state = 0, .external_lex_state = 50},
  [275] = {.lex_state = 0, .external_lex_state = 50},
  [276] = {.lex_state = 0, .external_lex_state = 50},
  [277] = {.lex_state = 0, .external_lex_state = 50},
  [278] = {.lex_state = 0, .external_lex_state = 50},
  [279] = {.lex_state = 0, .external_lex_state = 50},
  [280] = {.lex_state = 0, .external_lex_state = 50},
  [281] = {.lex_state = 0, .external_lex_state = 50},
  [282] = {.lex_state = 0, .external_lex_state = 50},
  [283] = {.lex_state = 0, .external_lex_state = 50},
  [284] = {.lex_state = 0, .external_lex_state = 50},
  [285] = {.lex_state = 0, .external_lex_state = 50},
  [286] = {.lex_state = 0, .external_lex_state = 50},
  [287] = {.lex_state = 0, .external_lex_state = 50},
  [288] = {.lex_state = 0, .external_lex_state = 50},
  [289] = {.lex_state = 0, .external_lex_state = 50},
  [290] = {.lex_state = 0, .external_lex_state = 50},
  [291] = {.lex_state = 0, .external_lex_state = 50},
  [292] = {.lex_state = 0, .external_lex_state = 50},
  [293] = {.lex_state = 0, .external_lex_state = 51},
  [294] = {.lex_state = 0, .external_lex_state = 51},
  [295] = {.lex_state = 0, .external_lex_state = 51},
  [296] = {.lex_state = 0, .external_lex_state = 51},
  [297] = {.lex_state = 0, .external_lex_state = 51},
  [298] = {.lex_state = 0, .external_lex_state = 51},
  [299] = {.lex_state = 0, .external_lex_state = 51},
  [300] = {.lex_state = 0, .external_lex_state = 51},
  [301] = {.lex_state = 0, .external_lex_state = 52},
  [302] = {.lex_state = 0, .external_lex_state = 52},
  [303] = {.lex_state = 0, .external_lex_state = 51},
  [304] = {.lex_state = 0, .external_lex_state = 53},
  [305] = {.lex_state = 0, .external_lex_state = 52},
  [306] = {.lex_state = 0, .external_lex_state = 52},
  [307] = {.lex_state = 0, .external_lex_state = 51},
  [308] = {.lex_state = 0, .external_lex_state = 51},
  [309] = {.lex_state = 0, .external_lex_state = 54},
  [310] = {.lex_state = 0, .external_lex_state = 51},
  [311] = {.lex_state = 0, .external_lex_state = 51},
  [312] = {.lex_state = 0, .external_lex_state = 51},
  [313] = {.lex_state = 0, .external_lex_state = 51},
  [314] = {.lex_state = 0, .external_lex_state = 52},
  [315] = {.lex_state = 0, .external_lex_state = 52},
  [316] = {.lex_state = 0, .external_lex_state = 51},
  [317] = {.lex_state = 0, .external_lex_state = 51},
  [318] = {.lex_state = 0, .external_lex_state = 51},
  [319] = {.lex_state = 0, .external_lex_state = 51},
  [320] = {.lex_state = 0, .external_lex_state = 51},
  [321] = {.lex_state = 0, .external_lex_state = 51},
  [322] = {.lex_state = 0, .external_lex_state = 51},
  [323] = {.lex_state = 0, .external_lex_state = 52},
  [324] = {.lex_state = 0, .external_lex_state = 52},
  [325] = {.lex_state = 0, .external_lex_state = 52},
  [326] = {.lex_state = 0, .external_lex_state = 52},
  [327] = {.lex_state = 0, .external_lex_state = 26},
  [328] = {.lex_state = 0, .external_lex_state = 52},
  [329] = {.lex_state = 0, .external_lex_state = 52},
  [330] = {.lex_state = 0, .external_lex_state = 52},
  [331] = {.lex_state = 0, .external_lex_state = 52},
  [332] = {.lex_state = 0, .external_lex_state = 52},
  [333] = {.lex_state = 0, .external_lex_state = 52},
  [334] = {.lex_state = 0, .external_lex_state = 52},
  [335] = {.lex_state = 0, .external_lex_state = 52},
  [336] = {.lex_state = 0, .external_lex_state = 52},
  [337] = {.lex_state = 0, .external_lex_state = 52},
  [338] = {.lex_state = 0, .external_lex_state = 52},
  [339] = {.lex_state = 0, .external_lex_state = 52},
  [340] = {.lex_state = 0, .external_lex_state = 52},
  [341] = {.lex_state = 0, .external_lex_state = 52},
  [342] = {.lex_state = 0, .external_lex_state = 52},
  [343] = {.lex_state = 0, .external_lex_state = 52},
  [344] = {.lex_state = 0, .external_lex_state = 52},
  [345] = {.lex_state = 0, .external_lex_state = 55},
  [346] = {.lex_state = 0, .external_lex_state = 52},
  [347] = {.lex_state = 0, .external_lex_state = 52},
  [348] = {.lex_state = 0, .external_lex_state = 52},
  [349] = {.lex_state = 0, .external_lex_state = 52},
  [350] = {.lex_state = 0, .external_lex_state = 52},
  [351] = {.lex_state = 0, .external_lex_state = 52},
  [352] = {.lex_state = 0, .external_lex_state = 52},
  [353] = {.lex_state = 0, .external_lex_state = 52},
  [354] = {.lex_state = 0, .external_lex_state = 52},
  [355] = {.lex_state = 0, .external_lex_state = 52},
  [356] = {.lex_state = 0, .external_lex_state = 52},
  [357] = {.lex_state = 0, .external_lex_state = 52},
  [358] = {.lex_state = 0, .external_lex_state = 52},
  [359] = {.lex_state = 0, .external_lex_state = 52},
  [360] = {.lex_state = 0, .external_lex_state = 52},
  [361] = {.lex_state = 0, .external_lex_state = 52},
  [362] = {.lex_state = 0, .external_lex_state = 52},
  [363] = {.lex_state = 0, .external_lex_state = 52},
  [364] = {.lex_state = 0, .external_lex_state = 52},
  [365] = {.lex_state = 0, .external_lex_state = 52},
  [366] = {.lex_state = 0, .external_lex_state = 52},
  [367] = {.lex_state = 0, .external_lex_state = 52},
  [368] = {.lex_state = 0, .external_lex_state = 52},
  [369] = {.lex_state = 0, .external_lex_state = 56},
  [370] = {.lex_state = 0, .external_lex_state = 26},
  [371] = {.lex_state = 0, .external_lex_state = 52},
  [372] = {.lex_state = 0, .external_lex_state = 52},
  [373] = {.lex_state = 0, .external_lex_state = 52},
  [374] = {.lex_state = 0, .external_lex_state = 52},
  [375] = {.lex_state = 0, .external_lex_state = 52},
  [376] = {.lex_state = 0, .external_lex_state = 52},
  [377] = {.lex_state = 0, .external_lex_state = 52},
  [378] = {.lex_state = 0, .external_lex_state = 52},
  [379] = {.lex_state = 0, .external_lex_state = 52},
  [380] = {.lex_state = 0, .external_lex_state = 52},
  [381] = {.lex_state = 0, .external_lex_state = 52},
  [382] = {.lex_state = 0, .external_lex_state = 52},
  [383] = {.lex_state = 0, .external_lex_state = 52},
  [384] = {.lex_state = 0, .external_lex_state = 52},
  [385] = {.lex_state = 0, .external_lex_state = 52},
  [386] = {.lex_state = 0, .external_lex_state = 52},
  [387] = {.lex_state = 0, .external_lex_state = 26},
  [388] = {.lex_state = 0, .external_lex_state = 52},
  [389] = {.lex_state = 0, .external_lex_state = 52},
  [390] = {.lex_state = 0, .external_lex_state = 52},
  [391] = {.lex_state = 0, .external_lex_state = 52},
  [392] = {.lex_state = 0, .external_lex_state = 52},
  [393] = {.lex_state = 0, .external_lex_state = 52},
  [394] = {.lex_state = 0, .external_lex_state = 52},
  [395] = {.lex_state = 0, .external_lex_state = 26},
  [396] = {.lex_state = 0, .external_lex_state = 26},
  [397] = {.lex_state = 0, .external_lex_state = 26},
  [398] = {.lex_state = 0, .external_lex_state = 26},
  [399] = {.lex_state = 0, .external_lex_state = 26},
  [400] = {.lex_state = 0, .external_lex_state = 26},
  [401] = {.lex_state = 0, .external_lex_state = 26},
  [402] = {.lex_state = 0, .external_lex_state = 26},
  [403] = {.lex_state = 0, .external_lex_state = 26},
  [404] = {.lex_state = 0, .external_lex_state = 26},
  [405] = {.lex_state = 0, .external_lex_state = 26},
  [406] = {.lex_state = 0, .external_lex_state = 26},
  [407] = {.lex_state = 0, .external_lex_state = 26},
  [408] = {.lex_state = 0, .external_lex_state = 26},
  [409] = {.lex_state = 0, .external_lex_state = 26},
  [410] = {.lex_state = 0, .external_lex_state = 26},
  [411] = {.lex_state = 0, .external_lex_state = 26},
  [412] = {.lex_state = 0, .external_lex_state = 26},
  [413] = {.lex_state = 0, .external_lex_state = 26},
  [414] = {.lex_state = 0, .external_lex_state = 26},
  [415] = {.lex_state = 0, .external_lex_state = 26},
  [416] = {.lex_state = 0, .external_lex_state = 26},
  [417] = {.lex_state = 0, .external_lex_state = 26},
  [418] = {.lex_state = 0, .external_lex_state = 26},
  [419] = {.lex_state = 0, .external_lex_state = 26},
  [420] = {.lex_state = 0, .external_lex_state = 26},
  [421] = {.lex_state = 0, .external_lex_state = 26},
  [422] = {.lex_state = 0, .external_lex_state = 26},
  [423] = {.lex_state = 0, .external_lex_state = 26},
  [424] = {.lex_state = 0, .external_lex_state = 26},
  [425] = {.lex_state = 0, .external_lex_state = 26},
  [426] = {.lex_state = 0, .external_lex_state = 26},
  [427] = {.lex_state = 0, .external_lex_state = 26},
  [428] = {.lex_state = 0, .external_lex_state = 26},
  [429] = {.lex_state = 0, .external_lex_state = 26},
  [430] = {.lex_state = 0, .external_lex_state = 26},
  [431] = {.lex_state = 0, .external_lex_state = 26},
  [432] = {.lex_state = 0, .external_lex_state = 26},
  [433] = {.lex_state = 0, .external_lex_state = 26},
  [434] = {.lex_state = 0, .external_lex_state = 26},
  [435] = {.lex_state = 0, .external_lex_state = 26},
  [436] = {.lex_state = 0, .external_lex_state = 26},
  [437] = {.lex_state = 0, .external_lex_state = 26},
  [438] = {.lex_state = 0, .external_lex_state = 26},
  [439] = {.lex_state = 0, .external_lex_state = 26},
  [440] = {.lex_state = 0, .external_lex_state = 26},
  [441] = {.lex_state = 0, .external_lex_state = 26},
  [442] = {.lex_state = 0, .external_lex_state = 26},
  [443] = {.lex_state = 0, .external_lex_state = 26},
  [444] = {.lex_state = 0, .external_lex_state = 57},
  [445] = {.lex_state = 0, .external_lex_state = 26},
  [446] = {.lex_state = 0, .external_lex_state = 26},
  [447] = {.lex_state = 0, .external_lex_state = 26},
  [448] = {.lex_state = 0, .external_lex_state = 26},
  [449] = {.lex_state = 0, .external_lex_state = 26},
  [450] = {.lex_state = 0, .external_lex_state = 26},
  [451] = {.lex_state = 0, .external_lex_state = 26},
  [452] = {.lex_state = 0, .external_lex_state = 26},
  [453] = {.lex_state = 0, .external_lex_state = 26},
  [454] = {.lex_state = 0, .external_lex_state = 26},
  [455] = {.lex_state = 0, .external_lex_state = 26},
  [456] = {.lex_state = 0, .external_lex_state = 26},
  [457] = {.lex_state = 0, .external_lex_state = 26},
  [458] = {.lex_state = 0, .external_lex_state = 26},
  [459] = {.lex_state = 0, .external_lex_state = 26},
  [460] = {.lex_state = 0, .external_lex_state = 26},
  [461] = {.lex_state = 0, .external_lex_state = 26},
  [462] = {.lex_state = 0, .external_lex_state = 26},
  [463] = {.lex_state = 0, .external_lex_state = 26},
  [464] = {.lex_state = 0, .external_lex_state = 26},
  [465] = {.lex_state = 0, .external_lex_state = 26},
  [466] = {.lex_state = 0, .external_lex_state = 26},
  [467] = {.lex_state = 0, .external_lex_state = 26},
  [468] = {.lex_state = 0, .external_lex_state = 26},
  [469] = {.lex_state = 0, .external_lex_state = 26},
  [470] = {.lex_state = 0, .external_lex_state = 26},
  [471] = {.lex_state = 0, .external_lex_state = 26},
  [472] = {.lex_state = 0, .external_lex_state = 26},
  [473] = {.lex_state = 0, .external_lex_state = 26},
  [474] = {.lex_state = 0, .external_lex_state = 26},
  [475] = {.lex_state = 0, .external_lex_state = 26},
  [476] = {.lex_state = 0, .external_lex_state = 26},
  [477] = {.lex_state = 0, .external_lex_state = 26},
  [478] = {.lex_state = 0, .external_lex_state = 26},
  [479] = {.lex_state = 0, .external_lex_state = 26},
  [480] = {.lex_state = 0, .external_lex_state = 26},
  [481] = {.lex_state = 0, .external_lex_state = 26},
  [482] = {.lex_state = 0, .external_lex_state = 26},
  [483] = {.lex_state = 0, .external_lex_state = 26},
  [484] = {.lex_state = 0, .external_lex_state = 26},
  [485] = {.lex_state = 0, .external_lex_state = 26},
  [486] = {.lex_state = 0, .external_lex_state = 26},
  [487] = {.lex_state = 0, .external_lex_state = 26},
  [488] = {.lex_state = 0, .external_lex_state = 26},
  [489] = {.lex_state = 0, .external_lex_state = 26},
  [490] = {.lex_state = 0, .external_lex_state = 26},
  [491] = {.lex_state = 0, .external_lex_state = 26},
  [492] = {.lex_state = 0, .external_lex_state = 26},
  [493] = {.lex_state = 0, .external_lex_state = 58},
  [494] = {.lex_state = 0, .external_lex_state = 59},
  [495] = {.lex_state = 0, .external_lex_state = 60},
  [496] = {.lex_state = 0, .external_lex_state = 61},
  [497] = {.lex_state = 0, .external_lex_state = 62},
  [498] = {.lex_state = 0, .external_lex_state = 62},
  [499] = {.lex_state = 0, .external_lex_state = 60},
  [500] = {.lex_state = 0, .external_lex_state = 63},
  [501] = {.lex_state = 0, .external_lex_state = 53},
  [502] = {.lex_state = 0, .external_lex_state = 64},
  [503] = {.lex_state = 0, .external_lex_state = 54},
  [504] = {.lex_state = 0, .external_lex_state = 64},
  [505] = {.lex_state = 0, .external_lex_state = 65},
  [506] = {.lex_state = 0, .external_lex_state = 53},
  [507] = {.lex_state = 0, .external_lex_state = 54},
  [508] = {.lex_state = 0, .external_lex_state = 66},
  [509] = {.lex_state = 0, .external_lex_state = 65},
  [510] = {.lex_state = 0, .external_lex_state = 67},
  [511] = {.lex_state = 0, .external_lex_state = 67},
  [512] = {.lex_state = 0, .external_lex_state = 67},
  [513] = {.lex_state = 0, .external_lex_state = 67},
  [514] = {.lex_state = 0, .external_lex_state = 67},
  [515] = {.lex_state = 0, .external_lex_state = 67},
  [516] = {.lex_state = 0, .external_lex_state = 67},
  [517] = {.lex_state = 0, .external_lex_state = 67},
  [518] = {.lex_state = 0, .external_lex_state = 67},
  [519] = {.lex_state = 0, .external_lex_state = 67},
  [520] = {.lex_state = 0, .external_lex_state = 56},
  [521] = {.lex_state = 0, .external_lex_state = 67},
  [522] = {.lex_state = 0, .external_lex_state = 67},
  [523] = {.lex_state = 0, .external_lex_state = 67},
  [524] = {.lex_state = 0, .external_lex_state = 67},
  [525] = {.lex_state = 0, .external_lex_state = 67},
  [526] = {.lex_state = 0, .external_lex_state = 67},
  [527] = {.lex_state = 0, .external_lex_state = 67},
  [528] = {.lex_state = 0, .external_lex_state = 67},
  [529] = {.lex_state = 0, .external_lex_state = 67},
  [530] = {.lex_state = 0, .external_lex_state = 67},
  [531] = {.lex_state = 0, .external_lex_state = 67},
  [532] = {.lex_state = 0, .external_lex_state = 67},
  [533] = {.lex_state = 0, .external_lex_state = 67},
  [534] = {.lex_state = 0, .external_lex_state = 67},
  [535] = {.lex_state = 0, .external_lex_state = 55},
  [536] = {.lex_state = 0, .external_lex_state = 68},
  [537] = {.lex_state = 0, .external_lex_state = 67},
  [538] = {.lex_state = 0, .external_lex_state = 67},
  [539] = {.lex_state = 0, .external_lex_state = 67},
  [540] = {.lex_state = 0, .external_lex_state = 67},
  [541] = {.lex_state = 0, .external_lex_state = 56},
  [542] = {.lex_state = 0, .external_lex_state = 55},
  [543] = {.lex_state = 0, .external_lex_state = 67},
  [544] = {.lex_state = 0, .external_lex_state = 67},
  [545] = {.lex_state = 0, .external_lex_state = 67},
  [546] = {.lex_state = 0, .external_lex_state = 67},
  [547] = {.lex_state = 0, .external_lex_state = 67},
  [548] = {.lex_state = 0, .external_lex_state = 67},
  [549] = {.lex_state = 0, .external_lex_state = 67},
  [550] = {.lex_state = 0, .external_lex_state = 67},
  [551] = {.lex_state = 0, .external_lex_state = 67},
  [552] = {.lex_state = 0, .external_lex_state = 68},
  [553] = {.lex_state = 0, .external_lex_state = 67},
  [554] = {.lex_state = 0, .external_lex_state = 67},
  [555] = {.lex_state = 0, .external_lex_state = 67},
  [556] = {.lex_state = 0, .external_lex_state = 67},
  [557] = {.lex_state = 0, .external_lex_state = 67},
  [558] = {.lex_state = 0, .external_lex_state = 67},
  [559] = {.lex_state = 0, .external_lex_state = 67},
  [560] = {.lex_state = 0, .external_lex_state = 67},
  [561] = {.lex_state = 0, .external_lex_state = 67},
  [562] = {.lex_state = 0, .external_lex_state = 67},
  [563] = {.lex_state = 0, .external_lex_state = 67},
  [564] = {.lex_state = 0, .external_lex_state = 57},
  [565] = {.lex_state = 0, .external_lex_state = 69},
  [566] = {.lex_state = 0, .external_lex_state = 69},
  [567] = {.lex_state = 0, .external_lex_state = 69},
  [568] = {.lex_state = 0, .external_lex_state = 70},
  [569] = {.lex_state = 0, .external_lex_state = 69},
  [570] = {.lex_state = 0, .external_lex_state = 57},
  [571] = {.lex_state = 0, .external_lex_state = 69},
  [572] = {.lex_state = 0, .external_lex_state = 71},
  [573] = {.lex_state = 0, .external_lex_state = 71},
  [574] = {.lex_state = 0, .external_lex_state = 71},
  [575] = {.lex_state = 0, .external_lex_state = 67},
  [576] = {.lex_state = 0, .external_lex_state = 71},
  [577] = {.lex_state = 0, .external_lex_state = 71},
  [578] = {.lex_state = 0, .external_lex_state = 71},
  [579] = {.lex_state = 0, .external_lex_state = 71},
  [580] = {.lex_state = 0, .external_lex_state = 71},
  [581] = {.lex_state = 0, .external_lex_state = 67},
  [582] = {.lex_state = 0, .external_lex_state = 71},
  [583] = {.lex_state = 0, .external_lex_state = 71},
  [584] = {.lex_state = 0, .external_lex_state = 71},
  [585] = {.lex_state = 0, .external_lex_state = 70},
  [586] = {.lex_state = 0, .external_lex_state = 67},
  [587] = {.lex_state = 0, .external_lex_state = 71},
  [588] = {.lex_state = 0, .external_lex_state = 71},
  [589] = {.lex_state = 0, .external_lex_state = 67},
  [590] = {.lex_state = 0, .external_lex_state = 71},
  [591] = {.lex_state = 0, .external_lex_state = 71},
  [592] = {.lex_state = 0, .external_lex_state = 71},
  [593] = {.lex_state = 0, .external_lex_state = 71},
  [594] = {.lex_state = 0, .external_lex_state = 71},
  [595] = {.lex_state = 0, .external_lex_state = 71},
  [596] = {.lex_state = 0, .external_lex_state = 71},
  [597] = {.lex_state = 0, .external_lex_state = 71},
  [598] = {.lex_state = 0, .external_lex_state = 71},
  [599] = {.lex_state = 0, .external_lex_state = 71},
  [600] = {.lex_state = 0, .external_lex_state = 72},
  [601] = {.lex_state = 0, .external_lex_state = 71},
  [602] = {.lex_state = 0, .external_lex_state = 71},
  [603] = {.lex_state = 0, .external_lex_state = 71},
  [604] = {.lex_state = 0, .external_lex_state = 71},
  [605] = {.lex_state = 0, .external_lex_state = 71},
  [606] = {.lex_state = 0, .external_lex_state = 71},
  [607] = {.lex_state = 0, .external_lex_state = 71},
  [608] = {.lex_state = 0, .external_lex_state = 71},
  [609] = {.lex_state = 0, .external_lex_state = 71},
  [610] = {.lex_state = 0, .external_lex_state = 71},
  [611] = {.lex_state = 0, .external_lex_state = 71},
  [612] = {.lex_state = 0, .external_lex_state = 71},
  [613] = {.lex_state = 0, .external_lex_state = 71},
  [614] = {.lex_state = 0, .external_lex_state = 71},
  [615] = {.lex_state = 0, .external_lex_state = 71},
  [616] = {.lex_state = 0, .external_lex_state = 71},
  [617] = {.lex_state = 0, .external_lex_state = 71},
  [618] = {.lex_state = 0, .external_lex_state = 71},
  [619] = {.lex_state = 0, .external_lex_state = 71},
  [620] = {.lex_state = 0, .external_lex_state = 71},
  [621] = {.lex_state = 0, .external_lex_state = 71},
  [622] = {.lex_state = 0, .external_lex_state = 71},
  [623] = {.lex_state = 0, .external_lex_state = 71},
  [624] = {.lex_state = 0, .external_lex_state = 71},
  [625] = {.lex_state = 0, .external_lex_state = 71},
  [626] = {.lex_state = 0, .external_lex_state = 73},
  [627] = {.lex_state = 0, .external_lex_state = 73},
  [628] = {.lex_state = 0, .external_lex_state = 72},
  [629] = {.lex_state = 0, .external_lex_state = 72},
  [630] = {.lex_state = 0, .external_lex_state = 74},
  [631] = {.lex_state = 0, .external_lex_state = 74},
  [632] = {.lex_state = 0, .external_lex_state = 74},
  [633] = {.lex_state = 0, .external_lex_state = 75},
  [634] = {.lex_state = 0, .external_lex_state = 74},
  [635] = {.lex_state = 0, .external_lex_state = 74},
  [636] = {.lex_state = 0, .external_lex_state = 74},
  [637] = {.lex_state = 0, .external_lex_state = 74},
  [638] = {.lex_state = 0, .external_lex_state = 74},
  [639] = {.lex_state = 0, .external_lex_state = 74},
  [640] = {.lex_state = 0, .external_lex_state = 74},
  [641] = {.lex_state = 0, .external_lex_state = 74},
  [642] = {.lex_state = 0, .external_lex_state = 71},
  [643] = {.lex_state = 0, .external_lex_state = 74},
  [644] = {.lex_state = 0, .external_lex_state = 74},
  [645] = {.lex_state = 0, .external_lex_state = 74},
  [646] = {.lex_state = 0, .external_lex_state = 74},
  [647] = {.lex_state = 0, .external_lex_state = 74},
  [648] = {.lex_state = 0, .external_lex_state = 74},
  [649] = {.lex_state = 0, .external_lex_state = 74},
  [650] = {.lex_state = 0, .external_lex_state = 74},
  [651] = {.lex_state = 0, .external_lex_state = 74},
  [652] = {.lex_state = 0, .external_lex_state = 74},
  [653] = {.lex_state = 0, .external_lex_state = 74},
  [654] = {.lex_state = 0, .external_lex_state = 74},
  [655] = {.lex_state = 0, .external_lex_state = 74},
  [656] = {.lex_state = 0, .external_lex_state = 74},
  [657] = {.lex_state = 0, .external_lex_state = 74},
  [658] = {.lex_state = 0, .external_lex_state = 74},
  [659] = {.lex_state = 0, .external_lex_state = 74},
  [660] = {.lex_state = 0, .external_lex_state = 74},
  [661] = {.lex_state = 0, .external_lex_state = 74},
  [662] = {.lex_state = 0, .external_lex_state = 74},
  [663] = {.lex_state = 0, .external_lex_state = 74},
  [664] = {.lex_state = 0, .external_lex_state = 74},
  [665] = {.lex_state = 0, .external_lex_state = 74},
  [666] = {.lex_state = 0, .external_lex_state = 74},
  [667] = {.lex_state = 0, .external_lex_state = 74},
  [668] = {.lex_state = 0, .external_lex_state = 74},
  [669] = {.lex_state = 0, .external_lex_state = 74},
  [670] = {.lex_state = 0, .external_lex_state = 71},
  [671] = {.lex_state = 0, .external_lex_state = 74},
  [672] = {.lex_state = 0, .external_lex_state = 74},
  [673] = {.lex_state = 0, .external_lex_state = 74},
  [674] = {.lex_state = 0, .external_lex_state = 74},
  [675] = {.lex_state = 0, .external_lex_state = 74},
  [676] = {.lex_state = 0, .external_lex_state = 74},
  [677] = {.lex_state = 0, .external_lex_state = 74},
  [678] = {.lex_state = 0, .external_lex_state = 74},
  [679] = {.lex_state = 0, .external_lex_state = 74},
  [680] = {.lex_state = 0, .external_lex_state = 74},
  [681] = {.lex_state = 0, .external_lex_state = 74},
  [682] = {.lex_state = 0, .external_lex_state = 74},
  [683] = {.lex_state = 0, .external_lex_state = 71},
  [684] = {.lex_state = 0, .external_lex_state = 74},
  [685] = {.lex_state = 0, .external_lex_state = 74},
  [686] = {.lex_state = 0, .external_lex_state = 74},
  [687] = {.lex_state = 0, .external_lex_state = 74},
  [688] = {.lex_state = 0, .external_lex_state = 74},
  [689] = {.lex_state = 0, .external_lex_state = 74},
  [690] = {.lex_state = 0, .external_lex_state = 74},
  [691] = {.lex_state = 0, .external_lex_state = 74},
  [692] = {.lex_state = 0, .external_lex_state = 74},
  [693] = {.lex_state = 0, .external_lex_state = 74},
  [694] = {.lex_state = 0, .external_lex_state = 74},
  [695] = {.lex_state = 0, .external_lex_state = 74},
  [696] = {.lex_state = 0, .external_lex_state = 74},
  [697] = {.lex_state = 0, .external_lex_state = 74},
  [698] = {.lex_state = 0, .external_lex_state = 74},
  [699] = {.lex_state = 0, .external_lex_state = 74},
  [700] = {.lex_state = 0, .external_lex_state = 74},
  [701] = {.lex_state = 0, .external_lex_state = 74},
  [702] = {.lex_state = 0, .external_lex_state = 74},
  [703] = {.lex_state = 0, .external_lex_state = 74},
  [704] = {.lex_state = 0, .external_lex_state = 74},
  [705] = {.lex_state = 0, .external_lex_state = 74},
  [706] = {.lex_state = 0, .external_lex_state = 74},
  [707] = {.lex_state = 0, .external_lex_state = 71},
  [708] = {.lex_state = 0, .external_lex_state = 74},
  [709] = {.lex_state = 0, .external_lex_state = 74},
  [710] = {.lex_state = 0, .external_lex_state = 74},
  [711] = {.lex_state = 0, .external_lex_state = 74},
  [712] = {.lex_state = 0, .external_lex_state = 74},
  [713] = {.lex_state = 0, .external_lex_state = 74},
  [714] = {.lex_state = 0, .external_lex_state = 74},
  [715] = {.lex_state = 0, .external_lex_state = 74},
  [716] = {.lex_state = 0, .external_lex_state = 76},
  [717] = {.lex_state = 0, .external_lex_state = 76},
  [718] = {.lex_state = 0, .external_lex_state = 77},
  [719] = {.lex_state = 0, .external_lex_state = 77},
  [720] = {.lex_state = 0, .external_lex_state = 73},
  [721] = {.lex_state = 0, .external_lex_state = 73},
  [722] = {.lex_state = 0, .external_lex_state = 78},
  [723] = {.lex_state = 0, .external_lex_state = 76},
  [724] = {.lex_state = 0, .external_lex_state = 72},
  [725] = {.lex_state = 0, .external_lex_state = 76},
  [726] = {.lex_state = 0, .external_lex_state = 76},
  [727] = {.lex_state = 0, .external_lex_state = 76},
  [728] = {.lex_state = 0, .external_lex_state = 76},
  [729] = {.lex_state = 0, .external_lex_state = 76},
  [730] = {.lex_state = 0, .external_lex_state = 76},
  [731] = {.lex_state = 0, .external_lex_state = 79},
  [732] = {.lex_state = 0, .external_lex_state = 76},
  [733] = {.lex_state = 0, .external_lex_state = 79},
  [734] = {.lex_state = 0, .external_lex_state = 75},
  [735] = {.lex_state = 0, .external_lex_state = 75},
  [736] = {.lex_state = 0, .external_lex_state = 76},
  [737] = {.lex_state = 0, .external_lex_state = 78},
  [738] = {.lex_state = 0, .external_lex_state = 78},
  [739] = {.lex_state = 0, .external_lex_state = 76},
  [740] = {.lex_state = 0, .external_lex_state = 76},
  [741] = {.lex_state = 0, .external_lex_state = 76},
  [742] = {.lex_state = 0, .external_lex_state = 76},
  [743] = {.lex_state = 0, .external_lex_state = 76},
  [744] = {.lex_state = 0, .external_lex_state = 76},
  [745] = {.lex_state = 0, .external_lex_state = 76},
  [746] = {.lex_state = 0, .external_lex_state = 72},
  [747] = {.lex_state = 0, .external_lex_state = 76},
  [748] = {.lex_state = 0, .external_lex_state = 76},
  [749] = {.lex_state = 0, .external_lex_state = 72},
  [750] = {.lex_state = 0, .external_lex_state = 72},
  [751] = {.lex_state = 0, .external_lex_state = 72},
  [752] = {.lex_state = 0, .external_lex_state = 73},
  [753] = {.lex_state = 0, .external_lex_state = 72},
  [754] = {.lex_state = 0, .external_lex_state = 73},
  [755] = {.lex_state = 0, .external_lex_state = 73},
  [756] = {.lex_state = 0, .external_lex_state = 76},
  [757] = {.lex_state = 0, .external_lex_state = 73},
  [758] = {.lex_state = 0, .external_lex_state = 73},
  [759] = {.lex_state = 0, .external_lex_state = 72},
  [760] = {.lex_state = 0, .external_lex_state = 72},
  [761] = {.lex_state = 0, .external_lex_state = 72},
  [762] = {.lex_state = 0, .external_lex_state = 73},
  [763] = {.lex_state = 0, .external_lex_state = 73},
  [764] = {.lex_state = 0, .external_lex_state = 73},
  [765] = {.lex_state = 0, .external_lex_state = 73},
  [766] = {.lex_state = 0, .external_lex_state = 73},
  [767] = {.lex_state = 0, .external_lex_state = 73},
  [768] = {.lex_state = 0, .external_lex_state = 76},
  [769] = {.lex_state = 0, .external_lex_state = 72},
  [770] = {.lex_state = 0, .external_lex_state = 73},
  [771] = {.lex_state = 0, .external_lex_state = 73},
  [772] = {.lex_state = 0, .external_lex_state = 73},
  [773] = {.lex_state = 0, .external_lex_state = 73},
  [774] = {.lex_state = 0, .external_lex_state = 73},
  [775] = {.lex_state = 0, .external_lex_state = 77},
  [776] = {.lex_state = 0, .external_lex_state = 73},
  [777] = {.lex_state = 0, .external_lex_state = 80},
  [778] = {.lex_state = 0, .external_lex_state = 80},
  [779] = {.lex_state = 0, .external_lex_state = 73},
  [780] = {.lex_state = 0, .external_lex_state = 77},
  [781] = {.lex_state = 0, .external_lex_state = 73},
  [782] = {.lex_state = 0, .external_lex_state = 76},
  [783] = {.lex_state = 0, .external_lex_state = 73},
  [784] = {.lex_state = 0, .external_lex_state = 73},
  [785] = {.lex_state = 0, .external_lex_state = 73},
  [786] = {.lex_state = 0, .external_lex_state = 73},
  [787] = {.lex_state = 0, .external_lex_state = 76},
  [788] = {.lex_state = 0, .external_lex_state = 76},
  [789] = {.lex_state = 0, .external_lex_state = 73},
  [790] = {.lex_state = 0, .external_lex_state = 73},
  [791] = {.lex_state = 0, .external_lex_state = 72},
  [792] = {.lex_state = 0, .external_lex_state = 73},
  [793] = {.lex_state = 0, .external_lex_state = 80},
  [794] = {.lex_state = 0, .external_lex_state = 73},
  [795] = {.lex_state = 0, .external_lex_state = 80},
  [796] = {.lex_state = 0, .external_lex_state = 73},
  [797] = {.lex_state = 0, .external_lex_state = 73},
  [798] = {.lex_state = 0, .external_lex_state = 72},
  [799] = {.lex_state = 0, .external_lex_state = 73},
  [800] = {.lex_state = 0, .external_lex_state = 76},
  [801] = {.lex_state = 0, .external_lex_state = 76},
  [802] = {.lex_state = 0, .external_lex_state = 73},
  [803] = {.lex_state = 0, .external_lex_state = 73},
  [804] = {.lex_state = 0, .external_lex_state = 76},
  [805] = {.lex_state = 0, .external_lex_state = 73},
  [806] = {.lex_state = 0, .external_lex_state = 73},
  [807] = {.lex_state = 0, .external_lex_state = 73},
  [808] = {.lex_state = 0, .external_lex_state = 76},
  [809] = {.lex_state = 0, .external_lex_state = 73},
  [810] = {.lex_state = 0, .external_lex_state = 73},
  [811] = {.lex_state = 0, .external_lex_state = 76},
  [812] = {.lex_state = 0, .external_lex_state = 76},
  [813] = {.lex_state = 0, .external_lex_state = 73},
  [814] = {.lex_state = 0, .external_lex_state = 76},
  [815] = {.lex_state = 0, .external_lex_state = 77},
  [816] = {.lex_state = 0, .external_lex_state = 73},
  [817] = {.lex_state = 0, .external_lex_state = 73},
  [818] = {.lex_state = 0, .external_lex_state = 80},
  [819] = {.lex_state = 0, .external_lex_state = 76},
  [820] = {.lex_state = 0, .external_lex_state = 72},
  [821] = {.lex_state = 0, .external_lex_state = 79},
  [822] = {.lex_state = 0, .external_lex_state = 79},
  [823] = {.lex_state = 0, .external_lex_state = 79},
  [824] = {.lex_state = 0, .external_lex_state = 79},
  [825] = {.lex_state = 0, .external_lex_state = 79},
  [826] = {.lex_state = 0, .external_lex_state = 79},
  [827] = {.lex_state = 0, .external_lex_state = 75},
  [828] = {.lex_state = 0, .external_lex_state = 74},
  [829] = {.lex_state = 0, .external_lex_state = 79},
  [830] = {.lex_state = 0, .external_lex_state = 79},
  [831] = {.lex_state = 0, .external_lex_state = 79},
  [832] = {.lex_state = 0, .external_lex_state = 79},
  [833] = {.lex_state = 0, .external_lex_state = 79},
  [834] = {.lex_state = 0, .external_lex_state = 79},
  [835] = {.lex_state = 0, .external_lex_state = 79},
  [836] = {.lex_state = 0, .external_lex_state = 79},
  [837] = {.lex_state = 0, .external_lex_state = 79},
  [838] = {.lex_state = 0, .external_lex_state = 79},
  [839] = {.lex_state = 0, .external_lex_state = 79},
  [840] = {.lex_state = 0, .external_lex_state = 79},
  [841] = {.lex_state = 0, .external_lex_state = 75},
  [842] = {.lex_state = 0, .external_lex_state = 79},
  [843] = {.lex_state = 0, .external_lex_state = 75},
  [844] = {.lex_state = 0, .external_lex_state = 79},
  [845] = {.lex_state = 0, .external_lex_state = 81},
  [846] = {.lex_state = 0, .external_lex_state = 75},
  [847] = {.lex_state = 0, .external_lex_state = 79},
  [848] = {.lex_state = 0, .external_lex_state = 79},
  [849] = {.lex_state = 0, .external_lex_state = 79},
  [850] = {.lex_state = 0, .external_lex_state = 79},
  [851] = {.lex_state = 0, .external_lex_state = 79},
  [852] = {.lex_state = 0, .external_lex_state = 79},
  [853] = {.lex_state = 0, .external_lex_state = 79},
  [854] = {.lex_state = 0, .external_lex_state = 79},
  [855] = {.lex_state = 0, .external_lex_state = 75},
  [856] = {.lex_state = 0, .external_lex_state = 75},
  [857] = {.lex_state = 0, .external_lex_state = 75},
  [858] = {.lex_state = 0, .external_lex_state = 74},
  [859] = {.lex_state = 0, .external_lex_state = 79},
  [860] = {.lex_state = 0, .external_lex_state = 79},
  [861] = {.lex_state = 0, .external_lex_state = 79},
  [862] = {.lex_state = 0, .external_lex_state = 79},
  [863] = {.lex_state = 0, .external_lex_state = 79},
  [864] = {.lex_state = 0, .external_lex_state = 79},
  [865] = {.lex_state = 0, .external_lex_state = 79},
  [866] = {.lex_state = 0, .external_lex_state = 79},
  [867] = {.lex_state = 0, .external_lex_state = 81},
  [868] = {.lex_state = 0, .external_lex_state = 75},
  [869] = {.lex_state = 0, .external_lex_state = 75},
  [870] = {.lex_state = 0, .external_lex_state = 79},
  [871] = {.lex_state = 0, .external_lex_state = 79},
  [872] = {.lex_state = 0, .external_lex_state = 79},
  [873] = {.lex_state = 0, .external_lex_state = 79},
  [874] = {.lex_state = 0, .external_lex_state = 75},
  [875] = {.lex_state = 0, .external_lex_state = 79},
  [876] = {.lex_state = 0, .external_lex_state = 79},
  [877] = {.lex_state = 0, .external_lex_state = 75},
  [878] = {.lex_state = 0, .external_lex_state = 75},
  [879] = {.lex_state = 0, .external_lex_state = 75},
  [880] = {.lex_state = 0, .external_lex_state = 77},
  [881] = {.lex_state = 0, .external_lex_state = 80},
  [882] = {.lex_state = 0, .external_lex_state = 80},
  [883] = {.lex_state = 0, .external_lex_state = 80},
  [884] = {.lex_state = 0, .external_lex_state = 82},
  [885] = {.lex_state = 0, .external_lex_state = 77},
  [886] = {.lex_state = 0, .external_lex_state = 80},
  [887] = {.lex_state = 0, .external_lex_state = 83},
  [888] = {.lex_state = 0, .external_lex_state = 82},
  [889] = {.lex_state = 0, .external_lex_state = 77},
  [890] = {.lex_state = 0, .external_lex_state = 83},
  [891] = {.lex_state = 0, .external_lex_state = 84},
  [892] = {.lex_state = 0, .external_lex_state = 85},
  [893] = {.lex_state = 0, .external_lex_state = 80},
  [894] = {.lex_state = 0, .external_lex_state = 70},
  [895] = {.lex_state = 0, .external_lex_state = 77},
  [896] = {.lex_state = 0, .external_lex_state = 77},
  [897] = {.lex_state = 0, .external_lex_state = 84},
  [898] = {.lex_state = 0, .external_lex_state = 86},
  [899] = {.lex_state = 0, .external_lex_state = 77},
  [900] = {.lex_state = 0, .external_lex_state = 80},
  [901] = {.lex_state = 0, .external_lex_state = 86},
  [902] = {.lex_state = 0, .external_lex_state = 80},
  [903] = {.lex_state = 0, .external_lex_state = 80},
  [904] = {.lex_state = 0, .external_lex_state = 80},
  [905] = {.lex_state = 0, .external_lex_state = 77},
  [906] = {.lex_state = 0, .external_lex_state = 83},
  [907] = {.lex_state = 0, .external_lex_state = 80},
  [908] = {.lex_state = 0, .external_lex_state = 80},
  [909] = {.lex_state = 0, .external_lex_state = 80},
  [910] = {.lex_state = 0, .external_lex_state = 80},
  [911] = {.lex_state = 0, .external_lex_state = 80},
  [912] = {.lex_state = 0, .external_lex_state = 80},
  [913] = {.lex_state = 0, .external_lex_state = 80},
  [914] = {.lex_state = 0, .external_lex_state = 77},
  [915] = {.lex_state = 0, .external_lex_state = 77},
  [916] = {.lex_state = 0, .external_lex_state = 80},
  [917] = {.lex_state = 0, .external_lex_state = 80},
  [918] = {.lex_state = 0, .external_lex_state = 80},
  [919] = {.lex_state = 0, .external_lex_state = 80},
  [920] = {.lex_state = 0, .external_lex_state = 80},
  [921] = {.lex_state = 0, .external_lex_state = 80},
  [922] = {.lex_state = 0, .external_lex_state = 77},
  [923] = {.lex_state = 0, .external_lex_state = 80},
  [924] = {.lex_state = 0, .external_lex_state = 77},
  [925] = {.lex_state = 0, .external_lex_state = 77},
  [926] = {.lex_state = 0, .external_lex_state = 80},
  [927] = {.lex_state = 0, .external_lex_state = 80},
  [928] = {.lex_state = 0, .external_lex_state = 80},
  [929] = {.lex_state = 0, .external_lex_state = 80},
  [930] = {.lex_state = 0, .external_lex_state = 84},
  [931] = {.lex_state = 0, .external_lex_state = 80},
  [932] = {.lex_state = 0, .external_lex_state = 80},
  [933] = {.lex_state = 0, .external_lex_state = 80},
  [934] = {.lex_state = 0, .external_lex_state = 83},
  [935] = {.lex_state = 0, .external_lex_state = 80},
  [936] = {.lex_state = 0, .external_lex_state = 77},
  [937] = {.lex_state = 0, .external_lex_state = 80},
  [938] = {.lex_state = 0, .external_lex_state = 80},
  [939] = {.lex_state = 0, .external_lex_state = 77},
  [940] = {.lex_state = 0, .external_lex_state = 80},
  [941] = {.lex_state = 0, .external_lex_state = 80},
  [942] = {.lex_state = 0, .external_lex_state = 81},
  [943] = {.lex_state = 0, .external_lex_state = 81},
  [944] = {.lex_state = 0, .external_lex_state = 80},
  [945] = {.lex_state = 0, .external_lex_state = 70},
  [946] = {.lex_state = 0, .external_lex_state = 80},
  [947] = {.lex_state = 0, .external_lex_state = 85},
  [948] = {.lex_state = 0, .external_lex_state = 83},
  [949] = {.lex_state = 0, .external_lex_state = 84},
  [950] = {.lex_state = 0, .external_lex_state = 80},
  [951] = {.lex_state = 0, .external_lex_state = 80},
  [952] = {.lex_state = 0, .external_lex_state = 83},
  [953] = {.lex_state = 0, .external_lex_state = 84},
  [954] = {.lex_state = 0, .external_lex_state = 82},
  [955] = {.lex_state = 0, .external_lex_state = 77},
  [956] = {.lex_state = 0, .external_lex_state = 82},
  [957] = {.lex_state = 0, .external_lex_state = 77},
  [958] = {.lex_state = 0, .external_lex_state = 82},
  [959] = {.lex_state = 0, .external_lex_state = 77},
  [960] = {.lex_state = 0, .external_lex_state = 82},
  [961] = {.lex_state = 0, .external_lex_state = 77},
  [962] = {.lex_state = 0, .external_lex_state = 82},
  [963] = {.lex_state = 0, .external_lex_state = 80},
  [964] = {.lex_state = 0, .external_lex_state = 77},
  [965] = {.lex_state = 0, .external_lex_state = 77},
  [966] = {.lex_state = 0, .external_lex_state = 77},
  [967] = {.lex_state = 0, .external_lex_state = 87},
  [968] = {.lex_state = 0, .external_lex_state = 77},
  [969] = {.lex_state = 0, .external_lex_state = 77},
  [970] = {.lex_state = 0, .external_lex_state = 77},
  [971] = {.lex_state = 0, .external_lex_state = 87},
  [972] = {.lex_state = 0, .external_lex_state = 77},
  [973] = {.lex_state = 0, .external_lex_state = 77},
  [974] = {.lex_state = 0, .external_lex_state = 88},
  [975] = {.lex_state = 0, .external_lex_state = 77},
  [976] = {.lex_state = 0, .external_lex_state = 89},
  [977] = {.lex_state = 0, .external_lex_state = 82},
  [978] = {.lex_state = 0, .external_lex_state = 82},
  [979] = {.lex_state = 0, .external_lex_state = 80},
  [980] = {.lex_state = 0, .external_lex_state = 83},
  [981] = {.lex_state = 0, .external_lex_state = 82},
  [982] = {.lex_state = 0, .external_lex_state = 83},
  [983] = {.lex_state = 0, .external_lex_state = 84},
  [984] = {.lex_state = 0, .external_lex_state = 84},
  [985] = {.lex_state = 0, .external_lex_state = 83},
  [986] = {.lex_state = 0, .external_lex_state = 77},
  [987] = {.lex_state = 0, .external_lex_state = 77},
  [988] = {.lex_state = 0, .external_lex_state = 83},
  [989] = {.lex_state = 0, .external_lex_state = 84},
  [990] = {.lex_state = 0, .external_lex_state = 84},
  [991] = {.lex_state = 0, .external_lex_state = 82},
  [992] = {.lex_state = 0, .external_lex_state = 77},
  [993] = {.lex_state = 0, .external_lex_state = 77},
  [994] = {.lex_state = 0, .external_lex_state = 77},
  [995] = {.lex_state = 0, .external_lex_state = 77},
  [996] = {.lex_state = 0, .external_lex_state = 84},
  [997] = {.lex_state = 0, .external_lex_state = 80},
  [998] = {.lex_state = 0, .external_lex_state = 83},
  [999] = {.lex_state = 0, .external_lex_state = 82},
  [1000] = {.lex_state = 0, .external_lex_state = 86},
  [1001] = {.lex_state = 0, .external_lex_state = 82},
  [1002] = {.lex_state = 0, .external_lex_state = 90},
  [1003] = {.lex_state = 0, .external_lex_state = 86},
  [1004] = {.lex_state = 0, .external_lex_state = 81},
  [1005] = {.lex_state = 0, .external_lex_state = 81},
  [1006] = {.lex_state = 0, .external_lex_state = 91},
  [1007] = {.lex_state = 0, .external_lex_state = 83},
  [1008] = {.lex_state = 0, .external_lex_state = 80},
  [1009] = {.lex_state = 0, .external_lex_state = 80},
  [1010] = {.lex_state = 0, .external_lex_state = 80},
  [1011] = {.lex_state = 0, .external_lex_state = 77},
  [1012] = {.lex_state = 0, .external_lex_state = 84},
  [1013] = {.lex_state = 0, .external_lex_state = 84},
  [1014] = {.lex_state = 0, .external_lex_state = 80},
  [1015] = {.lex_state = 0, .external_lex_state = 87},
  [1016] = {.lex_state = 0, .external_lex_state = 82},
  [1017] = {.lex_state = 0, .external_lex_state = 80},
  [1018] = {.lex_state = 0, .external_lex_state = 80},
  [1019] = {.lex_state = 0, .external_lex_state = 80},
  [1020] = {.lex_state = 0, .external_lex_state = 80},
  [1021] = {.lex_state = 0, .external_lex_state = 80},
  [1022] = {.lex_state = 0, .external_lex_state = 80},
  [1023] = {.lex_state = 0, .external_lex_state = 80},
  [1024] = {.lex_state = 0, .external_lex_state = 80},
  [1025] = {.lex_state = 0, .external_lex_state = 80},
  [1026] = {.lex_state = 0, .external_lex_state = 80},
  [1027] = {.lex_state = 0, .external_lex_state = 80},
  [1028] = {.lex_state = 0, .external_lex_state = 80},
  [1029] = {.lex_state = 0, .external_lex_state = 80},
  [1030] = {.lex_state = 0, .external_lex_state = 80},
  [1031] = {.lex_state = 0, .external_lex_state = 80},
  [1032] = {.lex_state = 0, .external_lex_state = 80},
  [1033] = {.lex_state = 0, .external_lex_state = 80},
  [1034] = {.lex_state = 0, .external_lex_state = 80},
  [1035] = {.lex_state = 0, .external_lex_state = 80},
  [1036] = {.lex_state = 0, .external_lex_state = 80},
  [1037] = {.lex_state = 0, .external_lex_state = 84},
  [1038] = {.lex_state = 0, .external_lex_state = 77},
  [1039] = {.lex_state = 0, .external_lex_state = 77},
  [1040] = {.lex_state = 0, .external_lex_state = 77},
  [1041] = {.lex_state = 0, .external_lex_state = 77},
  [1042] = {.lex_state = 0, .external_lex_state = 77},
  [1043] = {.lex_state = 0, .external_lex_state = 77},
  [1044] = {.lex_state = 0, .external_lex_state = 77},
  [1045] = {.lex_state = 0, .external_lex_state = 77},
  [1046] = {.lex_state = 0, .external_lex_state = 77},
  [1047] = {.lex_state = 0, .external_lex_state = 84},
  [1048] = {.lex_state = 0, .external_lex_state = 83},
  [1049] = {.lex_state = 0, .external_lex_state = 77},
  [1050] = {.lex_state = 0, .external_lex_state = 77},
  [1051] = {.lex_state = 0, .external_lex_state = 77},
  [1052] = {.lex_state = 0, .external_lex_state = 77},
  [1053] = {.lex_state = 0, .external_lex_state = 77},
  [1054] = {.lex_state = 0, .external_lex_state = 77},
  [1055] = {.lex_state = 0, .external_lex_state = 77},
  [1056] = {.lex_state = 0, .external_lex_state = 77},
  [1057] = {.lex_state = 0, .external_lex_state = 77},
  [1058] = {.lex_state = 0, .external_lex_state = 77},
  [1059] = {.lex_state = 0, .external_lex_state = 77},
  [1060] = {.lex_state = 0, .external_lex_state = 77},
  [1061] = {.lex_state = 0, .external_lex_state = 77},
  [1062] = {.lex_state = 0, .external_lex_state = 77},
  [1063] = {.lex_state = 0, .external_lex_state = 77},
  [1064] = {.lex_state = 0, .external_lex_state = 77},
  [1065] = {.lex_state = 0, .external_lex_state = 77},
  [1066] = {.lex_state = 0, .external_lex_state = 77},
  [1067] = {.lex_state = 0, .external_lex_state = 77},
  [1068] = {.lex_state = 0, .external_lex_state = 77},
  [1069] = {.lex_state = 0, .external_lex_state = 77},
  [1070] = {.lex_state = 0, .external_lex_state = 77},
  [1071] = {.lex_state = 0, .external_lex_state = 77},
  [1072] = {.lex_state = 0, .external_lex_state = 77},
  [1073] = {.lex_state = 0, .external_lex_state = 77},
  [1074] = {.lex_state = 0, .external_lex_state = 77},
  [1075] = {.lex_state = 0, .external_lex_state = 77},
  [1076] = {.lex_state = 0, .external_lex_state = 77},
  [1077] = {.lex_state = 0, .external_lex_state = 77},
  [1078] = {.lex_state = 0, .external_lex_state = 77},
  [1079] = {.lex_state = 0, .external_lex_state = 77},
  [1080] = {.lex_state = 0, .external_lex_state = 77},
  [1081] = {.lex_state = 0, .external_lex_state = 77},
  [1082] = {.lex_state = 0, .external_lex_state = 77},
  [1083] = {.lex_state = 0, .external_lex_state = 77},
  [1084] = {.lex_state = 0, .external_lex_state = 77},
  [1085] = {.lex_state = 0, .external_lex_state = 77},
  [1086] = {.lex_state = 0, .external_lex_state = 77},
  [1087] = {.lex_state = 0, .external_lex_state = 77},
  [1088] = {.lex_state = 0, .external_lex_state = 80},
  [1089] = {.lex_state = 0, .external_lex_state = 80},
  [1090] = {.lex_state = 0, .external_lex_state = 80},
  [1091] = {.lex_state = 0, .external_lex_state = 80},
  [1092] = {.lex_state = 0, .external_lex_state = 80},
  [1093] = {.lex_state = 0, .external_lex_state = 80},
  [1094] = {.lex_state = 0, .external_lex_state = 80},
  [1095] = {.lex_state = 0, .external_lex_state = 80},
  [1096] = {.lex_state = 0, .external_lex_state = 77},
  [1097] = {.lex_state = 0, .external_lex_state = 80},
  [1098] = {.lex_state = 0, .external_lex_state = 77},
  [1099] = {.lex_state = 0, .external_lex_state = 80},
  [1100] = {.lex_state = 0, .external_lex_state = 80},
  [1101] = {.lex_state = 0, .external_lex_state = 80},
  [1102] = {.lex_state = 0, .external_lex_state = 80},
  [1103] = {.lex_state = 0, .external_lex_state = 80},
  [1104] = {.lex_state = 0, .external_lex_state = 80},
  [1105] = {.lex_state = 0, .external_lex_state = 80},
  [1106] = {.lex_state = 0, .external_lex_state = 80},
  [1107] = {.lex_state = 0, .external_lex_state = 80},
  [1108] = {.lex_state = 0, .external_lex_state = 80},
  [1109] = {.lex_state = 0, .external_lex_state = 80},
  [1110] = {.lex_state = 0, .external_lex_state = 80},
  [1111] = {.lex_state = 0, .external_lex_state = 83},
  [1112] = {.lex_state = 0, .external_lex_state = 85},
  [1113] = {.lex_state = 0, .external_lex_state = 83},
  [1114] = {.lex_state = 0, .external_lex_state = 84},
  [1115] = {.lex_state = 0, .external_lex_state = 77},
  [1116] = {.lex_state = 0, .external_lex_state = 69},
  [1117] = {.lex_state = 0, .external_lex_state = 69},
  [1118] = {.lex_state = 0, .external_lex_state = 69},
  [1119] = {.lex_state = 0, .external_lex_state = 69},
  [1120] = {.lex_state = 0, .external_lex_state = 90},
  [1121] = {.lex_state = 0, .external_lex_state = 88},
  [1122] = {.lex_state = 0, .external_lex_state = 69},
  [1123] = {.lex_state = 0, .external_lex_state = 69},
  [1124] = {.lex_state = 0, .external_lex_state = 90},
  [1125] = {.lex_state = 0, .external_lex_state = 69},
  [1126] = {.lex_state = 0, .external_lex_state = 69},
  [1127] = {.lex_state = 0, .external_lex_state = 69},
  [1128] = {.lex_state = 0, .external_lex_state = 69},
  [1129] = {.lex_state = 0, .external_lex_state = 69},
  [1130] = {.lex_state = 0, .external_lex_state = 69},
  [1131] = {.lex_state = 0, .external_lex_state = 69},
  [1132] = {.lex_state = 0, .external_lex_state = 69},
  [1133] = {.lex_state = 0, .external_lex_state = 69},
  [1134] = {.lex_state = 0, .external_lex_state = 69},
  [1135] = {.lex_state = 0, .external_lex_state = 92},
  [1136] = {.lex_state = 0, .external_lex_state = 69},
  [1137] = {.lex_state = 0, .external_lex_state = 69},
  [1138] = {.lex_state = 0, .external_lex_state = 69},
  [1139] = {.lex_state = 0, .external_lex_state = 91},
  [1140] = {.lex_state = 0, .external_lex_state = 91},
  [1141] = {.lex_state = 0, .external_lex_state = 92},
  [1142] = {.lex_state = 0, .external_lex_state = 91},
  [1143] = {.lex_state = 0, .external_lex_state = 91},
  [1144] = {.lex_state = 0, .external_lex_state = 69},
  [1145] = {.lex_state = 0, .external_lex_state = 91},
  [1146] = {.lex_state = 0, .external_lex_state = 91},
  [1147] = {.lex_state = 0, .external_lex_state = 69},
  [1148] = {.lex_state = 0, .external_lex_state = 69},
  [1149] = {.lex_state = 0, .external_lex_state = 92},
  [1150] = {.lex_state = 0, .external_lex_state = 69},
  [1151] = {.lex_state = 0, .external_lex_state = 69},
  [1152] = {.lex_state = 0, .external_lex_state = 69},
  [1153] = {.lex_state = 0, .external_lex_state = 91},
  [1154] = {.lex_state = 0, .external_lex_state = 91},
  [1155] = {.lex_state = 0, .external_lex_state = 91},
  [1156] = {.lex_state = 0, .external_lex_state = 91},
  [1157] = {.lex_state = 0, .external_lex_state = 88},
  [1158] = {.lex_state = 0, .external_lex_state = 69},
  [1159] = {.lex_state = 0, .external_lex_state = 69},
  [1160] = {.lex_state = 0, .external_lex_state = 91},
  [1161] = {.lex_state = 0, .external_lex_state = 69},
  [1162] = {.lex_state = 0, .external_lex_state = 69},
  [1163] = {.lex_state = 0, .external_lex_state = 69},
  [1164] = {.lex_state = 0, .external_lex_state = 91},
  [1165] = {.lex_state = 0, .external_lex_state = 69},
  [1166] = {.lex_state = 0, .external_lex_state = 88},
  [1167] = {.lex_state = 0, .external_lex_state = 91},
  [1168] = {.lex_state = 0, .external_lex_state = 91},
  [1169] = {.lex_state = 0, .external_lex_state = 91},
  [1170] = {.lex_state = 0, .external_lex_state = 90},
  [1171] = {.lex_state = 0, .external_lex_state = 91},
  [1172] = {.lex_state = 0, .external_lex_state = 69},
  [1173] = {.lex_state = 0, .external_lex_state = 90},
  [1174] = {.lex_state = 0, .external_lex_state = 88},
  [1175] = {.lex_state = 0, .external_lex_state = 69},
  [1176] = {.lex_state = 0, .external_lex_state = 69},
  [1177] = {.lex_state = 0, .external_lex_state = 88},
  [1178] = {.lex_state = 0, .external_lex_state = 69},
  [1179] = {.lex_state = 0, .external_lex_state = 69},
  [1180] = {.lex_state = 0, .external_lex_state = 89},
  [1181] = {.lex_state = 0, .external_lex_state = 89},
  [1182] = {.lex_state = 0, .external_lex_state = 89},
  [1183] = {.lex_state = 0, .external_lex_state = 89},
  [1184] = {.lex_state = 0, .external_lex_state = 69},
  [1185] = {.lex_state = 0, .external_lex_state = 69},
  [1186] = {.lex_state = 0, .external_lex_state = 69},
  [1187] = {.lex_state = 0, .external_lex_state = 89},
  [1188] = {.lex_state = 0, .external_lex_state = 69},
  [1189] = {.lex_state = 0, .external_lex_state = 69},
  [1190] = {.lex_state = 0, .external_lex_state = 69},
  [1191] = {.lex_state = 0, .external_lex_state = 92},
  [1192] = {.lex_state = 0, .external_lex_state = 89},
  [1193] = {.lex_state = 0, .external_lex_state = 69},
  [1194] = {.lex_state = 0, .external_lex_state = 92},
  [1195] = {.lex_state = 0, .external_lex_state = 69},
  [1196] = {.lex_state = 0, .external_lex_state = 69},
  [1197] = {.lex_state = 0, .external_lex_state = 69},
  [1198] = {.lex_state = 0, .external_lex_state = 69},
  [1199] = {.lex_state = 0, .external_lex_state = 69},
  [1200] = {.lex_state = 0, .external_lex_state = 69},
  [1201] = {.lex_state = 0, .external_lex_state = 69},
  [1202] = {.lex_state = 0, .external_lex_state = 69},
  [1203] = {.lex_state = 0, .external_lex_state = 69},
  [1204] = {.lex_state = 0, .external_lex_state = 69},
  [1205] = {.lex_state = 0, .external_lex_state = 92},
  [1206] = {.lex_state = 0, .external_lex_state = 92},
  [1207] = {.lex_state = 0, .external_lex_state = 69},
  [1208] = {.lex_state = 0, .external_lex_state = 69},
  [1209] = {.lex_state = 0, .external_lex_state = 69},
  [1210] = {.lex_state = 0, .external_lex_state = 69},
  [1211] = {.lex_state = 0, .external_lex_state = 69},
  [1212] = {.lex_state = 0, .external_lex_state = 89},
  [1213] = {.lex_state = 0, .external_lex_state = 89},
  [1214] = {.lex_state = 0, .external_lex_state = 89},
  [1215] = {.lex_state = 0, .external_lex_state = 89},
  [1216] = {.lex_state = 0, .external_lex_state = 89},
  [1217] = {.lex_state = 0, .external_lex_state = 69},
  [1218] = {.lex_state = 0, .external_lex_state = 69},
  [1219] = {.lex_state = 0, .external_lex_state = 92},
  [1220] = {.lex_state = 0, .external_lex_state = 92},
  [1221] = {.lex_state = 0, .external_lex_state = 89},
  [1222] = {.lex_state = 0, .external_lex_state = 69},
  [1223] = {.lex_state = 0, .external_lex_state = 69},
  [1224] = {.lex_state = 0, .external_lex_state = 69},
  [1225] = {.lex_state = 0, .external_lex_state = 69},
  [1226] = {.lex_state = 0, .external_lex_state = 69},
  [1227] = {.lex_state = 0, .external_lex_state = 92},
  [1228] = {.lex_state = 0, .external_lex_state = 92},
  [1229] = {.lex_state = 0, .external_lex_state = 69},
  [1230] = {.lex_state = 0, .external_lex_state = 69},
  [1231] = {.lex_state = 0, .external_lex_state = 69},
  [1232] = {.lex_state = 0, .external_lex_state = 89},
  [1233] = {.lex_state = 0, .external_lex_state = 89},
  [1234] = {.lex_state = 0, .external_lex_state = 69},
  [1235] = {.lex_state = 0, .external_lex_state = 69},
  [1236] = {.lex_state = 0, .external_lex_state = 69},
  [1237] = {.lex_state = 0, .external_lex_state = 69},
  [1238] = {.lex_state = 0, .external_lex_state = 90},
  [1239] = {.lex_state = 0, .external_lex_state = 69},
  [1240] = {.lex_state = 0, .external_lex_state = 89},
  [1241] = {.lex_state = 0, .external_lex_state = 92},
  [1242] = {.lex_state = 0, .external_lex_state = 92},
  [1243] = {.lex_state = 0, .external_lex_state = 89},
  [1244] = {.lex_state = 0, .external_lex_state = 69},
  [1245] = {.lex_state = 0, .external_lex_state = 69},
  [1246] = {.lex_state = 0, .external_lex_state = 69},
  [1247] = {.lex_state = 0, .external_lex_state = 82},
  [1248] = {.lex_state = 0, .external_lex_state = 86},
  [1249] = {.lex_state = 0, .external_lex_state = 81},
  [1250] = {.lex_state = 0, .external_lex_state = 87},
  [1251] = {.lex_state = 0, .external_lex_state = 86},
  [1252] = {.lex_state = 0, .external_lex_state = 86},
  [1253] = {.lex_state = 0, .external_lex_state = 85},
  [1254] = {.lex_state = 0, .external_lex_state = 86},
  [1255] = {.lex_state = 0, .external_lex_state = 87},
  [1256] = {.lex_state = 0, .external_lex_state = 85},
  [1257] = {.lex_state = 0, .external_lex_state = 93},
  [1258] = {.lex_state = 0, .external_lex_state = 87},
  [1259] = {.lex_state = 0, .external_lex_state = 86},
  [1260] = {.lex_state = 0, .external_lex_state = 94},
  [1261] = {.lex_state = 0, .external_lex_state = 86},
  [1262] = {.lex_state = 0, .external_lex_state = 85},
  [1263] = {.lex_state = 0, .external_lex_state = 85},
  [1264] = {.lex_state = 0, .external_lex_state = 85},
  [1265] = {.lex_state = 0, .external_lex_state = 86},
  [1266] = {.lex_state = 0, .external_lex_state = 86},
  [1267] = {.lex_state = 0, .external_lex_state = 95},
  [1268] = {.lex_state = 0, .external_lex_state = 96},
  [1269] = {.lex_state = 0, .external_lex_state = 81},
  [1270] = {.lex_state = 0, .external_lex_state = 87},
  [1271] = {.lex_state = 0, .external_lex_state = 87},
  [1272] = {.lex_state = 0, .external_lex_state = 82},
  [1273] = {.lex_state = 0, .external_lex_state = 85},
  [1274] = {.lex_state = 0, .external_lex_state = 85},
  [1275] = {.lex_state = 0, .external_lex_state = 85},
  [1276] = {.lex_state = 0, .external_lex_state = 86},
  [1277] = {.lex_state = 0, .external_lex_state = 85},
  [1278] = {.lex_state = 0, .external_lex_state = 95},
  [1279] = {.lex_state = 0, .external_lex_state = 86},
  [1280] = {.lex_state = 0, .external_lex_state = 82},
  [1281] = {.lex_state = 0, .external_lex_state = 96},
  [1282] = {.lex_state = 0, .external_lex_state = 85},
  [1283] = {.lex_state = 0, .external_lex_state = 85},
  [1284] = {.lex_state = 0, .external_lex_state = 87},
  [1285] = {.lex_state = 0, .external_lex_state = 82},
  [1286] = {.lex_state = 0, .external_lex_state = 82},
  [1287] = {.lex_state = 0, .external_lex_state = 85},
  [1288] = {.lex_state = 0, .external_lex_state = 85},
  [1289] = {.lex_state = 0, .external_lex_state = 85},
  [1290] = {.lex_state = 0, .external_lex_state = 85},
  [1291] = {.lex_state = 0, .external_lex_state = 85},
  [1292] = {.lex_state = 0, .external_lex_state = 87},
  [1293] = {.lex_state = 0, .external_lex_state = 82},
  [1294] = {.lex_state = 0, .external_lex_state = 82},
  [1295] = {.lex_state = 0, .external_lex_state = 85},
  [1296] = {.lex_state = 0, .external_lex_state = 85},
  [1297] = {.lex_state = 0, .external_lex_state = 86},
  [1298] = {.lex_state = 0, .external_lex_state = 85},
  [1299] = {.lex_state = 0, .external_lex_state = 82},
  [1300] = {.lex_state = 0, .external_lex_state = 82},
  [1301] = {.lex_state = 0, .external_lex_state = 96},
  [1302] = {.lex_state = 0, .external_lex_state = 82},
  [1303] = {.lex_state = 0, .external_lex_state = 95},
  [1304] = {.lex_state = 0, .external_lex_state = 82},
  [1305] = {.lex_state = 0, .external_lex_state = 81},
  [1306] = {.lex_state = 0, .external_lex_state = 81},
  [1307] = {.lex_state = 0, .external_lex_state = 82},
  [1308] = {.lex_state = 0, .external_lex_state = 82},
  [1309] = {.lex_state = 0, .external_lex_state = 85},
  [1310] = {.lex_state = 0, .external_lex_state = 85},
  [1311] = {.lex_state = 0, .external_lex_state = 85},
  [1312] = {.lex_state = 0, .external_lex_state = 85},
  [1313] = {.lex_state = 0, .external_lex_state = 82},
  [1314] = {.lex_state = 0, .external_lex_state = 86},
  [1315] = {.lex_state = 0, .external_lex_state = 86},
  [1316] = {.lex_state = 0, .external_lex_state = 87},
  [1317] = {.lex_state = 0, .external_lex_state = 86},
  [1318] = {.lex_state = 0, .external_lex_state = 86},
  [1319] = {.lex_state = 0, .external_lex_state = 87},
  [1320] = {.lex_state = 0, .external_lex_state = 86},
  [1321] = {.lex_state = 0, .external_lex_state = 86},
  [1322] = {.lex_state = 0, .external_lex_state = 87},
  [1323] = {.lex_state = 0, .external_lex_state = 86},
  [1324] = {.lex_state = 0, .external_lex_state = 86},
  [1325] = {.lex_state = 0, .external_lex_state = 87},
  [1326] = {.lex_state = 0, .external_lex_state = 81},
  [1327] = {.lex_state = 0, .external_lex_state = 95},
  [1328] = {.lex_state = 0, .external_lex_state = 81},
  [1329] = {.lex_state = 0, .external_lex_state = 81},
  [1330] = {.lex_state = 0, .external_lex_state = 81},
  [1331] = {.lex_state = 0, .external_lex_state = 82},
  [1332] = {.lex_state = 0, .external_lex_state = 86},
  [1333] = {.lex_state = 0, .external_lex_state = 86},
  [1334] = {.lex_state = 0, .external_lex_state = 81},
  [1335] = {.lex_state = 0, .external_lex_state = 81},
  [1336] = {.lex_state = 0, .external_lex_state = 81},
  [1337] = {.lex_state = 0, .external_lex_state = 81},
  [1338] = {.lex_state = 0, .external_lex_state = 87},
  [1339] = {.lex_state = 0, .external_lex_state = 82},
  [1340] = {.lex_state = 0, .external_lex_state = 82},
  [1341] = {.lex_state = 0, .external_lex_state = 86},
  [1342] = {.lex_state = 0, .external_lex_state = 95},
  [1343] = {.lex_state = 0, .external_lex_state = 96},
  [1344] = {.lex_state = 0, .external_lex_state = 82},
  [1345] = {.lex_state = 0, .external_lex_state = 86},
  [1346] = {.lex_state = 0, .external_lex_state = 87},
  [1347] = {.lex_state = 0, .external_lex_state = 87},
  [1348] = {.lex_state = 0, .external_lex_state = 85},
  [1349] = {.lex_state = 0, .external_lex_state = 86},
  [1350] = {.lex_state = 0, .external_lex_state = 85},
  [1351] = {.lex_state = 0, .external_lex_state = 86},
  [1352] = {.lex_state = 0, .external_lex_state = 82},
  [1353] = {.lex_state = 0, .external_lex_state = 96},
  [1354] = {.lex_state = 0, .external_lex_state = 82},
  [1355] = {.lex_state = 0, .external_lex_state = 95},
  [1356] = {.lex_state = 0, .external_lex_state = 87},
  [1357] = {.lex_state = 0, .external_lex_state = 82},
  [1358] = {.lex_state = 0, .external_lex_state = 86},
  [1359] = {.lex_state = 0, .external_lex_state = 87},
  [1360] = {.lex_state = 0, .external_lex_state = 86},
  [1361] = {.lex_state = 0, .external_lex_state = 82},
  [1362] = {.lex_state = 0, .external_lex_state = 82},
  [1363] = {.lex_state = 0, .external_lex_state = 82},
  [1364] = {.lex_state = 0, .external_lex_state = 82},
  [1365] = {.lex_state = 0, .external_lex_state = 87},
  [1366] = {.lex_state = 0, .external_lex_state = 87},
  [1367] = {.lex_state = 0, .external_lex_state = 82},
  [1368] = {.lex_state = 0, .external_lex_state = 82},
  [1369] = {.lex_state = 0, .external_lex_state = 82},
  [1370] = {.lex_state = 0, .external_lex_state = 82},
  [1371] = {.lex_state = 0, .external_lex_state = 86},
  [1372] = {.lex_state = 0, .external_lex_state = 86},
  [1373] = {.lex_state = 0, .external_lex_state = 86},
  [1374] = {.lex_state = 0, .external_lex_state = 82},
  [1375] = {.lex_state = 0, .external_lex_state = 82},
  [1376] = {.lex_state = 0, .external_lex_state = 87},
  [1377] = {.lex_state = 0, .external_lex_state = 82},
  [1378] = {.lex_state = 0, .external_lex_state = 82},
  [1379] = {.lex_state = 0, .external_lex_state = 86},
  [1380] = {.lex_state = 0, .external_lex_state = 82},
  [1381] = {.lex_state = 0, .external_lex_state = 86},
  [1382] = {.lex_state = 0, .external_lex_state = 87},
  [1383] = {.lex_state = 0, .external_lex_state = 86},
  [1384] = {.lex_state = 0, .external_lex_state = 96},
  [1385] = {.lex_state = 0, .external_lex_state = 85},
  [1386] = {.lex_state = 0, .external_lex_state = 82},
  [1387] = {.lex_state = 0, .external_lex_state = 82},
  [1388] = {.lex_state = 0, .external_lex_state = 82},
  [1389] = {.lex_state = 0, .external_lex_state = 87},
  [1390] = {.lex_state = 0, .external_lex_state = 82},
  [1391] = {.lex_state = 0, .external_lex_state = 86},
  [1392] = {.lex_state = 0, .external_lex_state = 87},
  [1393] = {.lex_state = 0, .external_lex_state = 81},
  [1394] = {.lex_state = 0, .external_lex_state = 87},
  [1395] = {.lex_state = 0, .external_lex_state = 82},
  [1396] = {.lex_state = 0, .external_lex_state = 87},
  [1397] = {.lex_state = 0, .external_lex_state = 87},
  [1398] = {.lex_state = 0, .external_lex_state = 82},
  [1399] = {.lex_state = 0, .external_lex_state = 96},
  [1400] = {.lex_state = 0, .external_lex_state = 82},
  [1401] = {.lex_state = 0, .external_lex_state = 95},
  [1402] = {.lex_state = 0, .external_lex_state = 86},
  [1403] = {.lex_state = 0, .external_lex_state = 82},
  [1404] = {.lex_state = 0, .external_lex_state = 82},
  [1405] = {.lex_state = 0, .external_lex_state = 87},
  [1406] = {.lex_state = 0, .external_lex_state = 82},
  [1407] = {.lex_state = 0, .external_lex_state = 82},
  [1408] = {.lex_state = 0, .external_lex_state = 85},
  [1409] = {.lex_state = 0, .external_lex_state = 86},
  [1410] = {.lex_state = 0, .external_lex_state = 86},
  [1411] = {.lex_state = 0, .external_lex_state = 82},
  [1412] = {.lex_state = 0, .external_lex_state = 82},
  [1413] = {.lex_state = 0, .external_lex_state = 86},
  [1414] = {.lex_state = 0, .external_lex_state = 82},
  [1415] = {.lex_state = 0, .external_lex_state = 82},
  [1416] = {.lex_state = 0, .external_lex_state = 82},
  [1417] = {.lex_state = 0, .external_lex_state = 82},
  [1418] = {.lex_state = 0, .external_lex_state = 82},
  [1419] = {.lex_state = 0, .external_lex_state = 82},
  [1420] = {.lex_state = 0, .external_lex_state = 82},
  [1421] = {.lex_state = 0, .external_lex_state = 82},
  [1422] = {.lex_state = 0, .external_lex_state = 82},
  [1423] = {.lex_state = 0, .external_lex_state = 82},
  [1424] = {.lex_state = 0, .external_lex_state = 82},
  [1425] = {.lex_state = 0, .external_lex_state = 82},
  [1426] = {.lex_state = 0, .external_lex_state = 82},
  [1427] = {.lex_state = 0, .external_lex_state = 82},
  [1428] = {.lex_state = 0, .external_lex_state = 82},
  [1429] = {.lex_state = 0, .external_lex_state = 85},
  [1430] = {.lex_state = 0, .external_lex_state = 85},
  [1431] = {.lex_state = 0, .external_lex_state = 82},
  [1432] = {.lex_state = 0, .external_lex_state = 86},
  [1433] = {.lex_state = 0, .external_lex_state = 82},
  [1434] = {.lex_state = 0, .external_lex_state = 87},
  [1435] = {.lex_state = 0, .external_lex_state = 87},
  [1436] = {.lex_state = 0, .external_lex_state = 87},
  [1437] = {.lex_state = 0, .external_lex_state = 85},
  [1438] = {.lex_state = 0, .external_lex_state = 86},
  [1439] = {.lex_state = 0, .external_lex_state = 86},
  [1440] = {.lex_state = 0, .external_lex_state = 86},
  [1441] = {.lex_state = 0, .external_lex_state = 96},
  [1442] = {.lex_state = 0, .external_lex_state = 82},
  [1443] = {.lex_state = 0, .external_lex_state = 95},
  [1444] = {.lex_state = 0, .external_lex_state = 87},
  [1445] = {.lex_state = 0, .external_lex_state = 85},
  [1446] = {.lex_state = 0, .external_lex_state = 93},
  [1447] = {.lex_state = 0, .external_lex_state = 82},
  [1448] = {.lex_state = 0, .external_lex_state = 82},
  [1449] = {.lex_state = 0, .external_lex_state = 94},
  [1450] = {.lex_state = 0, .external_lex_state = 82},
  [1451] = {.lex_state = 0, .external_lex_state = 82},
  [1452] = {.lex_state = 0, .external_lex_state = 82},
  [1453] = {.lex_state = 0, .external_lex_state = 87},
  [1454] = {.lex_state = 0, .external_lex_state = 97},
  [1455] = {.lex_state = 0, .external_lex_state = 94},
  [1456] = {.lex_state = 0, .external_lex_state = 98},
  [1457] = {.lex_state = 0, .external_lex_state = 99},
  [1458] = {.lex_state = 0, .external_lex_state = 99},
  [1459] = {.lex_state = 0, .external_lex_state = 100},
  [1460] = {.lex_state = 0, .external_lex_state = 98},
  [1461] = {.lex_state = 0, .external_lex_state = 98},
  [1462] = {.lex_state = 0, .external_lex_state = 98},
  [1463] = {.lex_state = 0, .external_lex_state = 99},
  [1464] = {.lex_state = 0, .external_lex_state = 99},
  [1465] = {.lex_state = 0, .external_lex_state = 101},
  [1466] = {.lex_state = 0, .external_lex_state = 102},
  [1467] = {.lex_state = 0, .external_lex_state = 103},
  [1468] = {.lex_state = 0, .external_lex_state = 104},
  [1469] = {.lex_state = 0, .external_lex_state = 99},
  [1470] = {.lex_state = 0, .external_lex_state = 99},
  [1471] = {.lex_state = 0, .external_lex_state = 98},
  [1472] = {.lex_state = 0, .external_lex_state = 98},
  [1473] = {.lex_state = 0, .external_lex_state = 98},
  [1474] = {.lex_state = 0, .external_lex_state = 98},
  [1475] = {.lex_state = 0, .external_lex_state = 99},
  [1476] = {.lex_state = 0, .external_lex_state = 99},
  [1477] = {.lex_state = 0, .external_lex_state = 101},
  [1478] = {.lex_state = 0, .external_lex_state = 103},
  [1479] = {.lex_state = 0, .external_lex_state = 104},
  [1480] = {.lex_state = 0, .external_lex_state = 99},
  [1481] = {.lex_state = 0, .external_lex_state = 99},
  [1482] = {.lex_state = 0, .external_lex_state = 98},
  [1483] = {.lex_state = 0, .external_lex_state = 98},
  [1484] = {.lex_state = 0, .external_lex_state = 99},
  [1485] = {.lex_state = 0, .external_lex_state = 98},
  [1486] = {.lex_state = 0, .external_lex_state = 93},
  [1487] = {.lex_state = 0, .external_lex_state = 93},
  [1488] = {.lex_state = 0, .external_lex_state = 103},
  [1489] = {.lex_state = 0, .external_lex_state = 98},
  [1490] = {.lex_state = 0, .external_lex_state = 98},
  [1491] = {.lex_state = 0, .external_lex_state = 103},
  [1492] = {.lex_state = 0, .external_lex_state = 104},
  [1493] = {.lex_state = 0, .external_lex_state = 93},
  [1494] = {.lex_state = 0, .external_lex_state = 99},
  [1495] = {.lex_state = 0, .external_lex_state = 99},
  [1496] = {.lex_state = 0, .external_lex_state = 93},
  [1497] = {.lex_state = 0, .external_lex_state = 99},
  [1498] = {.lex_state = 0, .external_lex_state = 99},
  [1499] = {.lex_state = 0, .external_lex_state = 98},
  [1500] = {.lex_state = 0, .external_lex_state = 98},
  [1501] = {.lex_state = 0, .external_lex_state = 104},
  [1502] = {.lex_state = 0, .external_lex_state = 104},
  [1503] = {.lex_state = 0, .external_lex_state = 103},
  [1504] = {.lex_state = 0, .external_lex_state = 100},
  [1505] = {.lex_state = 0, .external_lex_state = 98},
  [1506] = {.lex_state = 0, .external_lex_state = 98},
  [1507] = {.lex_state = 0, .external_lex_state = 99},
  [1508] = {.lex_state = 0, .external_lex_state = 99},
  [1509] = {.lex_state = 0, .external_lex_state = 105},
  [1510] = {.lex_state = 0, .external_lex_state = 104},
  [1511] = {.lex_state = 0, .external_lex_state = 103},
  [1512] = {.lex_state = 0, .external_lex_state = 99},
  [1513] = {.lex_state = 0, .external_lex_state = 94},
  [1514] = {.lex_state = 0, .external_lex_state = 94},
  [1515] = {.lex_state = 0, .external_lex_state = 101},
  [1516] = {.lex_state = 0, .external_lex_state = 94},
  [1517] = {.lex_state = 0, .external_lex_state = 94},
  [1518] = {.lex_state = 0, .external_lex_state = 94},
  [1519] = {.lex_state = 0, .external_lex_state = 94},
  [1520] = {.lex_state = 0, .external_lex_state = 94},
  [1521] = {.lex_state = 0, .external_lex_state = 94},
  [1522] = {.lex_state = 0, .external_lex_state = 94},
  [1523] = {.lex_state = 0, .external_lex_state = 94},
  [1524] = {.lex_state = 0, .external_lex_state = 100},
  [1525] = {.lex_state = 0, .external_lex_state = 94},
  [1526] = {.lex_state = 0, .external_lex_state = 94},
  [1527] = {.lex_state = 0, .external_lex_state = 100},
  [1528] = {.lex_state = 0, .external_lex_state = 101},
  [1529] = {.lex_state = 0, .external_lex_state = 94},
  [1530] = {.lex_state = 0, .external_lex_state = 94},
  [1531] = {.lex_state = 0, .external_lex_state = 93},
  [1532] = {.lex_state = 0, .external_lex_state = 100},
  [1533] = {.lex_state = 0, .external_lex_state = 98},
  [1534] = {.lex_state = 0, .external_lex_state = 98},
  [1535] = {.lex_state = 0, .external_lex_state = 99},
  [1536] = {.lex_state = 0, .external_lex_state = 98},
  [1537] = {.lex_state = 0, .external_lex_state = 98},
  [1538] = {.lex_state = 0, .external_lex_state = 99},
  [1539] = {.lex_state = 0, .external_lex_state = 98},
  [1540] = {.lex_state = 0, .external_lex_state = 98},
  [1541] = {.lex_state = 0, .external_lex_state = 99},
  [1542] = {.lex_state = 0, .external_lex_state = 98},
  [1543] = {.lex_state = 0, .external_lex_state = 98},
  [1544] = {.lex_state = 0, .external_lex_state = 99},
  [1545] = {.lex_state = 0, .external_lex_state = 98},
  [1546] = {.lex_state = 0, .external_lex_state = 98},
  [1547] = {.lex_state = 0, .external_lex_state = 99},
  [1548] = {.lex_state = 0, .external_lex_state = 98},
  [1549] = {.lex_state = 0, .external_lex_state = 98},
  [1550] = {.lex_state = 0, .external_lex_state = 99},
  [1551] = {.lex_state = 0, .external_lex_state = 98},
  [1552] = {.lex_state = 0, .external_lex_state = 98},
  [1553] = {.lex_state = 0, .external_lex_state = 94},
  [1554] = {.lex_state = 0, .external_lex_state = 98},
  [1555] = {.lex_state = 0, .external_lex_state = 98},
  [1556] = {.lex_state = 0, .external_lex_state = 98},
  [1557] = {.lex_state = 0, .external_lex_state = 98},
  [1558] = {.lex_state = 0, .external_lex_state = 98},
  [1559] = {.lex_state = 0, .external_lex_state = 98},
  [1560] = {.lex_state = 0, .external_lex_state = 98},
  [1561] = {.lex_state = 0, .external_lex_state = 98},
  [1562] = {.lex_state = 0, .external_lex_state = 98},
  [1563] = {.lex_state = 0, .external_lex_state = 98},
  [1564] = {.lex_state = 0, .external_lex_state = 98},
  [1565] = {.lex_state = 0, .external_lex_state = 98},
  [1566] = {.lex_state = 0, .external_lex_state = 98},
  [1567] = {.lex_state = 0, .external_lex_state = 98},
  [1568] = {.lex_state = 0, .external_lex_state = 98},
};

enum {
  ts_external_token__eof = 0,
  ts_external_token__s_dir_yml_bgn = 1,
  ts_external_token__r_dir_yml_ver = 2,
  ts_external_token__s_dir_tag_bgn = 3,
  ts_external_token__r_dir_tag_hdl = 4,
  ts_external_token__r_dir_tag_pfx = 5,
  ts_external_token__s_dir_rsv_bgn = 6,
  ts_external_token__r_dir_rsv_prm = 7,
  ts_external_token__s_drs_end = 8,
  ts_external_token__s_doc_end = 9,
  ts_external_token__r_blk_seq_bgn = 10,
  ts_external_token__br_blk_seq_bgn = 11,
  ts_external_token__b_blk_seq_bgn = 12,
  ts_external_token__r_blk_key_bgn = 13,
  ts_external_token__br_blk_key_bgn = 14,
  ts_external_token__b_blk_key_bgn = 15,
  ts_external_token__r_blk_val_bgn = 16,
  ts_external_token__br_blk_val_bgn = 17,
  ts_external_token__b_blk_val_bgn = 18,
  ts_external_token__r_blk_imp_bgn = 19,
  ts_external_token__r_blk_lit_bgn = 20,
  ts_external_token__br_blk_lit_bgn = 21,
  ts_external_token__r_blk_fld_bgn = 22,
  ts_external_token__br_blk_fld_bgn = 23,
  ts_external_token__br_blk_str_ctn = 24,
  ts_external_token__r_flw_seq_bgn = 25,
  ts_external_token__br_flw_seq_bgn = 26,
  ts_external_token__b_flw_seq_bgn = 27,
  ts_external_token__r_flw_seq_end = 28,
  ts_external_token__br_flw_seq_end = 29,
  ts_external_token__r_flw_map_bgn = 30,
  ts_external_token__br_flw_map_bgn = 31,
  ts_external_token__b_flw_map_bgn = 32,
  ts_external_token__r_flw_map_end = 33,
  ts_external_token__br_flw_map_end = 34,
  ts_external_token__r_flw_sep_bgn = 35,
  ts_external_token__br_flw_sep_bgn = 36,
  ts_external_token__r_flw_key_bgn = 37,
  ts_external_token__br_flw_key_bgn = 38,
  ts_external_token__r_flw_jsv_bgn = 39,
  ts_external_token__br_flw_jsv_bgn = 40,
  ts_external_token__r_flw_njv_bgn = 41,
  ts_external_token__br_flw_njv_bgn = 42,
  ts_external_token__r_dqt_str_bgn = 43,
  ts_external_token__br_dqt_str_bgn = 44,
  ts_external_token__b_dqt_str_bgn = 45,
  ts_external_token__r_dqt_str_ctn = 46,
  ts_external_token__br_dqt_str_ctn = 47,
  ts_external_token__r_dqt_esc_nwl = 48,
  ts_external_token__br_dqt_esc_nwl = 49,
  ts_external_token__r_dqt_esc_seq = 50,
  ts_external_token__br_dqt_esc_seq = 51,
  ts_external_token__r_dqt_str_end = 52,
  ts_external_token__br_dqt_str_end = 53,
  ts_external_token__r_sqt_str_bgn = 54,
  ts_external_token__br_sqt_str_bgn = 55,
  ts_external_token__b_sqt_str_bgn = 56,
  ts_external_token__r_sqt_str_ctn = 57,
  ts_external_token__br_sqt_str_ctn = 58,
  ts_external_token__r_sqt_esc_sqt = 59,
  ts_external_token__br_sqt_esc_sqt = 60,
  ts_external_token__r_sqt_str_end = 61,
  ts_external_token__br_sqt_str_end = 62,
  ts_external_token__r_sgl_pln_nul_blk = 63,
  ts_external_token__br_sgl_pln_nul_blk = 64,
  ts_external_token__b_sgl_pln_nul_blk = 65,
  ts_external_token__r_sgl_pln_nul_flw = 66,
  ts_external_token__br_sgl_pln_nul_flw = 67,
  ts_external_token__r_sgl_pln_bol_blk = 68,
  ts_external_token__br_sgl_pln_bol_blk = 69,
  ts_external_token__b_sgl_pln_bol_blk = 70,
  ts_external_token__r_sgl_pln_bol_flw = 71,
  ts_external_token__br_sgl_pln_bol_flw = 72,
  ts_external_token__r_sgl_pln_int_blk = 73,
  ts_external_token__br_sgl_pln_int_blk = 74,
  ts_external_token__b_sgl_pln_int_blk = 75,
  ts_external_token__r_sgl_pln_int_flw = 76,
  ts_external_token__br_sgl_pln_int_flw = 77,
  ts_external_token__r_sgl_pln_flt_blk = 78,
  ts_external_token__br_sgl_pln_flt_blk = 79,
  ts_external_token__b_sgl_pln_flt_blk = 80,
  ts_external_token__r_sgl_pln_flt_flw = 81,
  ts_external_token__br_sgl_pln_flt_flw = 82,
  ts_external_token__r_sgl_pln_str_blk = 83,
  ts_external_token__br_sgl_pln_str_blk = 84,
  ts_external_token__b_sgl_pln_str_blk = 85,
  ts_external_token__r_sgl_pln_str_flw = 86,
  ts_external_token__br_sgl_pln_str_flw = 87,
  ts_external_token__r_mtl_pln_str_blk = 88,
  ts_external_token__br_mtl_pln_str_blk = 89,
  ts_external_token__r_mtl_pln_str_flw = 90,
  ts_external_token__br_mtl_pln_str_flw = 91,
  ts_external_token__r_tag = 92,
  ts_external_token__br_tag = 93,
  ts_external_token__b_tag = 94,
  ts_external_token__r_acr_bgn = 95,
  ts_external_token__br_acr_bgn = 96,
  ts_external_token__b_acr_bgn = 97,
  ts_external_token__r_acr_ctn = 98,
  ts_external_token__r_als_bgn = 99,
  ts_external_token__br_als_bgn = 100,
  ts_external_token__b_als_bgn = 101,
  ts_external_token__r_als_ctn = 102,
  ts_external_token__bl = 103,
  ts_external_token_comment = 104,
};

static const TSSymbol ts_external_scanner_symbol_map[EXTERNAL_TOKEN_COUNT] = {
  [ts_external_token__eof] = sym__eof,
  [ts_external_token__s_dir_yml_bgn] = sym__s_dir_yml_bgn,
  [ts_external_token__r_dir_yml_ver] = sym__r_dir_yml_ver,
  [ts_external_token__s_dir_tag_bgn] = sym__s_dir_tag_bgn,
  [ts_external_token__r_dir_tag_hdl] = sym__r_dir_tag_hdl,
  [ts_external_token__r_dir_tag_pfx] = sym__r_dir_tag_pfx,
  [ts_external_token__s_dir_rsv_bgn] = sym__s_dir_rsv_bgn,
  [ts_external_token__r_dir_rsv_prm] = sym__r_dir_rsv_prm,
  [ts_external_token__s_drs_end] = sym__s_drs_end,
  [ts_external_token__s_doc_end] = sym__s_doc_end,
  [ts_external_token__r_blk_seq_bgn] = sym__r_blk_seq_bgn,
  [ts_external_token__br_blk_seq_bgn] = sym__br_blk_seq_bgn,
  [ts_external_token__b_blk_seq_bgn] = sym__b_blk_seq_bgn,
  [ts_external_token__r_blk_key_bgn] = sym__r_blk_key_bgn,
  [ts_external_token__br_blk_key_bgn] = sym__br_blk_key_bgn,
  [ts_external_token__b_blk_key_bgn] = sym__b_blk_key_bgn,
  [ts_external_token__r_blk_val_bgn] = sym__r_blk_val_bgn,
  [ts_external_token__br_blk_val_bgn] = sym__br_blk_val_bgn,
  [ts_external_token__b_blk_val_bgn] = sym__b_blk_val_bgn,
  [ts_external_token__r_blk_imp_bgn] = sym__r_blk_imp_bgn,
  [ts_external_token__r_blk_lit_bgn] = sym__r_blk_lit_bgn,
  [ts_external_token__br_blk_lit_bgn] = sym__br_blk_lit_bgn,
  [ts_external_token__r_blk_fld_bgn] = sym__r_blk_fld_bgn,
  [ts_external_token__br_blk_fld_bgn] = sym__br_blk_fld_bgn,
  [ts_external_token__br_blk_str_ctn] = sym__br_blk_str_ctn,
  [ts_external_token__r_flw_seq_bgn] = sym__r_flw_seq_bgn,
  [ts_external_token__br_flw_seq_bgn] = sym__br_flw_seq_bgn,
  [ts_external_token__b_flw_seq_bgn] = sym__b_flw_seq_bgn,
  [ts_external_token__r_flw_seq_end] = sym__r_flw_seq_end,
  [ts_external_token__br_flw_seq_end] = sym__br_flw_seq_end,
  [ts_external_token__r_flw_map_bgn] = sym__r_flw_map_bgn,
  [ts_external_token__br_flw_map_bgn] = sym__br_flw_map_bgn,
  [ts_external_token__b_flw_map_bgn] = sym__b_flw_map_bgn,
  [ts_external_token__r_flw_map_end] = sym__r_flw_map_end,
  [ts_external_token__br_flw_map_end] = sym__br_flw_map_end,
  [ts_external_token__r_flw_sep_bgn] = sym__r_flw_sep_bgn,
  [ts_external_token__br_flw_sep_bgn] = sym__br_flw_sep_bgn,
  [ts_external_token__r_flw_key_bgn] = sym__r_flw_key_bgn,
  [ts_external_token__br_flw_key_bgn] = sym__br_flw_key_bgn,
  [ts_external_token__r_flw_jsv_bgn] = sym__r_flw_jsv_bgn,
  [ts_external_token__br_flw_jsv_bgn] = sym__br_flw_jsv_bgn,
  [ts_external_token__r_flw_njv_bgn] = sym__r_flw_njv_bgn,
  [ts_external_token__br_flw_njv_bgn] = sym__br_flw_njv_bgn,
  [ts_external_token__r_dqt_str_bgn] = sym__r_dqt_str_bgn,
  [ts_external_token__br_dqt_str_bgn] = sym__br_dqt_str_bgn,
  [ts_external_token__b_dqt_str_bgn] = sym__b_dqt_str_bgn,
  [ts_external_token__r_dqt_str_ctn] = sym__r_dqt_str_ctn,
  [ts_external_token__br_dqt_str_ctn] = sym__br_dqt_str_ctn,
  [ts_external_token__r_dqt_esc_nwl] = sym__r_dqt_esc_nwl,
  [ts_external_token__br_dqt_esc_nwl] = sym__br_dqt_esc_nwl,
  [ts_external_token__r_dqt_esc_seq] = sym__r_dqt_esc_seq,
  [ts_external_token__br_dqt_esc_seq] = sym__br_dqt_esc_seq,
  [ts_external_token__r_dqt_str_end] = sym__r_dqt_str_end,
  [ts_external_token__br_dqt_str_end] = sym__br_dqt_str_end,
  [ts_external_token__r_sqt_str_bgn] = sym__r_sqt_str_bgn,
  [ts_external_token__br_sqt_str_bgn] = sym__br_sqt_str_bgn,
  [ts_external_token__b_sqt_str_bgn] = sym__b_sqt_str_bgn,
  [ts_external_token__r_sqt_str_ctn] = sym__r_sqt_str_ctn,
  [ts_external_token__br_sqt_str_ctn] = sym__br_sqt_str_ctn,
  [ts_external_token__r_sqt_esc_sqt] = sym__r_sqt_esc_sqt,
  [ts_external_token__br_sqt_esc_sqt] = sym__br_sqt_esc_sqt,
  [ts_external_token__r_sqt_str_end] = sym__r_sqt_str_end,
  [ts_external_token__br_sqt_str_end] = sym__br_sqt_str_end,
  [ts_external_token__r_sgl_pln_nul_blk] = sym__r_sgl_pln_nul_blk,
  [ts_external_token__br_sgl_pln_nul_blk] = sym__br_sgl_pln_nul_blk,
  [ts_external_token__b_sgl_pln_nul_blk] = sym__b_sgl_pln_nul_blk,
  [ts_external_token__r_sgl_pln_nul_flw] = sym__r_sgl_pln_nul_flw,
  [ts_external_token__br_sgl_pln_nul_flw] = sym__br_sgl_pln_nul_flw,
  [ts_external_token__r_sgl_pln_bol_blk] = sym__r_sgl_pln_bol_blk,
  [ts_external_token__br_sgl_pln_bol_blk] = sym__br_sgl_pln_bol_blk,
  [ts_external_token__b_sgl_pln_bol_blk] = sym__b_sgl_pln_bol_blk,
  [ts_external_token__r_sgl_pln_bol_flw] = sym__r_sgl_pln_bol_flw,
  [ts_external_token__br_sgl_pln_bol_flw] = sym__br_sgl_pln_bol_flw,
  [ts_external_token__r_sgl_pln_int_blk] = sym__r_sgl_pln_int_blk,
  [ts_external_token__br_sgl_pln_int_blk] = sym__br_sgl_pln_int_blk,
  [ts_external_token__b_sgl_pln_int_blk] = sym__b_sgl_pln_int_blk,
  [ts_external_token__r_sgl_pln_int_flw] = sym__r_sgl_pln_int_flw,
  [ts_external_token__br_sgl_pln_int_flw] = sym__br_sgl_pln_int_flw,
  [ts_external_token__r_sgl_pln_flt_blk] = sym__r_sgl_pln_flt_blk,
  [ts_external_token__br_sgl_pln_flt_blk] = sym__br_sgl_pln_flt_blk,
  [ts_external_token__b_sgl_pln_flt_blk] = sym__b_sgl_pln_flt_blk,
  [ts_external_token__r_sgl_pln_flt_flw] = sym__r_sgl_pln_flt_flw,
  [ts_external_token__br_sgl_pln_flt_flw] = sym__br_sgl_pln_flt_flw,
  [ts_external_token__r_sgl_pln_str_blk] = sym__r_sgl_pln_str_blk,
  [ts_external_token__br_sgl_pln_str_blk] = sym__br_sgl_pln_str_blk,
  [ts_external_token__b_sgl_pln_str_blk] = sym__b_sgl_pln_str_blk,
  [ts_external_token__r_sgl_pln_str_flw] = sym__r_sgl_pln_str_flw,
  [ts_external_token__br_sgl_pln_str_flw] = sym__br_sgl_pln_str_flw,
  [ts_external_token__r_mtl_pln_str_blk] = sym__r_mtl_pln_str_blk,
  [ts_external_token__br_mtl_pln_str_blk] = sym__br_mtl_pln_str_blk,
  [ts_external_token__r_mtl_pln_str_flw] = sym__r_mtl_pln_str_flw,
  [ts_external_token__br_mtl_pln_str_flw] = sym__br_mtl_pln_str_flw,
  [ts_external_token__r_tag] = sym__r_tag,
  [ts_external_token__br_tag] = sym__br_tag,
  [ts_external_token__b_tag] = sym__b_tag,
  [ts_external_token__r_acr_bgn] = sym__r_acr_bgn,
  [ts_external_token__br_acr_bgn] = sym__br_acr_bgn,
  [ts_external_token__b_acr_bgn] = sym__b_acr_bgn,
  [ts_external_token__r_acr_ctn] = sym__r_acr_ctn,
  [ts_external_token__r_als_bgn] = sym__r_als_bgn,
  [ts_external_token__br_als_bgn] = sym__br_als_bgn,
  [ts_external_token__b_als_bgn] = sym__b_als_bgn,
  [ts_external_token__r_als_ctn] = sym__r_als_ctn,
  [ts_external_token__bl] = sym__bl,
  [ts_external_token_comment] = sym_comment,
};

static const bool ts_external_scanner_states[106][EXTERNAL_TOKEN_COUNT] = {
  [1] = {
    [ts_external_token__eof] = true,
    [ts_external_token__s_dir_yml_bgn] = true,
    [ts_external_token__r_dir_yml_ver] = true,
    [ts_external_token__s_dir_tag_bgn] = true,
    [ts_external_token__r_dir_tag_hdl] = true,
    [ts_external_token__r_dir_tag_pfx] = true,
    [ts_external_token__s_dir_rsv_bgn] = true,
    [ts_external_token__r_dir_rsv_prm] = true,
    [ts_external_token__s_drs_end] = true,
    [ts_external_token__s_doc_end] = true,
    [ts_external_token__r_blk_seq_bgn] = true,
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__b_blk_seq_bgn] = true,
    [ts_external_token__r_blk_key_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__r_blk_val_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_imp_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__br_blk_str_ctn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__br_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__br_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_flw_key_bgn] = true,
    [ts_external_token__br_flw_key_bgn] = true,
    [ts_external_token__r_flw_jsv_bgn] = true,
    [ts_external_token__br_flw_jsv_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__br_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_dqt_str_ctn] = true,
    [ts_external_token__br_dqt_str_ctn] = true,
    [ts_external_token__r_dqt_esc_nwl] = true,
    [ts_external_token__br_dqt_esc_nwl] = true,
    [ts_external_token__r_dqt_esc_seq] = true,
    [ts_external_token__br_dqt_esc_seq] = true,
    [ts_external_token__r_dqt_str_end] = true,
    [ts_external_token__br_dqt_str_end] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_ctn] = true,
    [ts_external_token__br_sqt_str_ctn] = true,
    [ts_external_token__r_sqt_esc_sqt] = true,
    [ts_external_token__br_sqt_esc_sqt] = true,
    [ts_external_token__r_sqt_str_end] = true,
    [ts_external_token__br_sqt_str_end] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__r_acr_ctn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__r_als_ctn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [2] = {
    [ts_external_token__eof] = true,
    [ts_external_token__s_dir_yml_bgn] = true,
    [ts_external_token__s_dir_tag_bgn] = true,
    [ts_external_token__s_dir_rsv_bgn] = true,
    [ts_external_token__s_drs_end] = true,
    [ts_external_token__s_doc_end] = true,
    [ts_external_token__r_blk_seq_bgn] = true,
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__r_blk_key_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__r_blk_val_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [3] = {
    [ts_external_token__r_blk_seq_bgn] = true,
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__b_blk_seq_bgn] = true,
    [ts_external_token__r_blk_key_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__r_blk_val_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [4] = {
    [ts_external_token__r_blk_seq_bgn] = true,
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__b_blk_seq_bgn] = true,
    [ts_external_token__r_blk_key_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__r_blk_val_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [5] = {
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__b_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [6] = {
    [ts_external_token__eof] = true,
    [ts_external_token__s_drs_end] = true,
    [ts_external_token__s_doc_end] = true,
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [7] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__br_flw_map_end] = true,
    [ts_external_token__r_flw_key_bgn] = true,
    [ts_external_token__br_flw_key_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__br_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [8] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__br_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_key_bgn] = true,
    [ts_external_token__br_flw_key_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__br_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [9] = {
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__b_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [10] = {
    [ts_external_token__eof] = true,
    [ts_external_token__s_dir_yml_bgn] = true,
    [ts_external_token__s_dir_tag_bgn] = true,
    [ts_external_token__s_dir_rsv_bgn] = true,
    [ts_external_token__s_drs_end] = true,
    [ts_external_token__s_doc_end] = true,
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [11] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__br_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [12] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__br_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [13] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_key_bgn] = true,
    [ts_external_token__br_flw_key_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__br_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [14] = {
    [ts_external_token__eof] = true,
    [ts_external_token__s_drs_end] = true,
    [ts_external_token__s_doc_end] = true,
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [15] = {
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__b_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [16] = {
    [ts_external_token__eof] = true,
    [ts_external_token__s_drs_end] = true,
    [ts_external_token__s_doc_end] = true,
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_imp_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [17] = {
    [ts_external_token__eof] = true,
    [ts_external_token__s_drs_end] = true,
    [ts_external_token__s_doc_end] = true,
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_imp_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [18] = {
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__b_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_imp_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [19] = {
    [ts_external_token__eof] = true,
    [ts_external_token__s_drs_end] = true,
    [ts_external_token__s_doc_end] = true,
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_imp_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [20] = {
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__b_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [21] = {
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__b_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_imp_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [22] = {
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__b_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_imp_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [23] = {
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__b_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [24] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__br_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__br_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token_comment] = true,
  },
  [25] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__br_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token_comment] = true,
  },
  [26] = {
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [27] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__r_flw_key_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [28] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_key_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [29] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__br_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token_comment] = true,
  },
  [30] = {
    [ts_external_token__eof] = true,
    [ts_external_token__s_drs_end] = true,
    [ts_external_token__s_doc_end] = true,
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [31] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__br_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token_comment] = true,
  },
  [32] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__br_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token_comment] = true,
  },
  [33] = {
    [ts_external_token__eof] = true,
    [ts_external_token__s_drs_end] = true,
    [ts_external_token__s_doc_end] = true,
    [ts_external_token__br_blk_seq_bgn] = true,
    [ts_external_token__br_blk_key_bgn] = true,
    [ts_external_token__br_blk_val_bgn] = true,
    [ts_external_token__r_blk_lit_bgn] = true,
    [ts_external_token__br_blk_lit_bgn] = true,
    [ts_external_token__r_blk_fld_bgn] = true,
    [ts_external_token__br_blk_fld_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__br_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__br_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__br_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__br_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__br_sgl_pln_str_blk] = true,
    [ts_external_token__r_mtl_pln_str_blk] = true,
    [ts_external_token__br_mtl_pln_str_blk] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token__br_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [34] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_key_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [35] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [36] = {
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_imp_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [37] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__br_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__br_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [38] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__r_als_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [39] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__br_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [40] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__br_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [41] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__br_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [42] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__br_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__br_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token_comment] = true,
  },
  [43] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__br_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token_comment] = true,
  },
  [44] = {
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_imp_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [45] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__br_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token_comment] = true,
  },
  [46] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__br_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token__br_acr_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [47] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__br_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token_comment] = true,
  },
  [48] = {
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_imp_bgn] = true,
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__r_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__r_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__r_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__r_sgl_pln_str_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [49] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__br_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__br_flw_map_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__br_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__br_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__br_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__br_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__br_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__br_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__br_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__br_sgl_pln_str_flw] = true,
    [ts_external_token__r_mtl_pln_str_flw] = true,
    [ts_external_token__br_mtl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token__br_tag] = true,
    [ts_external_token_comment] = true,
  },
  [50] = {
    [ts_external_token__eof] = true,
    [ts_external_token__s_drs_end] = true,
    [ts_external_token__s_doc_end] = true,
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_imp_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [51] = {
    [ts_external_token__b_blk_seq_bgn] = true,
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_imp_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [52] = {
    [ts_external_token__b_blk_key_bgn] = true,
    [ts_external_token__b_blk_val_bgn] = true,
    [ts_external_token__r_blk_imp_bgn] = true,
    [ts_external_token__b_flw_seq_bgn] = true,
    [ts_external_token__b_flw_map_bgn] = true,
    [ts_external_token__b_dqt_str_bgn] = true,
    [ts_external_token__b_sqt_str_bgn] = true,
    [ts_external_token__b_sgl_pln_nul_blk] = true,
    [ts_external_token__b_sgl_pln_bol_blk] = true,
    [ts_external_token__b_sgl_pln_int_blk] = true,
    [ts_external_token__b_sgl_pln_flt_blk] = true,
    [ts_external_token__b_sgl_pln_str_blk] = true,
    [ts_external_token__b_tag] = true,
    [ts_external_token__b_acr_bgn] = true,
    [ts_external_token__b_als_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [53] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token_comment] = true,
  },
  [54] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token_comment] = true,
  },
  [55] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token_comment] = true,
  },
  [56] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token_comment] = true,
  },
  [57] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token_comment] = true,
  },
  [58] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [59] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [60] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token_comment] = true,
  },
  [61] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [62] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token_comment] = true,
  },
  [63] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [64] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token_comment] = true,
  },
  [65] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token_comment] = true,
  },
  [66] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_acr_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [67] = {
    [ts_external_token__r_dqt_str_ctn] = true,
    [ts_external_token__br_dqt_str_ctn] = true,
    [ts_external_token__r_dqt_esc_nwl] = true,
    [ts_external_token__br_dqt_esc_nwl] = true,
    [ts_external_token__r_dqt_esc_seq] = true,
    [ts_external_token__br_dqt_esc_seq] = true,
    [ts_external_token__r_dqt_str_end] = true,
    [ts_external_token__br_dqt_str_end] = true,
    [ts_external_token_comment] = true,
  },
  [68] = {
    [ts_external_token__r_flw_seq_bgn] = true,
    [ts_external_token__r_flw_map_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__r_dqt_str_bgn] = true,
    [ts_external_token__r_sqt_str_bgn] = true,
    [ts_external_token__r_sgl_pln_nul_flw] = true,
    [ts_external_token__r_sgl_pln_bol_flw] = true,
    [ts_external_token__r_sgl_pln_int_flw] = true,
    [ts_external_token__r_sgl_pln_flt_flw] = true,
    [ts_external_token__r_sgl_pln_str_flw] = true,
    [ts_external_token__r_tag] = true,
    [ts_external_token_comment] = true,
  },
  [69] = {
    [ts_external_token__eof] = true,
    [ts_external_token__s_drs_end] = true,
    [ts_external_token__s_doc_end] = true,
    [ts_external_token_comment] = true,
  },
  [70] = {
    [ts_external_token__s_dir_yml_bgn] = true,
    [ts_external_token__s_dir_tag_bgn] = true,
    [ts_external_token__s_dir_rsv_bgn] = true,
    [ts_external_token__s_drs_end] = true,
    [ts_external_token_comment] = true,
  },
  [71] = {
    [ts_external_token__r_sqt_str_ctn] = true,
    [ts_external_token__br_sqt_str_ctn] = true,
    [ts_external_token__r_sqt_esc_sqt] = true,
    [ts_external_token__br_sqt_esc_sqt] = true,
    [ts_external_token__r_sqt_str_end] = true,
    [ts_external_token__br_sqt_str_end] = true,
    [ts_external_token_comment] = true,
  },
  [72] = {
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__br_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__br_flw_njv_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [73] = {
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__br_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_flw_jsv_bgn] = true,
    [ts_external_token__br_flw_jsv_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [74] = {
    [ts_external_token__br_dqt_str_ctn] = true,
    [ts_external_token__br_dqt_esc_nwl] = true,
    [ts_external_token__br_dqt_esc_seq] = true,
    [ts_external_token__r_dqt_str_end] = true,
    [ts_external_token__br_dqt_str_end] = true,
    [ts_external_token_comment] = true,
  },
  [75] = {
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__br_flw_seq_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [76] = {
    [ts_external_token__br_sqt_str_ctn] = true,
    [ts_external_token__br_sqt_esc_sqt] = true,
    [ts_external_token__r_sqt_str_end] = true,
    [ts_external_token__br_sqt_str_end] = true,
    [ts_external_token_comment] = true,
  },
  [77] = {
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__br_flw_seq_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [78] = {
    [ts_external_token__s_dir_yml_bgn] = true,
    [ts_external_token__s_dir_tag_bgn] = true,
    [ts_external_token__s_dir_rsv_bgn] = true,
    [ts_external_token__r_dir_rsv_prm] = true,
    [ts_external_token__s_drs_end] = true,
    [ts_external_token_comment] = true,
  },
  [79] = {
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__br_flw_seq_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token__r_flw_jsv_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [80] = {
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__br_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__br_flw_sep_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [81] = {
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token__br_flw_njv_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [82] = {
    [ts_external_token__b_blk_seq_bgn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [83] = {
    [ts_external_token__r_sqt_str_ctn] = true,
    [ts_external_token__r_sqt_esc_sqt] = true,
    [ts_external_token__r_sqt_str_end] = true,
    [ts_external_token_comment] = true,
  },
  [84] = {
    [ts_external_token__r_dqt_str_ctn] = true,
    [ts_external_token__r_dqt_esc_seq] = true,
    [ts_external_token__r_dqt_str_end] = true,
    [ts_external_token_comment] = true,
  },
  [85] = {
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [86] = {
    [ts_external_token__r_flw_jsv_bgn] = true,
    [ts_external_token__br_flw_jsv_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [87] = {
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [88] = {
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [89] = {
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_flw_jsv_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [90] = {
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [91] = {
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__r_flw_sep_bgn] = true,
    [ts_external_token__r_flw_jsv_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [92] = {
    [ts_external_token__br_blk_str_ctn] = true,
    [ts_external_token__bl] = true,
    [ts_external_token_comment] = true,
  },
  [93] = {
    [ts_external_token__r_flw_njv_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [94] = {
    [ts_external_token__r_flw_jsv_bgn] = true,
    [ts_external_token_comment] = true,
  },
  [95] = {
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token__br_flw_map_end] = true,
    [ts_external_token_comment] = true,
  },
  [96] = {
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token__br_flw_seq_end] = true,
    [ts_external_token_comment] = true,
  },
  [97] = {
    [ts_external_token__r_dir_yml_ver] = true,
    [ts_external_token_comment] = true,
  },
  [98] = {
    [ts_external_token__r_acr_ctn] = true,
    [ts_external_token_comment] = true,
  },
  [99] = {
    [ts_external_token__r_als_ctn] = true,
    [ts_external_token_comment] = true,
  },
  [100] = {
    [ts_external_token__eof] = true,
    [ts_external_token_comment] = true,
  },
  [101] = {
    [ts_external_token_comment] = true,
  },
  [102] = {
    [ts_external_token__r_dir_tag_pfx] = true,
    [ts_external_token_comment] = true,
  },
  [103] = {
    [ts_external_token__r_flw_map_end] = true,
    [ts_external_token_comment] = true,
  },
  [104] = {
    [ts_external_token__r_flw_seq_end] = true,
    [ts_external_token_comment] = true,
  },
  [105] = {
    [ts_external_token__r_dir_tag_hdl] = true,
    [ts_external_token_comment] = true,
  },
};

static const uint16_t ts_parse_table[LARGE_STATE_COUNT][SYMBOL_COUNT] = {
  [0] = {
    [ts_builtin_sym_end] = ACTIONS(1),
    [sym__eof] = ACTIONS(1),
    [sym__s_dir_yml_bgn] = ACTIONS(1),
    [sym__r_dir_yml_ver] = ACTIONS(1),
    [sym__s_dir_tag_bgn] = ACTIONS(1),
    [sym__r_dir_tag_hdl] = ACTIONS(1),
    [sym__r_dir_tag_pfx] = ACTIONS(1),
    [sym__s_dir_rsv_bgn] = ACTIONS(1),
    [sym__r_dir_rsv_prm] = ACTIONS(1),
    [sym__s_drs_end] = ACTIONS(1),
    [sym__s_doc_end] = ACTIONS(1),
    [sym__r_blk_seq_bgn] = ACTIONS(1),
    [sym__br_blk_seq_bgn] = ACTIONS(1),
    [sym__b_blk_seq_bgn] = ACTIONS(1),
    [sym__r_blk_key_bgn] = ACTIONS(1),
    [sym__br_blk_key_bgn] = ACTIONS(1),
    [sym__b_blk_key_bgn] = ACTIONS(1),
    [sym__r_blk_val_bgn] = ACTIONS(1),
    [sym__br_blk_val_bgn] = ACTIONS(1),
    [sym__b_blk_val_bgn] = ACTIONS(1),
    [sym__r_blk_imp_bgn] = ACTIONS(1),
    [sym__r_blk_lit_bgn] = ACTIONS(1),
    [sym__br_blk_lit_bgn] = ACTIONS(1),
    [sym__r_blk_fld_bgn] = ACTIONS(1),
    [sym__br_blk_fld_bgn] = ACTIONS(1),
    [sym__br_blk_str_ctn] = ACTIONS(1),
    [sym__r_flw_seq_bgn] = ACTIONS(1),
    [sym__br_flw_seq_bgn] = ACTIONS(1),
    [sym__b_flw_seq_bgn] = ACTIONS(1),
    [sym__r_flw_seq_end] = ACTIONS(1),
    [sym__br_flw_seq_end] = ACTIONS(1),
    [sym__r_flw_map_bgn] = ACTIONS(1),
    [sym__br_flw_map_bgn] = ACTIONS(1),
    [sym__b_flw_map_bgn] = ACTIONS(1),
    [sym__r_flw_map_end] = ACTIONS(1),
    [sym__br_flw_map_end] = ACTIONS(1),
    [sym__r_flw_sep_bgn] = ACTIONS(1),
    [sym__br_flw_sep_bgn] = ACTIONS(1),
    [sym__r_flw_key_bgn] = ACTIONS(1),
    [sym__br_flw_key_bgn] = ACTIONS(1),
    [sym__r_flw_jsv_bgn] = ACTIONS(1),
    [sym__br_flw_jsv_bgn] = ACTIONS(1),
    [sym__r_flw_njv_bgn] = ACTIONS(1),
    [sym__br_flw_njv_bgn] = ACTIONS(1),
    [sym__r_dqt_str_bgn] = ACTIONS(1),
    [sym__br_dqt_str_bgn] = ACTIONS(1),
    [sym__b_dqt_str_bgn] = ACTIONS(1),
    [sym__r_dqt_str_ctn] = ACTIONS(1),
    [sym__br_dqt_str_ctn] = ACTIONS(1),
    [sym__r_dqt_esc_nwl] = ACTIONS(1),
    [sym__br_dqt_esc_nwl] = ACTIONS(1),
    [sym__r_dqt_esc_seq] = ACTIONS(1),
    [sym__br_dqt_esc_seq] = ACTIONS(1),
    [sym__r_dqt_str_end] = ACTIONS(1),
    [sym__br_dqt_str_end] = ACTIONS(1),
    [sym__r_sqt_str_bgn] = ACTIONS(1),
    [sym__br_sqt_str_bgn] = ACTIONS(1),
    [sym__b_sqt_str_bgn] = ACTIONS(1),
    [sym__r_sqt_str_ctn] = ACTIONS(1),
    [sym__br_sqt_str_ctn] = ACTIONS(1),
    [sym__r_sqt_esc_sqt] = ACTIONS(1),
    [sym__br_sqt_esc_sqt] = ACTIONS(1),
    [sym__r_sqt_str_end] = ACTIONS(1),
    [sym__br_sqt_str_end] = ACTIONS(1),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(1),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(1),
    [sym__b_sgl_pln_nul_blk] = ACTIONS(1),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(1),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(1),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(1),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(1),
    [sym__b_sgl_pln_bol_blk] = ACTIONS(1),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(1),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(1),
    [sym__r_sgl_pln_int_blk] = ACTIONS(1),
    [sym__br_sgl_pln_int_blk] = ACTIONS(1),
    [sym__b_sgl_pln_int_blk] = ACTIONS(1),
    [sym__r_sgl_pln_int_flw] = ACTIONS(1),
    [sym__br_sgl_pln_int_flw] = ACTIONS(1),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(1),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(1),
    [sym__b_sgl_pln_flt_blk] = ACTIONS(1),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(1),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(1),
    [sym__r_sgl_pln_str_blk] = ACTIONS(1),
    [sym__br_sgl_pln_str_blk] = ACTIONS(1),
    [sym__b_sgl_pln_str_blk] = ACTIONS(1),
    [sym__r_sgl_pln_str_flw] = ACTIONS(1),
    [sym__br_sgl_pln_str_flw] = ACTIONS(1),
    [sym__r_mtl_pln_str_blk] = ACTIONS(1),
    [sym__br_mtl_pln_str_blk] = ACTIONS(1),
    [sym__r_mtl_pln_str_flw] = ACTIONS(1),
    [sym__br_mtl_pln_str_flw] = ACTIONS(1),
    [sym__r_tag] = ACTIONS(1),
    [sym__br_tag] = ACTIONS(1),
    [sym__b_tag] = ACTIONS(1),
    [sym__r_acr_bgn] = ACTIONS(1),
    [sym__br_acr_bgn] = ACTIONS(1),
    [sym__b_acr_bgn] = ACTIONS(1),
    [sym__r_acr_ctn] = ACTIONS(1),
    [sym__r_als_bgn] = ACTIONS(1),
    [sym__br_als_bgn] = ACTIONS(1),
    [sym__b_als_bgn] = ACTIONS(1),
    [sym__r_als_ctn] = ACTIONS(1),
    [sym__bl] = ACTIONS(1),
    [sym_comment] = ACTIONS(3),
  },
  [1] = {
    [sym_stream] = STATE(1465),
    [sym__bgn_imp_doc] = STATE(571),
    [sym__drs_doc] = STATE(569),
    [sym__exp_doc] = STATE(567),
    [sym__drs_doc_end] = STATE(39),
    [sym__exp_doc_end] = STATE(39),
    [sym__bgn_imp_doc_end] = STATE(39),
    [sym__doc_end] = STATE(39),
    [sym__exp_doc_tal] = STATE(1158),
    [sym__s_dir] = STATE(568),
    [sym__s_dir_yml] = STATE(568),
    [sym__s_dir_tag] = STATE(568),
    [sym__s_dir_rsv] = STATE(568),
    [sym__r_prp_val] = STATE(1158),
    [sym__br_prp_val] = STATE(1158),
    [sym__r_sgl_prp_val] = STATE(305),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__r_prp] = STATE(80),
    [sym__br_prp] = STATE(81),
    [sym__r_sgl_prp] = STATE(183),
    [sym__br_sgl_prp] = STATE(182),
    [sym__r_blk_seq_r_val] = STATE(1158),
    [sym__r_blk_seq_br_val] = STATE(1158),
    [sym__br_blk_seq_val] = STATE(1158),
    [sym__r_blk_seq] = STATE(1161),
    [sym__br_blk_seq] = STATE(1116),
    [sym__r_blk_seq_itm] = STATE(981),
    [sym__br_blk_seq_itm] = STATE(977),
    [sym__r_blk_map_r_val] = STATE(1158),
    [sym__r_blk_map_br_val] = STATE(1158),
    [sym__br_blk_map_val] = STATE(1158),
    [sym__r_blk_map] = STATE(1176),
    [sym__br_blk_map] = STATE(1178),
    [sym__r_blk_map_itm] = STATE(149),
    [sym__br_blk_map_itm] = STATE(132),
    [sym__r_blk_exp_itm] = STATE(421),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__r_blk_key_itm] = STATE(387),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__r_blk_val_itm] = STATE(435),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__r_blk_imp_itm_key_only] = STATE(450),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__r_blk_str_val] = STATE(1158),
    [sym__br_blk_str_val] = STATE(1158),
    [sym__r_blk_str] = STATE(1204),
    [sym__br_blk_str] = STATE(1207),
    [sym__r_flw_seq_val] = STATE(1158),
    [sym__br_flw_seq_val] = STATE(1158),
    [sym__r_sgl_flw_seq_val] = STATE(305),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(1217),
    [sym__br_flw_seq] = STATE(1218),
    [sym__r_sgl_flw_seq] = STATE(337),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__r_flw_map_val] = STATE(1158),
    [sym__br_flw_map_val] = STATE(1158),
    [sym__r_sgl_flw_map_val] = STATE(305),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(1223),
    [sym__br_flw_map] = STATE(1225),
    [sym__r_sgl_flw_map] = STATE(333),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__r_dqt_str_val] = STATE(1158),
    [sym__br_dqt_str_val] = STATE(1158),
    [sym__r_sgl_dqt_str_val] = STATE(305),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(1226),
    [sym__br_dqt_str] = STATE(1229),
    [sym__r_sgl_dqt_str] = STATE(326),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__r_sqt_str_val] = STATE(1158),
    [sym__br_sqt_str_val] = STATE(1158),
    [sym__r_sgl_sqt_str_val] = STATE(305),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(1234),
    [sym__br_sqt_str] = STATE(1235),
    [sym__r_sgl_sqt_str] = STATE(358),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__r_pln_blk_val] = STATE(1158),
    [sym__br_pln_blk_val] = STATE(1158),
    [sym__r_sgl_pln_blk_val] = STATE(305),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(291),
    [sym__br_sgl_pln_blk] = STATE(292),
    [sym__r_mtl_pln_blk] = STATE(1236),
    [sym__br_mtl_pln_blk] = STATE(1179),
    [sym__r_als_val] = STATE(272),
    [sym__br_als_val] = STATE(271),
    [sym__r_als] = STATE(276),
    [sym__br_als] = STATE(279),
    [sym__r_acr] = STATE(105),
    [sym__br_acr] = STATE(99),
    [aux_sym__drs_doc_repeat1] = STATE(568),
    [sym__eof] = ACTIONS(5),
    [sym__s_dir_yml_bgn] = ACTIONS(7),
    [sym__s_dir_tag_bgn] = ACTIONS(9),
    [sym__s_dir_rsv_bgn] = ACTIONS(11),
    [sym__s_drs_end] = ACTIONS(13),
    [sym__s_doc_end] = ACTIONS(15),
    [sym__r_blk_seq_bgn] = ACTIONS(17),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__r_blk_key_bgn] = ACTIONS(21),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__r_blk_val_bgn] = ACTIONS(25),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__r_blk_lit_bgn] = ACTIONS(29),
    [sym__br_blk_lit_bgn] = ACTIONS(31),
    [sym__r_blk_fld_bgn] = ACTIONS(29),
    [sym__br_blk_fld_bgn] = ACTIONS(31),
    [sym__r_flw_seq_bgn] = ACTIONS(33),
    [sym__br_flw_seq_bgn] = ACTIONS(35),
    [sym__r_flw_map_bgn] = ACTIONS(37),
    [sym__br_flw_map_bgn] = ACTIONS(39),
    [sym__r_dqt_str_bgn] = ACTIONS(41),
    [sym__br_dqt_str_bgn] = ACTIONS(43),
    [sym__r_sqt_str_bgn] = ACTIONS(45),
    [sym__br_sqt_str_bgn] = ACTIONS(47),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(49),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(51),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(49),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(51),
    [sym__r_sgl_pln_int_blk] = ACTIONS(49),
    [sym__br_sgl_pln_int_blk] = ACTIONS(51),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(49),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(51),
    [sym__r_sgl_pln_str_blk] = ACTIONS(49),
    [sym__br_sgl_pln_str_blk] = ACTIONS(51),
    [sym__r_mtl_pln_str_blk] = ACTIONS(53),
    [sym__br_mtl_pln_str_blk] = ACTIONS(55),
    [sym__r_tag] = ACTIONS(57),
    [sym__br_tag] = ACTIONS(59),
    [sym__r_acr_bgn] = ACTIONS(61),
    [sym__br_acr_bgn] = ACTIONS(63),
    [sym__r_als_bgn] = ACTIONS(65),
    [sym__br_als_bgn] = ACTIONS(67),
    [sym_comment] = ACTIONS(3),
  },
  [2] = {
    [sym__r_prp_val] = STATE(411),
    [sym__br_prp_val] = STATE(411),
    [sym__r_sgl_prp_val] = STATE(305),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__r_prp] = STATE(35),
    [sym__br_prp] = STATE(36),
    [sym__r_sgl_prp] = STATE(183),
    [sym__br_sgl_prp] = STATE(182),
    [sym__r_blk_seq_r_val] = STATE(411),
    [sym__r_blk_seq_br_val] = STATE(411),
    [sym__br_blk_seq_val] = STATE(411),
    [sym__r_blk_seq_spc_val] = STATE(411),
    [sym__br_blk_seq_spc_val] = STATE(411),
    [sym__b_blk_seq_spc_val] = STATE(411),
    [sym__r_blk_seq] = STATE(408),
    [sym__br_blk_seq] = STATE(409),
    [sym__b_blk_seq_spc] = STATE(438),
    [sym__r_blk_seq_itm] = STATE(958),
    [sym__br_blk_seq_itm] = STATE(960),
    [sym__b_blk_seq_itm] = STATE(978),
    [sym__blk_seq_itm_tal] = STATE(411),
    [sym__r_blk_map_r_val] = STATE(411),
    [sym__r_blk_map_br_val] = STATE(411),
    [sym__br_blk_map_val] = STATE(411),
    [sym__r_blk_map] = STATE(410),
    [sym__br_blk_map] = STATE(412),
    [sym__r_blk_map_itm] = STATE(157),
    [sym__br_blk_map_itm] = STATE(159),
    [sym__r_blk_exp_itm] = STATE(421),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__r_blk_key_itm] = STATE(387),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__r_blk_val_itm] = STATE(435),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__r_blk_imp_itm_key_only] = STATE(450),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__blk_exp_itm_tal] = STATE(411),
    [sym__r_blk_str_val] = STATE(411),
    [sym__br_blk_str_val] = STATE(411),
    [sym__r_blk_str] = STATE(413),
    [sym__br_blk_str] = STATE(416),
    [sym__r_flw_seq_val] = STATE(411),
    [sym__br_flw_seq_val] = STATE(411),
    [sym__r_sgl_flw_seq_val] = STATE(305),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(417),
    [sym__br_flw_seq] = STATE(418),
    [sym__r_sgl_flw_seq] = STATE(337),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__r_flw_map_val] = STATE(411),
    [sym__br_flw_map_val] = STATE(411),
    [sym__r_sgl_flw_map_val] = STATE(305),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(419),
    [sym__br_flw_map] = STATE(420),
    [sym__r_sgl_flw_map] = STATE(333),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__r_dqt_str_val] = STATE(411),
    [sym__br_dqt_str_val] = STATE(411),
    [sym__r_sgl_dqt_str_val] = STATE(305),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(426),
    [sym__br_dqt_str] = STATE(428),
    [sym__r_sgl_dqt_str] = STATE(326),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__r_sqt_str_val] = STATE(411),
    [sym__br_sqt_str_val] = STATE(411),
    [sym__r_sgl_sqt_str_val] = STATE(305),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(430),
    [sym__br_sqt_str] = STATE(434),
    [sym__r_sgl_sqt_str] = STATE(358),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__r_pln_blk_val] = STATE(411),
    [sym__br_pln_blk_val] = STATE(411),
    [sym__r_sgl_pln_blk_val] = STATE(305),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(386),
    [sym__br_sgl_pln_blk] = STATE(365),
    [sym__r_mtl_pln_blk] = STATE(436),
    [sym__br_mtl_pln_blk] = STATE(437),
    [sym__r_als_val] = STATE(315),
    [sym__br_als_val] = STATE(314),
    [sym__r_als] = STATE(364),
    [sym__br_als] = STATE(385),
    [sym__r_acr] = STATE(114),
    [sym__br_acr] = STATE(113),
    [aux_sym__r_blk_seq_repeat1] = STATE(978),
    [sym__r_blk_seq_bgn] = ACTIONS(17),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__b_blk_seq_bgn] = ACTIONS(69),
    [sym__r_blk_key_bgn] = ACTIONS(21),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__b_blk_key_bgn] = ACTIONS(71),
    [sym__r_blk_val_bgn] = ACTIONS(25),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__b_blk_val_bgn] = ACTIONS(71),
    [sym__r_blk_lit_bgn] = ACTIONS(73),
    [sym__br_blk_lit_bgn] = ACTIONS(75),
    [sym__r_blk_fld_bgn] = ACTIONS(73),
    [sym__br_blk_fld_bgn] = ACTIONS(75),
    [sym__r_flw_seq_bgn] = ACTIONS(77),
    [sym__br_flw_seq_bgn] = ACTIONS(79),
    [sym__b_flw_seq_bgn] = ACTIONS(71),
    [sym__r_flw_map_bgn] = ACTIONS(81),
    [sym__br_flw_map_bgn] = ACTIONS(83),
    [sym__b_flw_map_bgn] = ACTIONS(71),
    [sym__r_dqt_str_bgn] = ACTIONS(85),
    [sym__br_dqt_str_bgn] = ACTIONS(87),
    [sym__b_dqt_str_bgn] = ACTIONS(71),
    [sym__r_sqt_str_bgn] = ACTIONS(89),
    [sym__br_sqt_str_bgn] = ACTIONS(91),
    [sym__b_sqt_str_bgn] = ACTIONS(71),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(93),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(95),
    [sym__b_sgl_pln_nul_blk] = ACTIONS(71),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(93),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(95),
    [sym__b_sgl_pln_bol_blk] = ACTIONS(71),
    [sym__r_sgl_pln_int_blk] = ACTIONS(93),
    [sym__br_sgl_pln_int_blk] = ACTIONS(95),
    [sym__b_sgl_pln_int_blk] = ACTIONS(71),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(93),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(95),
    [sym__b_sgl_pln_flt_blk] = ACTIONS(71),
    [sym__r_sgl_pln_str_blk] = ACTIONS(93),
    [sym__br_sgl_pln_str_blk] = ACTIONS(95),
    [sym__b_sgl_pln_str_blk] = ACTIONS(71),
    [sym__r_mtl_pln_str_blk] = ACTIONS(97),
    [sym__br_mtl_pln_str_blk] = ACTIONS(99),
    [sym__r_tag] = ACTIONS(101),
    [sym__br_tag] = ACTIONS(103),
    [sym__b_tag] = ACTIONS(71),
    [sym__r_acr_bgn] = ACTIONS(105),
    [sym__br_acr_bgn] = ACTIONS(107),
    [sym__b_acr_bgn] = ACTIONS(71),
    [sym__r_als_bgn] = ACTIONS(109),
    [sym__br_als_bgn] = ACTIONS(111),
    [sym__b_als_bgn] = ACTIONS(71),
    [sym__bl] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
  },
  [3] = {
    [sym__r_prp_val] = STATE(415),
    [sym__br_prp_val] = STATE(415),
    [sym__r_sgl_prp_val] = STATE(305),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__r_prp] = STATE(35),
    [sym__br_prp] = STATE(36),
    [sym__r_sgl_prp] = STATE(183),
    [sym__br_sgl_prp] = STATE(182),
    [sym__r_blk_seq_r_val] = STATE(415),
    [sym__r_blk_seq_br_val] = STATE(415),
    [sym__br_blk_seq_val] = STATE(415),
    [sym__r_blk_seq_spc_val] = STATE(415),
    [sym__br_blk_seq_spc_val] = STATE(415),
    [sym__b_blk_seq_spc_val] = STATE(415),
    [sym__r_blk_seq] = STATE(408),
    [sym__br_blk_seq] = STATE(409),
    [sym__b_blk_seq_spc] = STATE(438),
    [sym__r_blk_seq_itm] = STATE(958),
    [sym__br_blk_seq_itm] = STATE(960),
    [sym__b_blk_seq_itm] = STATE(978),
    [sym__blk_seq_itm_tal] = STATE(415),
    [sym__r_blk_map_r_val] = STATE(415),
    [sym__r_blk_map_br_val] = STATE(415),
    [sym__br_blk_map_val] = STATE(415),
    [sym__r_blk_map] = STATE(410),
    [sym__br_blk_map] = STATE(412),
    [sym__r_blk_map_itm] = STATE(157),
    [sym__br_blk_map_itm] = STATE(159),
    [sym__r_blk_exp_itm] = STATE(421),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__r_blk_key_itm] = STATE(387),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__r_blk_val_itm] = STATE(435),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__r_blk_imp_itm_key_only] = STATE(450),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__blk_exp_itm_tal] = STATE(415),
    [sym__r_blk_str_val] = STATE(415),
    [sym__br_blk_str_val] = STATE(415),
    [sym__r_blk_str] = STATE(413),
    [sym__br_blk_str] = STATE(416),
    [sym__r_flw_seq_val] = STATE(415),
    [sym__br_flw_seq_val] = STATE(415),
    [sym__r_sgl_flw_seq_val] = STATE(305),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(417),
    [sym__br_flw_seq] = STATE(418),
    [sym__r_sgl_flw_seq] = STATE(337),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__r_flw_map_val] = STATE(415),
    [sym__br_flw_map_val] = STATE(415),
    [sym__r_sgl_flw_map_val] = STATE(305),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(419),
    [sym__br_flw_map] = STATE(420),
    [sym__r_sgl_flw_map] = STATE(333),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__r_dqt_str_val] = STATE(415),
    [sym__br_dqt_str_val] = STATE(415),
    [sym__r_sgl_dqt_str_val] = STATE(305),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(426),
    [sym__br_dqt_str] = STATE(428),
    [sym__r_sgl_dqt_str] = STATE(326),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__r_sqt_str_val] = STATE(415),
    [sym__br_sqt_str_val] = STATE(415),
    [sym__r_sgl_sqt_str_val] = STATE(305),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(430),
    [sym__br_sqt_str] = STATE(434),
    [sym__r_sgl_sqt_str] = STATE(358),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__r_pln_blk_val] = STATE(415),
    [sym__br_pln_blk_val] = STATE(415),
    [sym__r_sgl_pln_blk_val] = STATE(305),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(386),
    [sym__br_sgl_pln_blk] = STATE(365),
    [sym__r_mtl_pln_blk] = STATE(436),
    [sym__br_mtl_pln_blk] = STATE(437),
    [sym__r_als_val] = STATE(315),
    [sym__br_als_val] = STATE(314),
    [sym__r_als] = STATE(364),
    [sym__br_als] = STATE(385),
    [sym__r_acr] = STATE(114),
    [sym__br_acr] = STATE(113),
    [aux_sym__r_blk_seq_repeat1] = STATE(978),
    [sym__r_blk_seq_bgn] = ACTIONS(17),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__b_blk_seq_bgn] = ACTIONS(69),
    [sym__r_blk_key_bgn] = ACTIONS(21),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__b_blk_key_bgn] = ACTIONS(113),
    [sym__r_blk_val_bgn] = ACTIONS(25),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__b_blk_val_bgn] = ACTIONS(113),
    [sym__r_blk_lit_bgn] = ACTIONS(73),
    [sym__br_blk_lit_bgn] = ACTIONS(75),
    [sym__r_blk_fld_bgn] = ACTIONS(73),
    [sym__br_blk_fld_bgn] = ACTIONS(75),
    [sym__r_flw_seq_bgn] = ACTIONS(77),
    [sym__br_flw_seq_bgn] = ACTIONS(79),
    [sym__b_flw_seq_bgn] = ACTIONS(113),
    [sym__r_flw_map_bgn] = ACTIONS(81),
    [sym__br_flw_map_bgn] = ACTIONS(83),
    [sym__b_flw_map_bgn] = ACTIONS(113),
    [sym__r_dqt_str_bgn] = ACTIONS(85),
    [sym__br_dqt_str_bgn] = ACTIONS(87),
    [sym__b_dqt_str_bgn] = ACTIONS(113),
    [sym__r_sqt_str_bgn] = ACTIONS(89),
    [sym__br_sqt_str_bgn] = ACTIONS(91),
    [sym__b_sqt_str_bgn] = ACTIONS(113),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(93),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(95),
    [sym__b_sgl_pln_nul_blk] = ACTIONS(113),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(93),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(95),
    [sym__b_sgl_pln_bol_blk] = ACTIONS(113),
    [sym__r_sgl_pln_int_blk] = ACTIONS(93),
    [sym__br_sgl_pln_int_blk] = ACTIONS(95),
    [sym__b_sgl_pln_int_blk] = ACTIONS(113),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(93),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(95),
    [sym__b_sgl_pln_flt_blk] = ACTIONS(113),
    [sym__r_sgl_pln_str_blk] = ACTIONS(93),
    [sym__br_sgl_pln_str_blk] = ACTIONS(95),
    [sym__b_sgl_pln_str_blk] = ACTIONS(113),
    [sym__r_mtl_pln_str_blk] = ACTIONS(97),
    [sym__br_mtl_pln_str_blk] = ACTIONS(99),
    [sym__r_tag] = ACTIONS(101),
    [sym__br_tag] = ACTIONS(103),
    [sym__b_tag] = ACTIONS(113),
    [sym__r_acr_bgn] = ACTIONS(105),
    [sym__br_acr_bgn] = ACTIONS(107),
    [sym__b_acr_bgn] = ACTIONS(113),
    [sym__r_als_bgn] = ACTIONS(109),
    [sym__br_als_bgn] = ACTIONS(111),
    [sym__b_als_bgn] = ACTIONS(113),
    [sym__bl] = ACTIONS(113),
    [sym_comment] = ACTIONS(3),
  },
  [4] = {
    [sym__r_prp_val] = STATE(414),
    [sym__br_prp_val] = STATE(414),
    [sym__r_sgl_prp_val] = STATE(305),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__r_prp] = STATE(35),
    [sym__br_prp] = STATE(36),
    [sym__r_sgl_prp] = STATE(183),
    [sym__br_sgl_prp] = STATE(182),
    [sym__r_blk_seq_r_val] = STATE(414),
    [sym__r_blk_seq_br_val] = STATE(414),
    [sym__br_blk_seq_val] = STATE(414),
    [sym__r_blk_seq_spc_val] = STATE(414),
    [sym__br_blk_seq_spc_val] = STATE(414),
    [sym__b_blk_seq_spc_val] = STATE(414),
    [sym__r_blk_seq] = STATE(408),
    [sym__br_blk_seq] = STATE(409),
    [sym__b_blk_seq_spc] = STATE(438),
    [sym__r_blk_seq_itm] = STATE(958),
    [sym__br_blk_seq_itm] = STATE(960),
    [sym__b_blk_seq_itm] = STATE(978),
    [sym__blk_seq_itm_tal] = STATE(414),
    [sym__r_blk_map_r_val] = STATE(414),
    [sym__r_blk_map_br_val] = STATE(414),
    [sym__br_blk_map_val] = STATE(414),
    [sym__r_blk_map] = STATE(410),
    [sym__br_blk_map] = STATE(412),
    [sym__r_blk_map_itm] = STATE(157),
    [sym__br_blk_map_itm] = STATE(159),
    [sym__r_blk_exp_itm] = STATE(421),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__r_blk_key_itm] = STATE(387),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__r_blk_val_itm] = STATE(435),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__r_blk_imp_itm_key_only] = STATE(450),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__blk_exp_itm_tal] = STATE(414),
    [sym__r_blk_str_val] = STATE(414),
    [sym__br_blk_str_val] = STATE(414),
    [sym__r_blk_str] = STATE(413),
    [sym__br_blk_str] = STATE(416),
    [sym__r_flw_seq_val] = STATE(414),
    [sym__br_flw_seq_val] = STATE(414),
    [sym__r_sgl_flw_seq_val] = STATE(305),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(417),
    [sym__br_flw_seq] = STATE(418),
    [sym__r_sgl_flw_seq] = STATE(337),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__r_flw_map_val] = STATE(414),
    [sym__br_flw_map_val] = STATE(414),
    [sym__r_sgl_flw_map_val] = STATE(305),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(419),
    [sym__br_flw_map] = STATE(420),
    [sym__r_sgl_flw_map] = STATE(333),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__r_dqt_str_val] = STATE(414),
    [sym__br_dqt_str_val] = STATE(414),
    [sym__r_sgl_dqt_str_val] = STATE(305),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(426),
    [sym__br_dqt_str] = STATE(428),
    [sym__r_sgl_dqt_str] = STATE(326),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__r_sqt_str_val] = STATE(414),
    [sym__br_sqt_str_val] = STATE(414),
    [sym__r_sgl_sqt_str_val] = STATE(305),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(430),
    [sym__br_sqt_str] = STATE(434),
    [sym__r_sgl_sqt_str] = STATE(358),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__r_pln_blk_val] = STATE(414),
    [sym__br_pln_blk_val] = STATE(414),
    [sym__r_sgl_pln_blk_val] = STATE(305),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(386),
    [sym__br_sgl_pln_blk] = STATE(365),
    [sym__r_mtl_pln_blk] = STATE(436),
    [sym__br_mtl_pln_blk] = STATE(437),
    [sym__r_als_val] = STATE(315),
    [sym__br_als_val] = STATE(314),
    [sym__r_als] = STATE(364),
    [sym__br_als] = STATE(385),
    [sym__r_acr] = STATE(114),
    [sym__br_acr] = STATE(113),
    [aux_sym__r_blk_seq_repeat1] = STATE(978),
    [sym__r_blk_seq_bgn] = ACTIONS(17),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__b_blk_seq_bgn] = ACTIONS(69),
    [sym__r_blk_key_bgn] = ACTIONS(21),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__b_blk_key_bgn] = ACTIONS(115),
    [sym__r_blk_val_bgn] = ACTIONS(25),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__b_blk_val_bgn] = ACTIONS(115),
    [sym__r_blk_lit_bgn] = ACTIONS(73),
    [sym__br_blk_lit_bgn] = ACTIONS(75),
    [sym__r_blk_fld_bgn] = ACTIONS(73),
    [sym__br_blk_fld_bgn] = ACTIONS(75),
    [sym__r_flw_seq_bgn] = ACTIONS(77),
    [sym__br_flw_seq_bgn] = ACTIONS(79),
    [sym__b_flw_seq_bgn] = ACTIONS(115),
    [sym__r_flw_map_bgn] = ACTIONS(81),
    [sym__br_flw_map_bgn] = ACTIONS(83),
    [sym__b_flw_map_bgn] = ACTIONS(115),
    [sym__r_dqt_str_bgn] = ACTIONS(85),
    [sym__br_dqt_str_bgn] = ACTIONS(87),
    [sym__b_dqt_str_bgn] = ACTIONS(115),
    [sym__r_sqt_str_bgn] = ACTIONS(89),
    [sym__br_sqt_str_bgn] = ACTIONS(91),
    [sym__b_sqt_str_bgn] = ACTIONS(115),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(93),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(95),
    [sym__b_sgl_pln_nul_blk] = ACTIONS(115),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(93),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(95),
    [sym__b_sgl_pln_bol_blk] = ACTIONS(115),
    [sym__r_sgl_pln_int_blk] = ACTIONS(93),
    [sym__br_sgl_pln_int_blk] = ACTIONS(95),
    [sym__b_sgl_pln_int_blk] = ACTIONS(115),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(93),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(95),
    [sym__b_sgl_pln_flt_blk] = ACTIONS(115),
    [sym__r_sgl_pln_str_blk] = ACTIONS(93),
    [sym__br_sgl_pln_str_blk] = ACTIONS(95),
    [sym__b_sgl_pln_str_blk] = ACTIONS(115),
    [sym__r_mtl_pln_str_blk] = ACTIONS(97),
    [sym__br_mtl_pln_str_blk] = ACTIONS(99),
    [sym__r_tag] = ACTIONS(101),
    [sym__br_tag] = ACTIONS(103),
    [sym__b_tag] = ACTIONS(115),
    [sym__r_acr_bgn] = ACTIONS(105),
    [sym__br_acr_bgn] = ACTIONS(107),
    [sym__b_acr_bgn] = ACTIONS(115),
    [sym__r_als_bgn] = ACTIONS(109),
    [sym__br_als_bgn] = ACTIONS(111),
    [sym__b_als_bgn] = ACTIONS(115),
    [sym__bl] = ACTIONS(115),
    [sym_comment] = ACTIONS(3),
  },
  [5] = {
    [sym__r_prp_val] = STATE(427),
    [sym__br_prp_val] = STATE(427),
    [sym__r_sgl_prp_val] = STATE(305),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__r_prp] = STATE(35),
    [sym__br_prp] = STATE(36),
    [sym__r_sgl_prp] = STATE(183),
    [sym__br_sgl_prp] = STATE(182),
    [sym__r_blk_seq_r_val] = STATE(427),
    [sym__r_blk_seq_br_val] = STATE(427),
    [sym__br_blk_seq_val] = STATE(427),
    [sym__r_blk_seq_spc_val] = STATE(427),
    [sym__br_blk_seq_spc_val] = STATE(427),
    [sym__b_blk_seq_spc_val] = STATE(427),
    [sym__r_blk_seq] = STATE(408),
    [sym__br_blk_seq] = STATE(409),
    [sym__b_blk_seq_spc] = STATE(438),
    [sym__r_blk_seq_itm] = STATE(958),
    [sym__br_blk_seq_itm] = STATE(960),
    [sym__b_blk_seq_itm] = STATE(978),
    [sym__blk_seq_itm_tal] = STATE(427),
    [sym__r_blk_map_r_val] = STATE(427),
    [sym__r_blk_map_br_val] = STATE(427),
    [sym__br_blk_map_val] = STATE(427),
    [sym__r_blk_map] = STATE(410),
    [sym__br_blk_map] = STATE(412),
    [sym__r_blk_map_itm] = STATE(157),
    [sym__br_blk_map_itm] = STATE(159),
    [sym__r_blk_exp_itm] = STATE(421),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__r_blk_key_itm] = STATE(387),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__r_blk_val_itm] = STATE(435),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__r_blk_imp_itm_key_only] = STATE(450),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__blk_exp_itm_tal] = STATE(427),
    [sym__r_blk_str_val] = STATE(427),
    [sym__br_blk_str_val] = STATE(427),
    [sym__r_blk_str] = STATE(413),
    [sym__br_blk_str] = STATE(416),
    [sym__r_flw_seq_val] = STATE(427),
    [sym__br_flw_seq_val] = STATE(427),
    [sym__r_sgl_flw_seq_val] = STATE(305),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(417),
    [sym__br_flw_seq] = STATE(418),
    [sym__r_sgl_flw_seq] = STATE(337),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__r_flw_map_val] = STATE(427),
    [sym__br_flw_map_val] = STATE(427),
    [sym__r_sgl_flw_map_val] = STATE(305),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(419),
    [sym__br_flw_map] = STATE(420),
    [sym__r_sgl_flw_map] = STATE(333),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__r_dqt_str_val] = STATE(427),
    [sym__br_dqt_str_val] = STATE(427),
    [sym__r_sgl_dqt_str_val] = STATE(305),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(426),
    [sym__br_dqt_str] = STATE(428),
    [sym__r_sgl_dqt_str] = STATE(326),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__r_sqt_str_val] = STATE(427),
    [sym__br_sqt_str_val] = STATE(427),
    [sym__r_sgl_sqt_str_val] = STATE(305),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(430),
    [sym__br_sqt_str] = STATE(434),
    [sym__r_sgl_sqt_str] = STATE(358),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__r_pln_blk_val] = STATE(427),
    [sym__br_pln_blk_val] = STATE(427),
    [sym__r_sgl_pln_blk_val] = STATE(305),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(386),
    [sym__br_sgl_pln_blk] = STATE(365),
    [sym__r_mtl_pln_blk] = STATE(436),
    [sym__br_mtl_pln_blk] = STATE(437),
    [sym__r_als_val] = STATE(315),
    [sym__br_als_val] = STATE(314),
    [sym__r_als] = STATE(364),
    [sym__br_als] = STATE(385),
    [sym__r_acr] = STATE(114),
    [sym__br_acr] = STATE(113),
    [aux_sym__r_blk_seq_repeat1] = STATE(978),
    [sym__r_blk_seq_bgn] = ACTIONS(17),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__b_blk_seq_bgn] = ACTIONS(69),
    [sym__r_blk_key_bgn] = ACTIONS(21),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__b_blk_key_bgn] = ACTIONS(117),
    [sym__r_blk_val_bgn] = ACTIONS(25),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__b_blk_val_bgn] = ACTIONS(117),
    [sym__r_blk_lit_bgn] = ACTIONS(73),
    [sym__br_blk_lit_bgn] = ACTIONS(75),
    [sym__r_blk_fld_bgn] = ACTIONS(73),
    [sym__br_blk_fld_bgn] = ACTIONS(75),
    [sym__r_flw_seq_bgn] = ACTIONS(77),
    [sym__br_flw_seq_bgn] = ACTIONS(79),
    [sym__b_flw_seq_bgn] = ACTIONS(117),
    [sym__r_flw_map_bgn] = ACTIONS(81),
    [sym__br_flw_map_bgn] = ACTIONS(83),
    [sym__b_flw_map_bgn] = ACTIONS(117),
    [sym__r_dqt_str_bgn] = ACTIONS(85),
    [sym__br_dqt_str_bgn] = ACTIONS(87),
    [sym__b_dqt_str_bgn] = ACTIONS(117),
    [sym__r_sqt_str_bgn] = ACTIONS(89),
    [sym__br_sqt_str_bgn] = ACTIONS(91),
    [sym__b_sqt_str_bgn] = ACTIONS(117),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(93),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(95),
    [sym__b_sgl_pln_nul_blk] = ACTIONS(117),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(93),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(95),
    [sym__b_sgl_pln_bol_blk] = ACTIONS(117),
    [sym__r_sgl_pln_int_blk] = ACTIONS(93),
    [sym__br_sgl_pln_int_blk] = ACTIONS(95),
    [sym__b_sgl_pln_int_blk] = ACTIONS(117),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(93),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(95),
    [sym__b_sgl_pln_flt_blk] = ACTIONS(117),
    [sym__r_sgl_pln_str_blk] = ACTIONS(93),
    [sym__br_sgl_pln_str_blk] = ACTIONS(95),
    [sym__b_sgl_pln_str_blk] = ACTIONS(117),
    [sym__r_mtl_pln_str_blk] = ACTIONS(97),
    [sym__br_mtl_pln_str_blk] = ACTIONS(99),
    [sym__r_tag] = ACTIONS(101),
    [sym__br_tag] = ACTIONS(103),
    [sym__b_tag] = ACTIONS(117),
    [sym__r_acr_bgn] = ACTIONS(105),
    [sym__br_acr_bgn] = ACTIONS(107),
    [sym__b_acr_bgn] = ACTIONS(117),
    [sym__r_als_bgn] = ACTIONS(109),
    [sym__br_als_bgn] = ACTIONS(111),
    [sym__b_als_bgn] = ACTIONS(117),
    [sym__bl] = ACTIONS(117),
    [sym_comment] = ACTIONS(3),
  },
  [6] = {
    [sym__r_prp_val] = STATE(423),
    [sym__br_prp_val] = STATE(423),
    [sym__r_sgl_prp_val] = STATE(305),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__r_prp] = STATE(35),
    [sym__br_prp] = STATE(36),
    [sym__r_sgl_prp] = STATE(183),
    [sym__br_sgl_prp] = STATE(182),
    [sym__r_blk_seq_r_val] = STATE(423),
    [sym__r_blk_seq_br_val] = STATE(423),
    [sym__br_blk_seq_val] = STATE(423),
    [sym__r_blk_seq_spc_val] = STATE(423),
    [sym__br_blk_seq_spc_val] = STATE(423),
    [sym__b_blk_seq_spc_val] = STATE(423),
    [sym__r_blk_seq] = STATE(408),
    [sym__br_blk_seq] = STATE(409),
    [sym__b_blk_seq_spc] = STATE(438),
    [sym__r_blk_seq_itm] = STATE(958),
    [sym__br_blk_seq_itm] = STATE(960),
    [sym__b_blk_seq_itm] = STATE(978),
    [sym__blk_seq_itm_tal] = STATE(423),
    [sym__r_blk_map_r_val] = STATE(423),
    [sym__r_blk_map_br_val] = STATE(423),
    [sym__br_blk_map_val] = STATE(423),
    [sym__r_blk_map] = STATE(410),
    [sym__br_blk_map] = STATE(412),
    [sym__r_blk_map_itm] = STATE(157),
    [sym__br_blk_map_itm] = STATE(159),
    [sym__r_blk_exp_itm] = STATE(421),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__r_blk_key_itm] = STATE(387),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__r_blk_val_itm] = STATE(435),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__r_blk_imp_itm_key_only] = STATE(450),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__blk_exp_itm_tal] = STATE(423),
    [sym__r_blk_str_val] = STATE(423),
    [sym__br_blk_str_val] = STATE(423),
    [sym__r_blk_str] = STATE(413),
    [sym__br_blk_str] = STATE(416),
    [sym__r_flw_seq_val] = STATE(423),
    [sym__br_flw_seq_val] = STATE(423),
    [sym__r_sgl_flw_seq_val] = STATE(305),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(417),
    [sym__br_flw_seq] = STATE(418),
    [sym__r_sgl_flw_seq] = STATE(337),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__r_flw_map_val] = STATE(423),
    [sym__br_flw_map_val] = STATE(423),
    [sym__r_sgl_flw_map_val] = STATE(305),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(419),
    [sym__br_flw_map] = STATE(420),
    [sym__r_sgl_flw_map] = STATE(333),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__r_dqt_str_val] = STATE(423),
    [sym__br_dqt_str_val] = STATE(423),
    [sym__r_sgl_dqt_str_val] = STATE(305),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(426),
    [sym__br_dqt_str] = STATE(428),
    [sym__r_sgl_dqt_str] = STATE(326),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__r_sqt_str_val] = STATE(423),
    [sym__br_sqt_str_val] = STATE(423),
    [sym__r_sgl_sqt_str_val] = STATE(305),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(430),
    [sym__br_sqt_str] = STATE(434),
    [sym__r_sgl_sqt_str] = STATE(358),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__r_pln_blk_val] = STATE(423),
    [sym__br_pln_blk_val] = STATE(423),
    [sym__r_sgl_pln_blk_val] = STATE(305),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(386),
    [sym__br_sgl_pln_blk] = STATE(365),
    [sym__r_mtl_pln_blk] = STATE(436),
    [sym__br_mtl_pln_blk] = STATE(437),
    [sym__r_als_val] = STATE(315),
    [sym__br_als_val] = STATE(314),
    [sym__r_als] = STATE(364),
    [sym__br_als] = STATE(385),
    [sym__r_acr] = STATE(114),
    [sym__br_acr] = STATE(113),
    [aux_sym__r_blk_seq_repeat1] = STATE(978),
    [sym__r_blk_seq_bgn] = ACTIONS(17),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__b_blk_seq_bgn] = ACTIONS(69),
    [sym__r_blk_key_bgn] = ACTIONS(21),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__b_blk_key_bgn] = ACTIONS(119),
    [sym__r_blk_val_bgn] = ACTIONS(25),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__b_blk_val_bgn] = ACTIONS(119),
    [sym__r_blk_lit_bgn] = ACTIONS(73),
    [sym__br_blk_lit_bgn] = ACTIONS(75),
    [sym__r_blk_fld_bgn] = ACTIONS(73),
    [sym__br_blk_fld_bgn] = ACTIONS(75),
    [sym__r_flw_seq_bgn] = ACTIONS(77),
    [sym__br_flw_seq_bgn] = ACTIONS(79),
    [sym__b_flw_seq_bgn] = ACTIONS(119),
    [sym__r_flw_map_bgn] = ACTIONS(81),
    [sym__br_flw_map_bgn] = ACTIONS(83),
    [sym__b_flw_map_bgn] = ACTIONS(119),
    [sym__r_dqt_str_bgn] = ACTIONS(85),
    [sym__br_dqt_str_bgn] = ACTIONS(87),
    [sym__b_dqt_str_bgn] = ACTIONS(119),
    [sym__r_sqt_str_bgn] = ACTIONS(89),
    [sym__br_sqt_str_bgn] = ACTIONS(91),
    [sym__b_sqt_str_bgn] = ACTIONS(119),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(93),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(95),
    [sym__b_sgl_pln_nul_blk] = ACTIONS(119),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(93),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(95),
    [sym__b_sgl_pln_bol_blk] = ACTIONS(119),
    [sym__r_sgl_pln_int_blk] = ACTIONS(93),
    [sym__br_sgl_pln_int_blk] = ACTIONS(95),
    [sym__b_sgl_pln_int_blk] = ACTIONS(119),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(93),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(95),
    [sym__b_sgl_pln_flt_blk] = ACTIONS(119),
    [sym__r_sgl_pln_str_blk] = ACTIONS(93),
    [sym__br_sgl_pln_str_blk] = ACTIONS(95),
    [sym__b_sgl_pln_str_blk] = ACTIONS(119),
    [sym__r_mtl_pln_str_blk] = ACTIONS(97),
    [sym__br_mtl_pln_str_blk] = ACTIONS(99),
    [sym__r_tag] = ACTIONS(101),
    [sym__br_tag] = ACTIONS(103),
    [sym__b_tag] = ACTIONS(119),
    [sym__r_acr_bgn] = ACTIONS(105),
    [sym__br_acr_bgn] = ACTIONS(107),
    [sym__b_acr_bgn] = ACTIONS(119),
    [sym__r_als_bgn] = ACTIONS(109),
    [sym__br_als_bgn] = ACTIONS(111),
    [sym__b_als_bgn] = ACTIONS(119),
    [sym__bl] = ACTIONS(119),
    [sym_comment] = ACTIONS(3),
  },
  [7] = {
    [sym__r_prp_val] = STATE(422),
    [sym__br_prp_val] = STATE(422),
    [sym__r_sgl_prp_val] = STATE(305),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__r_prp] = STATE(35),
    [sym__br_prp] = STATE(36),
    [sym__r_sgl_prp] = STATE(183),
    [sym__br_sgl_prp] = STATE(182),
    [sym__r_blk_seq_r_val] = STATE(422),
    [sym__r_blk_seq_br_val] = STATE(422),
    [sym__br_blk_seq_val] = STATE(422),
    [sym__r_blk_seq_spc_val] = STATE(422),
    [sym__br_blk_seq_spc_val] = STATE(422),
    [sym__b_blk_seq_spc_val] = STATE(422),
    [sym__r_blk_seq] = STATE(408),
    [sym__br_blk_seq] = STATE(409),
    [sym__b_blk_seq_spc] = STATE(438),
    [sym__r_blk_seq_itm] = STATE(958),
    [sym__br_blk_seq_itm] = STATE(960),
    [sym__b_blk_seq_itm] = STATE(978),
    [sym__blk_seq_itm_tal] = STATE(422),
    [sym__r_blk_map_r_val] = STATE(422),
    [sym__r_blk_map_br_val] = STATE(422),
    [sym__br_blk_map_val] = STATE(422),
    [sym__r_blk_map] = STATE(410),
    [sym__br_blk_map] = STATE(412),
    [sym__r_blk_map_itm] = STATE(157),
    [sym__br_blk_map_itm] = STATE(159),
    [sym__r_blk_exp_itm] = STATE(421),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__r_blk_key_itm] = STATE(387),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__r_blk_val_itm] = STATE(435),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__r_blk_imp_itm_key_only] = STATE(450),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__blk_exp_itm_tal] = STATE(422),
    [sym__r_blk_str_val] = STATE(422),
    [sym__br_blk_str_val] = STATE(422),
    [sym__r_blk_str] = STATE(413),
    [sym__br_blk_str] = STATE(416),
    [sym__r_flw_seq_val] = STATE(422),
    [sym__br_flw_seq_val] = STATE(422),
    [sym__r_sgl_flw_seq_val] = STATE(305),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(417),
    [sym__br_flw_seq] = STATE(418),
    [sym__r_sgl_flw_seq] = STATE(337),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__r_flw_map_val] = STATE(422),
    [sym__br_flw_map_val] = STATE(422),
    [sym__r_sgl_flw_map_val] = STATE(305),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(419),
    [sym__br_flw_map] = STATE(420),
    [sym__r_sgl_flw_map] = STATE(333),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__r_dqt_str_val] = STATE(422),
    [sym__br_dqt_str_val] = STATE(422),
    [sym__r_sgl_dqt_str_val] = STATE(305),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(426),
    [sym__br_dqt_str] = STATE(428),
    [sym__r_sgl_dqt_str] = STATE(326),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__r_sqt_str_val] = STATE(422),
    [sym__br_sqt_str_val] = STATE(422),
    [sym__r_sgl_sqt_str_val] = STATE(305),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(430),
    [sym__br_sqt_str] = STATE(434),
    [sym__r_sgl_sqt_str] = STATE(358),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__r_pln_blk_val] = STATE(422),
    [sym__br_pln_blk_val] = STATE(422),
    [sym__r_sgl_pln_blk_val] = STATE(305),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(386),
    [sym__br_sgl_pln_blk] = STATE(365),
    [sym__r_mtl_pln_blk] = STATE(436),
    [sym__br_mtl_pln_blk] = STATE(437),
    [sym__r_als_val] = STATE(315),
    [sym__br_als_val] = STATE(314),
    [sym__r_als] = STATE(364),
    [sym__br_als] = STATE(385),
    [sym__r_acr] = STATE(114),
    [sym__br_acr] = STATE(113),
    [aux_sym__r_blk_seq_repeat1] = STATE(978),
    [sym__r_blk_seq_bgn] = ACTIONS(17),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__b_blk_seq_bgn] = ACTIONS(69),
    [sym__r_blk_key_bgn] = ACTIONS(21),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__b_blk_key_bgn] = ACTIONS(121),
    [sym__r_blk_val_bgn] = ACTIONS(25),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__b_blk_val_bgn] = ACTIONS(121),
    [sym__r_blk_lit_bgn] = ACTIONS(73),
    [sym__br_blk_lit_bgn] = ACTIONS(75),
    [sym__r_blk_fld_bgn] = ACTIONS(73),
    [sym__br_blk_fld_bgn] = ACTIONS(75),
    [sym__r_flw_seq_bgn] = ACTIONS(77),
    [sym__br_flw_seq_bgn] = ACTIONS(79),
    [sym__b_flw_seq_bgn] = ACTIONS(121),
    [sym__r_flw_map_bgn] = ACTIONS(81),
    [sym__br_flw_map_bgn] = ACTIONS(83),
    [sym__b_flw_map_bgn] = ACTIONS(121),
    [sym__r_dqt_str_bgn] = ACTIONS(85),
    [sym__br_dqt_str_bgn] = ACTIONS(87),
    [sym__b_dqt_str_bgn] = ACTIONS(121),
    [sym__r_sqt_str_bgn] = ACTIONS(89),
    [sym__br_sqt_str_bgn] = ACTIONS(91),
    [sym__b_sqt_str_bgn] = ACTIONS(121),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(93),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(95),
    [sym__b_sgl_pln_nul_blk] = ACTIONS(121),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(93),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(95),
    [sym__b_sgl_pln_bol_blk] = ACTIONS(121),
    [sym__r_sgl_pln_int_blk] = ACTIONS(93),
    [sym__br_sgl_pln_int_blk] = ACTIONS(95),
    [sym__b_sgl_pln_int_blk] = ACTIONS(121),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(93),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(95),
    [sym__b_sgl_pln_flt_blk] = ACTIONS(121),
    [sym__r_sgl_pln_str_blk] = ACTIONS(93),
    [sym__br_sgl_pln_str_blk] = ACTIONS(95),
    [sym__b_sgl_pln_str_blk] = ACTIONS(121),
    [sym__r_mtl_pln_str_blk] = ACTIONS(97),
    [sym__br_mtl_pln_str_blk] = ACTIONS(99),
    [sym__r_tag] = ACTIONS(101),
    [sym__br_tag] = ACTIONS(103),
    [sym__b_tag] = ACTIONS(121),
    [sym__r_acr_bgn] = ACTIONS(105),
    [sym__br_acr_bgn] = ACTIONS(107),
    [sym__b_acr_bgn] = ACTIONS(121),
    [sym__r_als_bgn] = ACTIONS(109),
    [sym__br_als_bgn] = ACTIONS(111),
    [sym__b_als_bgn] = ACTIONS(121),
    [sym__bl] = ACTIONS(121),
    [sym_comment] = ACTIONS(3),
  },
  [8] = {
    [sym__r_prp_val] = STATE(1395),
    [sym__br_prp_val] = STATE(1395),
    [sym__r_sgl_prp_val] = STATE(305),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__r_prp] = STATE(82),
    [sym__br_prp] = STATE(83),
    [sym__r_sgl_prp] = STATE(183),
    [sym__br_sgl_prp] = STATE(182),
    [sym__r_blk_seq_r_val] = STATE(1395),
    [sym__r_blk_seq_br_val] = STATE(1395),
    [sym__br_blk_seq_val] = STATE(1395),
    [sym__r_blk_seq] = STATE(1299),
    [sym__br_blk_seq] = STATE(1300),
    [sym__r_blk_seq_itm] = STATE(956),
    [sym__br_blk_seq_itm] = STATE(954),
    [sym__blk_seq_itm_tal] = STATE(1395),
    [sym__r_blk_map_r_val] = STATE(1395),
    [sym__r_blk_map_br_val] = STATE(1395),
    [sym__br_blk_map_val] = STATE(1395),
    [sym__r_blk_map] = STATE(1302),
    [sym__br_blk_map] = STATE(1304),
    [sym__r_blk_map_itm] = STATE(151),
    [sym__br_blk_map_itm] = STATE(146),
    [sym__r_blk_exp_itm] = STATE(421),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__r_blk_key_itm] = STATE(387),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__r_blk_val_itm] = STATE(435),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__r_blk_imp_itm_key_only] = STATE(450),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__r_blk_str_val] = STATE(1395),
    [sym__br_blk_str_val] = STATE(1395),
    [sym__r_blk_str] = STATE(1307),
    [sym__br_blk_str] = STATE(1308),
    [sym__r_flw_seq_val] = STATE(1395),
    [sym__br_flw_seq_val] = STATE(1395),
    [sym__r_sgl_flw_seq_val] = STATE(305),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(1313),
    [sym__br_flw_seq] = STATE(1331),
    [sym__r_sgl_flw_seq] = STATE(337),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__r_flw_map_val] = STATE(1395),
    [sym__br_flw_map_val] = STATE(1395),
    [sym__r_sgl_flw_map_val] = STATE(305),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(1339),
    [sym__br_flw_map] = STATE(1340),
    [sym__r_sgl_flw_map] = STATE(333),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__r_dqt_str_val] = STATE(1395),
    [sym__br_dqt_str_val] = STATE(1395),
    [sym__r_sgl_dqt_str_val] = STATE(305),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(1344),
    [sym__br_dqt_str] = STATE(1354),
    [sym__r_sgl_dqt_str] = STATE(326),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__r_sqt_str_val] = STATE(1395),
    [sym__br_sqt_str_val] = STATE(1395),
    [sym__r_sgl_sqt_str_val] = STATE(305),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(1387),
    [sym__br_sqt_str] = STATE(1388),
    [sym__r_sgl_sqt_str] = STATE(358),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__r_pln_blk_val] = STATE(1395),
    [sym__br_pln_blk_val] = STATE(1395),
    [sym__r_sgl_pln_blk_val] = STATE(305),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(318),
    [sym__br_sgl_pln_blk] = STATE(317),
    [sym__r_mtl_pln_blk] = STATE(1390),
    [sym__br_mtl_pln_blk] = STATE(1400),
    [sym__r_als_val] = STATE(293),
    [sym__br_als_val] = STATE(294),
    [sym__r_als] = STATE(303),
    [sym__br_als] = STATE(316),
    [sym__r_acr] = STATE(111),
    [sym__br_acr] = STATE(112),
    [sym__r_blk_seq_bgn] = ACTIONS(17),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__b_blk_seq_bgn] = ACTIONS(123),
    [sym__r_blk_key_bgn] = ACTIONS(21),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__r_blk_val_bgn] = ACTIONS(25),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__r_blk_lit_bgn] = ACTIONS(125),
    [sym__br_blk_lit_bgn] = ACTIONS(127),
    [sym__r_blk_fld_bgn] = ACTIONS(125),
    [sym__br_blk_fld_bgn] = ACTIONS(127),
    [sym__r_flw_seq_bgn] = ACTIONS(129),
    [sym__br_flw_seq_bgn] = ACTIONS(131),
    [sym__r_flw_map_bgn] = ACTIONS(133),
    [sym__br_flw_map_bgn] = ACTIONS(135),
    [sym__r_dqt_str_bgn] = ACTIONS(137),
    [sym__br_dqt_str_bgn] = ACTIONS(139),
    [sym__r_sqt_str_bgn] = ACTIONS(141),
    [sym__br_sqt_str_bgn] = ACTIONS(143),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(145),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(147),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(145),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(147),
    [sym__r_sgl_pln_int_blk] = ACTIONS(145),
    [sym__br_sgl_pln_int_blk] = ACTIONS(147),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(145),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(147),
    [sym__r_sgl_pln_str_blk] = ACTIONS(145),
    [sym__br_sgl_pln_str_blk] = ACTIONS(147),
    [sym__r_mtl_pln_str_blk] = ACTIONS(149),
    [sym__br_mtl_pln_str_blk] = ACTIONS(151),
    [sym__r_tag] = ACTIONS(153),
    [sym__br_tag] = ACTIONS(155),
    [sym__r_acr_bgn] = ACTIONS(105),
    [sym__br_acr_bgn] = ACTIONS(107),
    [sym__r_als_bgn] = ACTIONS(157),
    [sym__br_als_bgn] = ACTIONS(159),
    [sym__bl] = ACTIONS(123),
    [sym_comment] = ACTIONS(3),
  },
  [9] = {
    [sym__r_prp_val] = STATE(1293),
    [sym__br_prp_val] = STATE(1293),
    [sym__r_sgl_prp_val] = STATE(305),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__r_prp] = STATE(82),
    [sym__br_prp] = STATE(83),
    [sym__r_sgl_prp] = STATE(183),
    [sym__br_sgl_prp] = STATE(182),
    [sym__r_blk_seq_r_val] = STATE(1293),
    [sym__r_blk_seq_br_val] = STATE(1293),
    [sym__br_blk_seq_val] = STATE(1293),
    [sym__r_blk_seq] = STATE(1299),
    [sym__br_blk_seq] = STATE(1300),
    [sym__r_blk_seq_itm] = STATE(956),
    [sym__br_blk_seq_itm] = STATE(954),
    [sym__blk_seq_itm_tal] = STATE(1293),
    [sym__r_blk_map_r_val] = STATE(1293),
    [sym__r_blk_map_br_val] = STATE(1293),
    [sym__br_blk_map_val] = STATE(1293),
    [sym__r_blk_map] = STATE(1302),
    [sym__br_blk_map] = STATE(1304),
    [sym__r_blk_map_itm] = STATE(151),
    [sym__br_blk_map_itm] = STATE(146),
    [sym__r_blk_exp_itm] = STATE(421),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__r_blk_key_itm] = STATE(387),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__r_blk_val_itm] = STATE(435),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__r_blk_imp_itm_key_only] = STATE(450),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__r_blk_str_val] = STATE(1293),
    [sym__br_blk_str_val] = STATE(1293),
    [sym__r_blk_str] = STATE(1307),
    [sym__br_blk_str] = STATE(1308),
    [sym__r_flw_seq_val] = STATE(1293),
    [sym__br_flw_seq_val] = STATE(1293),
    [sym__r_sgl_flw_seq_val] = STATE(305),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(1313),
    [sym__br_flw_seq] = STATE(1331),
    [sym__r_sgl_flw_seq] = STATE(337),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__r_flw_map_val] = STATE(1293),
    [sym__br_flw_map_val] = STATE(1293),
    [sym__r_sgl_flw_map_val] = STATE(305),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(1339),
    [sym__br_flw_map] = STATE(1340),
    [sym__r_sgl_flw_map] = STATE(333),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__r_dqt_str_val] = STATE(1293),
    [sym__br_dqt_str_val] = STATE(1293),
    [sym__r_sgl_dqt_str_val] = STATE(305),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(1344),
    [sym__br_dqt_str] = STATE(1354),
    [sym__r_sgl_dqt_str] = STATE(326),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__r_sqt_str_val] = STATE(1293),
    [sym__br_sqt_str_val] = STATE(1293),
    [sym__r_sgl_sqt_str_val] = STATE(305),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(1387),
    [sym__br_sqt_str] = STATE(1388),
    [sym__r_sgl_sqt_str] = STATE(358),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__r_pln_blk_val] = STATE(1293),
    [sym__br_pln_blk_val] = STATE(1293),
    [sym__r_sgl_pln_blk_val] = STATE(305),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(318),
    [sym__br_sgl_pln_blk] = STATE(317),
    [sym__r_mtl_pln_blk] = STATE(1390),
    [sym__br_mtl_pln_blk] = STATE(1400),
    [sym__r_als_val] = STATE(293),
    [sym__br_als_val] = STATE(294),
    [sym__r_als] = STATE(303),
    [sym__br_als] = STATE(316),
    [sym__r_acr] = STATE(111),
    [sym__br_acr] = STATE(112),
    [sym__r_blk_seq_bgn] = ACTIONS(17),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__b_blk_seq_bgn] = ACTIONS(161),
    [sym__r_blk_key_bgn] = ACTIONS(21),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__r_blk_val_bgn] = ACTIONS(25),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__r_blk_lit_bgn] = ACTIONS(125),
    [sym__br_blk_lit_bgn] = ACTIONS(127),
    [sym__r_blk_fld_bgn] = ACTIONS(125),
    [sym__br_blk_fld_bgn] = ACTIONS(127),
    [sym__r_flw_seq_bgn] = ACTIONS(129),
    [sym__br_flw_seq_bgn] = ACTIONS(131),
    [sym__r_flw_map_bgn] = ACTIONS(133),
    [sym__br_flw_map_bgn] = ACTIONS(135),
    [sym__r_dqt_str_bgn] = ACTIONS(137),
    [sym__br_dqt_str_bgn] = ACTIONS(139),
    [sym__r_sqt_str_bgn] = ACTIONS(141),
    [sym__br_sqt_str_bgn] = ACTIONS(143),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(145),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(147),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(145),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(147),
    [sym__r_sgl_pln_int_blk] = ACTIONS(145),
    [sym__br_sgl_pln_int_blk] = ACTIONS(147),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(145),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(147),
    [sym__r_sgl_pln_str_blk] = ACTIONS(145),
    [sym__br_sgl_pln_str_blk] = ACTIONS(147),
    [sym__r_mtl_pln_str_blk] = ACTIONS(149),
    [sym__br_mtl_pln_str_blk] = ACTIONS(151),
    [sym__r_tag] = ACTIONS(153),
    [sym__br_tag] = ACTIONS(155),
    [sym__r_acr_bgn] = ACTIONS(105),
    [sym__br_acr_bgn] = ACTIONS(107),
    [sym__r_als_bgn] = ACTIONS(157),
    [sym__br_als_bgn] = ACTIONS(159),
    [sym__bl] = ACTIONS(161),
    [sym_comment] = ACTIONS(3),
  },
  [10] = {
    [sym__r_prp_val] = STATE(1294),
    [sym__br_prp_val] = STATE(1294),
    [sym__r_sgl_prp_val] = STATE(305),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__r_prp] = STATE(82),
    [sym__br_prp] = STATE(83),
    [sym__r_sgl_prp] = STATE(183),
    [sym__br_sgl_prp] = STATE(182),
    [sym__r_blk_seq_r_val] = STATE(1294),
    [sym__r_blk_seq_br_val] = STATE(1294),
    [sym__br_blk_seq_val] = STATE(1294),
    [sym__r_blk_seq] = STATE(1299),
    [sym__br_blk_seq] = STATE(1300),
    [sym__r_blk_seq_itm] = STATE(956),
    [sym__br_blk_seq_itm] = STATE(954),
    [sym__blk_seq_itm_tal] = STATE(1294),
    [sym__r_blk_map_r_val] = STATE(1294),
    [sym__r_blk_map_br_val] = STATE(1294),
    [sym__br_blk_map_val] = STATE(1294),
    [sym__r_blk_map] = STATE(1302),
    [sym__br_blk_map] = STATE(1304),
    [sym__r_blk_map_itm] = STATE(151),
    [sym__br_blk_map_itm] = STATE(146),
    [sym__r_blk_exp_itm] = STATE(421),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__r_blk_key_itm] = STATE(387),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__r_blk_val_itm] = STATE(435),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__r_blk_imp_itm_key_only] = STATE(450),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__r_blk_str_val] = STATE(1294),
    [sym__br_blk_str_val] = STATE(1294),
    [sym__r_blk_str] = STATE(1307),
    [sym__br_blk_str] = STATE(1308),
    [sym__r_flw_seq_val] = STATE(1294),
    [sym__br_flw_seq_val] = STATE(1294),
    [sym__r_sgl_flw_seq_val] = STATE(305),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(1313),
    [sym__br_flw_seq] = STATE(1331),
    [sym__r_sgl_flw_seq] = STATE(337),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__r_flw_map_val] = STATE(1294),
    [sym__br_flw_map_val] = STATE(1294),
    [sym__r_sgl_flw_map_val] = STATE(305),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(1339),
    [sym__br_flw_map] = STATE(1340),
    [sym__r_sgl_flw_map] = STATE(333),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__r_dqt_str_val] = STATE(1294),
    [sym__br_dqt_str_val] = STATE(1294),
    [sym__r_sgl_dqt_str_val] = STATE(305),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(1344),
    [sym__br_dqt_str] = STATE(1354),
    [sym__r_sgl_dqt_str] = STATE(326),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__r_sqt_str_val] = STATE(1294),
    [sym__br_sqt_str_val] = STATE(1294),
    [sym__r_sgl_sqt_str_val] = STATE(305),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(1387),
    [sym__br_sqt_str] = STATE(1388),
    [sym__r_sgl_sqt_str] = STATE(358),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__r_pln_blk_val] = STATE(1294),
    [sym__br_pln_blk_val] = STATE(1294),
    [sym__r_sgl_pln_blk_val] = STATE(305),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(318),
    [sym__br_sgl_pln_blk] = STATE(317),
    [sym__r_mtl_pln_blk] = STATE(1390),
    [sym__br_mtl_pln_blk] = STATE(1400),
    [sym__r_als_val] = STATE(293),
    [sym__br_als_val] = STATE(294),
    [sym__r_als] = STATE(303),
    [sym__br_als] = STATE(316),
    [sym__r_acr] = STATE(111),
    [sym__br_acr] = STATE(112),
    [sym__r_blk_seq_bgn] = ACTIONS(17),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__b_blk_seq_bgn] = ACTIONS(163),
    [sym__r_blk_key_bgn] = ACTIONS(21),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__r_blk_val_bgn] = ACTIONS(25),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__r_blk_lit_bgn] = ACTIONS(125),
    [sym__br_blk_lit_bgn] = ACTIONS(127),
    [sym__r_blk_fld_bgn] = ACTIONS(125),
    [sym__br_blk_fld_bgn] = ACTIONS(127),
    [sym__r_flw_seq_bgn] = ACTIONS(129),
    [sym__br_flw_seq_bgn] = ACTIONS(131),
    [sym__r_flw_map_bgn] = ACTIONS(133),
    [sym__br_flw_map_bgn] = ACTIONS(135),
    [sym__r_dqt_str_bgn] = ACTIONS(137),
    [sym__br_dqt_str_bgn] = ACTIONS(139),
    [sym__r_sqt_str_bgn] = ACTIONS(141),
    [sym__br_sqt_str_bgn] = ACTIONS(143),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(145),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(147),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(145),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(147),
    [sym__r_sgl_pln_int_blk] = ACTIONS(145),
    [sym__br_sgl_pln_int_blk] = ACTIONS(147),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(145),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(147),
    [sym__r_sgl_pln_str_blk] = ACTIONS(145),
    [sym__br_sgl_pln_str_blk] = ACTIONS(147),
    [sym__r_mtl_pln_str_blk] = ACTIONS(149),
    [sym__br_mtl_pln_str_blk] = ACTIONS(151),
    [sym__r_tag] = ACTIONS(153),
    [sym__br_tag] = ACTIONS(155),
    [sym__r_acr_bgn] = ACTIONS(105),
    [sym__br_acr_bgn] = ACTIONS(107),
    [sym__r_als_bgn] = ACTIONS(157),
    [sym__br_als_bgn] = ACTIONS(159),
    [sym__bl] = ACTIONS(163),
    [sym_comment] = ACTIONS(3),
  },
  [11] = {
    [sym__r_prp_val] = STATE(401),
    [sym__br_prp_val] = STATE(401),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__r_prp] = STATE(35),
    [sym__br_prp] = STATE(36),
    [sym__br_sgl_prp] = STATE(182),
    [sym__r_blk_seq_br_val] = STATE(401),
    [sym__br_blk_seq_val] = STATE(401),
    [sym__r_blk_seq_spc_val] = STATE(401),
    [sym__br_blk_seq_spc_val] = STATE(401),
    [sym__b_blk_seq_spc_val] = STATE(401),
    [sym__br_blk_seq] = STATE(409),
    [sym__b_blk_seq_spc] = STATE(438),
    [sym__br_blk_seq_itm] = STATE(960),
    [sym__b_blk_seq_itm] = STATE(978),
    [sym__r_blk_map_br_val] = STATE(401),
    [sym__br_blk_map_val] = STATE(401),
    [sym__br_blk_map] = STATE(412),
    [sym__br_blk_map_itm] = STATE(159),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__r_blk_str_val] = STATE(401),
    [sym__br_blk_str_val] = STATE(401),
    [sym__r_blk_str] = STATE(413),
    [sym__br_blk_str] = STATE(416),
    [sym__r_flw_seq_val] = STATE(401),
    [sym__br_flw_seq_val] = STATE(401),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(417),
    [sym__br_flw_seq] = STATE(418),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__r_flw_map_val] = STATE(401),
    [sym__br_flw_map_val] = STATE(401),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(419),
    [sym__br_flw_map] = STATE(420),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__r_dqt_str_val] = STATE(401),
    [sym__br_dqt_str_val] = STATE(401),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(426),
    [sym__br_dqt_str] = STATE(428),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__r_sqt_str_val] = STATE(401),
    [sym__br_sqt_str_val] = STATE(401),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(430),
    [sym__br_sqt_str] = STATE(434),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__r_pln_blk_val] = STATE(401),
    [sym__br_pln_blk_val] = STATE(401),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(436),
    [sym__br_sgl_pln_blk] = STATE(365),
    [sym__r_mtl_pln_blk] = STATE(436),
    [sym__br_mtl_pln_blk] = STATE(437),
    [sym__r_als_val] = STATE(401),
    [sym__br_als_val] = STATE(301),
    [sym__r_als] = STATE(399),
    [sym__br_als] = STATE(385),
    [sym__r_acr] = STATE(120),
    [sym__br_acr] = STATE(113),
    [aux_sym__r_blk_seq_repeat1] = STATE(978),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__b_blk_seq_bgn] = ACTIONS(69),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__b_blk_key_bgn] = ACTIONS(165),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__b_blk_val_bgn] = ACTIONS(165),
    [sym__r_blk_lit_bgn] = ACTIONS(73),
    [sym__br_blk_lit_bgn] = ACTIONS(75),
    [sym__r_blk_fld_bgn] = ACTIONS(73),
    [sym__br_blk_fld_bgn] = ACTIONS(75),
    [sym__r_flw_seq_bgn] = ACTIONS(167),
    [sym__br_flw_seq_bgn] = ACTIONS(79),
    [sym__b_flw_seq_bgn] = ACTIONS(165),
    [sym__r_flw_map_bgn] = ACTIONS(169),
    [sym__br_flw_map_bgn] = ACTIONS(83),
    [sym__b_flw_map_bgn] = ACTIONS(165),
    [sym__r_dqt_str_bgn] = ACTIONS(171),
    [sym__br_dqt_str_bgn] = ACTIONS(87),
    [sym__b_dqt_str_bgn] = ACTIONS(165),
    [sym__r_sqt_str_bgn] = ACTIONS(173),
    [sym__br_sqt_str_bgn] = ACTIONS(91),
    [sym__b_sqt_str_bgn] = ACTIONS(165),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(175),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(95),
    [sym__b_sgl_pln_nul_blk] = ACTIONS(165),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(175),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(95),
    [sym__b_sgl_pln_bol_blk] = ACTIONS(165),
    [sym__r_sgl_pln_int_blk] = ACTIONS(175),
    [sym__br_sgl_pln_int_blk] = ACTIONS(95),
    [sym__b_sgl_pln_int_blk] = ACTIONS(165),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(175),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(95),
    [sym__b_sgl_pln_flt_blk] = ACTIONS(165),
    [sym__r_sgl_pln_str_blk] = ACTIONS(175),
    [sym__br_sgl_pln_str_blk] = ACTIONS(95),
    [sym__b_sgl_pln_str_blk] = ACTIONS(165),
    [sym__r_mtl_pln_str_blk] = ACTIONS(97),
    [sym__br_mtl_pln_str_blk] = ACTIONS(99),
    [sym__r_tag] = ACTIONS(177),
    [sym__br_tag] = ACTIONS(103),
    [sym__b_tag] = ACTIONS(165),
    [sym__r_acr_bgn] = ACTIONS(179),
    [sym__br_acr_bgn] = ACTIONS(107),
    [sym__b_acr_bgn] = ACTIONS(165),
    [sym__r_als_bgn] = ACTIONS(181),
    [sym__br_als_bgn] = ACTIONS(111),
    [sym__b_als_bgn] = ACTIONS(165),
    [sym__bl] = ACTIONS(165),
    [sym_comment] = ACTIONS(3),
  },
  [12] = {
    [sym__exp_doc_tal] = STATE(1224),
    [sym__r_prp_val] = STATE(1224),
    [sym__br_prp_val] = STATE(1224),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__r_prp] = STATE(80),
    [sym__br_prp] = STATE(81),
    [sym__br_sgl_prp] = STATE(182),
    [sym__r_blk_seq_br_val] = STATE(1224),
    [sym__br_blk_seq_val] = STATE(1224),
    [sym__br_blk_seq] = STATE(1116),
    [sym__br_blk_seq_itm] = STATE(977),
    [sym__r_blk_map_br_val] = STATE(1224),
    [sym__br_blk_map_val] = STATE(1224),
    [sym__br_blk_map] = STATE(1178),
    [sym__br_blk_map_itm] = STATE(132),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__r_blk_str_val] = STATE(1224),
    [sym__br_blk_str_val] = STATE(1224),
    [sym__r_blk_str] = STATE(1204),
    [sym__br_blk_str] = STATE(1207),
    [sym__r_flw_seq_val] = STATE(1224),
    [sym__br_flw_seq_val] = STATE(1224),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(1217),
    [sym__br_flw_seq] = STATE(1218),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__r_flw_map_val] = STATE(1224),
    [sym__br_flw_map_val] = STATE(1224),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(1223),
    [sym__br_flw_map] = STATE(1225),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__r_dqt_str_val] = STATE(1224),
    [sym__br_dqt_str_val] = STATE(1224),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(1226),
    [sym__br_dqt_str] = STATE(1229),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__r_sqt_str_val] = STATE(1224),
    [sym__br_sqt_str_val] = STATE(1224),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(1234),
    [sym__br_sqt_str] = STATE(1235),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__r_pln_blk_val] = STATE(1224),
    [sym__br_pln_blk_val] = STATE(1224),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(1236),
    [sym__br_sgl_pln_blk] = STATE(292),
    [sym__r_mtl_pln_blk] = STATE(1236),
    [sym__br_mtl_pln_blk] = STATE(1179),
    [sym__r_als_val] = STATE(1224),
    [sym__br_als_val] = STATE(271),
    [sym__r_als] = STATE(1144),
    [sym__br_als] = STATE(279),
    [sym__r_acr] = STATE(168),
    [sym__br_acr] = STATE(99),
    [sym__eof] = ACTIONS(183),
    [sym__s_drs_end] = ACTIONS(183),
    [sym__s_doc_end] = ACTIONS(183),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__r_blk_lit_bgn] = ACTIONS(29),
    [sym__br_blk_lit_bgn] = ACTIONS(31),
    [sym__r_blk_fld_bgn] = ACTIONS(29),
    [sym__br_blk_fld_bgn] = ACTIONS(31),
    [sym__r_flw_seq_bgn] = ACTIONS(185),
    [sym__br_flw_seq_bgn] = ACTIONS(35),
    [sym__r_flw_map_bgn] = ACTIONS(187),
    [sym__br_flw_map_bgn] = ACTIONS(39),
    [sym__r_dqt_str_bgn] = ACTIONS(189),
    [sym__br_dqt_str_bgn] = ACTIONS(43),
    [sym__r_sqt_str_bgn] = ACTIONS(191),
    [sym__br_sqt_str_bgn] = ACTIONS(47),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(193),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(51),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(193),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(51),
    [sym__r_sgl_pln_int_blk] = ACTIONS(193),
    [sym__br_sgl_pln_int_blk] = ACTIONS(51),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(193),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(51),
    [sym__r_sgl_pln_str_blk] = ACTIONS(193),
    [sym__br_sgl_pln_str_blk] = ACTIONS(51),
    [sym__r_mtl_pln_str_blk] = ACTIONS(53),
    [sym__br_mtl_pln_str_blk] = ACTIONS(55),
    [sym__r_tag] = ACTIONS(195),
    [sym__br_tag] = ACTIONS(59),
    [sym__r_acr_bgn] = ACTIONS(197),
    [sym__br_acr_bgn] = ACTIONS(63),
    [sym__r_als_bgn] = ACTIONS(199),
    [sym__br_als_bgn] = ACTIONS(67),
    [sym_comment] = ACTIONS(3),
  },
  [13] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_sgl_prp_val] = STATE(974),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_sgl_prp] = STATE(309),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_sgl_flw_seq_val] = STATE(976),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_sgl_flw_seq] = STATE(1180),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_sgl_flw_map_val] = STATE(976),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_sgl_flw_map] = STATE(1181),
    [sym__flw_map_tal] = STATE(1053),
    [sym__sgl_flw_map_tal] = STATE(1287),
    [sym__r_flw_map_dat] = STATE(1342),
    [sym__br_flw_map_dat] = STATE(1342),
    [sym__r_sgl_flw_col_dat] = STATE(1467),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_sgl_flw_col_itm] = STATE(1015),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_sgl_flw_exp_par] = STATE(1405),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_sgl_flw_imp_par] = STATE(1435),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1436),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_sgl_dqt_str_val] = STATE(976),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sgl_dqt_str] = STATE(1182),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sgl_sqt_str_val] = STATE(976),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_sgl_sqt_str] = STATE(1183),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw_val] = STATE(974),
    [sym__r_sgl_pln_flw] = STATE(751),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(600),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(204),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(201),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(205),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(209),
    [sym__br_flw_map_end] = ACTIONS(211),
    [sym__r_flw_key_bgn] = ACTIONS(213),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(217),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(221),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(225),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(237),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [14] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_sgl_prp_val] = STATE(974),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_sgl_prp] = STATE(309),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_sgl_flw_seq_val] = STATE(976),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_sgl_flw_seq] = STATE(1180),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_sgl_flw_map_val] = STATE(976),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_sgl_flw_map] = STATE(1181),
    [sym__flw_map_tal] = STATE(1103),
    [sym__sgl_flw_map_tal] = STATE(1359),
    [sym__r_flw_map_dat] = STATE(1267),
    [sym__br_flw_map_dat] = STATE(1267),
    [sym__r_sgl_flw_col_dat] = STATE(1511),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_sgl_flw_col_itm] = STATE(1015),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_sgl_flw_exp_par] = STATE(1405),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_sgl_flw_imp_par] = STATE(1435),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1436),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_sgl_dqt_str_val] = STATE(976),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sgl_dqt_str] = STATE(1182),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sgl_sqt_str_val] = STATE(976),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_sgl_sqt_str] = STATE(1183),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw_val] = STATE(974),
    [sym__r_sgl_pln_flw] = STATE(751),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(600),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(204),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(201),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(205),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(249),
    [sym__br_flw_map_end] = ACTIONS(251),
    [sym__r_flw_key_bgn] = ACTIONS(213),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(217),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(221),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(225),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(237),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [15] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_sgl_prp_val] = STATE(974),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_sgl_prp] = STATE(309),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_sgl_flw_seq_val] = STATE(976),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_sgl_flw_seq] = STATE(1180),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_sgl_flw_map_val] = STATE(976),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_sgl_flw_map] = STATE(1181),
    [sym__flw_map_tal] = STATE(1442),
    [sym__sgl_flw_map_tal] = STATE(388),
    [sym__r_flw_map_dat] = STATE(1278),
    [sym__br_flw_map_dat] = STATE(1278),
    [sym__r_sgl_flw_col_dat] = STATE(1488),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_sgl_flw_col_itm] = STATE(1015),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_sgl_flw_exp_par] = STATE(1405),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_sgl_flw_imp_par] = STATE(1435),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1436),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_sgl_dqt_str_val] = STATE(976),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sgl_dqt_str] = STATE(1182),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sgl_sqt_str_val] = STATE(976),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_sgl_sqt_str] = STATE(1183),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw_val] = STATE(974),
    [sym__r_sgl_pln_flw] = STATE(751),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(600),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(204),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(201),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(205),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(253),
    [sym__br_flw_map_end] = ACTIONS(255),
    [sym__r_flw_key_bgn] = ACTIONS(213),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(217),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(221),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(225),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(237),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [16] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_sgl_prp_val] = STATE(974),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_sgl_prp] = STATE(309),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_sgl_flw_seq_val] = STATE(976),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_sgl_flw_seq] = STATE(1180),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_sgl_flw_map_val] = STATE(976),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_sgl_flw_map] = STATE(1181),
    [sym__flw_map_tal] = STATE(831),
    [sym__sgl_flw_map_tal] = STATE(1146),
    [sym__r_flw_map_dat] = STATE(1355),
    [sym__br_flw_map_dat] = STATE(1355),
    [sym__r_sgl_flw_col_dat] = STATE(1503),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_sgl_flw_col_itm] = STATE(1015),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_sgl_flw_exp_par] = STATE(1405),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_sgl_flw_imp_par] = STATE(1435),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1436),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_sgl_dqt_str_val] = STATE(976),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sgl_dqt_str] = STATE(1182),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sgl_sqt_str_val] = STATE(976),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_sgl_sqt_str] = STATE(1183),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw_val] = STATE(974),
    [sym__r_sgl_pln_flw] = STATE(751),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(600),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(204),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(201),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(205),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(257),
    [sym__br_flw_map_end] = ACTIONS(259),
    [sym__r_flw_key_bgn] = ACTIONS(213),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(217),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(221),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(225),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(237),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [17] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_sgl_prp_val] = STATE(974),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_sgl_prp] = STATE(309),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_sgl_flw_seq_val] = STATE(976),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_sgl_flw_seq] = STATE(1180),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_sgl_flw_map_val] = STATE(976),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_sgl_flw_map] = STATE(1181),
    [sym__flw_map_tal] = STATE(809),
    [sym__sgl_flw_map_tal] = STATE(1192),
    [sym__r_flw_map_dat] = STATE(1401),
    [sym__br_flw_map_dat] = STATE(1401),
    [sym__r_sgl_flw_col_dat] = STATE(1491),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_sgl_flw_col_itm] = STATE(1015),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_sgl_flw_exp_par] = STATE(1405),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_sgl_flw_imp_par] = STATE(1435),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1436),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_sgl_dqt_str_val] = STATE(976),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sgl_dqt_str] = STATE(1182),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sgl_sqt_str_val] = STATE(976),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_sgl_sqt_str] = STATE(1183),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw_val] = STATE(974),
    [sym__r_sgl_pln_flw] = STATE(751),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(600),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(204),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(201),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(205),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(261),
    [sym__br_flw_map_end] = ACTIONS(263),
    [sym__r_flw_key_bgn] = ACTIONS(213),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(217),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(221),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(225),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(237),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [18] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_sgl_prp_val] = STATE(974),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_sgl_prp] = STATE(309),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_sgl_flw_seq_val] = STATE(976),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_sgl_flw_seq] = STATE(1180),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_sgl_flw_map_val] = STATE(976),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_sgl_flw_map] = STATE(1181),
    [sym__flw_map_tal] = STATE(1188),
    [sym__sgl_flw_map_tal] = STATE(388),
    [sym__r_flw_map_dat] = STATE(1327),
    [sym__br_flw_map_dat] = STATE(1327),
    [sym__r_sgl_flw_col_dat] = STATE(1488),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_sgl_flw_col_itm] = STATE(1015),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_sgl_flw_exp_par] = STATE(1405),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_sgl_flw_imp_par] = STATE(1435),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1436),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_sgl_dqt_str_val] = STATE(976),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sgl_dqt_str] = STATE(1182),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sgl_sqt_str_val] = STATE(976),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_sgl_sqt_str] = STATE(1183),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw_val] = STATE(974),
    [sym__r_sgl_pln_flw] = STATE(751),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(600),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(204),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(201),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(205),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(265),
    [sym__br_flw_map_end] = ACTIONS(267),
    [sym__r_flw_key_bgn] = ACTIONS(213),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(217),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(221),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(225),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(237),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [19] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_sgl_prp_val] = STATE(974),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_sgl_prp] = STATE(309),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_sgl_flw_seq_val] = STATE(976),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_sgl_flw_seq] = STATE(1180),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_sgl_flw_map_val] = STATE(976),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_sgl_flw_map] = STATE(1181),
    [sym__flw_map_tal] = STATE(1239),
    [sym__sgl_flw_map_tal] = STATE(380),
    [sym__r_flw_map_dat] = STATE(1327),
    [sym__br_flw_map_dat] = STATE(1327),
    [sym__r_sgl_flw_col_dat] = STATE(1488),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_sgl_flw_col_itm] = STATE(1015),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_sgl_flw_exp_par] = STATE(1405),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_sgl_flw_imp_par] = STATE(1435),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1436),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_sgl_dqt_str_val] = STATE(976),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sgl_dqt_str] = STATE(1182),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sgl_sqt_str_val] = STATE(976),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_sgl_sqt_str] = STATE(1183),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw_val] = STATE(974),
    [sym__r_sgl_pln_flw] = STATE(751),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(600),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(204),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(201),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(205),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(265),
    [sym__br_flw_map_end] = ACTIONS(269),
    [sym__r_flw_key_bgn] = ACTIONS(213),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(217),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(221),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(225),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(237),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [20] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_sgl_prp_val] = STATE(974),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_sgl_prp] = STATE(309),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_sgl_flw_seq_val] = STATE(976),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_sgl_flw_seq] = STATE(1180),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_sgl_flw_map_val] = STATE(976),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_sgl_flw_map] = STATE(1181),
    [sym__flw_map_tal] = STATE(448),
    [sym__sgl_flw_map_tal] = STATE(380),
    [sym__r_flw_map_dat] = STATE(1303),
    [sym__br_flw_map_dat] = STATE(1303),
    [sym__r_sgl_flw_col_dat] = STATE(1488),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_sgl_flw_col_itm] = STATE(1015),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_sgl_flw_exp_par] = STATE(1405),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_sgl_flw_imp_par] = STATE(1435),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1436),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_sgl_dqt_str_val] = STATE(976),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sgl_dqt_str] = STATE(1182),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sgl_sqt_str_val] = STATE(976),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_sgl_sqt_str] = STATE(1183),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw_val] = STATE(974),
    [sym__r_sgl_pln_flw] = STATE(751),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(600),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(204),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(201),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(205),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(271),
    [sym__br_flw_map_end] = ACTIONS(273),
    [sym__r_flw_key_bgn] = ACTIONS(213),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(217),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(221),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(225),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(237),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [21] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_sgl_prp_val] = STATE(974),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_sgl_prp] = STATE(309),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_sgl_flw_seq_val] = STATE(976),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_sgl_flw_seq] = STATE(1180),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_sgl_flw_map_val] = STATE(976),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_sgl_flw_map] = STATE(1181),
    [sym__flw_map_tal] = STATE(447),
    [sym__sgl_flw_map_tal] = STATE(388),
    [sym__r_flw_map_dat] = STATE(1303),
    [sym__br_flw_map_dat] = STATE(1303),
    [sym__r_sgl_flw_col_dat] = STATE(1488),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_sgl_flw_col_itm] = STATE(1015),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_sgl_flw_exp_par] = STATE(1405),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_sgl_flw_imp_par] = STATE(1435),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1436),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_sgl_dqt_str_val] = STATE(976),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sgl_dqt_str] = STATE(1182),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sgl_sqt_str_val] = STATE(976),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_sgl_sqt_str] = STATE(1183),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw_val] = STATE(974),
    [sym__r_sgl_pln_flw] = STATE(751),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(600),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(204),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(201),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(205),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(271),
    [sym__br_flw_map_end] = ACTIONS(275),
    [sym__r_flw_key_bgn] = ACTIONS(213),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(217),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(221),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(225),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(237),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [22] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_sgl_prp_val] = STATE(974),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_sgl_prp] = STATE(309),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_sgl_flw_seq_val] = STATE(976),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_sgl_flw_seq] = STATE(1180),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_sgl_flw_map_val] = STATE(976),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_sgl_flw_map] = STATE(1181),
    [sym__flw_map_tal] = STATE(1452),
    [sym__sgl_flw_map_tal] = STATE(380),
    [sym__r_flw_map_dat] = STATE(1278),
    [sym__br_flw_map_dat] = STATE(1278),
    [sym__r_sgl_flw_col_dat] = STATE(1488),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_sgl_flw_col_itm] = STATE(1015),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_sgl_flw_exp_par] = STATE(1405),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_sgl_flw_imp_par] = STATE(1435),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1436),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_sgl_dqt_str_val] = STATE(976),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sgl_dqt_str] = STATE(1182),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sgl_sqt_str_val] = STATE(976),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_sgl_sqt_str] = STATE(1183),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw_val] = STATE(974),
    [sym__r_sgl_pln_flw] = STATE(751),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(600),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(204),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(201),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(205),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(253),
    [sym__br_flw_map_end] = ACTIONS(277),
    [sym__r_flw_key_bgn] = ACTIONS(213),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(217),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(221),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(225),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(237),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [23] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_sgl_prp_val] = STATE(974),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_sgl_prp] = STATE(309),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_sgl_flw_seq_val] = STATE(976),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_sgl_flw_seq] = STATE(1180),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_sgl_flw_map_val] = STATE(976),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_sgl_flw_map] = STATE(1181),
    [sym__flw_map_tal] = STATE(1358),
    [sym__sgl_flw_map_tal] = STATE(1518),
    [sym__r_flw_map_dat] = STATE(1443),
    [sym__br_flw_map_dat] = STATE(1443),
    [sym__r_sgl_flw_col_dat] = STATE(1478),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_sgl_flw_col_itm] = STATE(1015),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_sgl_flw_exp_par] = STATE(1405),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_sgl_flw_imp_par] = STATE(1435),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1436),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_sgl_dqt_str_val] = STATE(976),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sgl_dqt_str] = STATE(1182),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sgl_sqt_str_val] = STATE(976),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_sgl_sqt_str] = STATE(1183),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw_val] = STATE(974),
    [sym__r_sgl_pln_flw] = STATE(751),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(600),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(204),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(201),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(205),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(279),
    [sym__br_flw_map_end] = ACTIONS(281),
    [sym__r_flw_key_bgn] = ACTIONS(213),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(217),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(221),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(225),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(237),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [24] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_sgl_prp_val] = STATE(1002),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_sgl_prp] = STATE(304),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_sgl_flw_seq_val] = STATE(1006),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_sgl_flw_seq] = STATE(1139),
    [sym__flw_seq_tal] = STATE(1050),
    [sym__sgl_flw_seq_tal] = STATE(1445),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_sgl_flw_map_val] = STATE(1006),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_sgl_flw_map] = STATE(1140),
    [sym__r_flw_seq_dat] = STATE(1343),
    [sym__br_flw_seq_dat] = STATE(1343),
    [sym__r_sgl_flw_col_dat] = STATE(1468),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_sgl_flw_col_itm] = STATE(1112),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_sgl_flw_exp_par] = STATE(1256),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_sgl_flw_imp_par] = STATE(1283),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1437),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_sgl_dqt_str_val] = STATE(1006),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sgl_dqt_str] = STATE(1142),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sgl_sqt_str_val] = STATE(1006),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_sgl_sqt_str] = STATE(1143),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw_val] = STATE(1002),
    [sym__r_sgl_pln_flw] = STATE(879),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(633),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(211),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(283),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(287),
    [sym__br_flw_seq_end] = ACTIONS(289),
    [sym__r_flw_map_bgn] = ACTIONS(291),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(295),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(299),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(303),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(307),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(319),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [25] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_sgl_prp_val] = STATE(1002),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_sgl_prp] = STATE(304),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_sgl_flw_seq_val] = STATE(1006),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_sgl_flw_seq] = STATE(1139),
    [sym__flw_seq_tal] = STATE(1433),
    [sym__sgl_flw_seq_tal] = STATE(392),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_sgl_flw_map_val] = STATE(1006),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_sgl_flw_map] = STATE(1140),
    [sym__r_flw_seq_dat] = STATE(1281),
    [sym__br_flw_seq_dat] = STATE(1281),
    [sym__r_sgl_flw_col_dat] = STATE(1502),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_sgl_flw_col_itm] = STATE(1112),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_sgl_flw_exp_par] = STATE(1256),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_sgl_flw_imp_par] = STATE(1283),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1437),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_sgl_dqt_str_val] = STATE(1006),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sgl_dqt_str] = STATE(1142),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sgl_sqt_str_val] = STATE(1006),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_sgl_sqt_str] = STATE(1143),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw_val] = STATE(1002),
    [sym__r_sgl_pln_flw] = STATE(879),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(633),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(211),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(283),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(331),
    [sym__br_flw_seq_end] = ACTIONS(333),
    [sym__r_flw_map_bgn] = ACTIONS(291),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(295),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(299),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(303),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(307),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(319),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [26] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_sgl_prp_val] = STATE(1002),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_sgl_prp] = STATE(304),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_sgl_flw_seq_val] = STATE(1006),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_sgl_flw_seq] = STATE(1139),
    [sym__flw_seq_tal] = STATE(1100),
    [sym__sgl_flw_seq_tal] = STATE(1346),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_sgl_flw_map_val] = STATE(1006),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_sgl_flw_map] = STATE(1140),
    [sym__r_flw_seq_dat] = STATE(1268),
    [sym__br_flw_seq_dat] = STATE(1268),
    [sym__r_sgl_flw_col_dat] = STATE(1510),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_sgl_flw_col_itm] = STATE(1112),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_sgl_flw_exp_par] = STATE(1256),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_sgl_flw_imp_par] = STATE(1283),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1437),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_sgl_dqt_str_val] = STATE(1006),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sgl_dqt_str] = STATE(1142),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sgl_sqt_str_val] = STATE(1006),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_sgl_sqt_str] = STATE(1143),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw_val] = STATE(1002),
    [sym__r_sgl_pln_flw] = STATE(879),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(633),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(211),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(283),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(335),
    [sym__br_flw_seq_end] = ACTIONS(337),
    [sym__r_flw_map_bgn] = ACTIONS(291),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(295),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(299),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(303),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(307),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(319),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [27] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_sgl_prp_val] = STATE(1002),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_sgl_prp] = STATE(304),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_sgl_flw_seq_val] = STATE(1006),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_sgl_flw_seq] = STATE(1139),
    [sym__flw_seq_tal] = STATE(1163),
    [sym__sgl_flw_seq_tal] = STATE(392),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_sgl_flw_map_val] = STATE(1006),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_sgl_flw_map] = STATE(1140),
    [sym__r_flw_seq_dat] = STATE(1384),
    [sym__br_flw_seq_dat] = STATE(1384),
    [sym__r_sgl_flw_col_dat] = STATE(1502),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_sgl_flw_col_itm] = STATE(1112),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_sgl_flw_exp_par] = STATE(1256),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_sgl_flw_imp_par] = STATE(1283),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1437),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_sgl_dqt_str_val] = STATE(1006),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sgl_dqt_str] = STATE(1142),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sgl_sqt_str_val] = STATE(1006),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_sgl_sqt_str] = STATE(1143),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw_val] = STATE(1002),
    [sym__r_sgl_pln_flw] = STATE(879),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(633),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(211),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(283),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(339),
    [sym__br_flw_seq_end] = ACTIONS(341),
    [sym__r_flw_map_bgn] = ACTIONS(291),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(295),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(299),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(303),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(307),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(319),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [28] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_sgl_prp_val] = STATE(1002),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_sgl_prp] = STATE(304),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_sgl_flw_seq_val] = STATE(1006),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_sgl_flw_seq] = STATE(1139),
    [sym__flw_seq_tal] = STATE(1345),
    [sym__sgl_flw_seq_tal] = STATE(1517),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_sgl_flw_map_val] = STATE(1006),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_sgl_flw_map] = STATE(1140),
    [sym__r_flw_seq_dat] = STATE(1441),
    [sym__br_flw_seq_dat] = STATE(1441),
    [sym__r_sgl_flw_col_dat] = STATE(1479),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_sgl_flw_col_itm] = STATE(1112),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_sgl_flw_exp_par] = STATE(1256),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_sgl_flw_imp_par] = STATE(1283),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1437),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_sgl_dqt_str_val] = STATE(1006),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sgl_dqt_str] = STATE(1142),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sgl_sqt_str_val] = STATE(1006),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_sgl_sqt_str] = STATE(1143),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw_val] = STATE(1002),
    [sym__r_sgl_pln_flw] = STATE(879),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(633),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(211),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(283),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(343),
    [sym__br_flw_seq_end] = ACTIONS(345),
    [sym__r_flw_map_bgn] = ACTIONS(291),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(295),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(299),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(303),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(307),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(319),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [29] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_sgl_prp_val] = STATE(1002),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_sgl_prp] = STATE(304),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_sgl_flw_seq_val] = STATE(1006),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_sgl_flw_seq] = STATE(1139),
    [sym__flw_seq_tal] = STATE(445),
    [sym__sgl_flw_seq_tal] = STATE(392),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_sgl_flw_map_val] = STATE(1006),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_sgl_flw_map] = STATE(1140),
    [sym__r_flw_seq_dat] = STATE(1301),
    [sym__br_flw_seq_dat] = STATE(1301),
    [sym__r_sgl_flw_col_dat] = STATE(1502),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_sgl_flw_col_itm] = STATE(1112),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_sgl_flw_exp_par] = STATE(1256),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_sgl_flw_imp_par] = STATE(1283),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1437),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_sgl_dqt_str_val] = STATE(1006),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sgl_dqt_str] = STATE(1142),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sgl_sqt_str_val] = STATE(1006),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_sgl_sqt_str] = STATE(1143),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw_val] = STATE(1002),
    [sym__r_sgl_pln_flw] = STATE(879),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(633),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(211),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(283),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(347),
    [sym__br_flw_seq_end] = ACTIONS(349),
    [sym__r_flw_map_bgn] = ACTIONS(291),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(295),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(299),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(303),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(307),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(319),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [30] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_sgl_prp_val] = STATE(1002),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_sgl_prp] = STATE(304),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_sgl_flw_seq_val] = STATE(1006),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_sgl_flw_seq] = STATE(1139),
    [sym__flw_seq_tal] = STATE(446),
    [sym__sgl_flw_seq_tal] = STATE(391),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_sgl_flw_map_val] = STATE(1006),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_sgl_flw_map] = STATE(1140),
    [sym__r_flw_seq_dat] = STATE(1301),
    [sym__br_flw_seq_dat] = STATE(1301),
    [sym__r_sgl_flw_col_dat] = STATE(1502),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_sgl_flw_col_itm] = STATE(1112),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_sgl_flw_exp_par] = STATE(1256),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_sgl_flw_imp_par] = STATE(1283),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1437),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_sgl_dqt_str_val] = STATE(1006),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sgl_dqt_str] = STATE(1142),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sgl_sqt_str_val] = STATE(1006),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_sgl_sqt_str] = STATE(1143),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw_val] = STATE(1002),
    [sym__r_sgl_pln_flw] = STATE(879),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(633),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(211),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(283),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(347),
    [sym__br_flw_seq_end] = ACTIONS(351),
    [sym__r_flw_map_bgn] = ACTIONS(291),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(295),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(299),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(303),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(307),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(319),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [31] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_sgl_prp_val] = STATE(1002),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_sgl_prp] = STATE(304),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_sgl_flw_seq_val] = STATE(1006),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_sgl_flw_seq] = STATE(1139),
    [sym__flw_seq_tal] = STATE(1247),
    [sym__sgl_flw_seq_tal] = STATE(391),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_sgl_flw_map_val] = STATE(1006),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_sgl_flw_map] = STATE(1140),
    [sym__r_flw_seq_dat] = STATE(1281),
    [sym__br_flw_seq_dat] = STATE(1281),
    [sym__r_sgl_flw_col_dat] = STATE(1502),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_sgl_flw_col_itm] = STATE(1112),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_sgl_flw_exp_par] = STATE(1256),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_sgl_flw_imp_par] = STATE(1283),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1437),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_sgl_dqt_str_val] = STATE(1006),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sgl_dqt_str] = STATE(1142),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sgl_sqt_str_val] = STATE(1006),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_sgl_sqt_str] = STATE(1143),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw_val] = STATE(1002),
    [sym__r_sgl_pln_flw] = STATE(879),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(633),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(211),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(283),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(331),
    [sym__br_flw_seq_end] = ACTIONS(353),
    [sym__r_flw_map_bgn] = ACTIONS(291),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(295),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(299),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(303),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(307),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(319),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [32] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_sgl_prp_val] = STATE(1002),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_sgl_prp] = STATE(304),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_sgl_flw_seq_val] = STATE(1006),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_sgl_flw_seq] = STATE(1139),
    [sym__flw_seq_tal] = STATE(823),
    [sym__sgl_flw_seq_tal] = STATE(1145),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_sgl_flw_map_val] = STATE(1006),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_sgl_flw_map] = STATE(1140),
    [sym__r_flw_seq_dat] = STATE(1353),
    [sym__br_flw_seq_dat] = STATE(1353),
    [sym__r_sgl_flw_col_dat] = STATE(1501),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_sgl_flw_col_itm] = STATE(1112),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_sgl_flw_exp_par] = STATE(1256),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_sgl_flw_imp_par] = STATE(1283),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1437),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_sgl_dqt_str_val] = STATE(1006),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sgl_dqt_str] = STATE(1142),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sgl_sqt_str_val] = STATE(1006),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_sgl_sqt_str] = STATE(1143),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw_val] = STATE(1002),
    [sym__r_sgl_pln_flw] = STATE(879),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(633),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(211),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(283),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(355),
    [sym__br_flw_seq_end] = ACTIONS(357),
    [sym__r_flw_map_bgn] = ACTIONS(291),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(295),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(299),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(303),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(307),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(319),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [33] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_sgl_prp_val] = STATE(1002),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_sgl_prp] = STATE(304),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_sgl_flw_seq_val] = STATE(1006),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_sgl_flw_seq] = STATE(1139),
    [sym__flw_seq_tal] = STATE(816),
    [sym__sgl_flw_seq_tal] = STATE(1187),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_sgl_flw_map_val] = STATE(1006),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_sgl_flw_map] = STATE(1140),
    [sym__r_flw_seq_dat] = STATE(1399),
    [sym__br_flw_seq_dat] = STATE(1399),
    [sym__r_sgl_flw_col_dat] = STATE(1492),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_sgl_flw_col_itm] = STATE(1112),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_sgl_flw_exp_par] = STATE(1256),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_sgl_flw_imp_par] = STATE(1283),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1437),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_sgl_dqt_str_val] = STATE(1006),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sgl_dqt_str] = STATE(1142),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sgl_sqt_str_val] = STATE(1006),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_sgl_sqt_str] = STATE(1143),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw_val] = STATE(1002),
    [sym__r_sgl_pln_flw] = STATE(879),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(633),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(211),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(283),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(359),
    [sym__br_flw_seq_end] = ACTIONS(361),
    [sym__r_flw_map_bgn] = ACTIONS(291),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(295),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(299),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(303),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(307),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(319),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [34] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_sgl_prp_val] = STATE(1002),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_sgl_prp] = STATE(304),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_sgl_flw_seq_val] = STATE(1006),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_sgl_flw_seq] = STATE(1139),
    [sym__flw_seq_tal] = STATE(1159),
    [sym__sgl_flw_seq_tal] = STATE(391),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_sgl_flw_map_val] = STATE(1006),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_sgl_flw_map] = STATE(1140),
    [sym__r_flw_seq_dat] = STATE(1384),
    [sym__br_flw_seq_dat] = STATE(1384),
    [sym__r_sgl_flw_col_dat] = STATE(1502),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_sgl_flw_col_itm] = STATE(1112),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_sgl_flw_exp_par] = STATE(1256),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_sgl_flw_imp_par] = STATE(1283),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_sgl_flw_njl_ann_par] = STATE(1437),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_sgl_dqt_str_val] = STATE(1006),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sgl_dqt_str] = STATE(1142),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sgl_sqt_str_val] = STATE(1006),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_sgl_sqt_str] = STATE(1143),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw_val] = STATE(1002),
    [sym__r_sgl_pln_flw] = STATE(879),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(633),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(211),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(283),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(339),
    [sym__br_flw_seq_end] = ACTIONS(363),
    [sym__r_flw_map_bgn] = ACTIONS(291),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(295),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(299),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(303),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(307),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(319),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [35] = {
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__br_sgl_prp] = STATE(182),
    [sym__br_blk_seq] = STATE(455),
    [sym__b_blk_seq_spc] = STATE(432),
    [sym__br_blk_seq_itm] = STATE(960),
    [sym__b_blk_seq_itm] = STATE(978),
    [sym__br_blk_map] = STATE(456),
    [sym__br_blk_map_itm] = STATE(159),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__r_blk_str] = STATE(457),
    [sym__br_blk_str] = STATE(457),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(458),
    [sym__br_flw_seq] = STATE(458),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(459),
    [sym__br_flw_map] = STATE(459),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(460),
    [sym__br_dqt_str] = STATE(460),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(461),
    [sym__br_sqt_str] = STATE(461),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(462),
    [sym__br_sgl_pln_blk] = STATE(342),
    [sym__r_mtl_pln_blk] = STATE(462),
    [sym__br_mtl_pln_blk] = STATE(462),
    [sym__br_als_val] = STATE(306),
    [sym__br_als] = STATE(385),
    [sym__br_acr] = STATE(218),
    [aux_sym__r_blk_seq_repeat1] = STATE(978),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__b_blk_seq_bgn] = ACTIONS(69),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__b_blk_key_bgn] = ACTIONS(365),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__b_blk_val_bgn] = ACTIONS(365),
    [sym__r_blk_lit_bgn] = ACTIONS(73),
    [sym__br_blk_lit_bgn] = ACTIONS(75),
    [sym__r_blk_fld_bgn] = ACTIONS(73),
    [sym__br_blk_fld_bgn] = ACTIONS(75),
    [sym__r_flw_seq_bgn] = ACTIONS(167),
    [sym__br_flw_seq_bgn] = ACTIONS(79),
    [sym__b_flw_seq_bgn] = ACTIONS(365),
    [sym__r_flw_map_bgn] = ACTIONS(169),
    [sym__br_flw_map_bgn] = ACTIONS(83),
    [sym__b_flw_map_bgn] = ACTIONS(365),
    [sym__r_dqt_str_bgn] = ACTIONS(171),
    [sym__br_dqt_str_bgn] = ACTIONS(87),
    [sym__b_dqt_str_bgn] = ACTIONS(365),
    [sym__r_sqt_str_bgn] = ACTIONS(173),
    [sym__br_sqt_str_bgn] = ACTIONS(91),
    [sym__b_sqt_str_bgn] = ACTIONS(365),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(175),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(95),
    [sym__b_sgl_pln_nul_blk] = ACTIONS(365),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(175),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(95),
    [sym__b_sgl_pln_bol_blk] = ACTIONS(365),
    [sym__r_sgl_pln_int_blk] = ACTIONS(175),
    [sym__br_sgl_pln_int_blk] = ACTIONS(95),
    [sym__b_sgl_pln_int_blk] = ACTIONS(365),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(175),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(95),
    [sym__b_sgl_pln_flt_blk] = ACTIONS(365),
    [sym__r_sgl_pln_str_blk] = ACTIONS(175),
    [sym__br_sgl_pln_str_blk] = ACTIONS(95),
    [sym__b_sgl_pln_str_blk] = ACTIONS(365),
    [sym__r_mtl_pln_str_blk] = ACTIONS(97),
    [sym__br_mtl_pln_str_blk] = ACTIONS(99),
    [sym__br_tag] = ACTIONS(367),
    [sym__b_tag] = ACTIONS(365),
    [sym__br_acr_bgn] = ACTIONS(369),
    [sym__b_acr_bgn] = ACTIONS(365),
    [sym__br_als_bgn] = ACTIONS(111),
    [sym__b_als_bgn] = ACTIONS(365),
    [sym__bl] = ACTIONS(365),
    [sym_comment] = ACTIONS(3),
  },
  [36] = {
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__br_sgl_prp] = STATE(182),
    [sym__br_blk_seq] = STATE(464),
    [sym__b_blk_seq_spc] = STATE(433),
    [sym__br_blk_seq_itm] = STATE(960),
    [sym__b_blk_seq_itm] = STATE(978),
    [sym__br_blk_map] = STATE(465),
    [sym__br_blk_map_itm] = STATE(159),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__r_blk_str] = STATE(466),
    [sym__br_blk_str] = STATE(466),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(467),
    [sym__br_flw_seq] = STATE(467),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(468),
    [sym__br_flw_map] = STATE(468),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(469),
    [sym__br_dqt_str] = STATE(469),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(470),
    [sym__br_sqt_str] = STATE(470),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(471),
    [sym__br_sgl_pln_blk] = STATE(347),
    [sym__r_mtl_pln_blk] = STATE(471),
    [sym__br_mtl_pln_blk] = STATE(471),
    [sym__br_als_val] = STATE(306),
    [sym__br_als] = STATE(385),
    [sym__br_acr] = STATE(218),
    [aux_sym__r_blk_seq_repeat1] = STATE(978),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__b_blk_seq_bgn] = ACTIONS(69),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__b_blk_key_bgn] = ACTIONS(371),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__b_blk_val_bgn] = ACTIONS(371),
    [sym__r_blk_lit_bgn] = ACTIONS(73),
    [sym__br_blk_lit_bgn] = ACTIONS(75),
    [sym__r_blk_fld_bgn] = ACTIONS(73),
    [sym__br_blk_fld_bgn] = ACTIONS(75),
    [sym__r_flw_seq_bgn] = ACTIONS(167),
    [sym__br_flw_seq_bgn] = ACTIONS(79),
    [sym__b_flw_seq_bgn] = ACTIONS(371),
    [sym__r_flw_map_bgn] = ACTIONS(169),
    [sym__br_flw_map_bgn] = ACTIONS(83),
    [sym__b_flw_map_bgn] = ACTIONS(371),
    [sym__r_dqt_str_bgn] = ACTIONS(171),
    [sym__br_dqt_str_bgn] = ACTIONS(87),
    [sym__b_dqt_str_bgn] = ACTIONS(371),
    [sym__r_sqt_str_bgn] = ACTIONS(173),
    [sym__br_sqt_str_bgn] = ACTIONS(91),
    [sym__b_sqt_str_bgn] = ACTIONS(371),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(175),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(95),
    [sym__b_sgl_pln_nul_blk] = ACTIONS(371),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(175),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(95),
    [sym__b_sgl_pln_bol_blk] = ACTIONS(371),
    [sym__r_sgl_pln_int_blk] = ACTIONS(175),
    [sym__br_sgl_pln_int_blk] = ACTIONS(95),
    [sym__b_sgl_pln_int_blk] = ACTIONS(371),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(175),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(95),
    [sym__b_sgl_pln_flt_blk] = ACTIONS(371),
    [sym__r_sgl_pln_str_blk] = ACTIONS(175),
    [sym__br_sgl_pln_str_blk] = ACTIONS(95),
    [sym__b_sgl_pln_str_blk] = ACTIONS(371),
    [sym__r_mtl_pln_str_blk] = ACTIONS(97),
    [sym__br_mtl_pln_str_blk] = ACTIONS(99),
    [sym__br_tag] = ACTIONS(367),
    [sym__b_tag] = ACTIONS(371),
    [sym__br_acr_bgn] = ACTIONS(369),
    [sym__b_acr_bgn] = ACTIONS(371),
    [sym__br_als_bgn] = ACTIONS(111),
    [sym__b_als_bgn] = ACTIONS(371),
    [sym__bl] = ACTIONS(371),
    [sym_comment] = ACTIONS(3),
  },
  [37] = {
    [sym__doc_w_bgn_w_end_seq] = STATE(1459),
    [sym__doc_w_bgn_wo_end_seq] = STATE(1459),
    [sym__doc_wo_bgn_w_end_seq] = STATE(1459),
    [sym__doc_wo_bgn_wo_end_seq] = STATE(1459),
    [sym__doc_w_bgn_w_end] = STATE(38),
    [sym__doc_w_bgn_wo_end] = STATE(566),
    [sym__doc_wo_bgn_w_end] = STATE(37),
    [sym__doc_wo_bgn_wo_end] = STATE(565),
    [sym__drs_doc] = STATE(1231),
    [sym__exp_doc] = STATE(1237),
    [sym__imp_doc] = STATE(1230),
    [sym__drs_doc_end] = STATE(37),
    [sym__exp_doc_end] = STATE(38),
    [sym__imp_doc_end] = STATE(37),
    [sym__doc_end] = STATE(38),
    [sym__s_dir] = STATE(568),
    [sym__s_dir_yml] = STATE(568),
    [sym__s_dir_tag] = STATE(568),
    [sym__s_dir_rsv] = STATE(568),
    [sym__br_prp_val] = STATE(1222),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__br_prp] = STATE(81),
    [sym__br_sgl_prp] = STATE(182),
    [sym__br_blk_seq_val] = STATE(1222),
    [sym__br_blk_seq] = STATE(1116),
    [sym__br_blk_seq_itm] = STATE(977),
    [sym__br_blk_map_val] = STATE(1222),
    [sym__br_blk_map] = STATE(1178),
    [sym__br_blk_map_itm] = STATE(132),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__br_blk_str_val] = STATE(1222),
    [sym__br_blk_str] = STATE(1207),
    [sym__br_flw_seq_val] = STATE(1222),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__br_flw_seq] = STATE(1218),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__br_flw_map_val] = STATE(1222),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__br_flw_map] = STATE(1225),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__br_dqt_str_val] = STATE(1222),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__br_dqt_str] = STATE(1229),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__br_sqt_str_val] = STATE(1222),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__br_sqt_str] = STATE(1235),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__br_pln_blk_val] = STATE(1222),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__br_sgl_pln_blk] = STATE(292),
    [sym__br_mtl_pln_blk] = STATE(1179),
    [sym__br_als_val] = STATE(270),
    [sym__br_als] = STATE(279),
    [sym__br_acr] = STATE(99),
    [aux_sym__drs_doc_repeat1] = STATE(568),
    [sym__eof] = ACTIONS(373),
    [sym__s_dir_yml_bgn] = ACTIONS(7),
    [sym__s_dir_tag_bgn] = ACTIONS(9),
    [sym__s_dir_rsv_bgn] = ACTIONS(11),
    [sym__s_drs_end] = ACTIONS(13),
    [sym__s_doc_end] = ACTIONS(15),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__br_blk_lit_bgn] = ACTIONS(31),
    [sym__br_blk_fld_bgn] = ACTIONS(31),
    [sym__br_flw_seq_bgn] = ACTIONS(35),
    [sym__br_flw_map_bgn] = ACTIONS(39),
    [sym__br_dqt_str_bgn] = ACTIONS(43),
    [sym__br_sqt_str_bgn] = ACTIONS(47),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(51),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(51),
    [sym__br_sgl_pln_int_blk] = ACTIONS(51),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(51),
    [sym__br_sgl_pln_str_blk] = ACTIONS(51),
    [sym__br_mtl_pln_str_blk] = ACTIONS(55),
    [sym__br_tag] = ACTIONS(59),
    [sym__br_acr_bgn] = ACTIONS(63),
    [sym__br_als_bgn] = ACTIONS(67),
    [sym_comment] = ACTIONS(3),
  },
  [38] = {
    [sym__doc_w_bgn_w_end_seq] = STATE(1527),
    [sym__doc_w_bgn_wo_end_seq] = STATE(1527),
    [sym__doc_wo_bgn_w_end_seq] = STATE(1527),
    [sym__doc_wo_bgn_wo_end_seq] = STATE(1527),
    [sym__doc_w_bgn_w_end] = STATE(38),
    [sym__doc_w_bgn_wo_end] = STATE(566),
    [sym__doc_wo_bgn_w_end] = STATE(37),
    [sym__doc_wo_bgn_wo_end] = STATE(565),
    [sym__drs_doc] = STATE(1231),
    [sym__exp_doc] = STATE(1237),
    [sym__imp_doc] = STATE(1230),
    [sym__drs_doc_end] = STATE(37),
    [sym__exp_doc_end] = STATE(38),
    [sym__imp_doc_end] = STATE(37),
    [sym__doc_end] = STATE(38),
    [sym__s_dir] = STATE(568),
    [sym__s_dir_yml] = STATE(568),
    [sym__s_dir_tag] = STATE(568),
    [sym__s_dir_rsv] = STATE(568),
    [sym__br_prp_val] = STATE(1222),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__br_prp] = STATE(81),
    [sym__br_sgl_prp] = STATE(182),
    [sym__br_blk_seq_val] = STATE(1222),
    [sym__br_blk_seq] = STATE(1116),
    [sym__br_blk_seq_itm] = STATE(977),
    [sym__br_blk_map_val] = STATE(1222),
    [sym__br_blk_map] = STATE(1178),
    [sym__br_blk_map_itm] = STATE(132),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__br_blk_str_val] = STATE(1222),
    [sym__br_blk_str] = STATE(1207),
    [sym__br_flw_seq_val] = STATE(1222),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__br_flw_seq] = STATE(1218),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__br_flw_map_val] = STATE(1222),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__br_flw_map] = STATE(1225),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__br_dqt_str_val] = STATE(1222),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__br_dqt_str] = STATE(1229),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__br_sqt_str_val] = STATE(1222),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__br_sqt_str] = STATE(1235),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__br_pln_blk_val] = STATE(1222),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__br_sgl_pln_blk] = STATE(292),
    [sym__br_mtl_pln_blk] = STATE(1179),
    [sym__br_als_val] = STATE(270),
    [sym__br_als] = STATE(279),
    [sym__br_acr] = STATE(99),
    [aux_sym__drs_doc_repeat1] = STATE(568),
    [sym__eof] = ACTIONS(375),
    [sym__s_dir_yml_bgn] = ACTIONS(7),
    [sym__s_dir_tag_bgn] = ACTIONS(9),
    [sym__s_dir_rsv_bgn] = ACTIONS(11),
    [sym__s_drs_end] = ACTIONS(13),
    [sym__s_doc_end] = ACTIONS(15),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__br_blk_lit_bgn] = ACTIONS(31),
    [sym__br_blk_fld_bgn] = ACTIONS(31),
    [sym__br_flw_seq_bgn] = ACTIONS(35),
    [sym__br_flw_map_bgn] = ACTIONS(39),
    [sym__br_dqt_str_bgn] = ACTIONS(43),
    [sym__br_sqt_str_bgn] = ACTIONS(47),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(51),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(51),
    [sym__br_sgl_pln_int_blk] = ACTIONS(51),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(51),
    [sym__br_sgl_pln_str_blk] = ACTIONS(51),
    [sym__br_mtl_pln_str_blk] = ACTIONS(55),
    [sym__br_tag] = ACTIONS(59),
    [sym__br_acr_bgn] = ACTIONS(63),
    [sym__br_als_bgn] = ACTIONS(67),
    [sym_comment] = ACTIONS(3),
  },
  [39] = {
    [sym__doc_w_bgn_w_end_seq] = STATE(1504),
    [sym__doc_w_bgn_wo_end_seq] = STATE(1504),
    [sym__doc_wo_bgn_w_end_seq] = STATE(1504),
    [sym__doc_wo_bgn_wo_end_seq] = STATE(1504),
    [sym__doc_w_bgn_w_end] = STATE(38),
    [sym__doc_w_bgn_wo_end] = STATE(566),
    [sym__doc_wo_bgn_w_end] = STATE(37),
    [sym__doc_wo_bgn_wo_end] = STATE(565),
    [sym__drs_doc] = STATE(1231),
    [sym__exp_doc] = STATE(1237),
    [sym__imp_doc] = STATE(1230),
    [sym__drs_doc_end] = STATE(37),
    [sym__exp_doc_end] = STATE(38),
    [sym__imp_doc_end] = STATE(37),
    [sym__doc_end] = STATE(38),
    [sym__s_dir] = STATE(568),
    [sym__s_dir_yml] = STATE(568),
    [sym__s_dir_tag] = STATE(568),
    [sym__s_dir_rsv] = STATE(568),
    [sym__br_prp_val] = STATE(1222),
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__br_prp] = STATE(81),
    [sym__br_sgl_prp] = STATE(182),
    [sym__br_blk_seq_val] = STATE(1222),
    [sym__br_blk_seq] = STATE(1116),
    [sym__br_blk_seq_itm] = STATE(977),
    [sym__br_blk_map_val] = STATE(1222),
    [sym__br_blk_map] = STATE(1178),
    [sym__br_blk_map_itm] = STATE(132),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__br_blk_str_val] = STATE(1222),
    [sym__br_blk_str] = STATE(1207),
    [sym__br_flw_seq_val] = STATE(1222),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__br_flw_seq] = STATE(1218),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__br_flw_map_val] = STATE(1222),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__br_flw_map] = STATE(1225),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__br_dqt_str_val] = STATE(1222),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__br_dqt_str] = STATE(1229),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__br_sqt_str_val] = STATE(1222),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__br_sqt_str] = STATE(1235),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__br_pln_blk_val] = STATE(1222),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__br_sgl_pln_blk] = STATE(292),
    [sym__br_mtl_pln_blk] = STATE(1179),
    [sym__br_als_val] = STATE(270),
    [sym__br_als] = STATE(279),
    [sym__br_acr] = STATE(99),
    [aux_sym__drs_doc_repeat1] = STATE(568),
    [sym__eof] = ACTIONS(377),
    [sym__s_dir_yml_bgn] = ACTIONS(7),
    [sym__s_dir_tag_bgn] = ACTIONS(9),
    [sym__s_dir_rsv_bgn] = ACTIONS(11),
    [sym__s_drs_end] = ACTIONS(13),
    [sym__s_doc_end] = ACTIONS(15),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__br_blk_lit_bgn] = ACTIONS(31),
    [sym__br_blk_fld_bgn] = ACTIONS(31),
    [sym__br_flw_seq_bgn] = ACTIONS(35),
    [sym__br_flw_map_bgn] = ACTIONS(39),
    [sym__br_dqt_str_bgn] = ACTIONS(43),
    [sym__br_sqt_str_bgn] = ACTIONS(47),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(51),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(51),
    [sym__br_sgl_pln_int_blk] = ACTIONS(51),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(51),
    [sym__br_sgl_pln_str_blk] = ACTIONS(51),
    [sym__br_mtl_pln_str_blk] = ACTIONS(55),
    [sym__br_tag] = ACTIONS(59),
    [sym__br_acr_bgn] = ACTIONS(63),
    [sym__br_als_bgn] = ACTIONS(67),
    [sym_comment] = ACTIONS(3),
  },
  [40] = {
    [sym__r_prp_val] = STATE(1005),
    [sym__br_prp_val] = STATE(1004),
    [sym__r_sgl_prp_val] = STATE(1446),
    [sym__r_prp] = STATE(164),
    [sym__br_prp] = STATE(163),
    [sym__r_sgl_prp] = STATE(444),
    [sym__r_flw_seq_val] = STATE(1003),
    [sym__br_flw_seq_val] = STATE(1000),
    [sym__r_sgl_flw_seq_val] = STATE(1449),
    [sym__r_flw_seq] = STATE(1314),
    [sym__br_flw_seq] = STATE(1315),
    [sym__r_sgl_flw_seq] = STATE(1530),
    [sym__r_flw_map_val] = STATE(1003),
    [sym__br_flw_map_val] = STATE(1000),
    [sym__r_sgl_flw_map_val] = STATE(1449),
    [sym__r_flw_map] = STATE(1317),
    [sym__br_flw_map] = STATE(1318),
    [sym__r_sgl_flw_map] = STATE(1553),
    [sym__r_flw_imp_r_par] = STATE(995),
    [sym__r_flw_imp_br_par] = STATE(994),
    [sym__br_flw_imp_r_par] = STATE(992),
    [sym__br_flw_imp_br_par] = STATE(986),
    [sym__r_sgl_flw_imp_par] = STATE(1430),
    [sym__r_dqt_str_val] = STATE(1003),
    [sym__br_dqt_str_val] = STATE(1000),
    [sym__r_sgl_dqt_str_val] = STATE(1449),
    [sym__r_dqt_str] = STATE(1320),
    [sym__br_dqt_str] = STATE(1321),
    [sym__r_sgl_dqt_str] = STATE(1513),
    [sym__r_sqt_str_val] = STATE(1003),
    [sym__br_sqt_str_val] = STATE(1000),
    [sym__r_sgl_sqt_str_val] = STATE(1449),
    [sym__r_sqt_str] = STATE(1323),
    [sym__br_sqt_str] = STATE(1324),
    [sym__r_sgl_sqt_str] = STATE(1514),
    [sym__r_pln_flw_val] = STATE(1005),
    [sym__br_pln_flw_val] = STATE(1004),
    [sym__r_sgl_pln_flw_val] = STATE(1446),
    [sym__r_sgl_pln_flw] = STATE(1330),
    [sym__br_sgl_pln_flw] = STATE(1329),
    [sym__r_mtl_pln_flw] = STATE(1328),
    [sym__br_mtl_pln_flw] = STATE(1329),
    [sym__r_als_val] = STATE(867),
    [sym__br_als_val] = STATE(1004),
    [sym__r_als] = STATE(1269),
    [sym__br_als] = STATE(1326),
    [sym__r_acr] = STATE(251),
    [sym__br_acr] = STATE(246),
    [sym__r_flw_seq_bgn] = ACTIONS(379),
    [sym__br_flw_seq_bgn] = ACTIONS(381),
    [sym__r_flw_seq_end] = ACTIONS(383),
    [sym__br_flw_seq_end] = ACTIONS(386),
    [sym__r_flw_map_bgn] = ACTIONS(388),
    [sym__br_flw_map_bgn] = ACTIONS(390),
    [sym__r_flw_sep_bgn] = ACTIONS(383),
    [sym__br_flw_sep_bgn] = ACTIONS(386),
    [sym__r_dqt_str_bgn] = ACTIONS(392),
    [sym__br_dqt_str_bgn] = ACTIONS(394),
    [sym__r_sqt_str_bgn] = ACTIONS(396),
    [sym__br_sqt_str_bgn] = ACTIONS(398),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(400),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(402),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(400),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(402),
    [sym__r_sgl_pln_int_flw] = ACTIONS(400),
    [sym__br_sgl_pln_int_flw] = ACTIONS(402),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(400),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(402),
    [sym__r_sgl_pln_str_flw] = ACTIONS(400),
    [sym__br_sgl_pln_str_flw] = ACTIONS(402),
    [sym__r_mtl_pln_str_flw] = ACTIONS(404),
    [sym__br_mtl_pln_str_flw] = ACTIONS(406),
    [sym__r_tag] = ACTIONS(408),
    [sym__br_tag] = ACTIONS(410),
    [sym__r_acr_bgn] = ACTIONS(412),
    [sym__br_acr_bgn] = ACTIONS(414),
    [sym__r_als_bgn] = ACTIONS(416),
    [sym__br_als_bgn] = ACTIONS(418),
    [sym_comment] = ACTIONS(3),
  },
  [41] = {
    [sym__r_prp_val] = STATE(943),
    [sym__br_prp_val] = STATE(942),
    [sym__r_sgl_prp_val] = STATE(1257),
    [sym__r_prp] = STATE(164),
    [sym__br_prp] = STATE(163),
    [sym__r_sgl_prp] = STATE(444),
    [sym__r_flw_seq_val] = STATE(898),
    [sym__br_flw_seq_val] = STATE(901),
    [sym__r_sgl_flw_seq_val] = STATE(1260),
    [sym__r_flw_seq] = STATE(1314),
    [sym__br_flw_seq] = STATE(1315),
    [sym__r_sgl_flw_seq] = STATE(1530),
    [sym__r_flw_map_val] = STATE(898),
    [sym__br_flw_map_val] = STATE(901),
    [sym__r_sgl_flw_map_val] = STATE(1260),
    [sym__r_flw_map] = STATE(1317),
    [sym__br_flw_map] = STATE(1318),
    [sym__r_sgl_flw_map] = STATE(1553),
    [sym__r_flw_imp_r_par] = STATE(941),
    [sym__r_flw_imp_br_par] = STATE(940),
    [sym__br_flw_imp_r_par] = STATE(938),
    [sym__br_flw_imp_br_par] = STATE(937),
    [sym__r_sgl_flw_imp_par] = STATE(1397),
    [sym__r_dqt_str_val] = STATE(898),
    [sym__br_dqt_str_val] = STATE(901),
    [sym__r_sgl_dqt_str_val] = STATE(1260),
    [sym__r_dqt_str] = STATE(1320),
    [sym__br_dqt_str] = STATE(1321),
    [sym__r_sgl_dqt_str] = STATE(1513),
    [sym__r_sqt_str_val] = STATE(898),
    [sym__br_sqt_str_val] = STATE(901),
    [sym__r_sgl_sqt_str_val] = STATE(1260),
    [sym__r_sqt_str] = STATE(1323),
    [sym__br_sqt_str] = STATE(1324),
    [sym__r_sgl_sqt_str] = STATE(1514),
    [sym__r_pln_flw_val] = STATE(943),
    [sym__br_pln_flw_val] = STATE(942),
    [sym__r_sgl_pln_flw_val] = STATE(1257),
    [sym__r_sgl_pln_flw] = STATE(1330),
    [sym__br_sgl_pln_flw] = STATE(1329),
    [sym__r_mtl_pln_flw] = STATE(1328),
    [sym__br_mtl_pln_flw] = STATE(1329),
    [sym__r_als_val] = STATE(845),
    [sym__br_als_val] = STATE(942),
    [sym__r_als] = STATE(1269),
    [sym__br_als] = STATE(1326),
    [sym__r_acr] = STATE(251),
    [sym__br_acr] = STATE(246),
    [sym__r_flw_seq_bgn] = ACTIONS(379),
    [sym__br_flw_seq_bgn] = ACTIONS(381),
    [sym__r_flw_map_bgn] = ACTIONS(388),
    [sym__br_flw_map_bgn] = ACTIONS(390),
    [sym__r_flw_map_end] = ACTIONS(383),
    [sym__br_flw_map_end] = ACTIONS(386),
    [sym__r_flw_sep_bgn] = ACTIONS(383),
    [sym__br_flw_sep_bgn] = ACTIONS(386),
    [sym__r_dqt_str_bgn] = ACTIONS(392),
    [sym__br_dqt_str_bgn] = ACTIONS(394),
    [sym__r_sqt_str_bgn] = ACTIONS(396),
    [sym__br_sqt_str_bgn] = ACTIONS(398),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(400),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(402),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(400),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(402),
    [sym__r_sgl_pln_int_flw] = ACTIONS(400),
    [sym__br_sgl_pln_int_flw] = ACTIONS(402),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(400),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(402),
    [sym__r_sgl_pln_str_flw] = ACTIONS(400),
    [sym__br_sgl_pln_str_flw] = ACTIONS(402),
    [sym__r_mtl_pln_str_flw] = ACTIONS(404),
    [sym__br_mtl_pln_str_flw] = ACTIONS(406),
    [sym__r_tag] = ACTIONS(408),
    [sym__br_tag] = ACTIONS(410),
    [sym__r_acr_bgn] = ACTIONS(412),
    [sym__br_acr_bgn] = ACTIONS(414),
    [sym__r_als_bgn] = ACTIONS(416),
    [sym__br_als_bgn] = ACTIONS(418),
    [sym_comment] = ACTIONS(3),
  },
  [42] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__flw_map_tal] = STATE(831),
    [sym__r_flw_map_dat] = STATE(1355),
    [sym__br_flw_map_dat] = STATE(1355),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(259),
    [sym__br_flw_map_end] = ACTIONS(259),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [43] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__flw_map_tal] = STATE(1188),
    [sym__r_flw_map_dat] = STATE(1327),
    [sym__br_flw_map_dat] = STATE(1327),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(267),
    [sym__br_flw_map_end] = ACTIONS(267),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [44] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__flw_map_tal] = STATE(1103),
    [sym__r_flw_map_dat] = STATE(1267),
    [sym__br_flw_map_dat] = STATE(1267),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(251),
    [sym__br_flw_map_end] = ACTIONS(251),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [45] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__flw_map_tal] = STATE(447),
    [sym__r_flw_map_dat] = STATE(1303),
    [sym__br_flw_map_dat] = STATE(1303),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(275),
    [sym__br_flw_map_end] = ACTIONS(275),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [46] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__flw_map_tal] = STATE(833),
    [sym__r_flw_map_dat] = STATE(1355),
    [sym__br_flw_map_dat] = STATE(1355),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(434),
    [sym__br_flw_map_end] = ACTIONS(434),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [47] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__flw_map_tal] = STATE(1104),
    [sym__r_flw_map_dat] = STATE(1267),
    [sym__br_flw_map_dat] = STATE(1267),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(436),
    [sym__br_flw_map_end] = ACTIONS(436),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [48] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__flw_map_tal] = STATE(1053),
    [sym__r_flw_map_dat] = STATE(1342),
    [sym__br_flw_map_dat] = STATE(1342),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(211),
    [sym__br_flw_map_end] = ACTIONS(211),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [49] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__flw_map_tal] = STATE(809),
    [sym__r_flw_map_dat] = STATE(1401),
    [sym__br_flw_map_dat] = STATE(1401),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(263),
    [sym__br_flw_map_end] = ACTIONS(263),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [50] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__flw_map_tal] = STATE(1360),
    [sym__r_flw_map_dat] = STATE(1443),
    [sym__br_flw_map_dat] = STATE(1443),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(438),
    [sym__br_flw_map_end] = ACTIONS(438),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [51] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__flw_map_tal] = STATE(1358),
    [sym__r_flw_map_dat] = STATE(1443),
    [sym__br_flw_map_dat] = STATE(1443),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(281),
    [sym__br_flw_map_end] = ACTIONS(281),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [52] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__flw_map_tal] = STATE(1054),
    [sym__r_flw_map_dat] = STATE(1342),
    [sym__br_flw_map_dat] = STATE(1342),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(440),
    [sym__br_flw_map_end] = ACTIONS(440),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [53] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__flw_map_tal] = STATE(807),
    [sym__r_flw_map_dat] = STATE(1401),
    [sym__br_flw_map_dat] = STATE(1401),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(442),
    [sym__br_flw_map_end] = ACTIONS(442),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [54] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__flw_map_tal] = STATE(1442),
    [sym__r_flw_map_dat] = STATE(1278),
    [sym__br_flw_map_dat] = STATE(1278),
    [sym__r_flw_map_itm] = STATE(793),
    [sym__br_flw_map_itm] = STATE(795),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(255),
    [sym__br_flw_map_end] = ACTIONS(255),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [55] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__flw_seq_tal] = STATE(1050),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_dat] = STATE(1343),
    [sym__br_flw_seq_dat] = STATE(1343),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(289),
    [sym__br_flw_seq_end] = ACTIONS(289),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [56] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__flw_seq_tal] = STATE(823),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_dat] = STATE(1353),
    [sym__br_flw_seq_dat] = STATE(1353),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(357),
    [sym__br_flw_seq_end] = ACTIONS(357),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [57] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__flw_seq_tal] = STATE(1433),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_dat] = STATE(1281),
    [sym__br_flw_seq_dat] = STATE(1281),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(333),
    [sym__br_flw_seq_end] = ACTIONS(333),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [58] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__flw_seq_tal] = STATE(813),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_dat] = STATE(1399),
    [sym__br_flw_seq_dat] = STATE(1399),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(458),
    [sym__br_flw_seq_end] = ACTIONS(458),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [59] = {
    [sym__r_prp_val] = STATE(968),
    [sym__br_prp_val] = STATE(968),
    [sym__r_sgl_prp_val] = STATE(1429),
    [sym__r_prp] = STATE(144),
    [sym__br_prp] = STATE(147),
    [sym__r_sgl_prp] = STATE(369),
    [sym__r_flw_seq_val] = STATE(968),
    [sym__br_flw_seq_val] = STATE(968),
    [sym__r_sgl_flw_seq_val] = STATE(1429),
    [sym__r_flw_seq] = STATE(1038),
    [sym__br_flw_seq] = STATE(1039),
    [sym__r_sgl_flw_seq] = STATE(1273),
    [sym__r_flw_map_val] = STATE(968),
    [sym__br_flw_map_val] = STATE(968),
    [sym__r_sgl_flw_map_val] = STATE(1429),
    [sym__r_flw_map] = STATE(1040),
    [sym__br_flw_map] = STATE(1041),
    [sym__r_sgl_flw_map] = STATE(1274),
    [sym__flw_ann_par_tal] = STATE(968),
    [sym__sgl_flw_ann_par_tal] = STATE(1429),
    [sym__r_dqt_str_val] = STATE(968),
    [sym__br_dqt_str_val] = STATE(968),
    [sym__r_sgl_dqt_str_val] = STATE(1429),
    [sym__r_dqt_str] = STATE(1042),
    [sym__br_dqt_str] = STATE(1043),
    [sym__r_sgl_dqt_str] = STATE(1275),
    [sym__r_sqt_str_val] = STATE(968),
    [sym__br_sqt_str_val] = STATE(968),
    [sym__r_sgl_sqt_str_val] = STATE(1429),
    [sym__r_sqt_str] = STATE(1044),
    [sym__br_sqt_str] = STATE(1045),
    [sym__r_sgl_sqt_str] = STATE(1264),
    [sym__r_pln_flw_val] = STATE(968),
    [sym__br_pln_flw_val] = STATE(968),
    [sym__r_sgl_pln_flw_val] = STATE(1429),
    [sym__r_sgl_pln_flw] = STATE(915),
    [sym__br_sgl_pln_flw] = STATE(914),
    [sym__r_mtl_pln_flw] = STATE(905),
    [sym__br_mtl_pln_flw] = STATE(914),
    [sym__r_als_val] = STATE(961),
    [sym__br_als_val] = STATE(968),
    [sym__r_als] = STATE(1096),
    [sym__br_als] = STATE(1046),
    [sym__r_acr] = STATE(228),
    [sym__br_acr] = STATE(226),
    [sym__r_flw_seq_bgn] = ACTIONS(460),
    [sym__br_flw_seq_bgn] = ACTIONS(462),
    [sym__r_flw_seq_end] = ACTIONS(464),
    [sym__br_flw_seq_end] = ACTIONS(467),
    [sym__r_flw_map_bgn] = ACTIONS(469),
    [sym__br_flw_map_bgn] = ACTIONS(471),
    [sym__r_flw_sep_bgn] = ACTIONS(464),
    [sym__br_flw_sep_bgn] = ACTIONS(467),
    [sym__r_dqt_str_bgn] = ACTIONS(473),
    [sym__br_dqt_str_bgn] = ACTIONS(475),
    [sym__r_sqt_str_bgn] = ACTIONS(477),
    [sym__br_sqt_str_bgn] = ACTIONS(479),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(481),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(483),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(481),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(483),
    [sym__r_sgl_pln_int_flw] = ACTIONS(481),
    [sym__br_sgl_pln_int_flw] = ACTIONS(483),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(481),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(483),
    [sym__r_sgl_pln_str_flw] = ACTIONS(481),
    [sym__br_sgl_pln_str_flw] = ACTIONS(483),
    [sym__r_mtl_pln_str_flw] = ACTIONS(485),
    [sym__br_mtl_pln_str_flw] = ACTIONS(487),
    [sym__r_tag] = ACTIONS(489),
    [sym__br_tag] = ACTIONS(491),
    [sym__r_acr_bgn] = ACTIONS(493),
    [sym__br_acr_bgn] = ACTIONS(495),
    [sym__r_als_bgn] = ACTIONS(497),
    [sym__br_als_bgn] = ACTIONS(499),
    [sym_comment] = ACTIONS(3),
  },
  [60] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__flw_seq_tal] = STATE(1163),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_dat] = STATE(1384),
    [sym__br_flw_seq_dat] = STATE(1384),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(341),
    [sym__br_flw_seq_end] = ACTIONS(341),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [61] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__flw_seq_tal] = STATE(1345),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_dat] = STATE(1441),
    [sym__br_flw_seq_dat] = STATE(1441),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(345),
    [sym__br_flw_seq_end] = ACTIONS(345),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [62] = {
    [sym__r_prp_val] = STATE(926),
    [sym__br_prp_val] = STATE(926),
    [sym__r_sgl_prp_val] = STATE(1396),
    [sym__r_prp] = STATE(160),
    [sym__br_prp] = STATE(152),
    [sym__r_sgl_prp] = STATE(345),
    [sym__r_flw_seq_val] = STATE(926),
    [sym__br_flw_seq_val] = STATE(926),
    [sym__r_sgl_flw_seq_val] = STATE(1396),
    [sym__r_flw_seq] = STATE(1088),
    [sym__br_flw_seq] = STATE(1089),
    [sym__r_sgl_flw_seq] = STATE(1316),
    [sym__r_flw_map_val] = STATE(926),
    [sym__br_flw_map_val] = STATE(926),
    [sym__r_sgl_flw_map_val] = STATE(1396),
    [sym__r_flw_map] = STATE(1090),
    [sym__br_flw_map] = STATE(1091),
    [sym__r_sgl_flw_map] = STATE(1319),
    [sym__flw_ann_par_tal] = STATE(926),
    [sym__sgl_flw_ann_par_tal] = STATE(1396),
    [sym__r_dqt_str_val] = STATE(926),
    [sym__br_dqt_str_val] = STATE(926),
    [sym__r_sgl_dqt_str_val] = STATE(1396),
    [sym__r_dqt_str] = STATE(1092),
    [sym__br_dqt_str] = STATE(1093),
    [sym__r_sgl_dqt_str] = STATE(1322),
    [sym__r_sqt_str_val] = STATE(926),
    [sym__br_sqt_str_val] = STATE(926),
    [sym__r_sgl_sqt_str_val] = STATE(1396),
    [sym__r_sqt_str] = STATE(1094),
    [sym__br_sqt_str] = STATE(1095),
    [sym__r_sgl_sqt_str] = STATE(1325),
    [sym__r_pln_flw_val] = STATE(926),
    [sym__br_pln_flw_val] = STATE(926),
    [sym__r_sgl_pln_flw_val] = STATE(1396),
    [sym__r_sgl_pln_flw] = STATE(916),
    [sym__br_sgl_pln_flw] = STATE(917),
    [sym__r_mtl_pln_flw] = STATE(918),
    [sym__br_mtl_pln_flw] = STATE(917),
    [sym__r_als_val] = STATE(921),
    [sym__br_als_val] = STATE(926),
    [sym__r_als] = STATE(1009),
    [sym__br_als] = STATE(1097),
    [sym__r_acr] = STATE(224),
    [sym__br_acr] = STATE(234),
    [sym__r_flw_seq_bgn] = ACTIONS(501),
    [sym__br_flw_seq_bgn] = ACTIONS(503),
    [sym__r_flw_map_bgn] = ACTIONS(505),
    [sym__br_flw_map_bgn] = ACTIONS(507),
    [sym__r_flw_map_end] = ACTIONS(464),
    [sym__br_flw_map_end] = ACTIONS(467),
    [sym__r_flw_sep_bgn] = ACTIONS(464),
    [sym__br_flw_sep_bgn] = ACTIONS(467),
    [sym__r_dqt_str_bgn] = ACTIONS(509),
    [sym__br_dqt_str_bgn] = ACTIONS(511),
    [sym__r_sqt_str_bgn] = ACTIONS(513),
    [sym__br_sqt_str_bgn] = ACTIONS(515),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(517),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(519),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(517),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(519),
    [sym__r_sgl_pln_int_flw] = ACTIONS(517),
    [sym__br_sgl_pln_int_flw] = ACTIONS(519),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(517),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(519),
    [sym__r_sgl_pln_str_flw] = ACTIONS(517),
    [sym__br_sgl_pln_str_flw] = ACTIONS(519),
    [sym__r_mtl_pln_str_flw] = ACTIONS(521),
    [sym__br_mtl_pln_str_flw] = ACTIONS(523),
    [sym__r_tag] = ACTIONS(525),
    [sym__br_tag] = ACTIONS(527),
    [sym__r_acr_bgn] = ACTIONS(529),
    [sym__br_acr_bgn] = ACTIONS(531),
    [sym__r_als_bgn] = ACTIONS(533),
    [sym__br_als_bgn] = ACTIONS(535),
    [sym_comment] = ACTIONS(3),
  },
  [63] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__flw_seq_tal] = STATE(1051),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_dat] = STATE(1343),
    [sym__br_flw_seq_dat] = STATE(1343),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(537),
    [sym__br_flw_seq_end] = ACTIONS(537),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [64] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__flw_seq_tal] = STATE(1349),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_dat] = STATE(1441),
    [sym__br_flw_seq_dat] = STATE(1441),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(539),
    [sym__br_flw_seq_end] = ACTIONS(539),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [65] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__flw_seq_tal] = STATE(445),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_dat] = STATE(1301),
    [sym__br_flw_seq_dat] = STATE(1301),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(349),
    [sym__br_flw_seq_end] = ACTIONS(349),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [66] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__flw_seq_tal] = STATE(1101),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_dat] = STATE(1268),
    [sym__br_flw_seq_dat] = STATE(1268),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(541),
    [sym__br_flw_seq_end] = ACTIONS(541),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [67] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__flw_seq_tal] = STATE(829),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_dat] = STATE(1353),
    [sym__br_flw_seq_dat] = STATE(1353),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(543),
    [sym__br_flw_seq_end] = ACTIONS(543),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [68] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__flw_seq_tal] = STATE(816),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_dat] = STATE(1399),
    [sym__br_flw_seq_dat] = STATE(1399),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(361),
    [sym__br_flw_seq_end] = ACTIONS(361),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [69] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__flw_seq_tal] = STATE(1100),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_dat] = STATE(1268),
    [sym__br_flw_seq_dat] = STATE(1268),
    [sym__r_flw_seq_itm] = STATE(718),
    [sym__br_flw_seq_itm] = STATE(719),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(337),
    [sym__br_flw_seq_end] = ACTIONS(337),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [70] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_flw_map_itm] = STATE(923),
    [sym__br_flw_map_itm] = STATE(923),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(545),
    [sym__br_flw_map_end] = ACTIONS(545),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [71] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_flw_map_itm] = STATE(923),
    [sym__br_flw_map_itm] = STATE(923),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(547),
    [sym__br_flw_map_end] = ACTIONS(547),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [72] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_flw_map_itm] = STATE(923),
    [sym__br_flw_map_itm] = STATE(923),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(549),
    [sym__br_flw_map_end] = ACTIONS(549),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [73] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_flw_map_itm] = STATE(923),
    [sym__br_flw_map_itm] = STATE(923),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_map_end] = ACTIONS(551),
    [sym__br_flw_map_end] = ACTIONS(551),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [74] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_itm] = STATE(964),
    [sym__br_flw_seq_itm] = STATE(964),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(553),
    [sym__br_flw_seq_end] = ACTIONS(553),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [75] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_itm] = STATE(964),
    [sym__br_flw_seq_itm] = STATE(964),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(555),
    [sym__br_flw_seq_end] = ACTIONS(555),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [76] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_itm] = STATE(964),
    [sym__br_flw_seq_itm] = STATE(964),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(557),
    [sym__br_flw_seq_end] = ACTIONS(557),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [77] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_itm] = STATE(964),
    [sym__br_flw_seq_itm] = STATE(964),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_seq_end] = ACTIONS(559),
    [sym__br_flw_seq_end] = ACTIONS(559),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [78] = {
    [sym__r_prp_val] = STATE(629),
    [sym__br_prp_val] = STATE(628),
    [sym__r_prp] = STATE(128),
    [sym__br_prp] = STATE(127),
    [sym__r_flw_seq_val] = STATE(627),
    [sym__br_flw_seq_val] = STATE(626),
    [sym__r_flw_seq] = STATE(721),
    [sym__br_flw_seq] = STATE(720),
    [sym__r_flw_map_val] = STATE(627),
    [sym__br_flw_map_val] = STATE(626),
    [sym__r_flw_map] = STATE(797),
    [sym__br_flw_map] = STATE(755),
    [sym__r_flw_map_itm] = STATE(923),
    [sym__br_flw_map_itm] = STATE(923),
    [sym__r_flw_exp_par] = STATE(933),
    [sym__br_flw_exp_par] = STATE(932),
    [sym__r_flw_imp_r_par] = STATE(931),
    [sym__r_flw_imp_br_par] = STATE(919),
    [sym__br_flw_imp_r_par] = STATE(913),
    [sym__br_flw_imp_br_par] = STATE(912),
    [sym__r_flw_njl_ann_par] = STATE(911),
    [sym__br_flw_njl_ann_par] = STATE(910),
    [sym__r_dqt_str_val] = STATE(627),
    [sym__br_dqt_str_val] = STATE(626),
    [sym__r_dqt_str] = STATE(757),
    [sym__br_dqt_str] = STATE(776),
    [sym__r_sqt_str_val] = STATE(627),
    [sym__br_sqt_str_val] = STATE(626),
    [sym__r_sqt_str] = STATE(792),
    [sym__br_sqt_str] = STATE(794),
    [sym__r_pln_flw_val] = STATE(629),
    [sym__br_pln_flw_val] = STATE(628),
    [sym__r_sgl_pln_flw] = STATE(749),
    [sym__br_sgl_pln_flw] = STATE(750),
    [sym__r_mtl_pln_flw] = STATE(749),
    [sym__br_mtl_pln_flw] = STATE(750),
    [sym__r_als_val] = STATE(629),
    [sym__br_als_val] = STATE(628),
    [sym__r_als] = STATE(746),
    [sym__br_als] = STATE(820),
    [sym__r_acr] = STATE(197),
    [sym__br_acr] = STATE(202),
    [sym__r_flw_seq_bgn] = ACTIONS(420),
    [sym__br_flw_seq_bgn] = ACTIONS(203),
    [sym__r_flw_map_bgn] = ACTIONS(422),
    [sym__br_flw_map_bgn] = ACTIONS(207),
    [sym__r_flw_key_bgn] = ACTIONS(424),
    [sym__br_flw_key_bgn] = ACTIONS(215),
    [sym__r_flw_njv_bgn] = ACTIONS(426),
    [sym__br_flw_njv_bgn] = ACTIONS(219),
    [sym__r_dqt_str_bgn] = ACTIONS(428),
    [sym__br_dqt_str_bgn] = ACTIONS(223),
    [sym__r_sqt_str_bgn] = ACTIONS(430),
    [sym__br_sqt_str_bgn] = ACTIONS(227),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(229),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(231),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(229),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(231),
    [sym__r_sgl_pln_int_flw] = ACTIONS(229),
    [sym__br_sgl_pln_int_flw] = ACTIONS(231),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(229),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(231),
    [sym__r_sgl_pln_str_flw] = ACTIONS(229),
    [sym__br_sgl_pln_str_flw] = ACTIONS(231),
    [sym__r_mtl_pln_str_flw] = ACTIONS(233),
    [sym__br_mtl_pln_str_flw] = ACTIONS(235),
    [sym__r_tag] = ACTIONS(432),
    [sym__br_tag] = ACTIONS(239),
    [sym__r_acr_bgn] = ACTIONS(241),
    [sym__br_acr_bgn] = ACTIONS(243),
    [sym__r_als_bgn] = ACTIONS(245),
    [sym__br_als_bgn] = ACTIONS(247),
    [sym_comment] = ACTIONS(3),
  },
  [79] = {
    [sym__r_prp_val] = STATE(735),
    [sym__br_prp_val] = STATE(734),
    [sym__r_prp] = STATE(129),
    [sym__br_prp] = STATE(130),
    [sym__r_flw_seq_val] = STATE(733),
    [sym__br_flw_seq_val] = STATE(731),
    [sym__r_flw_seq] = STATE(821),
    [sym__br_flw_seq] = STATE(844),
    [sym__r_flw_map_val] = STATE(733),
    [sym__br_flw_map_val] = STATE(731),
    [sym__r_flw_map] = STATE(825),
    [sym__br_flw_map] = STATE(840),
    [sym__r_flw_seq_itm] = STATE(964),
    [sym__br_flw_seq_itm] = STATE(964),
    [sym__r_flw_exp_par] = STATE(1011),
    [sym__br_flw_exp_par] = STATE(1098),
    [sym__r_flw_imp_r_par] = STATE(1064),
    [sym__br_flw_imp_r_par] = STATE(1063),
    [sym__r_flw_njl_ann_par] = STATE(1062),
    [sym__br_flw_njl_ann_par] = STATE(1061),
    [sym__r_dqt_str_val] = STATE(733),
    [sym__br_dqt_str_val] = STATE(731),
    [sym__r_dqt_str] = STATE(826),
    [sym__br_dqt_str] = STATE(832),
    [sym__r_sqt_str_val] = STATE(733),
    [sym__br_sqt_str_val] = STATE(731),
    [sym__r_sqt_str] = STATE(842),
    [sym__br_sqt_str] = STATE(824),
    [sym__r_pln_flw_val] = STATE(735),
    [sym__br_pln_flw_val] = STATE(734),
    [sym__r_sgl_pln_flw] = STATE(877),
    [sym__br_sgl_pln_flw] = STATE(878),
    [sym__r_mtl_pln_flw] = STATE(877),
    [sym__br_mtl_pln_flw] = STATE(878),
    [sym__r_als_val] = STATE(735),
    [sym__br_als_val] = STATE(734),
    [sym__r_als] = STATE(841),
    [sym__br_als] = STATE(827),
    [sym__r_acr] = STATE(207),
    [sym__br_acr] = STATE(210),
    [sym__r_flw_seq_bgn] = ACTIONS(444),
    [sym__br_flw_seq_bgn] = ACTIONS(285),
    [sym__r_flw_map_bgn] = ACTIONS(446),
    [sym__br_flw_map_bgn] = ACTIONS(293),
    [sym__r_flw_key_bgn] = ACTIONS(448),
    [sym__br_flw_key_bgn] = ACTIONS(297),
    [sym__r_flw_njv_bgn] = ACTIONS(450),
    [sym__br_flw_njv_bgn] = ACTIONS(301),
    [sym__r_dqt_str_bgn] = ACTIONS(452),
    [sym__br_dqt_str_bgn] = ACTIONS(305),
    [sym__r_sqt_str_bgn] = ACTIONS(454),
    [sym__br_sqt_str_bgn] = ACTIONS(309),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(311),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(313),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(311),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(313),
    [sym__r_sgl_pln_int_flw] = ACTIONS(311),
    [sym__br_sgl_pln_int_flw] = ACTIONS(313),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(311),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(313),
    [sym__r_sgl_pln_str_flw] = ACTIONS(311),
    [sym__br_sgl_pln_str_flw] = ACTIONS(313),
    [sym__r_mtl_pln_str_flw] = ACTIONS(315),
    [sym__br_mtl_pln_str_flw] = ACTIONS(317),
    [sym__r_tag] = ACTIONS(456),
    [sym__br_tag] = ACTIONS(321),
    [sym__r_acr_bgn] = ACTIONS(323),
    [sym__br_acr_bgn] = ACTIONS(325),
    [sym__r_als_bgn] = ACTIONS(327),
    [sym__br_als_bgn] = ACTIONS(329),
    [sym_comment] = ACTIONS(3),
  },
  [80] = {
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__br_sgl_prp] = STATE(182),
    [sym__br_blk_seq] = STATE(1211),
    [sym__br_blk_seq_itm] = STATE(977),
    [sym__br_blk_map] = STATE(1210),
    [sym__br_blk_map_itm] = STATE(132),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__r_blk_str] = STATE(1209),
    [sym__br_blk_str] = STATE(1209),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(1208),
    [sym__br_flw_seq] = STATE(1208),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(1203),
    [sym__br_flw_map] = STATE(1203),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(1202),
    [sym__br_dqt_str] = STATE(1202),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(1201),
    [sym__br_sqt_str] = STATE(1201),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(1200),
    [sym__br_sgl_pln_blk] = STATE(289),
    [sym__r_mtl_pln_blk] = STATE(1200),
    [sym__br_mtl_pln_blk] = STATE(1200),
    [sym__br_als_val] = STATE(306),
    [sym__br_als] = STATE(385),
    [sym__br_acr] = STATE(218),
    [sym__eof] = ACTIONS(365),
    [sym__s_drs_end] = ACTIONS(365),
    [sym__s_doc_end] = ACTIONS(365),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__r_blk_lit_bgn] = ACTIONS(29),
    [sym__br_blk_lit_bgn] = ACTIONS(31),
    [sym__r_blk_fld_bgn] = ACTIONS(29),
    [sym__br_blk_fld_bgn] = ACTIONS(31),
    [sym__r_flw_seq_bgn] = ACTIONS(185),
    [sym__br_flw_seq_bgn] = ACTIONS(35),
    [sym__r_flw_map_bgn] = ACTIONS(187),
    [sym__br_flw_map_bgn] = ACTIONS(39),
    [sym__r_dqt_str_bgn] = ACTIONS(189),
    [sym__br_dqt_str_bgn] = ACTIONS(43),
    [sym__r_sqt_str_bgn] = ACTIONS(191),
    [sym__br_sqt_str_bgn] = ACTIONS(47),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(193),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(51),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(193),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(51),
    [sym__r_sgl_pln_int_blk] = ACTIONS(193),
    [sym__br_sgl_pln_int_blk] = ACTIONS(51),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(193),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(51),
    [sym__r_sgl_pln_str_blk] = ACTIONS(193),
    [sym__br_sgl_pln_str_blk] = ACTIONS(51),
    [sym__r_mtl_pln_str_blk] = ACTIONS(53),
    [sym__br_mtl_pln_str_blk] = ACTIONS(55),
    [sym__br_tag] = ACTIONS(367),
    [sym__br_acr_bgn] = ACTIONS(369),
    [sym__br_als_bgn] = ACTIONS(111),
    [sym_comment] = ACTIONS(3),
  },
  [81] = {
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__br_sgl_prp] = STATE(182),
    [sym__br_blk_seq] = STATE(1199),
    [sym__br_blk_seq_itm] = STATE(977),
    [sym__br_blk_map] = STATE(1198),
    [sym__br_blk_map_itm] = STATE(132),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__r_blk_str] = STATE(1197),
    [sym__br_blk_str] = STATE(1197),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(1195),
    [sym__br_flw_seq] = STATE(1195),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(1190),
    [sym__br_flw_map] = STATE(1190),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(1189),
    [sym__br_dqt_str] = STATE(1189),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(1186),
    [sym__br_sqt_str] = STATE(1186),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(1185),
    [sym__br_sgl_pln_blk] = STATE(290),
    [sym__r_mtl_pln_blk] = STATE(1185),
    [sym__br_mtl_pln_blk] = STATE(1185),
    [sym__br_als_val] = STATE(306),
    [sym__br_als] = STATE(385),
    [sym__br_acr] = STATE(218),
    [sym__eof] = ACTIONS(371),
    [sym__s_drs_end] = ACTIONS(371),
    [sym__s_doc_end] = ACTIONS(371),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__r_blk_lit_bgn] = ACTIONS(29),
    [sym__br_blk_lit_bgn] = ACTIONS(31),
    [sym__r_blk_fld_bgn] = ACTIONS(29),
    [sym__br_blk_fld_bgn] = ACTIONS(31),
    [sym__r_flw_seq_bgn] = ACTIONS(185),
    [sym__br_flw_seq_bgn] = ACTIONS(35),
    [sym__r_flw_map_bgn] = ACTIONS(187),
    [sym__br_flw_map_bgn] = ACTIONS(39),
    [sym__r_dqt_str_bgn] = ACTIONS(189),
    [sym__br_dqt_str_bgn] = ACTIONS(43),
    [sym__r_sqt_str_bgn] = ACTIONS(191),
    [sym__br_sqt_str_bgn] = ACTIONS(47),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(193),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(51),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(193),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(51),
    [sym__r_sgl_pln_int_blk] = ACTIONS(193),
    [sym__br_sgl_pln_int_blk] = ACTIONS(51),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(193),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(51),
    [sym__r_sgl_pln_str_blk] = ACTIONS(193),
    [sym__br_sgl_pln_str_blk] = ACTIONS(51),
    [sym__r_mtl_pln_str_blk] = ACTIONS(53),
    [sym__br_mtl_pln_str_blk] = ACTIONS(55),
    [sym__br_tag] = ACTIONS(367),
    [sym__br_acr_bgn] = ACTIONS(369),
    [sym__br_als_bgn] = ACTIONS(111),
    [sym_comment] = ACTIONS(3),
  },
  [82] = {
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__br_sgl_prp] = STATE(182),
    [sym__br_blk_seq] = STATE(1431),
    [sym__br_blk_seq_itm] = STATE(954),
    [sym__br_blk_map] = STATE(1428),
    [sym__br_blk_map_itm] = STATE(146),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__r_blk_str] = STATE(1427),
    [sym__br_blk_str] = STATE(1427),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(1426),
    [sym__br_flw_seq] = STATE(1426),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(1425),
    [sym__br_flw_map] = STATE(1425),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(1424),
    [sym__br_dqt_str] = STATE(1424),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(1423),
    [sym__br_sqt_str] = STATE(1423),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(1422),
    [sym__br_sgl_pln_blk] = STATE(311),
    [sym__r_mtl_pln_blk] = STATE(1422),
    [sym__br_mtl_pln_blk] = STATE(1422),
    [sym__br_als_val] = STATE(306),
    [sym__br_als] = STATE(385),
    [sym__br_acr] = STATE(218),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__b_blk_seq_bgn] = ACTIONS(365),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__r_blk_lit_bgn] = ACTIONS(125),
    [sym__br_blk_lit_bgn] = ACTIONS(127),
    [sym__r_blk_fld_bgn] = ACTIONS(125),
    [sym__br_blk_fld_bgn] = ACTIONS(127),
    [sym__r_flw_seq_bgn] = ACTIONS(561),
    [sym__br_flw_seq_bgn] = ACTIONS(131),
    [sym__r_flw_map_bgn] = ACTIONS(563),
    [sym__br_flw_map_bgn] = ACTIONS(135),
    [sym__r_dqt_str_bgn] = ACTIONS(565),
    [sym__br_dqt_str_bgn] = ACTIONS(139),
    [sym__r_sqt_str_bgn] = ACTIONS(567),
    [sym__br_sqt_str_bgn] = ACTIONS(143),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(569),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(147),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(569),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(147),
    [sym__r_sgl_pln_int_blk] = ACTIONS(569),
    [sym__br_sgl_pln_int_blk] = ACTIONS(147),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(569),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(147),
    [sym__r_sgl_pln_str_blk] = ACTIONS(569),
    [sym__br_sgl_pln_str_blk] = ACTIONS(147),
    [sym__r_mtl_pln_str_blk] = ACTIONS(149),
    [sym__br_mtl_pln_str_blk] = ACTIONS(151),
    [sym__br_tag] = ACTIONS(367),
    [sym__br_acr_bgn] = ACTIONS(369),
    [sym__br_als_bgn] = ACTIONS(111),
    [sym__bl] = ACTIONS(365),
    [sym_comment] = ACTIONS(3),
  },
  [83] = {
    [sym__br_sgl_prp_val] = STATE(306),
    [sym__br_sgl_prp] = STATE(182),
    [sym__br_blk_seq] = STATE(1421),
    [sym__br_blk_seq_itm] = STATE(954),
    [sym__br_blk_map] = STATE(1420),
    [sym__br_blk_map_itm] = STATE(146),
    [sym__br_blk_exp_itm] = STATE(429),
    [sym__br_blk_key_itm] = STATE(327),
    [sym__br_blk_val_itm] = STATE(396),
    [sym__br_blk_imp_itm_key_only] = STATE(452),
    [sym__r_blk_str] = STATE(1419),
    [sym__br_blk_str] = STATE(1419),
    [sym__br_sgl_flw_seq_val] = STATE(306),
    [sym__r_flw_seq] = STATE(1418),
    [sym__br_flw_seq] = STATE(1418),
    [sym__br_sgl_flw_seq] = STATE(336),
    [sym__br_sgl_flw_map_val] = STATE(306),
    [sym__r_flw_map] = STATE(1417),
    [sym__br_flw_map] = STATE(1417),
    [sym__br_sgl_flw_map] = STATE(331),
    [sym__br_sgl_dqt_str_val] = STATE(306),
    [sym__r_dqt_str] = STATE(1416),
    [sym__br_dqt_str] = STATE(1416),
    [sym__br_sgl_dqt_str] = STATE(325),
    [sym__br_sgl_sqt_str_val] = STATE(306),
    [sym__r_sqt_str] = STATE(1415),
    [sym__br_sqt_str] = STATE(1415),
    [sym__br_sgl_sqt_str] = STATE(362),
    [sym__br_sgl_pln_blk_val] = STATE(306),
    [sym__r_sgl_pln_blk] = STATE(1414),
    [sym__br_sgl_pln_blk] = STATE(310),
    [sym__r_mtl_pln_blk] = STATE(1414),
    [sym__br_mtl_pln_blk] = STATE(1414),
    [sym__br_als_val] = STATE(306),
    [sym__br_als] = STATE(385),
    [sym__br_acr] = STATE(218),
    [sym__br_blk_seq_bgn] = ACTIONS(19),
    [sym__b_blk_seq_bgn] = ACTIONS(371),
    [sym__br_blk_key_bgn] = ACTIONS(23),
    [sym__br_blk_val_bgn] = ACTIONS(27),
    [sym__r_blk_lit_bgn] = ACTIONS(125),
    [sym__br_blk_lit_bgn] = ACTIONS(127),
    [sym__r_blk_fld_bgn] = ACTIONS(125),
    [sym__br_blk_fld_bgn] = ACTIONS(127),
    [sym__r_flw_seq_bgn] = ACTIONS(561),
    [sym__br_flw_seq_bgn] = ACTIONS(131),
    [sym__r_flw_map_bgn] = ACTIONS(563),
    [sym__br_flw_map_bgn] = ACTIONS(135),
    [sym__r_dqt_str_bgn] = ACTIONS(565),
    [sym__br_dqt_str_bgn] = ACTIONS(139),
    [sym__r_sqt_str_bgn] = ACTIONS(567),
    [sym__br_sqt_str_bgn] = ACTIONS(143),
    [sym__r_sgl_pln_nul_blk] = ACTIONS(569),
    [sym__br_sgl_pln_nul_blk] = ACTIONS(147),
    [sym__r_sgl_pln_bol_blk] = ACTIONS(569),
    [sym__br_sgl_pln_bol_blk] = ACTIONS(147),
    [sym__r_sgl_pln_int_blk] = ACTIONS(569),
    [sym__br_sgl_pln_int_blk] = ACTIONS(147),
    [sym__r_sgl_pln_flt_blk] = ACTIONS(569),
    [sym__br_sgl_pln_flt_blk] = ACTIONS(147),
    [sym__r_sgl_pln_str_blk] = ACTIONS(569),
    [sym__br_sgl_pln_str_blk] = ACTIONS(147),
    [sym__r_mtl_pln_str_blk] = ACTIONS(149),
    [sym__br_mtl_pln_str_blk] = ACTIONS(151),
    [sym__br_tag] = ACTIONS(367),
    [sym__br_acr_bgn] = ACTIONS(369),
    [sym__br_als_bgn] = ACTIONS(111),
    [sym__bl] = ACTIONS(371),
    [sym_comment] = ACTIONS(3),
  },
  [84] = {
    [sym__r_prp_val] = STATE(1005),
    [sym__br_prp_val] = STATE(1004),
    [sym__r_prp] = STATE(164),
    [sym__br_prp] = STATE(163),
    [sym__r_flw_seq_val] = STATE(1003),
    [sym__br_flw_seq_val] = STATE(1000),
    [sym__r_flw_seq] = STATE(1314),
    [sym__br_flw_seq] = STATE(1315),
    [sym__r_flw_map_val] = STATE(1003),
    [sym__br_flw_map_val] = STATE(1000),
    [sym__r_flw_map] = STATE(1317),
    [sym__br_flw_map] = STATE(1318),
    [sym__r_flw_imp_r_par] = STATE(975),
    [sym__r_flw_imp_br_par] = STATE(972),
    [sym__br_flw_imp_r_par] = STATE(970),
    [sym__br_flw_imp_br_par] = STATE(969),
    [sym__r_dqt_str_val] = STATE(1003),
    [sym__br_dqt_str_val] = STATE(1000),
    [sym__r_dqt_str] = STATE(1320),
    [sym__br_dqt_str] = STATE(1321),
    [sym__r_sqt_str_val] = STATE(1003),
    [sym__br_sqt_str_val] = STATE(1000),
    [sym__r_sqt_str] = STATE(1323),
    [sym__br_sqt_str] = STATE(1324),
    [sym__r_pln_flw_val] = STATE(1005),
    [sym__br_pln_flw_val] = STATE(1004),
    [sym__r_sgl_pln_flw] = STATE(1328),
    [sym__br_sgl_pln_flw] = STATE(1329),
    [sym__r_mtl_pln_flw] = STATE(1328),
    [sym__br_mtl_pln_flw] = STATE(1329),
    [sym__r_als_val] = STATE(1005),
    [sym__br_als_val] = STATE(1004),
    [sym__r_als] = STATE(1269),
    [sym__br_als] = STATE(1326),
    [sym__r_acr] = STATE(261),
    [sym__br_acr] = STATE(246),
    [sym__r_flw_seq_bgn] = ACTIONS(571),
    [sym__br_flw_seq_bgn] = ACTIONS(381),
    [sym__r_flw_seq_end] = ACTIONS(573),
    [sym__br_flw_seq_end] = ACTIONS(573),
    [sym__r_flw_map_bgn] = ACTIONS(575),
    [sym__br_flw_map_bgn] = ACTIONS(390),
    [sym__r_flw_sep_bgn] = ACTIONS(573),
    [sym__br_flw_sep_bgn] = ACTIONS(573),
    [sym__r_dqt_str_bgn] = ACTIONS(577),
    [sym__br_dqt_str_bgn] = ACTIONS(394),
    [sym__r_sqt_str_bgn] = ACTIONS(579),
    [sym__br_sqt_str_bgn] = ACTIONS(398),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(400),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(402),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(400),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(402),
    [sym__r_sgl_pln_int_flw] = ACTIONS(400),
    [sym__br_sgl_pln_int_flw] = ACTIONS(402),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(400),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(402),
    [sym__r_sgl_pln_str_flw] = ACTIONS(400),
    [sym__br_sgl_pln_str_flw] = ACTIONS(402),
    [sym__r_mtl_pln_str_flw] = ACTIONS(404),
    [sym__br_mtl_pln_str_flw] = ACTIONS(406),
    [sym__r_tag] = ACTIONS(581),
    [sym__br_tag] = ACTIONS(410),
    [sym__r_acr_bgn] = ACTIONS(412),
    [sym__br_acr_bgn] = ACTIONS(414),
    [sym__r_als_bgn] = ACTIONS(416),
    [sym__br_als_bgn] = ACTIONS(418),
    [sym_comment] = ACTIONS(3),
  },
  [85] = {
    [sym__r_prp_val] = STATE(943),
    [sym__br_prp_val] = STATE(942),
    [sym__r_prp] = STATE(164),
    [sym__br_prp] = STATE(163),
    [sym__r_flw_seq_val] = STATE(898),
    [sym__br_flw_seq_val] = STATE(901),
    [sym__r_flw_seq] = STATE(1314),
    [sym__br_flw_seq] = STATE(1315),
    [sym__r_flw_map_val] = STATE(898),
    [sym__br_flw_map_val] = STATE(901),
    [sym__r_flw_map] = STATE(1317),
    [sym__br_flw_map] = STATE(1318),
    [sym__r_flw_imp_r_par] = STATE(941),
    [sym__r_flw_imp_br_par] = STATE(940),
    [sym__br_flw_imp_r_par] = STATE(938),
    [sym__br_flw_imp_br_par] = STATE(937),
    [sym__r_dqt_str_val] = STATE(898),
    [sym__br_dqt_str_val] = STATE(901),
    [sym__r_dqt_str] = STATE(1320),
    [sym__br_dqt_str] = STATE(1321),
    [sym__r_sqt_str_val] = STATE(898),
    [sym__br_sqt_str_val] = STATE(901),
    [sym__r_sqt_str] = STATE(1323),
    [sym__br_sqt_str] = STATE(1324),
    [sym__r_pln_flw_val] = STATE(943),
    [sym__br_pln_flw_val] = STATE(942),
    [sym__r_sgl_pln_flw] = STATE(1328),
    [sym__br_sgl_pln_flw] = STATE(1329),
    [sym__r_mtl_pln_flw] = STATE(1328),
    [sym__br_mtl_pln_flw] = STATE(1329),
    [sym__r_als_val] = STATE(943),
    [sym__br_als_val] = STATE(942),
    [sym__r_als] = STATE(1269),
    [sym__br_als] = STATE(1326),
    [sym__r_acr] = STATE(261),
    [sym__br_acr] = STATE(246),
    [sym__r_flw_seq_bgn] = ACTIONS(571),
    [sym__br_flw_seq_bgn] = ACTIONS(381),
    [sym__r_flw_map_bgn] = ACTIONS(575),
    [sym__br_flw_map_bgn] = ACTIONS(390),
    [sym__r_flw_map_end] = ACTIONS(386),
    [sym__br_flw_map_end] = ACTIONS(386),
    [sym__r_flw_sep_bgn] = ACTIONS(386),
    [sym__br_flw_sep_bgn] = ACTIONS(386),
    [sym__r_dqt_str_bgn] = ACTIONS(577),
    [sym__br_dqt_str_bgn] = ACTIONS(394),
    [sym__r_sqt_str_bgn] = ACTIONS(579),
    [sym__br_sqt_str_bgn] = ACTIONS(398),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(400),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(402),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(400),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(402),
    [sym__r_sgl_pln_int_flw] = ACTIONS(400),
    [sym__br_sgl_pln_int_flw] = ACTIONS(402),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(400),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(402),
    [sym__r_sgl_pln_str_flw] = ACTIONS(400),
    [sym__br_sgl_pln_str_flw] = ACTIONS(402),
    [sym__r_mtl_pln_str_flw] = ACTIONS(404),
    [sym__br_mtl_pln_str_flw] = ACTIONS(406),
    [sym__r_tag] = ACTIONS(581),
    [sym__br_tag] = ACTIONS(410),
    [sym__r_acr_bgn] = ACTIONS(412),
    [sym__br_acr_bgn] = ACTIONS(414),
    [sym__r_als_bgn] = ACTIONS(416),
    [sym__br_als_bgn] = ACTIONS(418),
    [sym_comment] = ACTIONS(3),
  },
  [86] = {
    [sym__r_prp_val] = STATE(943),
    [sym__br_prp_val] = STATE(942),
    [sym__r_prp] = STATE(164),
    [sym__br_prp] = STATE(163),
    [sym__r_flw_seq_val] = STATE(898),
    [sym__br_flw_seq_val] = STATE(901),
    [sym__r_flw_seq] = STATE(1314),
    [sym__br_flw_seq] = STATE(1315),
    [sym__r_flw_map_val] = STATE(898),
    [sym__br_flw_map_val] = STATE(901),
    [sym__r_flw_map] = STATE(1317),
    [sym__br_flw_map] = STATE(1318),
    [sym__r_flw_imp_r_par] = STATE(935),
    [sym__r_flw_imp_br_par] = STATE(929),
    [sym__br_flw_imp_r_par] = STATE(928),
    [sym__br_flw_imp_br_par] = STATE(927),
    [sym__r_dqt_str_val] = STATE(898),
    [sym__br_dqt_str_val] = STATE(901),
    [sym__r_dqt_str] = STATE(1320),
    [sym__br_dqt_str] = STATE(1321),
    [sym__r_sqt_str_val] = STATE(898),
    [sym__br_sqt_str_val] = STATE(901),
    [sym__r_sqt_str] = STATE(1323),
    [sym__br_sqt_str] = STATE(1324),
    [sym__r_pln_flw_val] = STATE(943),
    [sym__br_pln_flw_val] = STATE(942),
    [sym__r_sgl_pln_flw] = STATE(1328),
    [sym__br_sgl_pln_flw] = STATE(1329),
    [sym__r_mtl_pln_flw] = STATE(1328),
    [sym__br_mtl_pln_flw] = STATE(1329),
    [sym__r_als_val] = STATE(943),
    [sym__br_als_val] = STATE(942),
    [sym__r_als] = STATE(1269),
    [sym__br_als] = STATE(1326),
    [sym__r_acr] = STATE(261),
    [sym__br_acr] = STATE(246),
    [sym__r_flw_seq_bgn] = ACTIONS(571),
    [sym__br_flw_seq_bgn] = ACTIONS(381),
    [sym__r_flw_map_bgn] = ACTIONS(575),
    [sym__br_flw_map_bgn] = ACTIONS(390),
    [sym__r_flw_map_end] = ACTIONS(573),
    [sym__br_flw_map_end] = ACTIONS(573),
    [sym__r_flw_sep_bgn] = ACTIONS(573),
    [sym__br_flw_sep_bgn] = ACTIONS(573),
    [sym__r_dqt_str_bgn] = ACTIONS(577),
    [sym__br_dqt_str_bgn] = ACTIONS(394),
    [sym__r_sqt_str_bgn] = ACTIONS(579),
    [sym__br_sqt_str_bgn] = ACTIONS(398),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(400),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(402),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(400),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(402),
    [sym__r_sgl_pln_int_flw] = ACTIONS(400),
    [sym__br_sgl_pln_int_flw] = ACTIONS(402),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(400),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(402),
    [sym__r_sgl_pln_str_flw] = ACTIONS(400),
    [sym__br_sgl_pln_str_flw] = ACTIONS(402),
    [sym__r_mtl_pln_str_flw] = ACTIONS(404),
    [sym__br_mtl_pln_str_flw] = ACTIONS(406),
    [sym__r_tag] = ACTIONS(581),
    [sym__br_tag] = ACTIONS(410),
    [sym__r_acr_bgn] = ACTIONS(412),
    [sym__br_acr_bgn] = ACTIONS(414),
    [sym__r_als_bgn] = ACTIONS(416),
    [sym__br_als_bgn] = ACTIONS(418),
    [sym_comment] = ACTIONS(3),
  },
  [87] = {
    [sym__r_prp_val] = STATE(1005),
    [sym__br_prp_val] = STATE(1004),
    [sym__r_prp] = STATE(164),
    [sym__br_prp] = STATE(163),
    [sym__r_flw_seq_val] = STATE(1003),
    [sym__br_flw_seq_val] = STATE(1000),
    [sym__r_flw_seq] = STATE(1314),
    [sym__br_flw_seq] = STATE(1315),
    [sym__r_flw_map_val] = STATE(1003),
    [sym__br_flw_map_val] = STATE(1000),
    [sym__r_flw_map] = STATE(1317),
    [sym__br_flw_map] = STATE(1318),
    [sym__r_flw_imp_r_par] = STATE(995),
    [sym__r_flw_imp_br_par] = STATE(994),
    [sym__br_flw_imp_r_par] = STATE(992),
    [sym__br_flw_imp_br_par] = STATE(986),
    [sym__r_dqt_str_val] = STATE(1003),
    [sym__br_dqt_str_val] = STATE(1000),
    [sym__r_dqt_str] = STATE(1320),
    [sym__br_dqt_str] = STATE(1321),
    [sym__r_sqt_str_val] = STATE(1003),
    [sym__br_sqt_str_val] = STATE(1000),
    [sym__r_sqt_str] = STATE(1323),
    [sym__br_sqt_str] = STATE(1324),
    [sym__r_pln_flw_val] = STATE(1005),
    [sym__br_pln_flw_val] = STATE(1004),
    [sym__r_sgl_pln_flw] = STATE(1328),
    [sym__br_sgl_pln_flw] = STATE(1329),
    [sym__r_mtl_pln_flw] = STATE(1328),
    [sym__br_mtl_pln_flw] = STATE(1329),
    [sym__r_als_val] = STATE(1005),
    [sym__br_als_val] = STATE(1004),
    [sym__r_als] = STATE(1269),
    [sym__br_als] = STATE(1326),
    [sym__r_acr] = STATE(261),
    [sym__br_acr] = STATE(246),
    [sym__r_flw_seq_bgn] = ACTIONS(571),
    [sym__br_flw_seq_bgn] = ACTIONS(381),
    [sym__r_flw_seq_end] = ACTIONS(386),
    [sym__br_flw_seq_end] = ACTIONS(386),
    [sym__r_flw_map_bgn] = ACTIONS(575),
    [sym__br_flw_map_bgn] = ACTIONS(390),
    [sym__r_flw_sep_bgn] = ACTIONS(386),
    [sym__br_flw_sep_bgn] = ACTIONS(386),
    [sym__r_dqt_str_bgn] = ACTIONS(577),
    [sym__br_dqt_str_bgn] = ACTIONS(394),
    [sym__r_sqt_str_bgn] = ACTIONS(579),
    [sym__br_sqt_str_bgn] = ACTIONS(398),
    [sym__r_sgl_pln_nul_flw] = ACTIONS(400),
    [sym__br_sgl_pln_nul_flw] = ACTIONS(402),
    [sym__r_sgl_pln_bol_flw] = ACTIONS(400),
    [sym__br_sgl_pln_bol_flw] = ACTIONS(402),
    [sym__r_sgl_pln_int_flw] = ACTIONS(400),
    [sym__br_sgl_pln_int_flw] = ACTIONS(402),
    [sym__r_sgl_pln_flt_flw] = ACTIONS(400),
    [sym__br_sgl_pln_flt_flw] = ACTIONS(402),
    [sym__r_sgl_pln_str_flw] = ACTIONS(400),
    [sym__br_sgl_pln_str_flw] = ACTIONS(402),
    [sym__r_mtl_pln_str_flw] = ACTIONS(404),
    [sym__br_mtl_pln_str_flw] = ACTIONS(406),
    [sym__r_tag] = ACTIONS(581),
    [sym__br_tag] = ACTIONS(410),
    [sym__r_acr_bgn] = ACTIONS(412),
    [sym__br_acr_bgn] = ACTIONS(414),
    [sym__r_als_bgn] = ACTIONS(416),
    [sym__br_als_bgn] = ACTIONS(418),
    [sym_comment] = ACTIONS(3),
  },
};

static const uint16_t ts_small_parse_table[] = {
  [0] = 37,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(503), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(507), 1,
      sym__br_flw_map_bgn,
    ACTIONS(511), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(515), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(521), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(523), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(527), 1,
      sym__br_tag,
    ACTIONS(529), 1,
      sym__r_acr_bgn,
    ACTIONS(531), 1,
      sym__br_acr_bgn,
    ACTIONS(533), 1,
      sym__r_als_bgn,
    ACTIONS(535), 1,
      sym__br_als_bgn,
    ACTIONS(583), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(585), 1,
      sym__r_flw_map_bgn,
    ACTIONS(589), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(591), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(593), 1,
      sym__r_tag,
    STATE(152), 1,
      sym__br_prp,
    STATE(160), 1,
      sym__r_prp,
    STATE(213), 1,
      sym__r_acr,
    STATE(234), 1,
      sym__br_acr,
    STATE(1009), 1,
      sym__r_als,
    STATE(1088), 1,
      sym__r_flw_seq,
    STATE(1089), 1,
      sym__br_flw_seq,
    STATE(1090), 1,
      sym__r_flw_map,
    STATE(1091), 1,
      sym__br_flw_map,
    STATE(1092), 1,
      sym__r_dqt_str,
    STATE(1093), 1,
      sym__br_dqt_str,
    STATE(1094), 1,
      sym__r_sqt_str,
    STATE(1095), 1,
      sym__br_sqt_str,
    STATE(1097), 1,
      sym__br_als,
    STATE(917), 2,
      sym__br_sgl_pln_flw,
      sym__br_mtl_pln_flw,
    STATE(918), 2,
      sym__r_sgl_pln_flw,
      sym__r_mtl_pln_flw,
    ACTIONS(587), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    ACTIONS(517), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(519), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
    STATE(920), 15,
      sym__r_prp_val,
      sym__br_prp_val,
      sym__r_flw_seq_val,
      sym__br_flw_seq_val,
      sym__r_flw_map_val,
      sym__br_flw_map_val,
      sym__flw_ann_par_tal,
      sym__r_dqt_str_val,
      sym__br_dqt_str_val,
      sym__r_sqt_str_val,
      sym__br_sqt_str_val,
      sym__r_pln_flw_val,
      sym__br_pln_flw_val,
      sym__r_als_val,
      sym__br_als_val,
  [139] = 37,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(503), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(507), 1,
      sym__br_flw_map_bgn,
    ACTIONS(511), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(515), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(521), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(523), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(527), 1,
      sym__br_tag,
    ACTIONS(529), 1,
      sym__r_acr_bgn,
    ACTIONS(531), 1,
      sym__br_acr_bgn,
    ACTIONS(533), 1,
      sym__r_als_bgn,
    ACTIONS(535), 1,
      sym__br_als_bgn,
    ACTIONS(583), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(585), 1,
      sym__r_flw_map_bgn,
    ACTIONS(589), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(591), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(593), 1,
      sym__r_tag,
    STATE(152), 1,
      sym__br_prp,
    STATE(160), 1,
      sym__r_prp,
    STATE(213), 1,
      sym__r_acr,
    STATE(234), 1,
      sym__br_acr,
    STATE(1009), 1,
      sym__r_als,
    STATE(1088), 1,
      sym__r_flw_seq,
    STATE(1089), 1,
      sym__br_flw_seq,
    STATE(1090), 1,
      sym__r_flw_map,
    STATE(1091), 1,
      sym__br_flw_map,
    STATE(1092), 1,
      sym__r_dqt_str,
    STATE(1093), 1,
      sym__br_dqt_str,
    STATE(1094), 1,
      sym__r_sqt_str,
    STATE(1095), 1,
      sym__br_sqt_str,
    STATE(1097), 1,
      sym__br_als,
    STATE(917), 2,
      sym__br_sgl_pln_flw,
      sym__br_mtl_pln_flw,
    STATE(918), 2,
      sym__r_sgl_pln_flw,
      sym__r_mtl_pln_flw,
    ACTIONS(595), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    ACTIONS(517), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(519), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
    STATE(946), 15,
      sym__r_prp_val,
      sym__br_prp_val,
      sym__r_flw_seq_val,
      sym__br_flw_seq_val,
      sym__r_flw_map_val,
      sym__br_flw_map_val,
      sym__flw_ann_par_tal,
      sym__r_dqt_str_val,
      sym__br_dqt_str_val,
      sym__r_sqt_str_val,
      sym__br_sqt_str_val,
      sym__r_pln_flw_val,
      sym__br_pln_flw_val,
      sym__r_als_val,
      sym__br_als_val,
  [278] = 37,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(503), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(507), 1,
      sym__br_flw_map_bgn,
    ACTIONS(511), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(515), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(521), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(523), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(527), 1,
      sym__br_tag,
    ACTIONS(529), 1,
      sym__r_acr_bgn,
    ACTIONS(531), 1,
      sym__br_acr_bgn,
    ACTIONS(533), 1,
      sym__r_als_bgn,
    ACTIONS(535), 1,
      sym__br_als_bgn,
    ACTIONS(583), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(585), 1,
      sym__r_flw_map_bgn,
    ACTIONS(589), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(591), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(593), 1,
      sym__r_tag,
    STATE(152), 1,
      sym__br_prp,
    STATE(160), 1,
      sym__r_prp,
    STATE(213), 1,
      sym__r_acr,
    STATE(234), 1,
      sym__br_acr,
    STATE(1009), 1,
      sym__r_als,
    STATE(1088), 1,
      sym__r_flw_seq,
    STATE(1089), 1,
      sym__br_flw_seq,
    STATE(1090), 1,
      sym__r_flw_map,
    STATE(1091), 1,
      sym__br_flw_map,
    STATE(1092), 1,
      sym__r_dqt_str,
    STATE(1093), 1,
      sym__br_dqt_str,
    STATE(1094), 1,
      sym__r_sqt_str,
    STATE(1095), 1,
      sym__br_sqt_str,
    STATE(1097), 1,
      sym__br_als,
    STATE(917), 2,
      sym__br_sgl_pln_flw,
      sym__br_mtl_pln_flw,
    STATE(918), 2,
      sym__r_sgl_pln_flw,
      sym__r_mtl_pln_flw,
    ACTIONS(467), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    ACTIONS(517), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(519), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
    STATE(926), 15,
      sym__r_prp_val,
      sym__br_prp_val,
      sym__r_flw_seq_val,
      sym__br_flw_seq_val,
      sym__r_flw_map_val,
      sym__br_flw_map_val,
      sym__flw_ann_par_tal,
      sym__r_dqt_str_val,
      sym__br_dqt_str_val,
      sym__r_sqt_str_val,
      sym__br_sqt_str_val,
      sym__r_pln_flw_val,
      sym__br_pln_flw_val,
      sym__r_als_val,
      sym__br_als_val,
  [417] = 37,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(462), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(471), 1,
      sym__br_flw_map_bgn,
    ACTIONS(475), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(479), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(485), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(487), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(491), 1,
      sym__br_tag,
    ACTIONS(493), 1,
      sym__r_acr_bgn,
    ACTIONS(495), 1,
      sym__br_acr_bgn,
    ACTIONS(497), 1,
      sym__r_als_bgn,
    ACTIONS(499), 1,
      sym__br_als_bgn,
    ACTIONS(597), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(599), 1,
      sym__r_flw_map_bgn,
    ACTIONS(601), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(603), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(605), 1,
      sym__r_tag,
    STATE(144), 1,
      sym__r_prp,
    STATE(147), 1,
      sym__br_prp,
    STATE(226), 1,
      sym__br_acr,
    STATE(232), 1,
      sym__r_acr,
    STATE(1038), 1,
      sym__r_flw_seq,
    STATE(1039), 1,
      sym__br_flw_seq,
    STATE(1040), 1,
      sym__r_flw_map,
    STATE(1041), 1,
      sym__br_flw_map,
    STATE(1042), 1,
      sym__r_dqt_str,
    STATE(1043), 1,
      sym__br_dqt_str,
    STATE(1044), 1,
      sym__r_sqt_str,
    STATE(1045), 1,
      sym__br_sqt_str,
    STATE(1046), 1,
      sym__br_als,
    STATE(1096), 1,
      sym__r_als,
    STATE(905), 2,
      sym__r_sgl_pln_flw,
      sym__r_mtl_pln_flw,
    STATE(914), 2,
      sym__br_sgl_pln_flw,
      sym__br_mtl_pln_flw,
    ACTIONS(595), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    ACTIONS(481), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(483), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
    STATE(965), 15,
      sym__r_prp_val,
      sym__br_prp_val,
      sym__r_flw_seq_val,
      sym__br_flw_seq_val,
      sym__r_flw_map_val,
      sym__br_flw_map_val,
      sym__flw_ann_par_tal,
      sym__r_dqt_str_val,
      sym__br_dqt_str_val,
      sym__r_sqt_str_val,
      sym__br_sqt_str_val,
      sym__r_pln_flw_val,
      sym__br_pln_flw_val,
      sym__r_als_val,
      sym__br_als_val,
  [556] = 37,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(462), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(471), 1,
      sym__br_flw_map_bgn,
    ACTIONS(475), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(479), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(485), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(487), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(491), 1,
      sym__br_tag,
    ACTIONS(493), 1,
      sym__r_acr_bgn,
    ACTIONS(495), 1,
      sym__br_acr_bgn,
    ACTIONS(497), 1,
      sym__r_als_bgn,
    ACTIONS(499), 1,
      sym__br_als_bgn,
    ACTIONS(597), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(599), 1,
      sym__r_flw_map_bgn,
    ACTIONS(601), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(603), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(605), 1,
      sym__r_tag,
    STATE(144), 1,
      sym__r_prp,
    STATE(147), 1,
      sym__br_prp,
    STATE(226), 1,
      sym__br_acr,
    STATE(232), 1,
      sym__r_acr,
    STATE(1038), 1,
      sym__r_flw_seq,
    STATE(1039), 1,
      sym__br_flw_seq,
    STATE(1040), 1,
      sym__r_flw_map,
    STATE(1041), 1,
      sym__br_flw_map,
    STATE(1042), 1,
      sym__r_dqt_str,
    STATE(1043), 1,
      sym__br_dqt_str,
    STATE(1044), 1,
      sym__r_sqt_str,
    STATE(1045), 1,
      sym__br_sqt_str,
    STATE(1046), 1,
      sym__br_als,
    STATE(1096), 1,
      sym__r_als,
    STATE(905), 2,
      sym__r_sgl_pln_flw,
      sym__r_mtl_pln_flw,
    STATE(914), 2,
      sym__br_sgl_pln_flw,
      sym__br_mtl_pln_flw,
    ACTIONS(607), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    ACTIONS(481), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(483), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
    STATE(966), 15,
      sym__r_prp_val,
      sym__br_prp_val,
      sym__r_flw_seq_val,
      sym__br_flw_seq_val,
      sym__r_flw_map_val,
      sym__br_flw_map_val,
      sym__flw_ann_par_tal,
      sym__r_dqt_str_val,
      sym__br_dqt_str_val,
      sym__r_sqt_str_val,
      sym__br_sqt_str_val,
      sym__r_pln_flw_val,
      sym__br_pln_flw_val,
      sym__r_als_val,
      sym__br_als_val,
  [695] = 37,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(462), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(471), 1,
      sym__br_flw_map_bgn,
    ACTIONS(475), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(479), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(485), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(487), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(491), 1,
      sym__br_tag,
    ACTIONS(493), 1,
      sym__r_acr_bgn,
    ACTIONS(495), 1,
      sym__br_acr_bgn,
    ACTIONS(497), 1,
      sym__r_als_bgn,
    ACTIONS(499), 1,
      sym__br_als_bgn,
    ACTIONS(597), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(599), 1,
      sym__r_flw_map_bgn,
    ACTIONS(601), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(603), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(605), 1,
      sym__r_tag,
    STATE(144), 1,
      sym__r_prp,
    STATE(147), 1,
      sym__br_prp,
    STATE(226), 1,
      sym__br_acr,
    STATE(232), 1,
      sym__r_acr,
    STATE(1038), 1,
      sym__r_flw_seq,
    STATE(1039), 1,
      sym__br_flw_seq,
    STATE(1040), 1,
      sym__r_flw_map,
    STATE(1041), 1,
      sym__br_flw_map,
    STATE(1042), 1,
      sym__r_dqt_str,
    STATE(1043), 1,
      sym__br_dqt_str,
    STATE(1044), 1,
      sym__r_sqt_str,
    STATE(1045), 1,
      sym__br_sqt_str,
    STATE(1046), 1,
      sym__br_als,
    STATE(1096), 1,
      sym__r_als,
    STATE(905), 2,
      sym__r_sgl_pln_flw,
      sym__r_mtl_pln_flw,
    STATE(914), 2,
      sym__br_sgl_pln_flw,
      sym__br_mtl_pln_flw,
    ACTIONS(587), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    ACTIONS(481), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(483), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
    STATE(959), 15,
      sym__r_prp_val,
      sym__br_prp_val,
      sym__r_flw_seq_val,
      sym__br_flw_seq_val,
      sym__r_flw_map_val,
      sym__br_flw_map_val,
      sym__flw_ann_par_tal,
      sym__r_dqt_str_val,
      sym__br_dqt_str_val,
      sym__r_sqt_str_val,
      sym__br_sqt_str_val,
      sym__r_pln_flw_val,
      sym__br_pln_flw_val,
      sym__r_als_val,
      sym__br_als_val,
  [834] = 37,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(462), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(471), 1,
      sym__br_flw_map_bgn,
    ACTIONS(475), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(479), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(485), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(487), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(491), 1,
      sym__br_tag,
    ACTIONS(493), 1,
      sym__r_acr_bgn,
    ACTIONS(495), 1,
      sym__br_acr_bgn,
    ACTIONS(497), 1,
      sym__r_als_bgn,
    ACTIONS(499), 1,
      sym__br_als_bgn,
    ACTIONS(597), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(599), 1,
      sym__r_flw_map_bgn,
    ACTIONS(601), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(603), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(605), 1,
      sym__r_tag,
    STATE(144), 1,
      sym__r_prp,
    STATE(147), 1,
      sym__br_prp,
    STATE(226), 1,
      sym__br_acr,
    STATE(232), 1,
      sym__r_acr,
    STATE(1038), 1,
      sym__r_flw_seq,
    STATE(1039), 1,
      sym__br_flw_seq,
    STATE(1040), 1,
      sym__r_flw_map,
    STATE(1041), 1,
      sym__br_flw_map,
    STATE(1042), 1,
      sym__r_dqt_str,
    STATE(1043), 1,
      sym__br_dqt_str,
    STATE(1044), 1,
      sym__r_sqt_str,
    STATE(1045), 1,
      sym__br_sqt_str,
    STATE(1046), 1,
      sym__br_als,
    STATE(1096), 1,
      sym__r_als,
    STATE(905), 2,
      sym__r_sgl_pln_flw,
      sym__r_mtl_pln_flw,
    STATE(914), 2,
      sym__br_sgl_pln_flw,
      sym__br_mtl_pln_flw,
    ACTIONS(467), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    ACTIONS(481), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(483), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
    STATE(968), 15,
      sym__r_prp_val,
      sym__br_prp_val,
      sym__r_flw_seq_val,
      sym__br_flw_seq_val,
      sym__r_flw_map_val,
      sym__br_flw_map_val,
      sym__flw_ann_par_tal,
      sym__r_dqt_str_val,
      sym__br_dqt_str_val,
      sym__r_sqt_str_val,
      sym__br_sqt_str_val,
      sym__r_pln_flw_val,
      sym__br_pln_flw_val,
      sym__r_als_val,
      sym__br_als_val,
  [973] = 37,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(503), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(507), 1,
      sym__br_flw_map_bgn,
    ACTIONS(511), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(515), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(521), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(523), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(527), 1,
      sym__br_tag,
    ACTIONS(529), 1,
      sym__r_acr_bgn,
    ACTIONS(531), 1,
      sym__br_acr_bgn,
    ACTIONS(533), 1,
      sym__r_als_bgn,
    ACTIONS(535), 1,
      sym__br_als_bgn,
    ACTIONS(583), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(585), 1,
      sym__r_flw_map_bgn,
    ACTIONS(589), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(591), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(593), 1,
      sym__r_tag,
    STATE(152), 1,
      sym__br_prp,
    STATE(160), 1,
      sym__r_prp,
    STATE(213), 1,
      sym__r_acr,
    STATE(234), 1,
      sym__br_acr,
    STATE(1009), 1,
      sym__r_als,
    STATE(1088), 1,
      sym__r_flw_seq,
    STATE(1089), 1,
      sym__br_flw_seq,
    STATE(1090), 1,
      sym__r_flw_map,
    STATE(1091), 1,
      sym__br_flw_map,
    STATE(1092), 1,
      sym__r_dqt_str,
    STATE(1093), 1,
      sym__br_dqt_str,
    STATE(1094), 1,
      sym__r_sqt_str,
    STATE(1095), 1,
      sym__br_sqt_str,
    STATE(1097), 1,
      sym__br_als,
    STATE(917), 2,
      sym__br_sgl_pln_flw,
      sym__br_mtl_pln_flw,
    STATE(918), 2,
      sym__r_sgl_pln_flw,
      sym__r_mtl_pln_flw,
    ACTIONS(607), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    ACTIONS(517), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(519), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
    STATE(979), 15,
      sym__r_prp_val,
      sym__br_prp_val,
      sym__r_flw_seq_val,
      sym__br_flw_seq_val,
      sym__r_flw_map_val,
      sym__br_flw_map_val,
      sym__flw_ann_par_tal,
      sym__r_dqt_str_val,
      sym__br_dqt_str_val,
      sym__r_sqt_str_val,
      sym__br_sqt_str_val,
      sym__r_pln_flw_val,
      sym__br_pln_flw_val,
      sym__r_als_val,
      sym__br_als_val,
  [1112] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(616), 1,
      sym__r_acr_bgn,
    ACTIONS(618), 1,
      sym__br_acr_bgn,
    STATE(106), 1,
      sym__r_acr,
    STATE(173), 1,
      sym__br_acr,
    ACTIONS(613), 9,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
    ACTIONS(611), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
    ACTIONS(609), 23,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_als_bgn,
  [1182] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(616), 1,
      sym__r_acr_bgn,
    ACTIONS(628), 1,
      sym__br_acr_bgn,
    STATE(109), 1,
      sym__r_acr,
    STATE(174), 1,
      sym__br_acr,
    ACTIONS(625), 9,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
    ACTIONS(623), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
    ACTIONS(621), 23,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_als_bgn,
  [1252] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 50,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__b_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__r_tag,
      sym__br_tag,
      sym__b_tag,
      sym__br_acr_bgn,
      sym__b_acr_bgn,
      sym__br_als_bgn,
      sym__b_als_bgn,
      sym__bl,
  [1308] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(633), 1,
      sym__r_tag,
    ACTIONS(635), 1,
      sym__br_tag,
    ACTIONS(613), 9,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
    ACTIONS(611), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
    ACTIONS(609), 23,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [1372] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 50,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__b_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__r_tag,
      sym__br_tag,
      sym__b_tag,
      sym__br_acr_bgn,
      sym__b_acr_bgn,
      sym__br_als_bgn,
      sym__b_als_bgn,
      sym__bl,
  [1428] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(611), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(640), 1,
      sym__r_acr_bgn,
    ACTIONS(642), 1,
      sym__br_acr_bgn,
    STATE(117), 1,
      sym__r_acr,
    STATE(125), 1,
      sym__br_acr,
    ACTIONS(609), 21,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_als_bgn,
    ACTIONS(613), 24,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [1496] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(640), 1,
      sym__r_acr_bgn,
    ACTIONS(645), 1,
      sym__br_acr_bgn,
    STATE(118), 1,
      sym__r_acr,
    STATE(179), 1,
      sym__br_acr,
    ACTIONS(613), 10,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__bl,
    ACTIONS(611), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
    ACTIONS(609), 21,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_als_bgn,
  [1564] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(623), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(640), 1,
      sym__r_acr_bgn,
    ACTIONS(648), 1,
      sym__br_acr_bgn,
    STATE(116), 1,
      sym__r_acr,
    STATE(124), 1,
      sym__br_acr,
    ACTIONS(621), 21,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_als_bgn,
    ACTIONS(625), 24,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [1632] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(640), 1,
      sym__r_acr_bgn,
    ACTIONS(651), 1,
      sym__br_acr_bgn,
    STATE(119), 1,
      sym__r_acr,
    STATE(177), 1,
      sym__br_acr,
    ACTIONS(625), 10,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__bl,
    ACTIONS(623), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
    ACTIONS(621), 21,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_als_bgn,
  [1700] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(654), 1,
      sym__r_tag,
    ACTIONS(656), 1,
      sym__br_tag,
    ACTIONS(625), 9,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
    ACTIONS(623), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
    ACTIONS(621), 23,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [1764] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(663), 9,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
    ACTIONS(661), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
    ACTIONS(659), 24,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [1823] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(648), 1,
      sym__br_acr_bgn,
    ACTIONS(666), 1,
      sym__r_acr_bgn,
    STATE(124), 2,
      sym__r_acr,
      sym__br_acr,
    ACTIONS(621), 45,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__b_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__br_als_bgn,
      sym__b_als_bgn,
      sym__bl,
  [1884] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 49,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__b_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__b_tag,
      sym__br_acr_bgn,
      sym__b_acr_bgn,
      sym__br_als_bgn,
      sym__b_als_bgn,
      sym__bl,
  [1939] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(672), 9,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
    ACTIONS(670), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
    ACTIONS(668), 24,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [1998] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 48,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__b_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__r_tag,
      sym__br_tag,
      sym__b_tag,
      sym__br_acr_bgn,
      sym__b_acr_bgn,
      sym__br_als_bgn,
      sym__b_als_bgn,
      sym__bl,
  [2052] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(675), 1,
      sym__r_tag,
    ACTIONS(677), 1,
      sym__br_tag,
    ACTIONS(625), 10,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__bl,
    ACTIONS(623), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
    ACTIONS(621), 21,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [2114] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(680), 1,
      sym__r_tag,
    ACTIONS(682), 1,
      sym__br_tag,
    ACTIONS(613), 10,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__bl,
    ACTIONS(611), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
    ACTIONS(609), 21,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [2176] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(611), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(685), 1,
      sym__r_tag,
    ACTIONS(687), 1,
      sym__br_tag,
    ACTIONS(609), 21,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_acr_bgn,
      sym__br_als_bgn,
    ACTIONS(613), 24,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [2238] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(623), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(690), 1,
      sym__r_tag,
    ACTIONS(692), 1,
      sym__br_tag,
    ACTIONS(621), 21,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_acr_bgn,
      sym__br_als_bgn,
    ACTIONS(625), 24,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [2300] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 48,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__b_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__r_tag,
      sym__br_tag,
      sym__b_tag,
      sym__br_acr_bgn,
      sym__b_acr_bgn,
      sym__br_als_bgn,
      sym__b_als_bgn,
      sym__bl,
  [2354] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(670), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(668), 22,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
    ACTIONS(672), 24,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [2411] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(661), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(659), 22,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
    ACTIONS(663), 24,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [2468] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(663), 10,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__bl,
    ACTIONS(661), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
    ACTIONS(659), 22,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [2525] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(672), 10,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__bl,
    ACTIONS(670), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
    ACTIONS(668), 22,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [2582] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(692), 1,
      sym__br_tag,
    ACTIONS(695), 1,
      sym__r_tag,
    ACTIONS(621), 45,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__b_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__b_tag,
      sym__br_acr_bgn,
      sym__b_acr_bgn,
      sym__br_als_bgn,
      sym__b_als_bgn,
      sym__bl,
  [2639] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 47,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__b_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__r_tag,
      sym__br_tag,
      sym__b_tag,
      sym__br_acr_bgn,
      sym__b_acr_bgn,
      sym__br_als_bgn,
      sym__b_als_bgn,
      sym__bl,
  [2692] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 47,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__b_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__b_tag,
      sym__br_acr_bgn,
      sym__b_acr_bgn,
      sym__br_als_bgn,
      sym__b_als_bgn,
      sym__bl,
  [2745] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 46,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__b_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__b_tag,
      sym__br_acr_bgn,
      sym__b_acr_bgn,
      sym__br_als_bgn,
      sym__b_als_bgn,
      sym__bl,
  [2797] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(668), 46,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__b_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__b_tag,
      sym__br_acr_bgn,
      sym__b_acr_bgn,
      sym__br_als_bgn,
      sym__b_als_bgn,
      sym__bl,
  [2849] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(659), 46,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__b_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__b_tag,
      sym__br_acr_bgn,
      sym__b_acr_bgn,
      sym__br_als_bgn,
      sym__b_als_bgn,
      sym__bl,
  [2901] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 46,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__b_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__b_tag,
      sym__br_acr_bgn,
      sym__b_acr_bgn,
      sym__br_als_bgn,
      sym__b_als_bgn,
      sym__bl,
  [2953] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(203), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(207), 1,
      sym__br_flw_map_bgn,
    ACTIONS(223), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(227), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(233), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(235), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(420), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(422), 1,
      sym__r_flw_map_bgn,
    ACTIONS(428), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(430), 1,
      sym__r_sqt_str_bgn,
    STATE(779), 2,
      sym__r_sqt_str,
      sym__br_sqt_str,
    STATE(781), 2,
      sym__r_dqt_str,
      sym__br_dqt_str,
    STATE(783), 2,
      sym__r_flw_map,
      sym__br_flw_map,
    STATE(784), 2,
      sym__r_flw_seq,
      sym__br_flw_seq,
    STATE(798), 4,
      sym__r_sgl_pln_flw,
      sym__br_sgl_pln_flw,
      sym__r_mtl_pln_flw,
      sym__br_mtl_pln_flw,
    ACTIONS(229), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(231), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
    ACTIONS(371), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [3031] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(203), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(207), 1,
      sym__br_flw_map_bgn,
    ACTIONS(223), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(227), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(233), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(235), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(420), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(422), 1,
      sym__r_flw_map_bgn,
    ACTIONS(428), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(430), 1,
      sym__r_sqt_str_bgn,
    STATE(785), 2,
      sym__r_sqt_str,
      sym__br_sqt_str,
    STATE(786), 2,
      sym__r_dqt_str,
      sym__br_dqt_str,
    STATE(789), 2,
      sym__r_flw_map,
      sym__br_flw_map,
    STATE(790), 2,
      sym__r_flw_seq,
      sym__br_flw_seq,
    STATE(769), 4,
      sym__r_sgl_pln_flw,
      sym__br_sgl_pln_flw,
      sym__r_mtl_pln_flw,
      sym__br_mtl_pln_flw,
    ACTIONS(229), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(231), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
    ACTIONS(365), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [3109] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(285), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(293), 1,
      sym__br_flw_map_bgn,
    ACTIONS(305), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(309), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(315), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(317), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(444), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(446), 1,
      sym__r_flw_map_bgn,
    ACTIONS(452), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(454), 1,
      sym__r_sqt_str_bgn,
    STATE(847), 2,
      sym__r_flw_seq,
      sym__br_flw_seq,
    STATE(848), 2,
      sym__r_flw_map,
      sym__br_flw_map,
    STATE(849), 2,
      sym__r_dqt_str,
      sym__br_dqt_str,
    STATE(850), 2,
      sym__r_sqt_str,
      sym__br_sqt_str,
    STATE(857), 4,
      sym__r_sgl_pln_flw,
      sym__br_sgl_pln_flw,
      sym__r_mtl_pln_flw,
      sym__br_mtl_pln_flw,
    ACTIONS(311), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(313), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
    ACTIONS(365), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [3186] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(285), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(293), 1,
      sym__br_flw_map_bgn,
    ACTIONS(305), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(309), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(315), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(317), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(444), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(446), 1,
      sym__r_flw_map_bgn,
    ACTIONS(452), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(454), 1,
      sym__r_sqt_str_bgn,
    STATE(851), 2,
      sym__r_flw_seq,
      sym__br_flw_seq,
    STATE(852), 2,
      sym__r_flw_map,
      sym__br_flw_map,
    STATE(853), 2,
      sym__r_dqt_str,
      sym__br_dqt_str,
    STATE(854), 2,
      sym__r_sqt_str,
      sym__br_sqt_str,
    STATE(856), 4,
      sym__r_sgl_pln_flw,
      sym__br_sgl_pln_flw,
      sym__r_mtl_pln_flw,
      sym__br_mtl_pln_flw,
    ACTIONS(311), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(313), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
    ACTIONS(371), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [3263] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(697), 1,
      sym__b_blk_key_bgn,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    ACTIONS(701), 1,
      sym__b_flw_seq_bgn,
    ACTIONS(703), 1,
      sym__b_flw_map_bgn,
    ACTIONS(705), 1,
      sym__b_dqt_str_bgn,
    ACTIONS(707), 1,
      sym__b_sqt_str_bgn,
    ACTIONS(711), 1,
      sym__b_tag,
    ACTIONS(713), 1,
      sym__b_acr_bgn,
    ACTIONS(715), 1,
      sym__b_als_bgn,
    ACTIONS(717), 1,
      sym__bl,
    STATE(191), 1,
      sym__b_sgl_prp,
    STATE(219), 1,
      sym__b_acr,
    STATE(330), 1,
      sym__b_sgl_pln_blk,
    STATE(338), 1,
      sym__b_sgl_sqt_str,
    STATE(355), 1,
      sym__b_sgl_dqt_str,
    STATE(356), 1,
      sym__b_sgl_flw_map,
    STATE(363), 1,
      sym__b_sgl_flw_seq,
    STATE(367), 1,
      sym__b_als,
    STATE(370), 1,
      sym__b_blk_key_itm,
    STATE(403), 1,
      sym__b_blk_imp_itm_key_only,
    STATE(404), 1,
      sym__b_blk_val_itm,
    STATE(407), 1,
      sym__b_blk_exp_itm,
    STATE(161), 2,
      sym__b_blk_map_itm,
      aux_sym__r_blk_map_repeat1,
    ACTIONS(709), 5,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
    STATE(302), 7,
      sym__b_sgl_prp_val,
      sym__b_sgl_flw_seq_val,
      sym__b_sgl_flw_map_val,
      sym__b_sgl_dqt_str_val,
      sym__b_sgl_sqt_str_val,
      sym__b_sgl_pln_blk_val,
      sym__b_als_val,
  [3353] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(697), 1,
      sym__b_blk_key_bgn,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    ACTIONS(701), 1,
      sym__b_flw_seq_bgn,
    ACTIONS(703), 1,
      sym__b_flw_map_bgn,
    ACTIONS(705), 1,
      sym__b_dqt_str_bgn,
    ACTIONS(707), 1,
      sym__b_sqt_str_bgn,
    ACTIONS(711), 1,
      sym__b_tag,
    ACTIONS(713), 1,
      sym__b_acr_bgn,
    ACTIONS(715), 1,
      sym__b_als_bgn,
    ACTIONS(719), 1,
      sym__bl,
    STATE(191), 1,
      sym__b_sgl_prp,
    STATE(219), 1,
      sym__b_acr,
    STATE(330), 1,
      sym__b_sgl_pln_blk,
    STATE(338), 1,
      sym__b_sgl_sqt_str,
    STATE(355), 1,
      sym__b_sgl_dqt_str,
    STATE(356), 1,
      sym__b_sgl_flw_map,
    STATE(363), 1,
      sym__b_sgl_flw_seq,
    STATE(367), 1,
      sym__b_als,
    STATE(370), 1,
      sym__b_blk_key_itm,
    STATE(403), 1,
      sym__b_blk_imp_itm_key_only,
    STATE(404), 1,
      sym__b_blk_val_itm,
    STATE(407), 1,
      sym__b_blk_exp_itm,
    STATE(131), 2,
      sym__b_blk_map_itm,
      aux_sym__r_blk_map_repeat1,
    ACTIONS(709), 5,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
    STATE(302), 7,
      sym__b_sgl_prp_val,
      sym__b_sgl_flw_seq_val,
      sym__b_sgl_flw_map_val,
      sym__b_sgl_dqt_str_val,
      sym__b_sgl_sqt_str_val,
      sym__b_sgl_pln_blk_val,
      sym__b_als_val,
  [3443] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(721), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(723), 1,
      sym__r_flw_map_bgn,
    ACTIONS(725), 1,
      sym__r_flw_map_end,
    ACTIONS(727), 1,
      sym__r_flw_key_bgn,
    ACTIONS(729), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(731), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(733), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(737), 1,
      sym__r_tag,
    ACTIONS(739), 1,
      sym__r_acr_bgn,
    ACTIONS(741), 1,
      sym__r_als_bgn,
    STATE(309), 1,
      sym__r_sgl_prp,
    STATE(495), 1,
      sym__r_acr,
    STATE(1015), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1166), 1,
      sym__r_sgl_pln_flw,
    STATE(1174), 1,
      sym__r_als,
    STATE(1180), 1,
      sym__r_sgl_flw_seq,
    STATE(1181), 1,
      sym__r_sgl_flw_map,
    STATE(1182), 1,
      sym__r_sgl_dqt_str,
    STATE(1183), 1,
      sym__r_sgl_sqt_str,
    STATE(1405), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1435), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1436), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1478), 1,
      sym__r_sgl_flw_col_dat,
    STATE(1518), 1,
      sym__sgl_flw_map_tal,
    STATE(974), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(976), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(735), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [3537] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(721), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(723), 1,
      sym__r_flw_map_bgn,
    ACTIONS(727), 1,
      sym__r_flw_key_bgn,
    ACTIONS(729), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(731), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(733), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(737), 1,
      sym__r_tag,
    ACTIONS(739), 1,
      sym__r_acr_bgn,
    ACTIONS(741), 1,
      sym__r_als_bgn,
    ACTIONS(743), 1,
      sym__r_flw_map_end,
    STATE(309), 1,
      sym__r_sgl_prp,
    STATE(384), 1,
      sym__sgl_flw_map_tal,
    STATE(495), 1,
      sym__r_acr,
    STATE(1015), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1166), 1,
      sym__r_sgl_pln_flw,
    STATE(1174), 1,
      sym__r_als,
    STATE(1180), 1,
      sym__r_sgl_flw_seq,
    STATE(1181), 1,
      sym__r_sgl_flw_map,
    STATE(1182), 1,
      sym__r_sgl_dqt_str,
    STATE(1183), 1,
      sym__r_sgl_sqt_str,
    STATE(1405), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1435), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1436), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1488), 1,
      sym__r_sgl_flw_col_dat,
    STATE(974), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(976), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(735), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [3631] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(721), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(723), 1,
      sym__r_flw_map_bgn,
    ACTIONS(727), 1,
      sym__r_flw_key_bgn,
    ACTIONS(729), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(731), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(733), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(737), 1,
      sym__r_tag,
    ACTIONS(739), 1,
      sym__r_acr_bgn,
    ACTIONS(741), 1,
      sym__r_als_bgn,
    ACTIONS(745), 1,
      sym__r_flw_map_end,
    STATE(309), 1,
      sym__r_sgl_prp,
    STATE(495), 1,
      sym__r_acr,
    STATE(1015), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1146), 1,
      sym__sgl_flw_map_tal,
    STATE(1166), 1,
      sym__r_sgl_pln_flw,
    STATE(1174), 1,
      sym__r_als,
    STATE(1180), 1,
      sym__r_sgl_flw_seq,
    STATE(1181), 1,
      sym__r_sgl_flw_map,
    STATE(1182), 1,
      sym__r_sgl_dqt_str,
    STATE(1183), 1,
      sym__r_sgl_sqt_str,
    STATE(1405), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1435), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1436), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1503), 1,
      sym__r_sgl_flw_col_dat,
    STATE(974), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(976), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(735), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [3725] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(747), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(749), 1,
      sym__r_flw_seq_end,
    ACTIONS(751), 1,
      sym__r_flw_map_bgn,
    ACTIONS(753), 1,
      sym__r_flw_key_bgn,
    ACTIONS(755), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(757), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(759), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(763), 1,
      sym__r_tag,
    ACTIONS(765), 1,
      sym__r_acr_bgn,
    ACTIONS(767), 1,
      sym__r_als_bgn,
    STATE(304), 1,
      sym__r_sgl_prp,
    STATE(498), 1,
      sym__r_acr,
    STATE(1112), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1120), 1,
      sym__r_sgl_pln_flw,
    STATE(1139), 1,
      sym__r_sgl_flw_seq,
    STATE(1140), 1,
      sym__r_sgl_flw_map,
    STATE(1142), 1,
      sym__r_sgl_dqt_str,
    STATE(1143), 1,
      sym__r_sgl_sqt_str,
    STATE(1145), 1,
      sym__sgl_flw_seq_tal,
    STATE(1170), 1,
      sym__r_als,
    STATE(1256), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1283), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1437), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1501), 1,
      sym__r_sgl_flw_col_dat,
    STATE(1002), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(1006), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(761), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [3819] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(747), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(751), 1,
      sym__r_flw_map_bgn,
    ACTIONS(753), 1,
      sym__r_flw_key_bgn,
    ACTIONS(755), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(757), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(759), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(763), 1,
      sym__r_tag,
    ACTIONS(765), 1,
      sym__r_acr_bgn,
    ACTIONS(767), 1,
      sym__r_als_bgn,
    ACTIONS(769), 1,
      sym__r_flw_seq_end,
    STATE(304), 1,
      sym__r_sgl_prp,
    STATE(498), 1,
      sym__r_acr,
    STATE(1112), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1120), 1,
      sym__r_sgl_pln_flw,
    STATE(1139), 1,
      sym__r_sgl_flw_seq,
    STATE(1140), 1,
      sym__r_sgl_flw_map,
    STATE(1142), 1,
      sym__r_sgl_dqt_str,
    STATE(1143), 1,
      sym__r_sgl_sqt_str,
    STATE(1170), 1,
      sym__r_als,
    STATE(1256), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1283), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1437), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1445), 1,
      sym__sgl_flw_seq_tal,
    STATE(1468), 1,
      sym__r_sgl_flw_col_dat,
    STATE(1002), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(1006), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(761), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [3913] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(747), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(751), 1,
      sym__r_flw_map_bgn,
    ACTIONS(753), 1,
      sym__r_flw_key_bgn,
    ACTIONS(755), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(757), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(759), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(763), 1,
      sym__r_tag,
    ACTIONS(765), 1,
      sym__r_acr_bgn,
    ACTIONS(767), 1,
      sym__r_als_bgn,
    ACTIONS(771), 1,
      sym__r_flw_seq_end,
    STATE(304), 1,
      sym__r_sgl_prp,
    STATE(346), 1,
      sym__sgl_flw_seq_tal,
    STATE(498), 1,
      sym__r_acr,
    STATE(1112), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1120), 1,
      sym__r_sgl_pln_flw,
    STATE(1139), 1,
      sym__r_sgl_flw_seq,
    STATE(1140), 1,
      sym__r_sgl_flw_map,
    STATE(1142), 1,
      sym__r_sgl_dqt_str,
    STATE(1143), 1,
      sym__r_sgl_sqt_str,
    STATE(1170), 1,
      sym__r_als,
    STATE(1256), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1283), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1437), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1502), 1,
      sym__r_sgl_flw_col_dat,
    STATE(1002), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(1006), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(761), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [4007] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(697), 1,
      sym__b_blk_key_bgn,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    ACTIONS(701), 1,
      sym__b_flw_seq_bgn,
    ACTIONS(703), 1,
      sym__b_flw_map_bgn,
    ACTIONS(705), 1,
      sym__b_dqt_str_bgn,
    ACTIONS(707), 1,
      sym__b_sqt_str_bgn,
    ACTIONS(711), 1,
      sym__b_tag,
    ACTIONS(713), 1,
      sym__b_acr_bgn,
    ACTIONS(715), 1,
      sym__b_als_bgn,
    ACTIONS(773), 1,
      sym__bl,
    STATE(191), 1,
      sym__b_sgl_prp,
    STATE(219), 1,
      sym__b_acr,
    STATE(330), 1,
      sym__b_sgl_pln_blk,
    STATE(338), 1,
      sym__b_sgl_sqt_str,
    STATE(355), 1,
      sym__b_sgl_dqt_str,
    STATE(356), 1,
      sym__b_sgl_flw_map,
    STATE(363), 1,
      sym__b_sgl_flw_seq,
    STATE(367), 1,
      sym__b_als,
    STATE(370), 1,
      sym__b_blk_key_itm,
    STATE(403), 1,
      sym__b_blk_imp_itm_key_only,
    STATE(404), 1,
      sym__b_blk_val_itm,
    STATE(407), 1,
      sym__b_blk_exp_itm,
    STATE(161), 2,
      sym__b_blk_map_itm,
      aux_sym__r_blk_map_repeat1,
    ACTIONS(709), 5,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
    STATE(302), 7,
      sym__b_sgl_prp_val,
      sym__b_sgl_flw_seq_val,
      sym__b_sgl_flw_map_val,
      sym__b_sgl_dqt_str_val,
      sym__b_sgl_sqt_str_val,
      sym__b_sgl_pln_blk_val,
      sym__b_als_val,
  [4097] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(721), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(723), 1,
      sym__r_flw_map_bgn,
    ACTIONS(727), 1,
      sym__r_flw_key_bgn,
    ACTIONS(729), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(731), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(733), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(737), 1,
      sym__r_tag,
    ACTIONS(739), 1,
      sym__r_acr_bgn,
    ACTIONS(741), 1,
      sym__r_als_bgn,
    ACTIONS(775), 1,
      sym__r_flw_map_end,
    STATE(309), 1,
      sym__r_sgl_prp,
    STATE(495), 1,
      sym__r_acr,
    STATE(1015), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1166), 1,
      sym__r_sgl_pln_flw,
    STATE(1174), 1,
      sym__r_als,
    STATE(1180), 1,
      sym__r_sgl_flw_seq,
    STATE(1181), 1,
      sym__r_sgl_flw_map,
    STATE(1182), 1,
      sym__r_sgl_dqt_str,
    STATE(1183), 1,
      sym__r_sgl_sqt_str,
    STATE(1287), 1,
      sym__sgl_flw_map_tal,
    STATE(1405), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1435), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1436), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1467), 1,
      sym__r_sgl_flw_col_dat,
    STATE(974), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(976), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(735), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [4191] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(747), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(751), 1,
      sym__r_flw_map_bgn,
    ACTIONS(753), 1,
      sym__r_flw_key_bgn,
    ACTIONS(755), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(757), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(759), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(763), 1,
      sym__r_tag,
    ACTIONS(765), 1,
      sym__r_acr_bgn,
    ACTIONS(767), 1,
      sym__r_als_bgn,
    ACTIONS(777), 1,
      sym__r_flw_seq_end,
    STATE(304), 1,
      sym__r_sgl_prp,
    STATE(498), 1,
      sym__r_acr,
    STATE(1112), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1120), 1,
      sym__r_sgl_pln_flw,
    STATE(1139), 1,
      sym__r_sgl_flw_seq,
    STATE(1140), 1,
      sym__r_sgl_flw_map,
    STATE(1142), 1,
      sym__r_sgl_dqt_str,
    STATE(1143), 1,
      sym__r_sgl_sqt_str,
    STATE(1170), 1,
      sym__r_als,
    STATE(1256), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1283), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1437), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1479), 1,
      sym__r_sgl_flw_col_dat,
    STATE(1517), 1,
      sym__sgl_flw_seq_tal,
    STATE(1002), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(1006), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(761), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [4285] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(697), 1,
      sym__b_blk_key_bgn,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    ACTIONS(701), 1,
      sym__b_flw_seq_bgn,
    ACTIONS(703), 1,
      sym__b_flw_map_bgn,
    ACTIONS(705), 1,
      sym__b_dqt_str_bgn,
    ACTIONS(707), 1,
      sym__b_sqt_str_bgn,
    ACTIONS(711), 1,
      sym__b_tag,
    ACTIONS(713), 1,
      sym__b_acr_bgn,
    ACTIONS(715), 1,
      sym__b_als_bgn,
    ACTIONS(779), 1,
      sym__bl,
    STATE(191), 1,
      sym__b_sgl_prp,
    STATE(219), 1,
      sym__b_acr,
    STATE(330), 1,
      sym__b_sgl_pln_blk,
    STATE(338), 1,
      sym__b_sgl_sqt_str,
    STATE(355), 1,
      sym__b_sgl_dqt_str,
    STATE(356), 1,
      sym__b_sgl_flw_map,
    STATE(363), 1,
      sym__b_sgl_flw_seq,
    STATE(367), 1,
      sym__b_als,
    STATE(370), 1,
      sym__b_blk_key_itm,
    STATE(403), 1,
      sym__b_blk_imp_itm_key_only,
    STATE(404), 1,
      sym__b_blk_val_itm,
    STATE(407), 1,
      sym__b_blk_exp_itm,
    STATE(161), 2,
      sym__b_blk_map_itm,
      aux_sym__r_blk_map_repeat1,
    ACTIONS(709), 5,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
    STATE(302), 7,
      sym__b_sgl_prp_val,
      sym__b_sgl_flw_seq_val,
      sym__b_sgl_flw_map_val,
      sym__b_sgl_dqt_str_val,
      sym__b_sgl_sqt_str_val,
      sym__b_sgl_pln_blk_val,
      sym__b_als_val,
  [4375] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(721), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(723), 1,
      sym__r_flw_map_bgn,
    ACTIONS(727), 1,
      sym__r_flw_key_bgn,
    ACTIONS(729), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(731), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(733), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(737), 1,
      sym__r_tag,
    ACTIONS(739), 1,
      sym__r_acr_bgn,
    ACTIONS(741), 1,
      sym__r_als_bgn,
    ACTIONS(781), 1,
      sym__r_flw_map_end,
    STATE(309), 1,
      sym__r_sgl_prp,
    STATE(495), 1,
      sym__r_acr,
    STATE(1015), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1166), 1,
      sym__r_sgl_pln_flw,
    STATE(1174), 1,
      sym__r_als,
    STATE(1180), 1,
      sym__r_sgl_flw_seq,
    STATE(1181), 1,
      sym__r_sgl_flw_map,
    STATE(1182), 1,
      sym__r_sgl_dqt_str,
    STATE(1183), 1,
      sym__r_sgl_sqt_str,
    STATE(1192), 1,
      sym__sgl_flw_map_tal,
    STATE(1405), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1435), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1436), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1491), 1,
      sym__r_sgl_flw_col_dat,
    STATE(974), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(976), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(735), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [4469] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(462), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(471), 1,
      sym__br_flw_map_bgn,
    ACTIONS(475), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(479), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(485), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(487), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(597), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(599), 1,
      sym__r_flw_map_bgn,
    ACTIONS(601), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(603), 1,
      sym__r_sqt_str_bgn,
    STATE(1066), 2,
      sym__r_flw_seq,
      sym__br_flw_seq,
    STATE(1067), 2,
      sym__r_flw_map,
      sym__br_flw_map,
    STATE(1068), 2,
      sym__r_dqt_str,
      sym__br_dqt_str,
    STATE(1069), 2,
      sym__r_sqt_str,
      sym__br_sqt_str,
    ACTIONS(365), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    STATE(889), 4,
      sym__r_sgl_pln_flw,
      sym__br_sgl_pln_flw,
      sym__r_mtl_pln_flw,
      sym__br_mtl_pln_flw,
    ACTIONS(481), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(483), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
  [4545] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(697), 1,
      sym__b_blk_key_bgn,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    ACTIONS(701), 1,
      sym__b_flw_seq_bgn,
    ACTIONS(703), 1,
      sym__b_flw_map_bgn,
    ACTIONS(705), 1,
      sym__b_dqt_str_bgn,
    ACTIONS(707), 1,
      sym__b_sqt_str_bgn,
    ACTIONS(711), 1,
      sym__b_tag,
    ACTIONS(713), 1,
      sym__b_acr_bgn,
    ACTIONS(715), 1,
      sym__b_als_bgn,
    ACTIONS(783), 1,
      sym__bl,
    STATE(191), 1,
      sym__b_sgl_prp,
    STATE(219), 1,
      sym__b_acr,
    STATE(330), 1,
      sym__b_sgl_pln_blk,
    STATE(338), 1,
      sym__b_sgl_sqt_str,
    STATE(355), 1,
      sym__b_sgl_dqt_str,
    STATE(356), 1,
      sym__b_sgl_flw_map,
    STATE(363), 1,
      sym__b_sgl_flw_seq,
    STATE(367), 1,
      sym__b_als,
    STATE(370), 1,
      sym__b_blk_key_itm,
    STATE(403), 1,
      sym__b_blk_imp_itm_key_only,
    STATE(404), 1,
      sym__b_blk_val_itm,
    STATE(407), 1,
      sym__b_blk_exp_itm,
    STATE(161), 2,
      sym__b_blk_map_itm,
      aux_sym__r_blk_map_repeat1,
    ACTIONS(709), 5,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
    STATE(302), 7,
      sym__b_sgl_prp_val,
      sym__b_sgl_flw_seq_val,
      sym__b_sgl_flw_map_val,
      sym__b_sgl_dqt_str_val,
      sym__b_sgl_sqt_str_val,
      sym__b_sgl_pln_blk_val,
      sym__b_als_val,
  [4635] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(697), 1,
      sym__b_blk_key_bgn,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    ACTIONS(701), 1,
      sym__b_flw_seq_bgn,
    ACTIONS(703), 1,
      sym__b_flw_map_bgn,
    ACTIONS(705), 1,
      sym__b_dqt_str_bgn,
    ACTIONS(707), 1,
      sym__b_sqt_str_bgn,
    ACTIONS(711), 1,
      sym__b_tag,
    ACTIONS(713), 1,
      sym__b_acr_bgn,
    ACTIONS(715), 1,
      sym__b_als_bgn,
    ACTIONS(785), 1,
      sym__bl,
    STATE(191), 1,
      sym__b_sgl_prp,
    STATE(219), 1,
      sym__b_acr,
    STATE(330), 1,
      sym__b_sgl_pln_blk,
    STATE(338), 1,
      sym__b_sgl_sqt_str,
    STATE(355), 1,
      sym__b_sgl_dqt_str,
    STATE(356), 1,
      sym__b_sgl_flw_map,
    STATE(363), 1,
      sym__b_sgl_flw_seq,
    STATE(367), 1,
      sym__b_als,
    STATE(370), 1,
      sym__b_blk_key_itm,
    STATE(403), 1,
      sym__b_blk_imp_itm_key_only,
    STATE(404), 1,
      sym__b_blk_val_itm,
    STATE(407), 1,
      sym__b_blk_exp_itm,
    STATE(145), 2,
      sym__b_blk_map_itm,
      aux_sym__r_blk_map_repeat1,
    ACTIONS(709), 5,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
    STATE(302), 7,
      sym__b_sgl_prp_val,
      sym__b_sgl_flw_seq_val,
      sym__b_sgl_flw_map_val,
      sym__b_sgl_dqt_str_val,
      sym__b_sgl_sqt_str_val,
      sym__b_sgl_pln_blk_val,
      sym__b_als_val,
  [4725] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(462), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(471), 1,
      sym__br_flw_map_bgn,
    ACTIONS(475), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(479), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(485), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(487), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(597), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(599), 1,
      sym__r_flw_map_bgn,
    ACTIONS(601), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(603), 1,
      sym__r_sqt_str_bgn,
    STATE(1070), 2,
      sym__r_flw_seq,
      sym__br_flw_seq,
    STATE(1071), 2,
      sym__r_flw_map,
      sym__br_flw_map,
    STATE(1072), 2,
      sym__r_dqt_str,
      sym__br_dqt_str,
    STATE(1073), 2,
      sym__r_sqt_str,
      sym__br_sqt_str,
    ACTIONS(371), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    STATE(885), 4,
      sym__r_sgl_pln_flw,
      sym__br_sgl_pln_flw,
      sym__r_mtl_pln_flw,
      sym__br_mtl_pln_flw,
    ACTIONS(481), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(483), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
  [4801] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(747), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(751), 1,
      sym__r_flw_map_bgn,
    ACTIONS(753), 1,
      sym__r_flw_key_bgn,
    ACTIONS(755), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(757), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(759), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(763), 1,
      sym__r_tag,
    ACTIONS(765), 1,
      sym__r_acr_bgn,
    ACTIONS(767), 1,
      sym__r_als_bgn,
    ACTIONS(787), 1,
      sym__r_flw_seq_end,
    STATE(304), 1,
      sym__r_sgl_prp,
    STATE(498), 1,
      sym__r_acr,
    STATE(1112), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1120), 1,
      sym__r_sgl_pln_flw,
    STATE(1139), 1,
      sym__r_sgl_flw_seq,
    STATE(1140), 1,
      sym__r_sgl_flw_map,
    STATE(1142), 1,
      sym__r_sgl_dqt_str,
    STATE(1143), 1,
      sym__r_sgl_sqt_str,
    STATE(1170), 1,
      sym__r_als,
    STATE(1187), 1,
      sym__sgl_flw_seq_tal,
    STATE(1256), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1283), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1437), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1492), 1,
      sym__r_sgl_flw_col_dat,
    STATE(1002), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(1006), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(761), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [4895] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(697), 1,
      sym__b_blk_key_bgn,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    ACTIONS(701), 1,
      sym__b_flw_seq_bgn,
    ACTIONS(703), 1,
      sym__b_flw_map_bgn,
    ACTIONS(705), 1,
      sym__b_dqt_str_bgn,
    ACTIONS(707), 1,
      sym__b_sqt_str_bgn,
    ACTIONS(711), 1,
      sym__b_tag,
    ACTIONS(713), 1,
      sym__b_acr_bgn,
    ACTIONS(715), 1,
      sym__b_als_bgn,
    ACTIONS(789), 1,
      sym__bl,
    STATE(191), 1,
      sym__b_sgl_prp,
    STATE(219), 1,
      sym__b_acr,
    STATE(330), 1,
      sym__b_sgl_pln_blk,
    STATE(338), 1,
      sym__b_sgl_sqt_str,
    STATE(355), 1,
      sym__b_sgl_dqt_str,
    STATE(356), 1,
      sym__b_sgl_flw_map,
    STATE(363), 1,
      sym__b_sgl_flw_seq,
    STATE(367), 1,
      sym__b_als,
    STATE(370), 1,
      sym__b_blk_key_itm,
    STATE(403), 1,
      sym__b_blk_imp_itm_key_only,
    STATE(404), 1,
      sym__b_blk_val_itm,
    STATE(407), 1,
      sym__b_blk_exp_itm,
    STATE(142), 2,
      sym__b_blk_map_itm,
      aux_sym__r_blk_map_repeat1,
    ACTIONS(709), 5,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
    STATE(302), 7,
      sym__b_sgl_prp_val,
      sym__b_sgl_flw_seq_val,
      sym__b_sgl_flw_map_val,
      sym__b_sgl_dqt_str_val,
      sym__b_sgl_sqt_str_val,
      sym__b_sgl_pln_blk_val,
      sym__b_als_val,
  [4985] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(628), 1,
      sym__br_acr_bgn,
    ACTIONS(791), 1,
      sym__r_acr_bgn,
    STATE(174), 2,
      sym__r_acr,
      sym__br_acr,
    ACTIONS(621), 32,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_als_bgn,
  [5033] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(697), 1,
      sym__b_blk_key_bgn,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    ACTIONS(701), 1,
      sym__b_flw_seq_bgn,
    ACTIONS(703), 1,
      sym__b_flw_map_bgn,
    ACTIONS(705), 1,
      sym__b_dqt_str_bgn,
    ACTIONS(707), 1,
      sym__b_sqt_str_bgn,
    ACTIONS(711), 1,
      sym__b_tag,
    ACTIONS(713), 1,
      sym__b_acr_bgn,
    ACTIONS(715), 1,
      sym__b_als_bgn,
    ACTIONS(793), 1,
      sym__bl,
    STATE(191), 1,
      sym__b_sgl_prp,
    STATE(219), 1,
      sym__b_acr,
    STATE(330), 1,
      sym__b_sgl_pln_blk,
    STATE(338), 1,
      sym__b_sgl_sqt_str,
    STATE(355), 1,
      sym__b_sgl_dqt_str,
    STATE(356), 1,
      sym__b_sgl_flw_map,
    STATE(363), 1,
      sym__b_sgl_flw_seq,
    STATE(367), 1,
      sym__b_als,
    STATE(370), 1,
      sym__b_blk_key_itm,
    STATE(403), 1,
      sym__b_blk_imp_itm_key_only,
    STATE(404), 1,
      sym__b_blk_val_itm,
    STATE(407), 1,
      sym__b_blk_exp_itm,
    STATE(139), 2,
      sym__b_blk_map_itm,
      aux_sym__r_blk_map_repeat1,
    ACTIONS(709), 5,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
    STATE(302), 7,
      sym__b_sgl_prp_val,
      sym__b_sgl_flw_seq_val,
      sym__b_sgl_flw_map_val,
      sym__b_sgl_dqt_str_val,
      sym__b_sgl_sqt_str_val,
      sym__b_sgl_pln_blk_val,
      sym__b_als_val,
  [5123] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(503), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(507), 1,
      sym__br_flw_map_bgn,
    ACTIONS(511), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(515), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(521), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(523), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(583), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(585), 1,
      sym__r_flw_map_bgn,
    ACTIONS(589), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(591), 1,
      sym__r_sqt_str_bgn,
    STATE(1029), 2,
      sym__r_sqt_str,
      sym__br_sqt_str,
    STATE(1030), 2,
      sym__r_dqt_str,
      sym__br_dqt_str,
    STATE(1031), 2,
      sym__r_flw_map,
      sym__br_flw_map,
    STATE(1032), 2,
      sym__r_flw_seq,
      sym__br_flw_seq,
    ACTIONS(371), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    STATE(951), 4,
      sym__r_sgl_pln_flw,
      sym__br_sgl_pln_flw,
      sym__r_mtl_pln_flw,
      sym__br_mtl_pln_flw,
    ACTIONS(517), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(519), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
  [5199] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(747), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(751), 1,
      sym__r_flw_map_bgn,
    ACTIONS(753), 1,
      sym__r_flw_key_bgn,
    ACTIONS(755), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(757), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(759), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(763), 1,
      sym__r_tag,
    ACTIONS(765), 1,
      sym__r_acr_bgn,
    ACTIONS(767), 1,
      sym__r_als_bgn,
    ACTIONS(795), 1,
      sym__r_flw_seq_end,
    STATE(304), 1,
      sym__r_sgl_prp,
    STATE(498), 1,
      sym__r_acr,
    STATE(1112), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1120), 1,
      sym__r_sgl_pln_flw,
    STATE(1139), 1,
      sym__r_sgl_flw_seq,
    STATE(1140), 1,
      sym__r_sgl_flw_map,
    STATE(1142), 1,
      sym__r_sgl_dqt_str,
    STATE(1143), 1,
      sym__r_sgl_sqt_str,
    STATE(1170), 1,
      sym__r_als,
    STATE(1256), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1283), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1346), 1,
      sym__sgl_flw_seq_tal,
    STATE(1437), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1510), 1,
      sym__r_sgl_flw_col_dat,
    STATE(1002), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(1006), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(761), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [5293] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(697), 1,
      sym__b_blk_key_bgn,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    ACTIONS(701), 1,
      sym__b_flw_seq_bgn,
    ACTIONS(703), 1,
      sym__b_flw_map_bgn,
    ACTIONS(705), 1,
      sym__b_dqt_str_bgn,
    ACTIONS(707), 1,
      sym__b_sqt_str_bgn,
    ACTIONS(711), 1,
      sym__b_tag,
    ACTIONS(713), 1,
      sym__b_acr_bgn,
    ACTIONS(715), 1,
      sym__b_als_bgn,
    ACTIONS(797), 1,
      sym__bl,
    STATE(191), 1,
      sym__b_sgl_prp,
    STATE(219), 1,
      sym__b_acr,
    STATE(330), 1,
      sym__b_sgl_pln_blk,
    STATE(338), 1,
      sym__b_sgl_sqt_str,
    STATE(355), 1,
      sym__b_sgl_dqt_str,
    STATE(356), 1,
      sym__b_sgl_flw_map,
    STATE(363), 1,
      sym__b_sgl_flw_seq,
    STATE(367), 1,
      sym__b_als,
    STATE(370), 1,
      sym__b_blk_key_itm,
    STATE(403), 1,
      sym__b_blk_imp_itm_key_only,
    STATE(404), 1,
      sym__b_blk_val_itm,
    STATE(407), 1,
      sym__b_blk_exp_itm,
    STATE(161), 2,
      sym__b_blk_map_itm,
      aux_sym__r_blk_map_repeat1,
    ACTIONS(709), 5,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
    STATE(302), 7,
      sym__b_sgl_prp_val,
      sym__b_sgl_flw_seq_val,
      sym__b_sgl_flw_map_val,
      sym__b_sgl_dqt_str_val,
      sym__b_sgl_sqt_str_val,
      sym__b_sgl_pln_blk_val,
      sym__b_als_val,
  [5383] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(721), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(723), 1,
      sym__r_flw_map_bgn,
    ACTIONS(727), 1,
      sym__r_flw_key_bgn,
    ACTIONS(729), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(731), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(733), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(737), 1,
      sym__r_tag,
    ACTIONS(739), 1,
      sym__r_acr_bgn,
    ACTIONS(741), 1,
      sym__r_als_bgn,
    ACTIONS(799), 1,
      sym__r_flw_map_end,
    STATE(309), 1,
      sym__r_sgl_prp,
    STATE(388), 1,
      sym__sgl_flw_map_tal,
    STATE(495), 1,
      sym__r_acr,
    STATE(1015), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1166), 1,
      sym__r_sgl_pln_flw,
    STATE(1174), 1,
      sym__r_als,
    STATE(1180), 1,
      sym__r_sgl_flw_seq,
    STATE(1181), 1,
      sym__r_sgl_flw_map,
    STATE(1182), 1,
      sym__r_sgl_dqt_str,
    STATE(1183), 1,
      sym__r_sgl_sqt_str,
    STATE(1405), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1435), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1436), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1488), 1,
      sym__r_sgl_flw_col_dat,
    STATE(974), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(976), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(735), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [5477] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(747), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(751), 1,
      sym__r_flw_map_bgn,
    ACTIONS(753), 1,
      sym__r_flw_key_bgn,
    ACTIONS(755), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(757), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(759), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(763), 1,
      sym__r_tag,
    ACTIONS(765), 1,
      sym__r_acr_bgn,
    ACTIONS(767), 1,
      sym__r_als_bgn,
    ACTIONS(801), 1,
      sym__r_flw_seq_end,
    STATE(304), 1,
      sym__r_sgl_prp,
    STATE(392), 1,
      sym__sgl_flw_seq_tal,
    STATE(498), 1,
      sym__r_acr,
    STATE(1112), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1120), 1,
      sym__r_sgl_pln_flw,
    STATE(1139), 1,
      sym__r_sgl_flw_seq,
    STATE(1140), 1,
      sym__r_sgl_flw_map,
    STATE(1142), 1,
      sym__r_sgl_dqt_str,
    STATE(1143), 1,
      sym__r_sgl_sqt_str,
    STATE(1170), 1,
      sym__r_als,
    STATE(1256), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1283), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1437), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1502), 1,
      sym__r_sgl_flw_col_dat,
    STATE(1002), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(1006), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(761), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [5571] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(697), 1,
      sym__b_blk_key_bgn,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    ACTIONS(701), 1,
      sym__b_flw_seq_bgn,
    ACTIONS(703), 1,
      sym__b_flw_map_bgn,
    ACTIONS(705), 1,
      sym__b_dqt_str_bgn,
    ACTIONS(707), 1,
      sym__b_sqt_str_bgn,
    ACTIONS(711), 1,
      sym__b_tag,
    ACTIONS(713), 1,
      sym__b_acr_bgn,
    ACTIONS(715), 1,
      sym__b_als_bgn,
    ACTIONS(803), 1,
      sym__bl,
    STATE(191), 1,
      sym__b_sgl_prp,
    STATE(219), 1,
      sym__b_acr,
    STATE(330), 1,
      sym__b_sgl_pln_blk,
    STATE(338), 1,
      sym__b_sgl_sqt_str,
    STATE(355), 1,
      sym__b_sgl_dqt_str,
    STATE(356), 1,
      sym__b_sgl_flw_map,
    STATE(363), 1,
      sym__b_sgl_flw_seq,
    STATE(367), 1,
      sym__b_als,
    STATE(370), 1,
      sym__b_blk_key_itm,
    STATE(403), 1,
      sym__b_blk_imp_itm_key_only,
    STATE(404), 1,
      sym__b_blk_val_itm,
    STATE(407), 1,
      sym__b_blk_exp_itm,
    STATE(162), 2,
      sym__b_blk_map_itm,
      aux_sym__r_blk_map_repeat1,
    ACTIONS(709), 5,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
    STATE(302), 7,
      sym__b_sgl_prp_val,
      sym__b_sgl_flw_seq_val,
      sym__b_sgl_flw_map_val,
      sym__b_sgl_dqt_str_val,
      sym__b_sgl_sqt_str_val,
      sym__b_sgl_pln_blk_val,
      sym__b_als_val,
  [5661] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(721), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(723), 1,
      sym__r_flw_map_bgn,
    ACTIONS(727), 1,
      sym__r_flw_key_bgn,
    ACTIONS(729), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(731), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(733), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(737), 1,
      sym__r_tag,
    ACTIONS(739), 1,
      sym__r_acr_bgn,
    ACTIONS(741), 1,
      sym__r_als_bgn,
    ACTIONS(805), 1,
      sym__r_flw_map_end,
    STATE(309), 1,
      sym__r_sgl_prp,
    STATE(495), 1,
      sym__r_acr,
    STATE(1015), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1166), 1,
      sym__r_sgl_pln_flw,
    STATE(1174), 1,
      sym__r_als,
    STATE(1180), 1,
      sym__r_sgl_flw_seq,
    STATE(1181), 1,
      sym__r_sgl_flw_map,
    STATE(1182), 1,
      sym__r_sgl_dqt_str,
    STATE(1183), 1,
      sym__r_sgl_sqt_str,
    STATE(1359), 1,
      sym__sgl_flw_map_tal,
    STATE(1405), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1435), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1436), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1511), 1,
      sym__r_sgl_flw_col_dat,
    STATE(974), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(976), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(735), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [5755] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(697), 1,
      sym__b_blk_key_bgn,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    ACTIONS(701), 1,
      sym__b_flw_seq_bgn,
    ACTIONS(703), 1,
      sym__b_flw_map_bgn,
    ACTIONS(705), 1,
      sym__b_dqt_str_bgn,
    ACTIONS(707), 1,
      sym__b_sqt_str_bgn,
    ACTIONS(711), 1,
      sym__b_tag,
    ACTIONS(713), 1,
      sym__b_acr_bgn,
    ACTIONS(715), 1,
      sym__b_als_bgn,
    ACTIONS(807), 1,
      sym__bl,
    STATE(191), 1,
      sym__b_sgl_prp,
    STATE(219), 1,
      sym__b_acr,
    STATE(330), 1,
      sym__b_sgl_pln_blk,
    STATE(338), 1,
      sym__b_sgl_sqt_str,
    STATE(355), 1,
      sym__b_sgl_dqt_str,
    STATE(356), 1,
      sym__b_sgl_flw_map,
    STATE(363), 1,
      sym__b_sgl_flw_seq,
    STATE(367), 1,
      sym__b_als,
    STATE(370), 1,
      sym__b_blk_key_itm,
    STATE(403), 1,
      sym__b_blk_imp_itm_key_only,
    STATE(404), 1,
      sym__b_blk_val_itm,
    STATE(407), 1,
      sym__b_blk_exp_itm,
    STATE(154), 2,
      sym__b_blk_map_itm,
      aux_sym__r_blk_map_repeat1,
    ACTIONS(709), 5,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
    STATE(302), 7,
      sym__b_sgl_prp_val,
      sym__b_sgl_flw_seq_val,
      sym__b_sgl_flw_map_val,
      sym__b_sgl_dqt_str_val,
      sym__b_sgl_sqt_str_val,
      sym__b_sgl_pln_blk_val,
      sym__b_als_val,
  [5845] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(503), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(507), 1,
      sym__br_flw_map_bgn,
    ACTIONS(511), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(515), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(521), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(523), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(583), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(585), 1,
      sym__r_flw_map_bgn,
    ACTIONS(589), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(591), 1,
      sym__r_sqt_str_bgn,
    STATE(1033), 2,
      sym__r_sqt_str,
      sym__br_sqt_str,
    STATE(1034), 2,
      sym__r_dqt_str,
      sym__br_dqt_str,
    STATE(1035), 2,
      sym__r_flw_map,
      sym__br_flw_map,
    STATE(1036), 2,
      sym__r_flw_seq,
      sym__br_flw_seq,
    ACTIONS(365), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    STATE(950), 4,
      sym__r_sgl_pln_flw,
      sym__br_sgl_pln_flw,
      sym__r_mtl_pln_flw,
      sym__br_mtl_pln_flw,
    ACTIONS(517), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(519), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
  [5921] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(809), 1,
      sym__b_blk_key_bgn,
    ACTIONS(812), 1,
      sym__b_blk_val_bgn,
    ACTIONS(815), 1,
      sym__b_flw_seq_bgn,
    ACTIONS(818), 1,
      sym__b_flw_map_bgn,
    ACTIONS(821), 1,
      sym__b_dqt_str_bgn,
    ACTIONS(824), 1,
      sym__b_sqt_str_bgn,
    ACTIONS(830), 1,
      sym__b_tag,
    ACTIONS(833), 1,
      sym__b_acr_bgn,
    ACTIONS(836), 1,
      sym__b_als_bgn,
    ACTIONS(839), 1,
      sym__bl,
    STATE(191), 1,
      sym__b_sgl_prp,
    STATE(219), 1,
      sym__b_acr,
    STATE(330), 1,
      sym__b_sgl_pln_blk,
    STATE(338), 1,
      sym__b_sgl_sqt_str,
    STATE(355), 1,
      sym__b_sgl_dqt_str,
    STATE(356), 1,
      sym__b_sgl_flw_map,
    STATE(363), 1,
      sym__b_sgl_flw_seq,
    STATE(367), 1,
      sym__b_als,
    STATE(370), 1,
      sym__b_blk_key_itm,
    STATE(403), 1,
      sym__b_blk_imp_itm_key_only,
    STATE(404), 1,
      sym__b_blk_val_itm,
    STATE(407), 1,
      sym__b_blk_exp_itm,
    STATE(161), 2,
      sym__b_blk_map_itm,
      aux_sym__r_blk_map_repeat1,
    ACTIONS(827), 5,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
    STATE(302), 7,
      sym__b_sgl_prp_val,
      sym__b_sgl_flw_seq_val,
      sym__b_sgl_flw_map_val,
      sym__b_sgl_dqt_str_val,
      sym__b_sgl_sqt_str_val,
      sym__b_sgl_pln_blk_val,
      sym__b_als_val,
  [6011] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(697), 1,
      sym__b_blk_key_bgn,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    ACTIONS(701), 1,
      sym__b_flw_seq_bgn,
    ACTIONS(703), 1,
      sym__b_flw_map_bgn,
    ACTIONS(705), 1,
      sym__b_dqt_str_bgn,
    ACTIONS(707), 1,
      sym__b_sqt_str_bgn,
    ACTIONS(711), 1,
      sym__b_tag,
    ACTIONS(713), 1,
      sym__b_acr_bgn,
    ACTIONS(715), 1,
      sym__b_als_bgn,
    ACTIONS(841), 1,
      sym__bl,
    STATE(191), 1,
      sym__b_sgl_prp,
    STATE(219), 1,
      sym__b_acr,
    STATE(330), 1,
      sym__b_sgl_pln_blk,
    STATE(338), 1,
      sym__b_sgl_sqt_str,
    STATE(355), 1,
      sym__b_sgl_dqt_str,
    STATE(356), 1,
      sym__b_sgl_flw_map,
    STATE(363), 1,
      sym__b_sgl_flw_seq,
    STATE(367), 1,
      sym__b_als,
    STATE(370), 1,
      sym__b_blk_key_itm,
    STATE(403), 1,
      sym__b_blk_imp_itm_key_only,
    STATE(404), 1,
      sym__b_blk_val_itm,
    STATE(407), 1,
      sym__b_blk_exp_itm,
    STATE(161), 2,
      sym__b_blk_map_itm,
      aux_sym__r_blk_map_repeat1,
    ACTIONS(709), 5,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
    STATE(302), 7,
      sym__b_sgl_prp_val,
      sym__b_sgl_flw_seq_val,
      sym__b_sgl_flw_map_val,
      sym__b_sgl_dqt_str_val,
      sym__b_sgl_sqt_str_val,
      sym__b_sgl_pln_blk_val,
      sym__b_als_val,
  [6101] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(381), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(390), 1,
      sym__br_flw_map_bgn,
    ACTIONS(394), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(398), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(404), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(406), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(571), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(575), 1,
      sym__r_flw_map_bgn,
    ACTIONS(577), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(579), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(371), 2,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
    STATE(1432), 2,
      sym__r_flw_seq,
      sym__br_flw_seq,
    STATE(1438), 2,
      sym__r_flw_map,
      sym__br_flw_map,
    STATE(1439), 2,
      sym__r_dqt_str,
      sym__br_dqt_str,
    STATE(1440), 2,
      sym__r_sqt_str,
      sym__br_sqt_str,
    STATE(1305), 4,
      sym__r_sgl_pln_flw,
      sym__br_sgl_pln_flw,
      sym__r_mtl_pln_flw,
      sym__br_mtl_pln_flw,
    ACTIONS(400), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(402), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
  [6175] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(381), 1,
      sym__br_flw_seq_bgn,
    ACTIONS(390), 1,
      sym__br_flw_map_bgn,
    ACTIONS(394), 1,
      sym__br_dqt_str_bgn,
    ACTIONS(398), 1,
      sym__br_sqt_str_bgn,
    ACTIONS(404), 1,
      sym__r_mtl_pln_str_flw,
    ACTIONS(406), 1,
      sym__br_mtl_pln_str_flw,
    ACTIONS(571), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(575), 1,
      sym__r_flw_map_bgn,
    ACTIONS(577), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(579), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(365), 2,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
    STATE(1276), 2,
      sym__r_flw_seq,
      sym__br_flw_seq,
    STATE(1409), 2,
      sym__r_flw_map,
      sym__br_flw_map,
    STATE(1410), 2,
      sym__r_dqt_str,
      sym__br_dqt_str,
    STATE(1413), 2,
      sym__r_sqt_str,
      sym__br_sqt_str,
    STATE(1306), 4,
      sym__r_sgl_pln_flw,
      sym__br_sgl_pln_flw,
      sym__r_mtl_pln_flw,
      sym__br_mtl_pln_flw,
    ACTIONS(400), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(402), 5,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
  [6249] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 34,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__r_tag,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [6289] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(747), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(751), 1,
      sym__r_flw_map_bgn,
    ACTIONS(753), 1,
      sym__r_flw_key_bgn,
    ACTIONS(755), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(757), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(759), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(763), 1,
      sym__r_tag,
    ACTIONS(765), 1,
      sym__r_acr_bgn,
    ACTIONS(767), 1,
      sym__r_als_bgn,
    ACTIONS(843), 1,
      sym__r_flw_seq_end,
    STATE(304), 1,
      sym__r_sgl_prp,
    STATE(498), 1,
      sym__r_acr,
    STATE(1120), 1,
      sym__r_sgl_pln_flw,
    STATE(1139), 1,
      sym__r_sgl_flw_seq,
    STATE(1140), 1,
      sym__r_sgl_flw_map,
    STATE(1142), 1,
      sym__r_sgl_dqt_str,
    STATE(1143), 1,
      sym__r_sgl_sqt_str,
    STATE(1170), 1,
      sym__r_als,
    STATE(1256), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1263), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1283), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1437), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1002), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(1006), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(761), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [6377] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(721), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(723), 1,
      sym__r_flw_map_bgn,
    ACTIONS(727), 1,
      sym__r_flw_key_bgn,
    ACTIONS(729), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(731), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(733), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(737), 1,
      sym__r_tag,
    ACTIONS(739), 1,
      sym__r_acr_bgn,
    ACTIONS(741), 1,
      sym__r_als_bgn,
    ACTIONS(845), 1,
      sym__r_flw_map_end,
    STATE(309), 1,
      sym__r_sgl_prp,
    STATE(495), 1,
      sym__r_acr,
    STATE(1166), 1,
      sym__r_sgl_pln_flw,
    STATE(1174), 1,
      sym__r_als,
    STATE(1180), 1,
      sym__r_sgl_flw_seq,
    STATE(1181), 1,
      sym__r_sgl_flw_map,
    STATE(1182), 1,
      sym__r_sgl_dqt_str,
    STATE(1183), 1,
      sym__r_sgl_sqt_str,
    STATE(1365), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1405), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1435), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1436), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(974), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(976), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(735), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [6465] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(656), 1,
      sym__br_tag,
    ACTIONS(847), 1,
      sym__r_tag,
    ACTIONS(621), 32,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [6509] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(721), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(723), 1,
      sym__r_flw_map_bgn,
    ACTIONS(727), 1,
      sym__r_flw_key_bgn,
    ACTIONS(729), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(731), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(733), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(737), 1,
      sym__r_tag,
    ACTIONS(739), 1,
      sym__r_acr_bgn,
    ACTIONS(741), 1,
      sym__r_als_bgn,
    ACTIONS(843), 1,
      sym__r_flw_map_end,
    STATE(309), 1,
      sym__r_sgl_prp,
    STATE(495), 1,
      sym__r_acr,
    STATE(1166), 1,
      sym__r_sgl_pln_flw,
    STATE(1174), 1,
      sym__r_als,
    STATE(1180), 1,
      sym__r_sgl_flw_seq,
    STATE(1181), 1,
      sym__r_sgl_flw_map,
    STATE(1182), 1,
      sym__r_sgl_dqt_str,
    STATE(1183), 1,
      sym__r_sgl_sqt_str,
    STATE(1365), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1405), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1435), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1436), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(974), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(976), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(735), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [6597] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(747), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(751), 1,
      sym__r_flw_map_bgn,
    ACTIONS(753), 1,
      sym__r_flw_key_bgn,
    ACTIONS(755), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(757), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(759), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(763), 1,
      sym__r_tag,
    ACTIONS(765), 1,
      sym__r_acr_bgn,
    ACTIONS(767), 1,
      sym__r_als_bgn,
    ACTIONS(845), 1,
      sym__r_flw_seq_end,
    STATE(304), 1,
      sym__r_sgl_prp,
    STATE(498), 1,
      sym__r_acr,
    STATE(1120), 1,
      sym__r_sgl_pln_flw,
    STATE(1139), 1,
      sym__r_sgl_flw_seq,
    STATE(1140), 1,
      sym__r_sgl_flw_map,
    STATE(1142), 1,
      sym__r_sgl_dqt_str,
    STATE(1143), 1,
      sym__r_sgl_sqt_str,
    STATE(1170), 1,
      sym__r_als,
    STATE(1256), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1263), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1283), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1437), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1002), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(1006), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(761), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [6685] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 33,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [6724] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(747), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(751), 1,
      sym__r_flw_map_bgn,
    ACTIONS(753), 1,
      sym__r_flw_key_bgn,
    ACTIONS(755), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(757), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(759), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(763), 1,
      sym__r_tag,
    ACTIONS(765), 1,
      sym__r_acr_bgn,
    ACTIONS(767), 1,
      sym__r_als_bgn,
    STATE(304), 1,
      sym__r_sgl_prp,
    STATE(498), 1,
      sym__r_acr,
    STATE(1120), 1,
      sym__r_sgl_pln_flw,
    STATE(1139), 1,
      sym__r_sgl_flw_seq,
    STATE(1140), 1,
      sym__r_sgl_flw_map,
    STATE(1142), 1,
      sym__r_sgl_dqt_str,
    STATE(1143), 1,
      sym__r_sgl_sqt_str,
    STATE(1170), 1,
      sym__r_als,
    STATE(1256), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1263), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1283), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1437), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(1002), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(1006), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(761), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [6809] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(659), 33,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [6848] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(668), 33,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [6887] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(721), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(723), 1,
      sym__r_flw_map_bgn,
    ACTIONS(727), 1,
      sym__r_flw_key_bgn,
    ACTIONS(729), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(731), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(733), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(737), 1,
      sym__r_tag,
    ACTIONS(739), 1,
      sym__r_acr_bgn,
    ACTIONS(741), 1,
      sym__r_als_bgn,
    STATE(309), 1,
      sym__r_sgl_prp,
    STATE(495), 1,
      sym__r_acr,
    STATE(1166), 1,
      sym__r_sgl_pln_flw,
    STATE(1174), 1,
      sym__r_als,
    STATE(1180), 1,
      sym__r_sgl_flw_seq,
    STATE(1181), 1,
      sym__r_sgl_flw_map,
    STATE(1182), 1,
      sym__r_sgl_dqt_str,
    STATE(1183), 1,
      sym__r_sgl_sqt_str,
    STATE(1365), 1,
      sym__r_sgl_flw_col_itm,
    STATE(1405), 1,
      sym__r_sgl_flw_exp_par,
    STATE(1435), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1436), 1,
      sym__r_sgl_flw_njl_ann_par,
    STATE(974), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(976), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(735), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [6972] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 33,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [7011] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(668), 32,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
      sym__bl,
  [7049] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 32,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
      sym__bl,
  [7087] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(659), 32,
      sym__br_blk_seq_bgn,
      sym__b_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__r_blk_lit_bgn,
      sym__br_blk_lit_bgn,
      sym__r_blk_fld_bgn,
      sym__br_blk_fld_bgn,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__br_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__br_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__br_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__br_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__br_sgl_pln_str_blk,
      sym__r_mtl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
      sym__bl,
  [7125] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(849), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(853), 1,
      sym__r_flw_map_bgn,
    ACTIONS(855), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(857), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(861), 1,
      sym__r_tag,
    ACTIONS(863), 1,
      sym__r_acr_bgn,
    ACTIONS(865), 1,
      sym__r_als_bgn,
    STATE(369), 1,
      sym__r_sgl_prp,
    STATE(502), 1,
      sym__r_acr,
    STATE(1264), 1,
      sym__r_sgl_sqt_str,
    STATE(1273), 1,
      sym__r_sgl_flw_seq,
    STATE(1274), 1,
      sym__r_sgl_flw_map,
    STATE(1275), 1,
      sym__r_sgl_dqt_str,
    STATE(1282), 1,
      sym__r_sgl_pln_flw,
    STATE(1350), 1,
      sym__r_als,
    ACTIONS(851), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
    ACTIONS(859), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    STATE(1429), 8,
      sym__r_sgl_prp_val,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__sgl_flw_ann_par_tal,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
  [7195] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(867), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(871), 1,
      sym__r_flw_map_bgn,
    ACTIONS(873), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(875), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(879), 1,
      sym__r_tag,
    ACTIONS(881), 1,
      sym__r_acr_bgn,
    ACTIONS(883), 1,
      sym__r_als_bgn,
    STATE(444), 1,
      sym__r_sgl_prp,
    STATE(552), 1,
      sym__r_acr,
    STATE(1430), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1486), 1,
      sym__r_als,
    STATE(1496), 1,
      sym__r_sgl_pln_flw,
    STATE(1513), 1,
      sym__r_sgl_dqt_str,
    STATE(1514), 1,
      sym__r_sgl_sqt_str,
    STATE(1530), 1,
      sym__r_sgl_flw_seq,
    STATE(1553), 1,
      sym__r_sgl_flw_map,
    ACTIONS(869), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
    STATE(1446), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(1449), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(877), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [7269] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(889), 1,
      sym__r_flw_map_bgn,
    ACTIONS(891), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(893), 1,
      sym__r_sqt_str_bgn,
    STATE(340), 1,
      sym__r_sgl_dqt_str,
    STATE(344), 1,
      sym__r_sgl_flw_map,
    STATE(350), 1,
      sym__r_sgl_flw_seq,
    STATE(368), 1,
      sym__r_sgl_sqt_str,
    STATE(377), 1,
      sym__r_sgl_pln_blk,
    ACTIONS(93), 5,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
    ACTIONS(885), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [7325] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(889), 1,
      sym__r_flw_map_bgn,
    ACTIONS(891), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(893), 1,
      sym__r_sqt_str_bgn,
    STATE(323), 1,
      sym__r_sgl_flw_map,
    STATE(328), 1,
      sym__r_sgl_flw_seq,
    STATE(357), 1,
      sym__r_sgl_dqt_str,
    STATE(383), 1,
      sym__r_sgl_sqt_str,
    STATE(390), 1,
      sym__r_sgl_pln_blk,
    ACTIONS(93), 5,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
    ACTIONS(895), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [7381] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(897), 1,
      sym__r_acr_bgn,
    ACTIONS(899), 1,
      sym__br_acr_bgn,
    STATE(230), 2,
      sym__r_acr,
      sym__br_acr,
    ACTIONS(609), 26,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [7423] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(897), 1,
      sym__r_acr_bgn,
    ACTIONS(899), 1,
      sym__br_acr_bgn,
    STATE(233), 2,
      sym__r_acr,
      sym__br_acr,
    ACTIONS(621), 26,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [7465] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(867), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(871), 1,
      sym__r_flw_map_bgn,
    ACTIONS(873), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(875), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(879), 1,
      sym__r_tag,
    ACTIONS(881), 1,
      sym__r_acr_bgn,
    ACTIONS(883), 1,
      sym__r_als_bgn,
    STATE(444), 1,
      sym__r_sgl_prp,
    STATE(552), 1,
      sym__r_acr,
    STATE(1397), 1,
      sym__r_sgl_flw_imp_par,
    STATE(1486), 1,
      sym__r_als,
    STATE(1496), 1,
      sym__r_sgl_pln_flw,
    STATE(1513), 1,
      sym__r_sgl_dqt_str,
    STATE(1514), 1,
      sym__r_sgl_sqt_str,
    STATE(1530), 1,
      sym__r_sgl_flw_seq,
    STATE(1553), 1,
      sym__r_sgl_flw_map,
    ACTIONS(869), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
    STATE(1257), 3,
      sym__r_sgl_prp_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
    STATE(1260), 4,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
    ACTIONS(877), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [7539] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(897), 1,
      sym__r_acr_bgn,
    ACTIONS(899), 1,
      sym__br_acr_bgn,
    STATE(214), 1,
      sym__r_acr,
    STATE(233), 1,
      sym__br_acr,
    ACTIONS(625), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(621), 14,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_flw_map_end,
      sym__br_flw_sep_bgn,
      sym__br_flw_njv_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [7585] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(901), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(903), 1,
      sym__r_flw_map_bgn,
    ACTIONS(907), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(909), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(913), 1,
      sym__r_tag,
    ACTIONS(915), 1,
      sym__r_acr_bgn,
    ACTIONS(917), 1,
      sym__r_als_bgn,
    STATE(345), 1,
      sym__r_sgl_prp,
    STATE(509), 1,
      sym__r_acr,
    STATE(1250), 1,
      sym__r_sgl_pln_flw,
    STATE(1316), 1,
      sym__r_sgl_flw_seq,
    STATE(1319), 1,
      sym__r_sgl_flw_map,
    STATE(1322), 1,
      sym__r_sgl_dqt_str,
    STATE(1325), 1,
      sym__r_sgl_sqt_str,
    STATE(1347), 1,
      sym__r_als,
    ACTIONS(905), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
    ACTIONS(911), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    STATE(1366), 8,
      sym__r_sgl_prp_val,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__sgl_flw_ann_par_tal,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
  [7655] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(849), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(853), 1,
      sym__r_flw_map_bgn,
    ACTIONS(855), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(857), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(861), 1,
      sym__r_tag,
    ACTIONS(863), 1,
      sym__r_acr_bgn,
    ACTIONS(865), 1,
      sym__r_als_bgn,
    STATE(369), 1,
      sym__r_sgl_prp,
    STATE(502), 1,
      sym__r_acr,
    STATE(1264), 1,
      sym__r_sgl_sqt_str,
    STATE(1273), 1,
      sym__r_sgl_flw_seq,
    STATE(1274), 1,
      sym__r_sgl_flw_map,
    STATE(1275), 1,
      sym__r_sgl_dqt_str,
    STATE(1282), 1,
      sym__r_sgl_pln_flw,
    STATE(1350), 1,
      sym__r_als,
    ACTIONS(905), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
    ACTIONS(859), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    STATE(1262), 8,
      sym__r_sgl_prp_val,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__sgl_flw_ann_par_tal,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
  [7725] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(901), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(903), 1,
      sym__r_flw_map_bgn,
    ACTIONS(907), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(909), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(913), 1,
      sym__r_tag,
    ACTIONS(915), 1,
      sym__r_acr_bgn,
    ACTIONS(917), 1,
      sym__r_als_bgn,
    STATE(345), 1,
      sym__r_sgl_prp,
    STATE(509), 1,
      sym__r_acr,
    STATE(1250), 1,
      sym__r_sgl_pln_flw,
    STATE(1316), 1,
      sym__r_sgl_flw_seq,
    STATE(1319), 1,
      sym__r_sgl_flw_map,
    STATE(1322), 1,
      sym__r_sgl_dqt_str,
    STATE(1325), 1,
      sym__r_sgl_sqt_str,
    STATE(1347), 1,
      sym__r_als,
    ACTIONS(851), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
    ACTIONS(911), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    STATE(1396), 8,
      sym__r_sgl_prp_val,
      sym__r_sgl_flw_seq_val,
      sym__r_sgl_flw_map_val,
      sym__sgl_flw_ann_par_tal,
      sym__r_sgl_dqt_str_val,
      sym__r_sgl_sqt_str_val,
      sym__r_sgl_pln_flw_val,
      sym__r_als_val,
  [7795] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(889), 1,
      sym__r_flw_map_bgn,
    ACTIONS(891), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(893), 1,
      sym__r_sqt_str_bgn,
    STATE(372), 1,
      sym__r_sgl_pln_blk,
    STATE(373), 1,
      sym__r_sgl_sqt_str,
    STATE(374), 1,
      sym__r_sgl_dqt_str,
    STATE(375), 1,
      sym__r_sgl_flw_map,
    STATE(376), 1,
      sym__r_sgl_flw_seq,
    ACTIONS(93), 5,
      sym__r_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
    ACTIONS(919), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [7851] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(921), 1,
      sym__r_acr_bgn,
    ACTIONS(923), 1,
      sym__br_acr_bgn,
    STATE(238), 2,
      sym__r_acr,
      sym__br_acr,
    ACTIONS(621), 25,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [7892] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(921), 1,
      sym__r_acr_bgn,
    ACTIONS(923), 1,
      sym__br_acr_bgn,
    STATE(237), 2,
      sym__r_acr,
      sym__br_acr,
    ACTIONS(609), 25,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [7933] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(921), 1,
      sym__r_acr_bgn,
    ACTIONS(923), 1,
      sym__br_acr_bgn,
    STATE(238), 1,
      sym__br_acr,
    STATE(239), 1,
      sym__r_acr,
    ACTIONS(625), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(621), 13,
      sym__br_flw_seq_bgn,
      sym__br_flw_seq_end,
      sym__br_flw_map_bgn,
      sym__br_flw_sep_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [7978] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(925), 1,
      sym__r_acr_bgn,
    ACTIONS(927), 1,
      sym__br_acr_bgn,
    STATE(259), 2,
      sym__r_acr,
      sym__br_acr,
    ACTIONS(621), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8018] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(929), 1,
      sym__r_acr_bgn,
    ACTIONS(931), 1,
      sym__br_acr_bgn,
    STATE(243), 1,
      sym__br_acr,
    STATE(254), 1,
      sym__r_acr,
    ACTIONS(625), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(621), 13,
      sym__br_flw_seq_bgn,
      sym__br_flw_seq_end,
      sym__br_flw_map_bgn,
      sym__br_flw_sep_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8062] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(933), 2,
      sym__r_tag,
      sym__br_tag,
    ACTIONS(621), 26,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8098] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(925), 1,
      sym__r_acr_bgn,
    ACTIONS(927), 1,
      sym__br_acr_bgn,
    STATE(244), 1,
      sym__r_acr,
    STATE(259), 1,
      sym__br_acr,
    ACTIONS(625), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(621), 13,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_flw_map_end,
      sym__br_flw_sep_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8142] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 28,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
      sym__r_tag,
      sym__br_tag,
  [8176] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 28,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
      sym__r_tag,
      sym__br_tag,
  [8210] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(929), 1,
      sym__r_acr_bgn,
    ACTIONS(931), 1,
      sym__br_acr_bgn,
    STATE(256), 2,
      sym__r_acr,
      sym__br_acr,
    ACTIONS(609), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8250] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(935), 2,
      sym__r_tag,
      sym__br_tag,
    ACTIONS(609), 26,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8286] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(925), 1,
      sym__r_acr_bgn,
    ACTIONS(927), 1,
      sym__br_acr_bgn,
    STATE(260), 2,
      sym__r_acr,
      sym__br_acr,
    ACTIONS(609), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8326] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(933), 1,
      sym__br_tag,
    ACTIONS(937), 1,
      sym__r_tag,
    ACTIONS(625), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(621), 14,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_flw_map_end,
      sym__br_flw_sep_bgn,
      sym__br_flw_njv_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8366] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(929), 1,
      sym__r_acr_bgn,
    ACTIONS(931), 1,
      sym__br_acr_bgn,
    STATE(243), 2,
      sym__r_acr,
      sym__br_acr,
    ACTIONS(621), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8406] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 27,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
      sym__r_tag,
      sym__br_tag,
  [8439] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(939), 2,
      sym__r_tag,
      sym__br_tag,
    ACTIONS(621), 25,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8474] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      sym__r_acr_bgn,
    STATE(236), 1,
      sym__r_acr,
    ACTIONS(941), 25,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [8511] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      sym__r_acr_bgn,
    STATE(241), 1,
      sym__r_acr,
    ACTIONS(611), 25,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [8548] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 2,
      sym__r_tag,
      sym__br_tag,
    ACTIONS(609), 25,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8583] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(939), 1,
      sym__br_tag,
    ACTIONS(947), 1,
      sym__r_tag,
    ACTIONS(625), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(621), 13,
      sym__br_flw_seq_bgn,
      sym__br_flw_seq_end,
      sym__br_flw_map_bgn,
      sym__br_flw_sep_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8622] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 27,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
      sym__r_tag,
      sym__br_tag,
  [8655] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(949), 2,
      sym__r_tag,
      sym__br_tag,
    ACTIONS(621), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8689] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(672), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(668), 14,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_flw_map_end,
      sym__br_flw_sep_bgn,
      sym__br_flw_njv_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8723] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(951), 1,
      sym__r_acr_bgn,
    ACTIONS(953), 1,
      sym__br_acr_bgn,
    STATE(268), 2,
      sym__r_acr,
      sym__br_acr,
    ACTIONS(609), 22,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8761] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 26,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
      sym__r_tag,
      sym__br_tag,
  [8793] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 26,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
      sym__r_tag,
      sym__br_tag,
  [8825] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(955), 1,
      sym__r_tag,
    ACTIONS(611), 25,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [8859] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(957), 1,
      sym__r_tag,
    ACTIONS(941), 25,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [8893] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 26,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
      sym__r_tag,
      sym__br_tag,
  [8925] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 26,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8957] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(951), 1,
      sym__r_acr_bgn,
    ACTIONS(953), 1,
      sym__br_acr_bgn,
    STATE(267), 2,
      sym__r_acr,
      sym__br_acr,
    ACTIONS(621), 22,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [8995] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 26,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_tag,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [9027] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(949), 1,
      sym__br_tag,
    ACTIONS(959), 1,
      sym__r_tag,
    ACTIONS(625), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(621), 13,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_flw_map_end,
      sym__br_flw_sep_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9065] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 26,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
      sym__r_tag,
      sym__br_tag,
  [9097] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(961), 2,
      sym__r_tag,
      sym__br_tag,
    ACTIONS(609), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9131] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 26,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9163] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(963), 1,
      sym__r_tag,
    ACTIONS(965), 1,
      sym__br_tag,
    ACTIONS(625), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(621), 13,
      sym__br_flw_seq_bgn,
      sym__br_flw_seq_end,
      sym__br_flw_map_bgn,
      sym__br_flw_sep_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9201] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(951), 1,
      sym__r_acr_bgn,
    ACTIONS(953), 1,
      sym__br_acr_bgn,
    STATE(266), 1,
      sym__r_acr,
    STATE(267), 1,
      sym__br_acr,
    ACTIONS(625), 10,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(621), 12,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_flw_njv_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9243] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(659), 26,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9275] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(967), 26,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__r_tag,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [9307] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(965), 2,
      sym__r_tag,
      sym__br_tag,
    ACTIONS(621), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9341] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(668), 26,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9373] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(969), 2,
      sym__r_tag,
      sym__br_tag,
    ACTIONS(609), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9407] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 25,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [9438] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(971), 25,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [9469] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(659), 25,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9500] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(668), 25,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9531] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(672), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(668), 13,
      sym__br_flw_seq_bgn,
      sym__br_flw_seq_end,
      sym__br_flw_map_bgn,
      sym__br_flw_sep_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9564] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 25,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9595] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(661), 25,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__r_flw_seq_bgn,
      sym__b_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__b_flw_map_bgn,
      sym__r_dqt_str_bgn,
      sym__b_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__r_sgl_pln_nul_blk,
      sym__b_sgl_pln_nul_blk,
      sym__r_sgl_pln_bol_blk,
      sym__b_sgl_pln_bol_blk,
      sym__r_sgl_pln_int_blk,
      sym__b_sgl_pln_int_blk,
      sym__r_sgl_pln_flt_blk,
      sym__b_sgl_pln_flt_blk,
      sym__r_sgl_pln_str_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [9626] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 25,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9657] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(668), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9687] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(672), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(668), 13,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_flw_map_end,
      sym__br_flw_sep_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9719] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(973), 24,
      sym__eof,
      sym__s_dir_yml_bgn,
      sym__s_dir_tag_bgn,
      sym__s_dir_rsv_bgn,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__br_blk_lit_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [9749] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(975), 2,
      sym__r_tag,
      sym__br_tag,
    ACTIONS(609), 22,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9781] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9811] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9841] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(977), 24,
      sym__eof,
      sym__s_dir_yml_bgn,
      sym__s_dir_tag_bgn,
      sym__s_dir_rsv_bgn,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__br_blk_lit_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [9871] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
      sym__r_tag,
      sym__br_tag,
  [9901] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(979), 1,
      sym__r_tag,
    ACTIONS(981), 1,
      sym__br_tag,
    ACTIONS(625), 10,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(621), 12,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_flw_njv_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [9937] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(983), 24,
      sym__eof,
      sym__s_dir_yml_bgn,
      sym__s_dir_tag_bgn,
      sym__s_dir_rsv_bgn,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__br_blk_lit_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [9967] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(985), 24,
      sym__eof,
      sym__s_dir_yml_bgn,
      sym__s_dir_tag_bgn,
      sym__s_dir_rsv_bgn,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__br_blk_lit_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [9997] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(672), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(668), 13,
      sym__br_flw_seq_bgn,
      sym__br_flw_seq_end,
      sym__br_flw_map_bgn,
      sym__br_flw_sep_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [10029] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [10059] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(659), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [10089] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
      sym__r_tag,
      sym__br_tag,
  [10119] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(987), 24,
      sym__eof,
      sym__s_dir_yml_bgn,
      sym__s_dir_tag_bgn,
      sym__s_dir_rsv_bgn,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__br_blk_lit_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [10149] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(668), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [10179] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(659), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [10209] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(981), 2,
      sym__r_tag,
      sym__br_tag,
    ACTIONS(621), 22,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [10241] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(985), 24,
      sym__eof,
      sym__s_dir_yml_bgn,
      sym__s_dir_tag_bgn,
      sym__s_dir_rsv_bgn,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__br_blk_lit_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [10271] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(987), 24,
      sym__eof,
      sym__s_dir_yml_bgn,
      sym__s_dir_tag_bgn,
      sym__s_dir_rsv_bgn,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__br_blk_seq_bgn,
      sym__br_blk_key_bgn,
      sym__br_blk_val_bgn,
      sym__br_blk_lit_bgn,
      sym__br_blk_fld_bgn,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_blk,
      sym__br_sgl_pln_bol_blk,
      sym__br_sgl_pln_int_blk,
      sym__br_sgl_pln_flt_blk,
      sym__br_sgl_pln_str_blk,
      sym__br_mtl_pln_str_blk,
      sym__br_tag,
      sym__br_acr_bgn,
      sym__br_als_bgn,
  [10301] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 24,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [10331] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(638), 22,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [10359] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(672), 10,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
    ACTIONS(668), 12,
      sym__br_flw_seq_bgn,
      sym__br_flw_map_bgn,
      sym__br_flw_njv_bgn,
      sym__br_dqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__br_sgl_pln_nul_flw,
      sym__br_sgl_pln_bol_flw,
      sym__br_sgl_pln_int_flw,
      sym__br_sgl_pln_flt_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [10389] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(668), 22,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [10417] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(659), 22,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [10445] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 22,
      sym__r_flw_seq_bgn,
      sym__br_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__br_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__br_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__br_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__br_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__br_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__br_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__br_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__br_sgl_pln_str_flw,
      sym__r_mtl_pln_str_flw,
      sym__br_mtl_pln_str_flw,
  [10473] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(993), 1,
      sym__r_blk_imp_bgn,
    STATE(440), 1,
      sym__blk_imp_itm_tal,
    ACTIONS(989), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(991), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10505] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(993), 1,
      sym__r_blk_imp_bgn,
    STATE(440), 1,
      sym__blk_imp_itm_tal,
    ACTIONS(995), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(991), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10537] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(993), 1,
      sym__r_blk_imp_bgn,
    STATE(397), 1,
      sym__blk_imp_itm_tal,
    ACTIONS(995), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(997), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10569] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(1001), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10596] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1003), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(1005), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10623] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1007), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(1009), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10650] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 19,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10675] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1013), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(1015), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10702] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(1019), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10729] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1021), 19,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10754] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1023), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(1025), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10781] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(1029), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10808] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(1033), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10835] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1035), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(1037), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10862] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1039), 19,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10887] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1041), 19,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10912] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1043), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(1045), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10939] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 19,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10964] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1049), 19,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [10989] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1051), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(1053), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11016] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1055), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(1053), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11043] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1057), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(1059), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11070] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1061), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
    ACTIONS(1053), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11097] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(993), 1,
      sym__r_blk_imp_bgn,
    STATE(397), 1,
      sym__blk_imp_itm_tal,
    ACTIONS(1063), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
    ACTIONS(997), 14,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11127] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(993), 1,
      sym__r_blk_imp_bgn,
    STATE(440), 1,
      sym__blk_imp_itm_tal,
    ACTIONS(1063), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
    ACTIONS(991), 14,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11157] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1039), 17,
      sym__b_blk_seq_bgn,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11180] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1007), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(1065), 1,
      sym__bl,
    ACTIONS(1009), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11207] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1013), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
    ACTIONS(1015), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11232] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
    ACTIONS(1019), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11257] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1023), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
    ACTIONS(1025), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11282] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
    ACTIONS(1029), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11307] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(993), 1,
      sym__r_blk_imp_bgn,
    STATE(440), 1,
      sym__blk_imp_itm_tal,
    ACTIONS(1068), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11334] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(993), 1,
      sym__r_blk_imp_bgn,
    STATE(425), 1,
      sym__blk_imp_itm_tal,
    ACTIONS(1070), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11361] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 17,
      sym__b_blk_seq_bgn,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11384] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(747), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(751), 1,
      sym__r_flw_map_bgn,
    ACTIONS(757), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(759), 1,
      sym__r_sqt_str_bgn,
    STATE(1124), 1,
      sym__r_sgl_pln_flw,
    STATE(1153), 1,
      sym__r_sgl_flw_seq,
    STATE(1154), 1,
      sym__r_sgl_flw_map,
    STATE(1155), 1,
      sym__r_sgl_dqt_str,
    STATE(1156), 1,
      sym__r_sgl_sqt_str,
    ACTIONS(895), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
    ACTIONS(761), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [11427] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(993), 1,
      sym__r_blk_imp_bgn,
    STATE(441), 1,
      sym__blk_imp_itm_tal,
    ACTIONS(997), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11454] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(993), 1,
      sym__r_blk_imp_bgn,
    STATE(463), 1,
      sym__blk_imp_itm_tal,
    ACTIONS(991), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11481] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1041), 17,
      sym__b_blk_seq_bgn,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11504] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 17,
      sym__b_blk_seq_bgn,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11527] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(721), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(723), 1,
      sym__r_flw_map_bgn,
    ACTIONS(731), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(733), 1,
      sym__r_sqt_str_bgn,
    STATE(1157), 1,
      sym__r_sgl_pln_flw,
    STATE(1212), 1,
      sym__r_sgl_flw_seq,
    STATE(1213), 1,
      sym__r_sgl_flw_map,
    STATE(1214), 1,
      sym__r_sgl_dqt_str,
    STATE(1215), 1,
      sym__r_sgl_sqt_str,
    ACTIONS(895), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
    ACTIONS(735), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [11570] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1055), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
    ACTIONS(1053), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11595] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1051), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
    ACTIONS(1053), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11620] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1049), 17,
      sym__b_blk_seq_bgn,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11643] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1043), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(1072), 1,
      sym__bl,
    ACTIONS(1045), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11670] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(993), 1,
      sym__r_blk_imp_bgn,
    STATE(440), 1,
      sym__blk_imp_itm_tal,
    ACTIONS(1063), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11697] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(993), 1,
      sym__r_blk_imp_bgn,
    STATE(397), 1,
      sym__blk_imp_itm_tal,
    ACTIONS(1063), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11724] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1021), 17,
      sym__b_blk_seq_bgn,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11747] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1061), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
    ACTIONS(1053), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11772] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1057), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
    ACTIONS(1059), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11797] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1003), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
    ACTIONS(1005), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11822] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
    ACTIONS(1001), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11847] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1035), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
    ACTIONS(1037), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11872] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
    ACTIONS(1033), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
  [11897] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1075), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11919] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1077), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11941] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1079), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11963] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1081), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [11985] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    STATE(398), 1,
      sym__b_blk_val_itm,
    ACTIONS(1083), 14,
      sym__b_blk_key_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12011] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1085), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12033] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1045), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(1072), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12057] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1087), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12079] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1089), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12101] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1009), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(1065), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12125] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1091), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12147] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1033), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(1031), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12171] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1037), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(1035), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12195] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1093), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12217] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1095), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12239] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1097), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12261] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1005), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(1003), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12285] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1099), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12307] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1049), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12329] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1053), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(1051), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12353] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1001), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(999), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12377] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1101), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12399] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(901), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(903), 1,
      sym__r_flw_map_bgn,
    ACTIONS(907), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(909), 1,
      sym__r_sqt_str_bgn,
    STATE(1284), 1,
      sym__r_sgl_flw_seq,
    STATE(1292), 1,
      sym__r_sgl_pln_flw,
    STATE(1434), 1,
      sym__r_sgl_sqt_str,
    STATE(1444), 1,
      sym__r_sgl_dqt_str,
    STATE(1453), 1,
      sym__r_sgl_flw_map,
    ACTIONS(895), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
    ACTIONS(911), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [12441] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1103), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12463] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1053), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(1055), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12487] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1029), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(1027), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12511] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1025), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(1023), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12535] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1105), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12557] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1107), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12579] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1019), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(1017), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12603] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1015), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(1013), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12627] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1039), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12649] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1109), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12671] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1111), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12693] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1113), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12715] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1115), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12737] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1117), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12759] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1019), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12781] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1029), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12803] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1119), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12825] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1121), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12847] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12869] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1053), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(1061), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12893] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1123), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12915] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1125), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12937] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1127), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [12959] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(849), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(853), 1,
      sym__r_flw_map_bgn,
    ACTIONS(855), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(857), 1,
      sym__r_sqt_str_bgn,
    STATE(1253), 1,
      sym__r_sgl_pln_flw,
    STATE(1288), 1,
      sym__r_sgl_flw_seq,
    STATE(1289), 1,
      sym__r_sgl_flw_map,
    STATE(1290), 1,
      sym__r_sgl_dqt_str,
    STATE(1291), 1,
      sym__r_sgl_sqt_str,
    ACTIONS(895), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
    ACTIONS(859), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [13001] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    STATE(431), 1,
      sym__b_blk_val_itm,
    ACTIONS(1129), 14,
      sym__b_blk_key_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13027] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1041), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13049] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1131), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13071] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1133), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13093] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1135), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13115] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1137), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13137] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1139), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13159] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1141), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13181] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1143), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13203] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13225] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1145), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13247] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1147), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13269] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1149), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13291] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1151), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13313] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1153), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13335] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1021), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13357] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1059), 1,
      sym__r_blk_imp_bgn,
    ACTIONS(1057), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13381] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(699), 1,
      sym__b_blk_val_bgn,
    STATE(402), 1,
      sym__b_blk_val_itm,
    ACTIONS(1155), 14,
      sym__b_blk_key_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13407] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1157), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13429] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1159), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13451] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1161), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13473] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1163), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13495] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1165), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13517] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1033), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13539] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1001), 16,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__r_blk_imp_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13561] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1167), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13582] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1169), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13603] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1171), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13624] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1173), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13645] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13666] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13687] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1068), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13708] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1175), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13729] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1177), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13750] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1179), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13771] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1181), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13792] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1183), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13813] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1177), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13834] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1185), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13855] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1187), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13876] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1189), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13897] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1191), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13918] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1193), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13939] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1195), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13960] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1197), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [13981] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1199), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14002] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1201), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14023] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1203), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14044] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1205), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14065] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1207), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14086] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1209), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14107] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1211), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14128] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1213), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14149] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1215), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14170] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1039), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14191] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1217), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14212] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1219), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14233] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1221), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14254] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1223), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14275] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1225), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14296] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1227), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14317] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1229), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14338] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1231), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14359] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1233), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14380] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1235), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14401] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1237), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14422] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1057), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14443] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1061), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14464] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1239), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14485] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1241), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14506] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1243), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14527] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1171), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14548] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1245), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14569] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1247), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14590] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(867), 1,
      sym__r_flw_seq_bgn,
    ACTIONS(871), 1,
      sym__r_flw_map_bgn,
    ACTIONS(873), 1,
      sym__r_dqt_str_bgn,
    ACTIONS(875), 1,
      sym__r_sqt_str_bgn,
    ACTIONS(895), 1,
      sym__r_flw_njv_bgn,
    STATE(1493), 1,
      sym__r_sgl_pln_flw,
    STATE(1521), 1,
      sym__r_sgl_flw_seq,
    STATE(1522), 1,
      sym__r_sgl_flw_map,
    STATE(1523), 1,
      sym__r_sgl_dqt_str,
    STATE(1525), 1,
      sym__r_sgl_sqt_str,
    ACTIONS(877), 5,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [14631] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1249), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14652] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1251), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14673] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1253), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14694] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1255), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14715] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14736] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1211), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14757] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1035), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14778] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1225), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14799] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14820] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1003), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14841] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1257), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14862] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1259), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14883] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1261), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14904] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1263), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14925] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1265), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14946] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1267), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14967] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1269), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [14988] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1051), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15009] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1243), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15030] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1271), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15051] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1273), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15072] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1275), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15093] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1277), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15114] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1279), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15135] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1281), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15156] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1283), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15177] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1055), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15198] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1285), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15219] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1287), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15240] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1289), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15261] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1291), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15282] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15303] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1295), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15324] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1297), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15345] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1299), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15366] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1301), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15387] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1303), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15408] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1305), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15429] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1307), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15450] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1309), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15471] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1311), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15492] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1313), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15513] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1315), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15534] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1317), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15555] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15576] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1023), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15597] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15618] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1013), 15,
      sym__b_blk_key_bgn,
      sym__b_blk_val_bgn,
      sym__b_flw_seq_bgn,
      sym__b_flw_map_bgn,
      sym__b_dqt_str_bgn,
      sym__b_sqt_str_bgn,
      sym__b_sgl_pln_nul_blk,
      sym__b_sgl_pln_bol_blk,
      sym__b_sgl_pln_int_blk,
      sym__b_sgl_pln_flt_blk,
      sym__b_sgl_pln_str_blk,
      sym__b_tag,
      sym__b_acr_bgn,
      sym__b_als_bgn,
      sym__bl,
  [15639] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1319), 1,
      sym__r_acr_bgn,
    STATE(503), 1,
      sym__r_acr,
    ACTIONS(623), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [15663] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1321), 1,
      sym__r_acr_bgn,
    STATE(501), 1,
      sym__r_acr,
    ACTIONS(623), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [15687] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1323), 1,
      sym__r_tag,
    ACTIONS(623), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [15708] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1325), 1,
      sym__r_acr_bgn,
    STATE(520), 1,
      sym__r_acr,
    ACTIONS(623), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [15731] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 13,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__r_tag,
  [15750] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1327), 1,
      sym__r_tag,
    ACTIONS(623), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [15771] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 13,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__r_tag,
  [15790] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1329), 1,
      sym__r_acr_bgn,
    STATE(535), 1,
      sym__r_acr,
    ACTIONS(623), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [15813] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(670), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [15831] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1331), 1,
      sym__r_tag,
    ACTIONS(623), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [15851] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(670), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [15869] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__r_tag,
  [15887] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__r_tag,
  [15905] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [15923] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 12,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [15941] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1333), 1,
      sym__r_acr_bgn,
    STATE(564), 1,
      sym__r_acr,
    ACTIONS(623), 10,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [15963] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1335), 1,
      sym__r_tag,
    ACTIONS(623), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [15983] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1341), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1343), 1,
      sym__br_dqt_esc_nwl,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1345), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(641), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16012] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1349), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1351), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1353), 1,
      sym__r_dqt_str_end,
    ACTIONS(1355), 1,
      sym__br_dqt_str_end,
    STATE(531), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1347), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(666), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16043] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1359), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1361), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1363), 1,
      sym__r_dqt_str_end,
    ACTIONS(1365), 1,
      sym__br_dqt_str_end,
    STATE(528), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1357), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(669), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16074] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1369), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1371), 1,
      sym__br_dqt_esc_nwl,
    STATE(558), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1367), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1373), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(702), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16103] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1375), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1377), 1,
      sym__br_dqt_esc_nwl,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1379), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(676), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16132] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1383), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1385), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1387), 1,
      sym__r_dqt_str_end,
    ACTIONS(1389), 1,
      sym__br_dqt_str_end,
    STATE(521), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1381), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(703), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16163] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1391), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1393), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1395), 1,
      sym__r_dqt_str_end,
    ACTIONS(1397), 1,
      sym__br_dqt_str_end,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    STATE(680), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16194] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1399), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1401), 1,
      sym__br_dqt_esc_nwl,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1403), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(671), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16223] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1391), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1393), 1,
      sym__br_dqt_esc_nwl,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1397), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(680), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16252] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1405), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1407), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1409), 1,
      sym__r_dqt_str_end,
    ACTIONS(1411), 1,
      sym__br_dqt_str_end,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    STATE(635), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16283] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(670), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [16300] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1413), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1415), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1417), 1,
      sym__r_dqt_str_end,
    ACTIONS(1419), 1,
      sym__br_dqt_str_end,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    STATE(711), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16331] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1421), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1423), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1425), 1,
      sym__r_dqt_str_end,
    ACTIONS(1427), 1,
      sym__br_dqt_str_end,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    STATE(709), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16362] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1431), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1433), 1,
      sym__br_dqt_esc_nwl,
    STATE(514), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1429), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1435), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(681), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16391] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1439), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1441), 1,
      sym__br_dqt_esc_nwl,
    STATE(534), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1437), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1443), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(654), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16420] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1447), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1449), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1451), 1,
      sym__r_dqt_str_end,
    ACTIONS(1453), 1,
      sym__br_dqt_str_end,
    STATE(548), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1445), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(686), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16451] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1455), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1457), 1,
      sym__br_dqt_esc_nwl,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1459), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(647), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16480] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1461), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1463), 1,
      sym__br_dqt_esc_nwl,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1465), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(712), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16509] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1469), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1471), 1,
      sym__r_dqt_str_end,
    ACTIONS(1473), 1,
      sym__br_dqt_str_end,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    STATE(644), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16540] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1479), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1481), 1,
      sym__r_dqt_str_end,
    ACTIONS(1483), 1,
      sym__br_dqt_str_end,
    STATE(516), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1475), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(684), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16571] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1485), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1487), 1,
      sym__br_dqt_esc_nwl,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1489), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(715), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16600] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1455), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1457), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1459), 1,
      sym__br_dqt_str_end,
    ACTIONS(1491), 1,
      sym__r_dqt_str_end,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    STATE(647), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16631] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1383), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1385), 1,
      sym__br_dqt_esc_nwl,
    STATE(560), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1389), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    ACTIONS(1493), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(703), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16660] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1405), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1407), 1,
      sym__br_dqt_esc_nwl,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1411), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(635), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16689] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1495), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1497), 1,
      sym__br_dqt_esc_nwl,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1499), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(637), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16718] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(670), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [16735] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
      sym__r_tag,
  [16752] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1503), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1505), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1507), 1,
      sym__r_dqt_str_end,
    ACTIONS(1509), 1,
      sym__br_dqt_str_end,
    STATE(559), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1501), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(650), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16783] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1447), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1449), 1,
      sym__br_dqt_esc_nwl,
    STATE(554), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1453), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    ACTIONS(1511), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(686), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16812] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1479), 1,
      sym__br_dqt_esc_nwl,
    STATE(518), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1483), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    ACTIONS(1513), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(684), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16841] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1517), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1519), 1,
      sym__br_dqt_esc_nwl,
    STATE(517), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1515), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1521), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(653), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16870] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_seq_end,
      sym__r_flw_map_bgn,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [16887] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 11,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [16904] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1517), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1519), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1521), 1,
      sym__br_dqt_str_end,
    ACTIONS(1525), 1,
      sym__r_dqt_str_end,
    STATE(557), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1523), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(653), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16935] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1503), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1505), 1,
      sym__br_dqt_esc_nwl,
    STATE(510), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1509), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    ACTIONS(1527), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(650), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16964] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1485), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1487), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1489), 1,
      sym__br_dqt_str_end,
    ACTIONS(1529), 1,
      sym__r_dqt_str_end,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    STATE(715), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [16995] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1533), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1535), 1,
      sym__br_dqt_esc_nwl,
    STATE(561), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1531), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1537), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(648), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17024] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1541), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1543), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1545), 1,
      sym__r_dqt_str_end,
    ACTIONS(1547), 1,
      sym__br_dqt_str_end,
    STATE(549), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1539), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(661), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17055] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1549), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1551), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1553), 1,
      sym__r_dqt_str_end,
    ACTIONS(1555), 1,
      sym__br_dqt_str_end,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    STATE(697), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17086] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1557), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1559), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1561), 1,
      sym__r_dqt_str_end,
    ACTIONS(1563), 1,
      sym__br_dqt_str_end,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    STATE(674), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17117] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1567), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1569), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1571), 1,
      sym__r_dqt_str_end,
    ACTIONS(1573), 1,
      sym__br_dqt_str_end,
    STATE(519), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1565), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(657), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17148] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1577), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1579), 1,
      sym__br_dqt_esc_nwl,
    STATE(530), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1575), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1581), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(631), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17177] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1583), 1,
      sym__r_tag,
    ACTIONS(623), 10,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [17196] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1577), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1579), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1581), 1,
      sym__br_dqt_str_end,
    ACTIONS(1587), 1,
      sym__r_dqt_str_end,
    STATE(545), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1585), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(631), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17227] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1549), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1551), 1,
      sym__br_dqt_esc_nwl,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1555), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(697), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17256] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1591), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1593), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1595), 1,
      sym__r_dqt_str_end,
    ACTIONS(1597), 1,
      sym__br_dqt_str_end,
    STATE(522), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1589), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(698), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17287] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1567), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1569), 1,
      sym__br_dqt_esc_nwl,
    STATE(533), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1573), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    ACTIONS(1599), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(657), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17316] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1399), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1401), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1403), 1,
      sym__br_dqt_str_end,
    ACTIONS(1601), 1,
      sym__r_dqt_str_end,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    STATE(671), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17347] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1603), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1605), 1,
      sym__br_dqt_esc_nwl,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1607), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(704), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17376] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1341), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1343), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1345), 1,
      sym__br_dqt_str_end,
    ACTIONS(1609), 1,
      sym__r_dqt_str_end,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    STATE(641), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17407] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1413), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1415), 1,
      sym__br_dqt_esc_nwl,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1419), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(711), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17436] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1611), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1613), 1,
      sym__br_dqt_esc_nwl,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1337), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1615), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(639), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17465] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1619), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1621), 1,
      sym__br_dqt_esc_nwl,
    STATE(527), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1617), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1623), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(688), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17494] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1349), 1,
      sym__r_dqt_esc_nwl,
    ACTIONS(1351), 1,
      sym__br_dqt_esc_nwl,
    STATE(526), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1355), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    ACTIONS(1625), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    STATE(666), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [17523] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(670), 10,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [17539] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(13), 1,
      sym__s_drs_end,
    ACTIONS(15), 1,
      sym__s_doc_end,
    ACTIONS(1627), 1,
      sym__eof,
    STATE(566), 1,
      sym__doc_w_bgn_wo_end,
    STATE(1237), 1,
      sym__exp_doc,
    STATE(1532), 2,
      sym__doc_w_bgn_w_end_seq,
      sym__doc_w_bgn_wo_end_seq,
    STATE(38), 3,
      sym__doc_w_bgn_w_end,
      sym__exp_doc_end,
      sym__doc_end,
  [17567] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(13), 1,
      sym__s_drs_end,
    ACTIONS(15), 1,
      sym__s_doc_end,
    ACTIONS(1629), 1,
      sym__eof,
    STATE(566), 1,
      sym__doc_w_bgn_wo_end,
    STATE(1237), 1,
      sym__exp_doc,
    STATE(1524), 2,
      sym__doc_w_bgn_w_end_seq,
      sym__doc_w_bgn_wo_end_seq,
    STATE(38), 3,
      sym__doc_w_bgn_w_end,
      sym__exp_doc_end,
      sym__doc_end,
  [17595] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(13), 1,
      sym__s_drs_end,
    ACTIONS(377), 1,
      sym__eof,
    ACTIONS(1631), 1,
      sym__s_doc_end,
    STATE(566), 1,
      sym__doc_w_bgn_wo_end,
    STATE(1237), 1,
      sym__exp_doc,
    STATE(1504), 2,
      sym__doc_w_bgn_w_end_seq,
      sym__doc_w_bgn_wo_end_seq,
    STATE(38), 3,
      sym__doc_w_bgn_w_end,
      sym__exp_doc_end,
      sym__doc_end,
  [17623] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(7), 1,
      sym__s_dir_yml_bgn,
    ACTIONS(9), 1,
      sym__s_dir_tag_bgn,
    ACTIONS(11), 1,
      sym__s_dir_rsv_bgn,
    ACTIONS(13), 1,
      sym__s_drs_end,
    STATE(1165), 1,
      sym__exp_doc,
    STATE(585), 5,
      sym__s_dir,
      sym__s_dir_yml,
      sym__s_dir_tag,
      sym__s_dir_rsv,
      aux_sym__drs_doc_repeat1,
  [17649] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(13), 1,
      sym__s_drs_end,
    ACTIONS(377), 1,
      sym__eof,
    ACTIONS(1633), 1,
      sym__s_doc_end,
    STATE(566), 1,
      sym__doc_w_bgn_wo_end,
    STATE(1237), 1,
      sym__exp_doc,
    STATE(1504), 2,
      sym__doc_w_bgn_w_end_seq,
      sym__doc_w_bgn_wo_end_seq,
    STATE(38), 3,
      sym__doc_w_bgn_w_end,
      sym__exp_doc_end,
      sym__doc_end,
  [17677] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(631), 10,
      sym__r_flw_seq_bgn,
      sym__r_flw_map_bgn,
      sym__r_flw_njv_bgn,
      sym__r_dqt_str_bgn,
      sym__r_sqt_str_bgn,
      sym__r_sgl_pln_nul_flw,
      sym__r_sgl_pln_bol_flw,
      sym__r_sgl_pln_int_flw,
      sym__r_sgl_pln_flt_flw,
      sym__r_sgl_pln_str_flw,
  [17693] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(13), 1,
      sym__s_drs_end,
    ACTIONS(377), 1,
      sym__eof,
    ACTIONS(1635), 1,
      sym__s_doc_end,
    STATE(566), 1,
      sym__doc_w_bgn_wo_end,
    STATE(1237), 1,
      sym__exp_doc,
    STATE(1504), 2,
      sym__doc_w_bgn_w_end_seq,
      sym__doc_w_bgn_wo_end_seq,
    STATE(38), 3,
      sym__doc_w_bgn_w_end,
      sym__exp_doc_end,
      sym__doc_end,
  [17721] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1641), 1,
      sym__r_sqt_str_end,
    ACTIONS(1643), 1,
      sym__br_sqt_str_end,
    STATE(612), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1637), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    STATE(811), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [17746] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1647), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(743), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [17769] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1649), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(728), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [17792] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1655), 1,
      sym__r_dqt_esc_nwl,
    STATE(586), 1,
      aux_sym__br_mtl_dqt_ctn_repeat1,
    ACTIONS(1651), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1653), 5,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_nwl,
      sym__br_dqt_esc_seq,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
  [17813] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1659), 1,
      sym__r_sqt_str_end,
    ACTIONS(1661), 1,
      sym__br_sqt_str_end,
    STATE(592), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1657), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(736), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [17838] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1649), 1,
      sym__br_sqt_str_end,
    ACTIONS(1663), 1,
      sym__r_sqt_str_end,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(728), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [17863] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1667), 1,
      sym__r_sqt_str_end,
    ACTIONS(1669), 1,
      sym__br_sqt_str_end,
    STATE(593), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1665), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(744), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [17888] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(609), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1669), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    ACTIONS(1671), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(744), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [17911] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1675), 1,
      sym__r_sqt_str_end,
    ACTIONS(1677), 1,
      sym__br_sqt_str_end,
    STATE(619), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1673), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(814), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [17936] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(581), 1,
      aux_sym__br_mtl_dqt_ctn_repeat1,
    ACTIONS(1679), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1682), 6,
      sym__br_dqt_str_ctn,
      sym__r_dqt_esc_nwl,
      sym__br_dqt_esc_nwl,
      sym__br_dqt_esc_seq,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
  [17955] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1686), 1,
      sym__r_sqt_str_end,
    ACTIONS(1688), 1,
      sym__br_sqt_str_end,
    STATE(588), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1684), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(723), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [17980] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(596), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1690), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1692), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(748), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18003] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(597), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1694), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1696), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(747), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18026] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1698), 1,
      sym__s_dir_yml_bgn,
    ACTIONS(1701), 1,
      sym__s_dir_tag_bgn,
    ACTIONS(1704), 1,
      sym__s_dir_rsv_bgn,
    ACTIONS(1707), 1,
      sym__s_drs_end,
    STATE(585), 5,
      sym__s_dir,
      sym__s_dir_yml,
      sym__s_dir_tag,
      sym__s_dir_rsv,
      aux_sym__drs_doc_repeat1,
  [18049] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1713), 1,
      sym__r_dqt_esc_nwl,
    STATE(581), 1,
      aux_sym__br_mtl_dqt_ctn_repeat1,
    ACTIONS(1709), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1711), 5,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_nwl,
      sym__br_dqt_esc_seq,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
  [18070] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1715), 1,
      sym__r_sqt_str_end,
    ACTIONS(1717), 1,
      sym__br_sqt_str_end,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(739), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18095] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1719), 1,
      sym__r_sqt_str_end,
    ACTIONS(1721), 1,
      sym__br_sqt_str_end,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(801), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18120] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(589), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(1723), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
    ACTIONS(1726), 6,
      sym__br_dqt_str_ctn,
      sym__r_dqt_esc_nwl,
      sym__br_dqt_esc_nwl,
      sym__br_dqt_esc_seq,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
  [18139] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1728), 1,
      sym__r_sqt_str_end,
    ACTIONS(1730), 1,
      sym__br_sqt_str_end,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(725), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18164] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1732), 1,
      sym__r_sqt_str_end,
    ACTIONS(1734), 1,
      sym__br_sqt_str_end,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(726), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18189] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1736), 1,
      sym__r_sqt_str_end,
    ACTIONS(1738), 1,
      sym__br_sqt_str_end,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(800), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18214] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1740), 1,
      sym__r_sqt_str_end,
    ACTIONS(1742), 1,
      sym__br_sqt_str_end,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(768), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18239] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1692), 1,
      sym__br_sqt_str_end,
    ACTIONS(1746), 1,
      sym__r_sqt_str_end,
    STATE(587), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1744), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(748), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18264] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(607), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1643), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    ACTIONS(1748), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(811), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18287] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1717), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(739), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18310] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1750), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(732), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18333] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1754), 1,
      sym__r_sqt_str_end,
    ACTIONS(1756), 1,
      sym__br_sqt_str_end,
    STATE(613), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1752), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(804), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18358] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(574), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1758), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1760), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(782), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18381] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(217), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(219), 1,
      sym__br_flw_njv_bgn,
    STATE(893), 1,
      sym__r_flw_njl_ann_par,
    STATE(963), 1,
      sym__br_flw_njl_ann_par,
    STATE(1382), 1,
      sym__r_sgl_flw_njl_ann_par,
    ACTIONS(1762), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
    ACTIONS(1765), 2,
      sym__br_flw_map_end,
      sym__br_flw_sep_bgn,
  [18408] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(603), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1677), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    ACTIONS(1767), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(814), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18431] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(615), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1769), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1771), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(741), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18454] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1773), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(742), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18477] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(622), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1688), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    ACTIONS(1775), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(723), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18500] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1779), 1,
      sym__r_sqt_str_end,
    ACTIONS(1781), 1,
      sym__br_sqt_str_end,
    STATE(621), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1777), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(787), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18525] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1783), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(717), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18548] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1785), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(727), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18571] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1789), 1,
      sym__r_sqt_str_end,
    ACTIONS(1791), 1,
      sym__br_sqt_str_end,
    STATE(590), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1787), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(740), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18596] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1742), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(768), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18619] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1793), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(729), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18642] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1760), 1,
      sym__br_sqt_str_end,
    ACTIONS(1797), 1,
      sym__r_sqt_str_end,
    STATE(577), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1795), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(782), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18667] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1785), 1,
      sym__br_sqt_str_end,
    ACTIONS(1799), 1,
      sym__r_sqt_str_end,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(727), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18692] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1801), 1,
      sym__r_sqt_str_end,
    ACTIONS(1803), 1,
      sym__br_sqt_str_end,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(716), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18717] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(573), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1805), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1807), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(756), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18740] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1734), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(726), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18763] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(625), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1809), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1811), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(788), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18786] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1813), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(812), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18809] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(610), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1815), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1817), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(808), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18832] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1773), 1,
      sym__br_sqt_str_end,
    ACTIONS(1819), 1,
      sym__r_sqt_str_end,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(742), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18857] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1771), 1,
      sym__br_sqt_str_end,
    ACTIONS(1823), 1,
      sym__r_sqt_str_end,
    STATE(591), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1821), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(741), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18882] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1813), 1,
      sym__br_sqt_str_end,
    ACTIONS(1825), 1,
      sym__r_sqt_str_end,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(812), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18907] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1721), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(801), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18930] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(606), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1827), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1829), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(745), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18953] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(617), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1781), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    ACTIONS(1831), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    STATE(787), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18976] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1645), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1833), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(819), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [18999] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1837), 1,
      sym__r_flw_jsv_bgn,
    ACTIONS(1839), 1,
      sym__br_flw_jsv_bgn,
    STATE(886), 1,
      sym__r_flw_jsl_ann_par,
    STATE(903), 1,
      sym__br_flw_jsl_ann_par,
    ACTIONS(1835), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [19021] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1837), 1,
      sym__r_flw_jsv_bgn,
    ACTIONS(1839), 1,
      sym__br_flw_jsv_bgn,
    STATE(882), 1,
      sym__br_flw_jsl_ann_par,
    STATE(900), 1,
      sym__r_flw_jsl_ann_par,
    ACTIONS(1765), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [19043] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(219), 1,
      sym__br_flw_njv_bgn,
    ACTIONS(426), 1,
      sym__r_flw_njv_bgn,
    STATE(902), 1,
      sym__r_flw_njl_ann_par,
    STATE(997), 1,
      sym__br_flw_njl_ann_par,
    ACTIONS(1835), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [19065] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(219), 1,
      sym__br_flw_njv_bgn,
    ACTIONS(426), 1,
      sym__r_flw_njv_bgn,
    STATE(883), 1,
      sym__br_flw_njl_ann_par,
    STATE(904), 1,
      sym__r_flw_njl_ann_par,
    ACTIONS(1765), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [19087] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1463), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1465), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(712), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19106] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1489), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19125] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1843), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1845), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(660), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19144] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(299), 1,
      sym__r_flw_njv_bgn,
    STATE(895), 1,
      sym__r_flw_njl_ann_par,
    STATE(1277), 1,
      sym__r_sgl_flw_njl_ann_par,
    ACTIONS(1847), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
    ACTIONS(1850), 2,
      sym__br_flw_seq_end,
      sym__br_flw_sep_bgn,
  [19165] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1852), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19184] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1845), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19203] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1854), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1856), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(664), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19222] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1856), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19241] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1858), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1860), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(663), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19260] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1860), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19279] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1862), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1864), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(634), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19298] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1864), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19317] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(707), 1,
      aux_sym__br_mtl_sqt_ctn_repeat1,
    ACTIONS(1866), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1868), 4,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
  [19334] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1870), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1872), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(662), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19353] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1872), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19372] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1877), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1874), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1880), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19391] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1882), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1884), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(658), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19410] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1884), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19429] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1615), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19448] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1613), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1615), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(639), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19467] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1345), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19486] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1343), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1345), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(641), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19505] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1401), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1403), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(671), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19524] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1403), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19543] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1499), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19562] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1497), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1499), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(637), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19581] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1559), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1563), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(674), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19600] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1411), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19619] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1886), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19638] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1407), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1411), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(635), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19657] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1888), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19676] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1563), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19695] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1890), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19714] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1892), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19733] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1894), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19752] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1896), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19771] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1459), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19790] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1469), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1473), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(644), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19809] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1898), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19828] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1473), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19847] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(642), 1,
      aux_sym__br_mtl_sqt_ctn_repeat1,
    ACTIONS(1900), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1902), 4,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
  [19864] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1904), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19883] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1906), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1904), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(678), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19902] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1908), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1910), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(665), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19921] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1912), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19940] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1914), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1912), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(679), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19959] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1910), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19978] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1916), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1918), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(668), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [19997] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1920), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20016] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1922), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20035] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1918), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20054] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1379), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20073] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1377), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1379), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(676), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20092] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(683), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(1924), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1927), 4,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
  [20109] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1397), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20128] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1551), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1555), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(697), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20147] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1555), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20166] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1393), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1397), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(680), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20185] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1465), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20204] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1929), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20223] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1931), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20242] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1933), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1935), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(689), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20261] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1415), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1419), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(711), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20280] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1937), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1939), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(690), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20299] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1941), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20318] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1943), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20337] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1605), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1607), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(704), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20356] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1945), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20375] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1427), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20394] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1947), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20413] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1949), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20432] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1487), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1489), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(715), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20451] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1607), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20470] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1419), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20489] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1935), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20508] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1457), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1459), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(647), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20527] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1951), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1945), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(694), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20546] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(707), 1,
      aux_sym__br_mtl_sqt_ctn_repeat1,
    ACTIONS(1953), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
    ACTIONS(1956), 4,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
  [20563] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1958), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1960), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(699), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20582] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1960), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20601] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1962), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1964), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(700), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20620] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1964), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20639] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1966), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20658] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1968), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1966), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(695), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20677] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1423), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1427), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(709), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20696] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1841), 1,
      sym__br_dqt_esc_nwl,
    ACTIONS(1339), 2,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_seq,
    ACTIONS(1939), 2,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
    STATE(645), 2,
      sym__br_mtl_dqt_ctn,
      aux_sym__r_dqt_str_repeat1,
  [20715] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1970), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [20731] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1972), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [20747] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1974), 2,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
    ACTIONS(1976), 2,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    STATE(815), 2,
      sym__flw_seq_dat_rpt,
      aux_sym__r_flw_seq_dat_repeat1,
  [20763] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1978), 2,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
    ACTIONS(1980), 2,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    STATE(780), 2,
      sym__flw_seq_dat_rpt,
      aux_sym__r_flw_seq_dat_repeat1,
  [20779] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1205), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [20791] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1203), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [20803] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1984), 1,
      sym__r_dir_rsv_prm,
    STATE(722), 1,
      aux_sym__s_dir_rsv_repeat1,
    ACTIONS(1982), 4,
      sym__s_dir_yml_bgn,
      sym__s_dir_tag_bgn,
      sym__s_dir_rsv_bgn,
      sym__s_drs_end,
  [20819] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1721), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [20835] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [20847] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1987), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [20863] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1989), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [20879] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1991), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [20895] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1993), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [20911] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1995), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [20927] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1997), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(2000), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [20943] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2004), 1,
      sym__r_flw_jsv_bgn,
    STATE(936), 1,
      sym__r_flw_jsl_ann_par,
    ACTIONS(2002), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [20959] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(2006), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [20975] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2004), 1,
      sym__r_flw_jsv_bgn,
    STATE(939), 1,
      sym__r_flw_jsl_ann_par,
    ACTIONS(1850), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [20991] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(450), 1,
      sym__r_flw_njv_bgn,
    STATE(955), 1,
      sym__r_flw_njl_ann_par,
    ACTIONS(2002), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [21007] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(450), 1,
      sym__r_flw_njv_bgn,
    STATE(957), 1,
      sym__r_flw_njl_ann_par,
    ACTIONS(1850), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [21023] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1738), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21039] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2010), 1,
      sym__r_dir_rsv_prm,
    STATE(722), 1,
      aux_sym__s_dir_rsv_repeat1,
    ACTIONS(2008), 4,
      sym__s_dir_yml_bgn,
      sym__s_dir_tag_bgn,
      sym__s_dir_rsv_bgn,
      sym__s_drs_end,
  [21055] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2014), 1,
      sym__r_dir_rsv_prm,
    STATE(737), 1,
      aux_sym__s_dir_rsv_repeat1,
    ACTIONS(2012), 4,
      sym__s_dir_yml_bgn,
      sym__s_dir_tag_bgn,
      sym__s_dir_rsv_bgn,
      sym__s_drs_end,
  [21071] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(2016), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21087] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1730), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21103] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1734), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21119] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(2018), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21135] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(2020), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21151] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1742), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21167] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1783), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21183] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [21195] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1750), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21211] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1717), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21227] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2022), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [21239] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2024), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [21251] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2022), 3,
      sym__br_flw_map_end,
      sym__br_flw_sep_bgn,
      sym__br_flw_njv_bgn,
    ACTIONS(2026), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [21265] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1285), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21277] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2029), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [21289] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1287), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21301] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1209), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21313] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1647), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21329] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1219), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21341] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1291), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21353] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2031), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [21365] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2033), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [21377] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2035), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [21389] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21401] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1299), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21413] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1301), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21425] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1013), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21437] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21449] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21461] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(2037), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21477] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2039), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [21489] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1023), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21501] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21513] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21525] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1317), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21537] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1315), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21549] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2041), 2,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
    ACTIONS(2043), 2,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    STATE(775), 2,
      sym__flw_seq_dat_rpt,
      aux_sym__r_flw_seq_dat_repeat1,
  [21565] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1223), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21577] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(545), 2,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
    ACTIONS(2046), 2,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    STATE(818), 2,
      sym__flw_map_dat_rpt,
      aux_sym__r_flw_map_dat_repeat1,
  [21593] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(549), 2,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
    ACTIONS(2048), 2,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    STATE(818), 2,
      sym__flw_map_dat_rpt,
      aux_sym__r_flw_map_dat_repeat1,
  [21609] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1283), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21621] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(557), 2,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
    ACTIONS(2050), 2,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    STATE(775), 2,
      sym__flw_seq_dat_rpt,
      aux_sym__r_flw_seq_dat_repeat1,
  [21637] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1281), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21649] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1649), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21665] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1279), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21677] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1277), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21689] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1269), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21701] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1267), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21713] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1813), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21729] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1833), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21745] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1265), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21757] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1263), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21769] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1049), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [21781] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1227), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21793] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2052), 2,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
    ACTIONS(2054), 2,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    STATE(778), 2,
      sym__flw_map_dat_rpt,
      aux_sym__r_flw_map_dat_repeat1,
  [21809] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1235), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21821] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2056), 2,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
    ACTIONS(2058), 2,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    STATE(777), 2,
      sym__flw_map_dat_rpt,
      aux_sym__r_flw_map_dat_repeat1,
  [21837] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1003), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21849] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1207), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21861] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2060), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [21873] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21885] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(2062), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21901] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(2064), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21917] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21929] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1035), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21941] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1803), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [21957] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21969] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21981] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1255), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [21993] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1793), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [22009] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1253), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [22021] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1007), 3,
      sym__br_flw_map_end,
      sym__br_flw_sep_bgn,
      sym__br_flw_jsv_bgn,
    ACTIONS(1065), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22035] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1785), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [22051] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(2066), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [22067] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1251), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [22079] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(1773), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [22095] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(559), 2,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
    ACTIONS(2068), 2,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    STATE(775), 2,
      sym__flw_seq_dat_rpt,
      aux_sym__r_flw_seq_dat_repeat1,
  [22111] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1249), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [22123] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1043), 3,
      sym__br_flw_map_end,
      sym__br_flw_sep_bgn,
      sym__br_flw_jsv_bgn,
    ACTIONS(1072), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22137] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2070), 2,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
    ACTIONS(2072), 2,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
    STATE(818), 2,
      sym__flw_map_dat_rpt,
      aux_sym__r_flw_map_dat_repeat1,
  [22153] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1639), 2,
      sym__br_sqt_str_ctn,
      sym__br_sqt_esc_sqt,
    ACTIONS(2075), 2,
      sym__r_sqt_str_end,
      sym__br_sqt_str_end,
    STATE(730), 2,
      sym__br_mtl_sqt_ctn,
      aux_sym__r_sqt_str_repeat1,
  [22169] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1021), 6,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [22181] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1203), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22192] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1043), 2,
      sym__br_flw_seq_end,
      sym__br_flw_sep_bgn,
    ACTIONS(1072), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22205] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1249), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22216] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1235), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22227] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1207), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22238] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1219), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22249] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1021), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [22260] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2077), 5,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_nwl,
      sym__br_dqt_esc_seq,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
  [22271] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1251), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22282] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1007), 2,
      sym__br_flw_seq_end,
      sym__br_flw_sep_bgn,
    ACTIONS(1065), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22295] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1253), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22306] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1223), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22317] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1255), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22328] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22339] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22350] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1035), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22361] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22372] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22383] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1003), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22394] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1209), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22405] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [22416] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1227), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22427] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2029), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [22438] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1205), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22449] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(217), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(219), 1,
      sym__br_flw_njv_bgn,
    STATE(893), 1,
      sym__r_flw_njl_ann_par,
    STATE(963), 1,
      sym__br_flw_njl_ann_par,
    STATE(1382), 1,
      sym__r_sgl_flw_njl_ann_par,
  [22468] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1049), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [22479] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1263), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22490] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1265), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22501] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1267), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22512] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1269), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22523] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1277), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22534] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1279), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22545] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1281), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22556] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1283), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22567] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [22578] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2060), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [22589] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2039), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [22600] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1711), 5,
      sym__br_dqt_str_ctn,
      sym__br_dqt_esc_nwl,
      sym__br_dqt_esc_seq,
      sym__r_dqt_str_end,
      sym__br_dqt_str_end,
  [22611] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1315), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22622] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1317), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22633] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22644] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22655] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1023), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22666] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22677] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22688] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1013), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22699] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(299), 1,
      sym__r_flw_njv_bgn,
    ACTIONS(301), 1,
      sym__br_flw_njv_bgn,
    STATE(895), 1,
      sym__r_flw_njl_ann_par,
    STATE(993), 1,
      sym__br_flw_njl_ann_par,
    STATE(1277), 1,
      sym__r_sgl_flw_njl_ann_par,
  [22718] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2035), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [22729] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2033), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [22740] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1301), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22751] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1299), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22762] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22773] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1291), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22784] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2031), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [22795] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1287), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22806] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1285), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [22817] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2022), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [22828] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2024), 5,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [22839] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2022), 2,
      sym__br_flw_seq_end,
      sym__br_flw_sep_bgn,
    ACTIONS(2026), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [22852] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2033), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [22862] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2035), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [22872] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2079), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [22882] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2079), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [22892] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(2081), 1,
      sym__bl,
    STATE(1016), 2,
      sym__b_blk_seq_itm,
      aux_sym__r_blk_seq_repeat1,
  [22906] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2060), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [22916] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2083), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [22926] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2087), 1,
      sym__r_sqt_str_end,
    STATE(934), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2085), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [22940] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(2089), 1,
      sym__bl,
    STATE(1016), 2,
      sym__b_blk_seq_itm,
      aux_sym__r_blk_seq_repeat1,
  [22954] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2039), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [22964] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1927), 1,
      sym__r_sqt_str_end,
    STATE(890), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2091), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [22978] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2096), 1,
      sym__r_dqt_str_end,
    STATE(930), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2094), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [22992] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(845), 1,
      sym__r_flw_seq_end,
    ACTIONS(2098), 1,
      sym__r_flw_sep_bgn,
    STATE(947), 2,
      sym__sgl_flw_col_dat_rpt,
      aux_sym__r_sgl_flw_col_dat_repeat1,
  [23006] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2100), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23016] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2102), 4,
      sym__s_dir_yml_bgn,
      sym__s_dir_tag_bgn,
      sym__s_dir_rsv_bgn,
      sym__s_drs_end,
  [23026] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2100), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23036] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2079), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23046] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2106), 1,
      sym__r_dqt_str_end,
    STATE(953), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2104), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [23060] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1837), 1,
      sym__r_flw_jsv_bgn,
    ACTIONS(1839), 1,
      sym__br_flw_jsv_bgn,
    STATE(882), 1,
      sym__br_flw_jsl_ann_par,
    STATE(900), 1,
      sym__r_flw_jsl_ann_par,
  [23076] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2108), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23086] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2100), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23096] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1837), 1,
      sym__r_flw_jsv_bgn,
    ACTIONS(1839), 1,
      sym__br_flw_jsv_bgn,
    STATE(886), 1,
      sym__r_flw_jsl_ann_par,
    STATE(903), 1,
      sym__br_flw_jsl_ann_par,
  [23112] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2083), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23122] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2108), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23132] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2100), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23142] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2022), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23152] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2112), 1,
      sym__r_sqt_str_end,
    STATE(890), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2110), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [23166] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2033), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23176] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2031), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23186] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2029), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23196] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2114), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23206] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2116), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23216] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2118), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23226] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2118), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23236] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2024), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23246] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2022), 2,
      sym__br_flw_seq_end,
      sym__br_flw_sep_bgn,
    ACTIONS(2026), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [23258] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2022), 2,
      sym__br_flw_map_end,
      sym__br_flw_sep_bgn,
    ACTIONS(2026), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [23270] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2024), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23280] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2022), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23290] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2120), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23300] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2122), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23310] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2124), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
    ACTIONS(2127), 2,
      sym__br_flw_map_end,
      sym__br_flw_sep_bgn,
  [23322] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2029), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23332] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2129), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23342] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2031), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23352] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2035), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23362] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2131), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23372] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2133), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23382] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2133), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23392] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2133), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23402] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2137), 1,
      sym__r_dqt_str_end,
    STATE(1013), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2135), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [23416] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2120), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23426] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2118), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23436] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2120), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23446] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2139), 1,
      sym__r_sqt_str_end,
    STATE(890), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2110), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [23460] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2133), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23470] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2083), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23480] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2141), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23490] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2141), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23500] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2100), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23510] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2141), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23520] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2141), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23530] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(219), 1,
      sym__br_flw_njv_bgn,
    ACTIONS(426), 1,
      sym__r_flw_njv_bgn,
    STATE(902), 1,
      sym__r_flw_njl_ann_par,
    STATE(997), 1,
      sym__br_flw_njl_ann_par,
  [23546] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(219), 1,
      sym__br_flw_njv_bgn,
    ACTIONS(426), 1,
      sym__r_flw_njv_bgn,
    STATE(883), 1,
      sym__br_flw_njl_ann_par,
    STATE(904), 1,
      sym__r_flw_njl_ann_par,
  [23562] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1049), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23572] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2143), 4,
      sym__s_dir_yml_bgn,
      sym__s_dir_tag_bgn,
      sym__s_dir_rsv_bgn,
      sym__s_drs_end,
  [23582] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2145), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23592] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2147), 1,
      sym__r_flw_seq_end,
    ACTIONS(2149), 1,
      sym__r_flw_sep_bgn,
    STATE(947), 2,
      sym__sgl_flw_col_dat_rpt,
      aux_sym__r_sgl_flw_col_dat_repeat1,
  [23606] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2154), 1,
      sym__r_sqt_str_end,
    STATE(982), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2152), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [23620] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2158), 1,
      sym__r_dqt_str_end,
    STATE(984), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2156), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [23634] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2039), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23644] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2060), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23654] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2160), 1,
      sym__r_sqt_str_end,
    STATE(890), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2110), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [23668] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2162), 1,
      sym__r_dqt_str_end,
    STATE(1013), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2135), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [23682] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(2164), 1,
      sym__bl,
    STATE(888), 2,
      sym__b_blk_seq_itm,
      aux_sym__r_blk_seq_repeat1,
  [23696] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2083), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23706] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(2166), 1,
      sym__bl,
    STATE(884), 2,
      sym__b_blk_seq_itm,
      aux_sym__r_blk_seq_repeat1,
  [23720] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2100), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23730] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(2168), 1,
      sym__bl,
    STATE(999), 2,
      sym__b_blk_seq_itm,
      aux_sym__r_blk_seq_repeat1,
  [23744] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2122), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23754] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(2170), 1,
      sym__bl,
    STATE(1001), 2,
      sym__b_blk_seq_itm,
      aux_sym__r_blk_seq_repeat1,
  [23768] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2124), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
    ACTIONS(2127), 2,
      sym__br_flw_seq_end,
      sym__br_flw_sep_bgn,
  [23780] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(2172), 1,
      sym__bl,
    STATE(1016), 2,
      sym__b_blk_seq_itm,
      aux_sym__r_blk_seq_repeat1,
  [23794] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2079), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23804] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2174), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23814] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2145), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23824] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2176), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23834] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2147), 1,
      sym__r_flw_map_end,
    ACTIONS(2178), 1,
      sym__r_flw_sep_bgn,
    STATE(967), 2,
      sym__sgl_flw_col_dat_rpt,
      aux_sym__r_sgl_flw_col_dat_repeat1,
  [23848] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2131), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23858] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2133), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23868] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2133), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23878] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(845), 1,
      sym__r_flw_map_end,
    ACTIONS(2181), 1,
      sym__r_flw_sep_bgn,
    STATE(967), 2,
      sym__sgl_flw_col_dat_rpt,
      aux_sym__r_sgl_flw_col_dat_repeat1,
  [23892] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2133), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23902] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2079), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23912] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(729), 1,
      sym__r_flw_njv_bgn,
    STATE(1394), 1,
      sym__r_sgl_flw_njl_ann_par,
    ACTIONS(2183), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [23926] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2133), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23936] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2185), 1,
      sym__r_flw_jsv_bgn,
    STATE(1392), 1,
      sym__r_sgl_flw_jsl_ann_par,
    ACTIONS(2183), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [23950] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(2187), 1,
      sym__bl,
    STATE(962), 2,
      sym__b_blk_seq_itm,
      aux_sym__r_blk_seq_repeat1,
  [23964] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(2189), 1,
      sym__bl,
    STATE(1016), 2,
      sym__b_blk_seq_itm,
      aux_sym__r_blk_seq_repeat1,
  [23978] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2176), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [23988] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2193), 1,
      sym__r_sqt_str_end,
    STATE(952), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2191), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [24002] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(2195), 1,
      sym__bl,
    STATE(991), 2,
      sym__b_blk_seq_itm,
      aux_sym__r_blk_seq_repeat1,
  [24016] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2197), 1,
      sym__r_sqt_str_end,
    STATE(890), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2110), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [24030] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2201), 1,
      sym__r_dqt_str_end,
    STATE(989), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2199), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [24044] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2203), 1,
      sym__r_dqt_str_end,
    STATE(1013), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2135), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [24058] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2207), 1,
      sym__r_sqt_str_end,
    STATE(906), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2205), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [24072] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2141), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24082] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2108), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24092] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2209), 1,
      sym__r_sqt_str_end,
    STATE(890), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2110), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [24106] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2211), 1,
      sym__r_dqt_str_end,
    STATE(1013), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2135), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [24120] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2213), 1,
      sym__r_dqt_str_end,
    STATE(1013), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2135), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [24134] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(2215), 1,
      sym__bl,
    STATE(1016), 2,
      sym__b_blk_seq_itm,
      aux_sym__r_blk_seq_repeat1,
  [24148] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2141), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24158] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2079), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24168] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2141), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24178] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2141), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24188] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2219), 1,
      sym__r_dqt_str_end,
    STATE(1047), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2217), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [24202] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2108), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24212] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2223), 1,
      sym__r_sqt_str_end,
    STATE(1048), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2221), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [24226] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(2225), 1,
      sym__bl,
    STATE(1016), 2,
      sym__b_blk_seq_itm,
      aux_sym__r_blk_seq_repeat1,
  [24240] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2004), 1,
      sym__r_flw_jsv_bgn,
    ACTIONS(2227), 1,
      sym__br_flw_jsv_bgn,
    STATE(936), 1,
      sym__r_flw_jsl_ann_par,
    STATE(987), 1,
      sym__br_flw_jsl_ann_par,
  [24256] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(2229), 1,
      sym__bl,
    STATE(1016), 2,
      sym__b_blk_seq_itm,
      aux_sym__r_blk_seq_repeat1,
  [24270] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      sym__r_flw_njv_bgn,
    STATE(1408), 1,
      sym__r_sgl_flw_njl_ann_par,
    ACTIONS(2183), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [24284] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2004), 1,
      sym__r_flw_jsv_bgn,
    ACTIONS(2227), 1,
      sym__br_flw_jsv_bgn,
    STATE(939), 1,
      sym__r_flw_jsl_ann_par,
    STATE(973), 1,
      sym__br_flw_jsl_ann_par,
  [24300] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(301), 1,
      sym__br_flw_njv_bgn,
    ACTIONS(450), 1,
      sym__r_flw_njv_bgn,
    STATE(899), 1,
      sym__br_flw_njl_ann_par,
    STATE(955), 1,
      sym__r_flw_njl_ann_par,
  [24316] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(301), 1,
      sym__br_flw_njv_bgn,
    ACTIONS(450), 1,
      sym__r_flw_njv_bgn,
    STATE(896), 1,
      sym__br_flw_njl_ann_par,
    STATE(957), 1,
      sym__r_flw_njl_ann_par,
  [24332] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2231), 1,
      sym__r_flw_jsv_bgn,
    STATE(1298), 1,
      sym__r_sgl_flw_jsl_ann_par,
    ACTIONS(2183), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [24346] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2233), 1,
      sym__r_sqt_str_end,
    STATE(890), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2110), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [24360] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24370] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24380] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1285), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24390] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2235), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24400] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2237), 1,
      sym__r_dqt_str_end,
    STATE(1013), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2135), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [24414] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1726), 1,
      sym__r_dqt_str_end,
    STATE(1013), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2239), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [24428] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1287), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24438] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2242), 1,
      sym__r_flw_map_end,
    ACTIONS(2244), 1,
      sym__r_flw_sep_bgn,
    STATE(971), 2,
      sym__sgl_flw_col_dat_rpt,
      aux_sym__r_sgl_flw_col_dat_repeat1,
  [24452] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2246), 1,
      sym__b_blk_seq_bgn,
    ACTIONS(2249), 1,
      sym__bl,
    STATE(1016), 2,
      sym__b_blk_seq_itm,
      aux_sym__r_blk_seq_repeat1,
  [24466] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1291), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24476] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24486] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1299), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24496] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1301), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24506] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1013), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24516] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24526] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24536] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1023), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24546] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24556] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24566] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1317), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24576] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1315), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24586] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1283), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24596] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1281), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24606] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1279), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24616] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1277), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24626] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1269), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24636] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1267), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24646] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1265), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24656] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1263), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24666] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2253), 1,
      sym__r_dqt_str_end,
    STATE(990), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2251), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [24680] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1203), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24690] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1205), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24700] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1207), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24710] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1209), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24720] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1219), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24730] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1223), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24740] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1227), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24750] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1235), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24760] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1021), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24770] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2255), 1,
      sym__r_dqt_str_end,
    STATE(1013), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2135), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [24784] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2257), 1,
      sym__r_sqt_str_end,
    STATE(890), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2110), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [24798] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1043), 2,
      sym__br_flw_seq_end,
      sym__br_flw_sep_bgn,
    ACTIONS(1072), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [24810] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1249), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24820] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1251), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24830] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1007), 2,
      sym__br_flw_seq_end,
      sym__br_flw_sep_bgn,
    ACTIONS(1065), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [24842] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1253), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24852] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1255), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24862] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24872] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24882] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1035), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24892] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24902] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24912] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1003), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24922] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2259), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24932] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2261), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24942] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2263), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24952] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2235), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24962] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1049), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24972] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1263), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24982] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1265), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [24992] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1267), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25002] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1269), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25012] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1277), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25022] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1279), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25032] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1281), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25042] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1283), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25052] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1315), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25062] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1317), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25072] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25082] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25092] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1023), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25102] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25112] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25122] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1013), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25132] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1301), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25142] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1299), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25152] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25162] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1291), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25172] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1287), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25182] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1285), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25192] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1203), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25202] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1205), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25212] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1207), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25222] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1209), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25232] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1219), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25242] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1223), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25252] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1227), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25262] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1235), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25272] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25282] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1021), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25292] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2263), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25302] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1043), 2,
      sym__br_flw_map_end,
      sym__br_flw_sep_bgn,
    ACTIONS(1072), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [25314] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1249), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25324] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1251), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25334] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1007), 2,
      sym__br_flw_map_end,
      sym__br_flw_sep_bgn,
    ACTIONS(1065), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [25346] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1253), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25356] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1255), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25366] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25376] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25386] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1035), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25396] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25406] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25416] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1003), 4,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25426] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2267), 1,
      sym__r_sqt_str_end,
    STATE(988), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2265), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [25440] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2242), 1,
      sym__r_flw_seq_end,
    ACTIONS(2269), 1,
      sym__r_flw_sep_bgn,
    STATE(892), 2,
      sym__sgl_flw_col_dat_rpt,
      aux_sym__r_sgl_flw_col_dat_repeat1,
  [25454] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2273), 1,
      sym__r_sqt_str_end,
    STATE(1007), 1,
      aux_sym__r_sgl_sqt_ctn,
    ACTIONS(2271), 2,
      sym__r_sqt_str_ctn,
      sym__r_sqt_esc_sqt,
  [25468] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2277), 1,
      sym__r_dqt_str_end,
    STATE(1012), 1,
      aux_sym__r_sgl_dqt_ctn,
    ACTIONS(2275), 2,
      sym__r_dqt_str_ctn,
      sym__r_dqt_esc_seq,
  [25482] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 4,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__br_flw_sep_bgn,
  [25492] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1187), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25501] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1035), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25510] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25519] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1013), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25528] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2279), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [25537] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2035), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [25546] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1309), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25555] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1307), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25564] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2281), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [25573] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1023), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25582] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25591] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1167), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25600] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1305), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25609] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1317), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25618] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1301), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25627] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1299), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25636] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25645] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1291), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25654] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1287), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25663] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2283), 1,
      sym__br_blk_str_ctn,
    ACTIONS(2285), 1,
      sym__bl,
    STATE(1194), 1,
      aux_sym__r_blk_str_repeat1,
  [25676] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1285), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25685] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1039), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25694] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1315), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25703] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1095), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25712] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1091), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25721] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2287), 1,
      sym__br_blk_str_ctn,
    ACTIONS(2289), 1,
      sym__bl,
    STATE(1191), 1,
      aux_sym__r_blk_str_repeat1,
  [25734] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1081), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25743] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1115), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25752] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25761] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1165), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25770] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1157), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25779] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1181), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25788] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1313), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25797] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2291), 1,
      sym__br_blk_str_ctn,
    ACTIONS(2294), 1,
      sym__bl,
    STATE(1149), 1,
      aux_sym__r_blk_str_repeat1,
  [25810] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25819] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1183), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25828] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1311), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25837] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1085), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25846] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1075), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25855] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1113), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25864] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1151), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25873] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2281), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [25882] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2296), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25891] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1251), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25900] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1107), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25909] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1185), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25918] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1289), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25927] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1249), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25936] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1123), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25945] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2298), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [25954] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2279), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [25963] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1033), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25972] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1001), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25981] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1029), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [25990] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [25999] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1019), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26008] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1303), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26017] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [26026] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [26035] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1297), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26044] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1189), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26053] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [26062] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1193), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26071] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1061), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26080] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1095), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26089] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1091), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26098] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1081), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26107] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1115), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26116] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1295), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26125] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1055), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26134] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1283), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26143] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1165), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26152] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1253), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26161] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1281), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26170] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1279), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26179] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2300), 1,
      sym__br_blk_str_ctn,
    ACTIONS(2302), 1,
      sym__bl,
    STATE(1149), 1,
      aux_sym__r_blk_str_repeat1,
  [26192] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1157), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26201] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1247), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26210] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2300), 1,
      sym__br_blk_str_ctn,
    ACTIONS(2304), 1,
      sym__bl,
    STATE(1149), 1,
      aux_sym__r_blk_str_repeat1,
  [26223] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1277), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26232] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1245), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26241] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1275), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26250] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1273), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26259] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1271), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26268] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1051), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26277] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1269), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26286] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1267), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26295] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1265), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26304] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1195), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26313] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2306), 1,
      sym__br_blk_str_ctn,
    ACTIONS(2308), 1,
      sym__bl,
    STATE(1227), 1,
      aux_sym__r_blk_str_repeat1,
  [26326] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2310), 1,
      sym__br_blk_str_ctn,
    ACTIONS(2312), 1,
      sym__bl,
    STATE(1228), 1,
      aux_sym__r_blk_str_repeat1,
  [26339] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1201), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26348] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1263), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26357] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1261), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26366] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1259), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26375] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1257), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26384] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1085), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26393] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1075), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26402] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1113), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26411] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1151), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26420] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1107), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26429] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1203), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26438] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1205), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26447] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2300), 1,
      sym__br_blk_str_ctn,
    ACTIONS(2314), 1,
      sym__bl,
    STATE(1149), 1,
      aux_sym__r_blk_str_repeat1,
  [26460] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2300), 1,
      sym__br_blk_str_ctn,
    ACTIONS(2316), 1,
      sym__bl,
    STATE(1149), 1,
      aux_sym__r_blk_str_repeat1,
  [26473] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1123), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26482] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(989), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26491] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1207), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26500] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2318), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26509] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1209), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26518] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1219), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26527] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2300), 1,
      sym__br_blk_str_ctn,
    ACTIONS(2320), 1,
      sym__bl,
    STATE(1149), 1,
      aux_sym__r_blk_str_repeat1,
  [26540] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2300), 1,
      sym__br_blk_str_ctn,
    ACTIONS(2322), 1,
      sym__bl,
    STATE(1149), 1,
      aux_sym__r_blk_str_repeat1,
  [26553] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1223), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26562] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2326), 1,
      sym__s_doc_end,
    ACTIONS(2324), 2,
      sym__eof,
      sym__s_drs_end,
  [26573] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2328), 1,
      sym__s_doc_end,
    ACTIONS(2324), 2,
      sym__eof,
      sym__s_drs_end,
  [26584] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1033), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26593] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1001), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26602] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1227), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26611] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1235), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26620] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1057), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26629] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2332), 1,
      sym__s_doc_end,
    ACTIONS(2330), 2,
      sym__eof,
      sym__s_drs_end,
  [26640] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2035), 3,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_njv_bgn,
  [26649] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1255), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26658] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1029), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26667] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2334), 1,
      sym__br_blk_str_ctn,
    ACTIONS(2336), 1,
      sym__bl,
    STATE(1219), 1,
      aux_sym__r_blk_str_repeat1,
  [26680] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2338), 1,
      sym__br_blk_str_ctn,
    ACTIONS(2340), 1,
      sym__bl,
    STATE(1220), 1,
      aux_sym__r_blk_str_repeat1,
  [26693] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1019), 3,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
      sym__r_flw_jsv_bgn,
  [26702] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1003), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26711] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26720] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 3,
      sym__eof,
      sym__s_drs_end,
      sym__s_doc_end,
  [26729] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1251), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [26737] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1285), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [26745] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 2,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [26753] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2279), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [26761] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [26769] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [26777] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2281), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [26785] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1013), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [26793] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1033), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [26801] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2342), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [26809] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(729), 1,
      sym__r_flw_njv_bgn,
    STATE(1394), 1,
      sym__r_sgl_flw_njl_ann_par,
  [26819] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1001), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [26827] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1301), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [26835] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2185), 1,
      sym__r_flw_jsv_bgn,
    STATE(1392), 1,
      sym__r_sgl_flw_jsl_ann_par,
  [26845] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1299), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [26853] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2344), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [26861] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2346), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [26869] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1115), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [26877] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [26885] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1291), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [26893] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2348), 2,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
  [26901] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2350), 2,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
  [26909] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 2,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [26917] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1029), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [26925] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1019), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [26933] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1039), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [26941] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1095), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [26949] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1091), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [26957] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1081), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [26965] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1263), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [26973] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2352), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [26981] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2354), 2,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
  [26989] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1287), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [26997] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1299), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27005] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2356), 2,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
  [27013] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2279), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27021] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2342), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27029] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1085), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27037] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1181), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27045] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1183), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27053] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1157), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27061] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1085), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27069] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1075), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27077] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1113), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27085] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1151), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27093] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2281), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27101] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2358), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27109] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2360), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27117] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1107), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27125] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1123), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27133] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1023), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27141] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2352), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27149] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1185), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27157] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1187), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27165] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2362), 2,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
  [27173] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1189), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27181] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2364), 2,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
  [27189] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1193), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27197] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2060), 2,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [27205] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2039), 2,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [27213] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1195), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27221] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1201), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27229] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1033), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27237] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1001), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27245] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1029), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27253] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1019), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27261] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1203), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27269] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1203), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27277] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1205), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27285] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1095), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27293] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1207), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27301] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1209), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27309] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1091), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27317] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1219), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27325] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1223), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27333] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1081), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27341] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1227), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27349] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1235), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27357] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1115), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27365] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1021), 2,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [27373] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2366), 2,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
  [27381] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2022), 2,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [27389] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2024), 2,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [27397] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2022), 1,
      sym__br_flw_njv_bgn,
    ACTIONS(2026), 1,
      sym__r_flw_njv_bgn,
  [27407] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1205), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27415] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1043), 1,
      sym__br_flw_jsv_bgn,
    ACTIONS(1072), 1,
      sym__r_flw_jsv_bgn,
  [27425] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27433] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2029), 2,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [27441] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2031), 2,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [27449] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2033), 2,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [27457] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2035), 2,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [27465] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27473] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1207), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27481] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1209), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27489] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27497] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2368), 2,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
  [27505] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2370), 2,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
  [27513] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1219), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27521] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1249), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27529] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1165), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27537] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27545] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27553] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1251), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27561] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27569] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1007), 1,
      sym__br_flw_jsv_bgn,
    ACTIONS(1065), 1,
      sym__r_flw_jsv_bgn,
  [27579] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1285), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27587] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2372), 2,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
  [27595] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1223), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27603] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2374), 2,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
  [27611] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1123), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27619] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1287), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27627] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1253), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27635] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1157), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27643] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1255), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27651] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1291), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27659] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27667] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1297), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27675] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1301), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27683] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2346), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27691] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2344), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27699] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1305), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27707] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1167), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27715] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1307), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27723] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1309), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27731] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27739] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27747] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1035), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27755] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1013), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27763] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1017), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27771] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2035), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27779] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1023), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27787] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1027), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27795] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27803] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1317), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27811] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27819] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2352), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27827] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1317), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27835] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2376), 2,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
  [27843] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2035), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [27851] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1315), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27859] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1227), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27867] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1235), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27875] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1107), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27883] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1057), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27891] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1003), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27899] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2352), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27907] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1049), 2,
      sym__r_flw_njv_bgn,
      sym__br_flw_njv_bgn,
  [27915] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2352), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27923] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2378), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27931] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2380), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27939] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2382), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [27947] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1313), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27955] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2384), 2,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
  [27963] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1061), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27971] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2386), 2,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
  [27979] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1315), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [27987] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1311), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [27995] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1303), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28003] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2342), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [28011] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1295), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28019] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1289), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28027] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2352), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [28035] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1265), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [28043] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1267), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [28051] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1245), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28059] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1247), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28067] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1269), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [28075] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1055), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28083] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1283), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28091] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1281), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28099] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1279), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28107] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1277), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28115] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1275), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28123] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1273), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28131] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1271), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28139] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1051), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28147] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1269), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28155] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1267), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28163] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1265), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28171] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1263), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28179] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1261), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28187] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1259), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28195] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2380), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [28203] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2382), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [28211] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1257), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28219] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1277), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [28227] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1249), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28235] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1151), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [28243] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2342), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [28251] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2388), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [28259] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2388), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [28267] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1279), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [28275] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1281), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [28283] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1283), 2,
      sym__r_flw_jsv_bgn,
      sym__br_flw_jsv_bgn,
  [28291] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2390), 2,
      sym__r_flw_seq_end,
      sym__br_flw_seq_end,
  [28299] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1253), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28307] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2392), 2,
      sym__r_flw_map_end,
      sym__br_flw_map_end,
  [28315] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1113), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [28323] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1165), 2,
      sym__r_flw_seq_end,
      sym__r_flw_sep_bgn,
  [28331] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      sym__r_flw_njv_bgn,
    STATE(1408), 1,
      sym__r_sgl_flw_njl_ann_par,
  [28341] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1003), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28349] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(999), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28357] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2231), 1,
      sym__r_flw_jsv_bgn,
    STATE(1298), 1,
      sym__r_sgl_flw_jsl_ann_par,
  [28367] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1035), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28375] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1031), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28383] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1255), 2,
      sym__b_blk_seq_bgn,
      sym__bl,
  [28391] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1075), 2,
      sym__r_flw_map_end,
      sym__r_flw_sep_bgn,
  [28399] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2394), 1,
      sym__r_dir_yml_ver,
  [28406] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1033), 1,
      sym__r_flw_jsv_bgn,
  [28413] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2396), 1,
      sym__r_acr_ctn,
  [28420] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2398), 1,
      sym__r_als_ctn,
  [28427] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2400), 1,
      sym__r_als_ctn,
  [28434] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2402), 1,
      sym__eof,
  [28441] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2404), 1,
      sym__r_acr_ctn,
  [28448] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2406), 1,
      sym__r_acr_ctn,
  [28455] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2408), 1,
      sym__r_acr_ctn,
  [28462] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2410), 1,
      sym__r_als_ctn,
  [28469] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2412), 1,
      sym__r_als_ctn,
  [28476] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2414), 1,
      ts_builtin_sym_end,
  [28483] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2416), 1,
      sym__r_dir_tag_pfx,
  [28490] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2418), 1,
      sym__r_flw_map_end,
  [28497] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2420), 1,
      sym__r_flw_seq_end,
  [28504] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2422), 1,
      sym__r_als_ctn,
  [28511] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2424), 1,
      sym__r_als_ctn,
  [28518] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2426), 1,
      sym__r_acr_ctn,
  [28525] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2428), 1,
      sym__r_acr_ctn,
  [28532] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2430), 1,
      sym__r_acr_ctn,
  [28539] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2432), 1,
      sym__r_acr_ctn,
  [28546] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2434), 1,
      sym__r_als_ctn,
  [28553] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2436), 1,
      sym__r_als_ctn,
  [28560] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2438), 1,
      ts_builtin_sym_end,
  [28567] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2440), 1,
      sym__r_flw_map_end,
  [28574] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2442), 1,
      sym__r_flw_seq_end,
  [28581] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2444), 1,
      sym__r_als_ctn,
  [28588] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2446), 1,
      sym__r_als_ctn,
  [28595] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2448), 1,
      sym__r_acr_ctn,
  [28602] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2450), 1,
      sym__r_acr_ctn,
  [28609] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2452), 1,
      sym__r_als_ctn,
  [28616] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2454), 1,
      sym__r_acr_ctn,
  [28623] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1011), 1,
      sym__r_flw_njv_bgn,
  [28630] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1047), 1,
      sym__r_flw_njv_bgn,
  [28637] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2456), 1,
      sym__r_flw_map_end,
  [28644] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2458), 1,
      sym__r_acr_ctn,
  [28651] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2460), 1,
      sym__r_acr_ctn,
  [28658] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2462), 1,
      sym__r_flw_map_end,
  [28665] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2464), 1,
      sym__r_flw_seq_end,
  [28672] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2281), 1,
      sym__r_flw_njv_bgn,
  [28679] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2466), 1,
      sym__r_als_ctn,
  [28686] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2468), 1,
      sym__r_als_ctn,
  [28693] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2279), 1,
      sym__r_flw_njv_bgn,
  [28700] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2470), 1,
      sym__r_als_ctn,
  [28707] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2472), 1,
      sym__r_als_ctn,
  [28714] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2474), 1,
      sym__r_acr_ctn,
  [28721] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2476), 1,
      sym__r_acr_ctn,
  [28728] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2478), 1,
      sym__r_flw_seq_end,
  [28735] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2480), 1,
      sym__r_flw_seq_end,
  [28742] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2482), 1,
      sym__r_flw_map_end,
  [28749] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2484), 1,
      sym__eof,
  [28756] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2486), 1,
      sym__r_acr_ctn,
  [28763] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2488), 1,
      sym__r_acr_ctn,
  [28770] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2490), 1,
      sym__r_als_ctn,
  [28777] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2492), 1,
      sym__r_als_ctn,
  [28784] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2494), 1,
      sym__r_dir_tag_hdl,
  [28791] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2496), 1,
      sym__r_flw_seq_end,
  [28798] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2498), 1,
      sym__r_flw_map_end,
  [28805] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2500), 1,
      sym__r_als_ctn,
  [28812] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1081), 1,
      sym__r_flw_jsv_bgn,
  [28819] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1115), 1,
      sym__r_flw_jsv_bgn,
  [28826] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2502), 1,
      ts_builtin_sym_end,
  [28833] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1019), 1,
      sym__r_flw_jsv_bgn,
  [28840] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1165), 1,
      sym__r_flw_jsv_bgn,
  [28847] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1157), 1,
      sym__r_flw_jsv_bgn,
  [28854] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1001), 1,
      sym__r_flw_jsv_bgn,
  [28861] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1029), 1,
      sym__r_flw_jsv_bgn,
  [28868] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1085), 1,
      sym__r_flw_jsv_bgn,
  [28875] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1075), 1,
      sym__r_flw_jsv_bgn,
  [28882] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1113), 1,
      sym__r_flw_jsv_bgn,
  [28889] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2504), 1,
      sym__eof,
  [28896] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1151), 1,
      sym__r_flw_jsv_bgn,
  [28903] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1107), 1,
      sym__r_flw_jsv_bgn,
  [28910] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2506), 1,
      sym__eof,
  [28917] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2508), 1,
      ts_builtin_sym_end,
  [28924] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1123), 1,
      sym__r_flw_jsv_bgn,
  [28931] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1095), 1,
      sym__r_flw_jsv_bgn,
  [28938] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2035), 1,
      sym__r_flw_njv_bgn,
  [28945] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2510), 1,
      sym__eof,
  [28952] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2512), 1,
      sym__r_acr_ctn,
  [28959] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2514), 1,
      sym__r_acr_ctn,
  [28966] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2516), 1,
      sym__r_als_ctn,
  [28973] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2518), 1,
      sym__r_acr_ctn,
  [28980] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2520), 1,
      sym__r_acr_ctn,
  [28987] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2522), 1,
      sym__r_als_ctn,
  [28994] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2524), 1,
      sym__r_acr_ctn,
  [29001] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2526), 1,
      sym__r_acr_ctn,
  [29008] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2528), 1,
      sym__r_als_ctn,
  [29015] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2530), 1,
      sym__r_acr_ctn,
  [29022] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2532), 1,
      sym__r_acr_ctn,
  [29029] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2534), 1,
      sym__r_als_ctn,
  [29036] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2536), 1,
      sym__r_acr_ctn,
  [29043] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2538), 1,
      sym__r_acr_ctn,
  [29050] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2540), 1,
      sym__r_als_ctn,
  [29057] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2542), 1,
      sym__r_acr_ctn,
  [29064] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2544), 1,
      sym__r_acr_ctn,
  [29071] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2546), 1,
      sym__r_als_ctn,
  [29078] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2548), 1,
      sym__r_acr_ctn,
  [29085] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2550), 1,
      sym__r_acr_ctn,
  [29092] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1091), 1,
      sym__r_flw_jsv_bgn,
  [29099] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2552), 1,
      sym__r_acr_ctn,
  [29106] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2554), 1,
      sym__r_acr_ctn,
  [29113] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2556), 1,
      sym__r_acr_ctn,
  [29120] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2558), 1,
      sym__r_acr_ctn,
  [29127] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2560), 1,
      sym__r_acr_ctn,
  [29134] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2562), 1,
      sym__r_acr_ctn,
  [29141] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2564), 1,
      sym__r_acr_ctn,
  [29148] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2566), 1,
      sym__r_acr_ctn,
  [29155] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2568), 1,
      sym__r_acr_ctn,
  [29162] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2570), 1,
      sym__r_acr_ctn,
  [29169] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2572), 1,
      sym__r_acr_ctn,
  [29176] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2574), 1,
      sym__r_acr_ctn,
  [29183] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2576), 1,
      sym__r_acr_ctn,
  [29190] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2578), 1,
      sym__r_acr_ctn,
  [29197] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2580), 1,
      sym__r_acr_ctn,
};

static const uint32_t ts_small_parse_table_map[] = {
  [SMALL_STATE(88)] = 0,
  [SMALL_STATE(89)] = 139,
  [SMALL_STATE(90)] = 278,
  [SMALL_STATE(91)] = 417,
  [SMALL_STATE(92)] = 556,
  [SMALL_STATE(93)] = 695,
  [SMALL_STATE(94)] = 834,
  [SMALL_STATE(95)] = 973,
  [SMALL_STATE(96)] = 1112,
  [SMALL_STATE(97)] = 1182,
  [SMALL_STATE(98)] = 1252,
  [SMALL_STATE(99)] = 1308,
  [SMALL_STATE(100)] = 1372,
  [SMALL_STATE(101)] = 1428,
  [SMALL_STATE(102)] = 1496,
  [SMALL_STATE(103)] = 1564,
  [SMALL_STATE(104)] = 1632,
  [SMALL_STATE(105)] = 1700,
  [SMALL_STATE(106)] = 1764,
  [SMALL_STATE(107)] = 1823,
  [SMALL_STATE(108)] = 1884,
  [SMALL_STATE(109)] = 1939,
  [SMALL_STATE(110)] = 1998,
  [SMALL_STATE(111)] = 2052,
  [SMALL_STATE(112)] = 2114,
  [SMALL_STATE(113)] = 2176,
  [SMALL_STATE(114)] = 2238,
  [SMALL_STATE(115)] = 2300,
  [SMALL_STATE(116)] = 2354,
  [SMALL_STATE(117)] = 2411,
  [SMALL_STATE(118)] = 2468,
  [SMALL_STATE(119)] = 2525,
  [SMALL_STATE(120)] = 2582,
  [SMALL_STATE(121)] = 2639,
  [SMALL_STATE(122)] = 2692,
  [SMALL_STATE(123)] = 2745,
  [SMALL_STATE(124)] = 2797,
  [SMALL_STATE(125)] = 2849,
  [SMALL_STATE(126)] = 2901,
  [SMALL_STATE(127)] = 2953,
  [SMALL_STATE(128)] = 3031,
  [SMALL_STATE(129)] = 3109,
  [SMALL_STATE(130)] = 3186,
  [SMALL_STATE(131)] = 3263,
  [SMALL_STATE(132)] = 3353,
  [SMALL_STATE(133)] = 3443,
  [SMALL_STATE(134)] = 3537,
  [SMALL_STATE(135)] = 3631,
  [SMALL_STATE(136)] = 3725,
  [SMALL_STATE(137)] = 3819,
  [SMALL_STATE(138)] = 3913,
  [SMALL_STATE(139)] = 4007,
  [SMALL_STATE(140)] = 4097,
  [SMALL_STATE(141)] = 4191,
  [SMALL_STATE(142)] = 4285,
  [SMALL_STATE(143)] = 4375,
  [SMALL_STATE(144)] = 4469,
  [SMALL_STATE(145)] = 4545,
  [SMALL_STATE(146)] = 4635,
  [SMALL_STATE(147)] = 4725,
  [SMALL_STATE(148)] = 4801,
  [SMALL_STATE(149)] = 4895,
  [SMALL_STATE(150)] = 4985,
  [SMALL_STATE(151)] = 5033,
  [SMALL_STATE(152)] = 5123,
  [SMALL_STATE(153)] = 5199,
  [SMALL_STATE(154)] = 5293,
  [SMALL_STATE(155)] = 5383,
  [SMALL_STATE(156)] = 5477,
  [SMALL_STATE(157)] = 5571,
  [SMALL_STATE(158)] = 5661,
  [SMALL_STATE(159)] = 5755,
  [SMALL_STATE(160)] = 5845,
  [SMALL_STATE(161)] = 5921,
  [SMALL_STATE(162)] = 6011,
  [SMALL_STATE(163)] = 6101,
  [SMALL_STATE(164)] = 6175,
  [SMALL_STATE(165)] = 6249,
  [SMALL_STATE(166)] = 6289,
  [SMALL_STATE(167)] = 6377,
  [SMALL_STATE(168)] = 6465,
  [SMALL_STATE(169)] = 6509,
  [SMALL_STATE(170)] = 6597,
  [SMALL_STATE(171)] = 6685,
  [SMALL_STATE(172)] = 6724,
  [SMALL_STATE(173)] = 6809,
  [SMALL_STATE(174)] = 6848,
  [SMALL_STATE(175)] = 6887,
  [SMALL_STATE(176)] = 6972,
  [SMALL_STATE(177)] = 7011,
  [SMALL_STATE(178)] = 7049,
  [SMALL_STATE(179)] = 7087,
  [SMALL_STATE(180)] = 7125,
  [SMALL_STATE(181)] = 7195,
  [SMALL_STATE(182)] = 7269,
  [SMALL_STATE(183)] = 7325,
  [SMALL_STATE(184)] = 7381,
  [SMALL_STATE(185)] = 7423,
  [SMALL_STATE(186)] = 7465,
  [SMALL_STATE(187)] = 7539,
  [SMALL_STATE(188)] = 7585,
  [SMALL_STATE(189)] = 7655,
  [SMALL_STATE(190)] = 7725,
  [SMALL_STATE(191)] = 7795,
  [SMALL_STATE(192)] = 7851,
  [SMALL_STATE(193)] = 7892,
  [SMALL_STATE(194)] = 7933,
  [SMALL_STATE(195)] = 7978,
  [SMALL_STATE(196)] = 8018,
  [SMALL_STATE(197)] = 8062,
  [SMALL_STATE(198)] = 8098,
  [SMALL_STATE(199)] = 8142,
  [SMALL_STATE(200)] = 8176,
  [SMALL_STATE(201)] = 8210,
  [SMALL_STATE(202)] = 8250,
  [SMALL_STATE(203)] = 8286,
  [SMALL_STATE(204)] = 8326,
  [SMALL_STATE(205)] = 8366,
  [SMALL_STATE(206)] = 8406,
  [SMALL_STATE(207)] = 8439,
  [SMALL_STATE(208)] = 8474,
  [SMALL_STATE(209)] = 8511,
  [SMALL_STATE(210)] = 8548,
  [SMALL_STATE(211)] = 8583,
  [SMALL_STATE(212)] = 8622,
  [SMALL_STATE(213)] = 8655,
  [SMALL_STATE(214)] = 8689,
  [SMALL_STATE(215)] = 8723,
  [SMALL_STATE(216)] = 8761,
  [SMALL_STATE(217)] = 8793,
  [SMALL_STATE(218)] = 8825,
  [SMALL_STATE(219)] = 8859,
  [SMALL_STATE(220)] = 8893,
  [SMALL_STATE(221)] = 8925,
  [SMALL_STATE(222)] = 8957,
  [SMALL_STATE(223)] = 8995,
  [SMALL_STATE(224)] = 9027,
  [SMALL_STATE(225)] = 9065,
  [SMALL_STATE(226)] = 9097,
  [SMALL_STATE(227)] = 9131,
  [SMALL_STATE(228)] = 9163,
  [SMALL_STATE(229)] = 9201,
  [SMALL_STATE(230)] = 9243,
  [SMALL_STATE(231)] = 9275,
  [SMALL_STATE(232)] = 9307,
  [SMALL_STATE(233)] = 9341,
  [SMALL_STATE(234)] = 9373,
  [SMALL_STATE(235)] = 9407,
  [SMALL_STATE(236)] = 9438,
  [SMALL_STATE(237)] = 9469,
  [SMALL_STATE(238)] = 9500,
  [SMALL_STATE(239)] = 9531,
  [SMALL_STATE(240)] = 9564,
  [SMALL_STATE(241)] = 9595,
  [SMALL_STATE(242)] = 9626,
  [SMALL_STATE(243)] = 9657,
  [SMALL_STATE(244)] = 9687,
  [SMALL_STATE(245)] = 9719,
  [SMALL_STATE(246)] = 9749,
  [SMALL_STATE(247)] = 9781,
  [SMALL_STATE(248)] = 9811,
  [SMALL_STATE(249)] = 9841,
  [SMALL_STATE(250)] = 9871,
  [SMALL_STATE(251)] = 9901,
  [SMALL_STATE(252)] = 9937,
  [SMALL_STATE(253)] = 9967,
  [SMALL_STATE(254)] = 9997,
  [SMALL_STATE(255)] = 10029,
  [SMALL_STATE(256)] = 10059,
  [SMALL_STATE(257)] = 10089,
  [SMALL_STATE(258)] = 10119,
  [SMALL_STATE(259)] = 10149,
  [SMALL_STATE(260)] = 10179,
  [SMALL_STATE(261)] = 10209,
  [SMALL_STATE(262)] = 10241,
  [SMALL_STATE(263)] = 10271,
  [SMALL_STATE(264)] = 10301,
  [SMALL_STATE(265)] = 10331,
  [SMALL_STATE(266)] = 10359,
  [SMALL_STATE(267)] = 10389,
  [SMALL_STATE(268)] = 10417,
  [SMALL_STATE(269)] = 10445,
  [SMALL_STATE(270)] = 10473,
  [SMALL_STATE(271)] = 10505,
  [SMALL_STATE(272)] = 10537,
  [SMALL_STATE(273)] = 10569,
  [SMALL_STATE(274)] = 10596,
  [SMALL_STATE(275)] = 10623,
  [SMALL_STATE(276)] = 10650,
  [SMALL_STATE(277)] = 10675,
  [SMALL_STATE(278)] = 10702,
  [SMALL_STATE(279)] = 10729,
  [SMALL_STATE(280)] = 10754,
  [SMALL_STATE(281)] = 10781,
  [SMALL_STATE(282)] = 10808,
  [SMALL_STATE(283)] = 10835,
  [SMALL_STATE(284)] = 10862,
  [SMALL_STATE(285)] = 10887,
  [SMALL_STATE(286)] = 10912,
  [SMALL_STATE(287)] = 10939,
  [SMALL_STATE(288)] = 10964,
  [SMALL_STATE(289)] = 10989,
  [SMALL_STATE(290)] = 11016,
  [SMALL_STATE(291)] = 11043,
  [SMALL_STATE(292)] = 11070,
  [SMALL_STATE(293)] = 11097,
  [SMALL_STATE(294)] = 11127,
  [SMALL_STATE(295)] = 11157,
  [SMALL_STATE(296)] = 11180,
  [SMALL_STATE(297)] = 11207,
  [SMALL_STATE(298)] = 11232,
  [SMALL_STATE(299)] = 11257,
  [SMALL_STATE(300)] = 11282,
  [SMALL_STATE(301)] = 11307,
  [SMALL_STATE(302)] = 11334,
  [SMALL_STATE(303)] = 11361,
  [SMALL_STATE(304)] = 11384,
  [SMALL_STATE(305)] = 11427,
  [SMALL_STATE(306)] = 11454,
  [SMALL_STATE(307)] = 11481,
  [SMALL_STATE(308)] = 11504,
  [SMALL_STATE(309)] = 11527,
  [SMALL_STATE(310)] = 11570,
  [SMALL_STATE(311)] = 11595,
  [SMALL_STATE(312)] = 11620,
  [SMALL_STATE(313)] = 11643,
  [SMALL_STATE(314)] = 11670,
  [SMALL_STATE(315)] = 11697,
  [SMALL_STATE(316)] = 11724,
  [SMALL_STATE(317)] = 11747,
  [SMALL_STATE(318)] = 11772,
  [SMALL_STATE(319)] = 11797,
  [SMALL_STATE(320)] = 11822,
  [SMALL_STATE(321)] = 11847,
  [SMALL_STATE(322)] = 11872,
  [SMALL_STATE(323)] = 11897,
  [SMALL_STATE(324)] = 11919,
  [SMALL_STATE(325)] = 11941,
  [SMALL_STATE(326)] = 11963,
  [SMALL_STATE(327)] = 11985,
  [SMALL_STATE(328)] = 12011,
  [SMALL_STATE(329)] = 12033,
  [SMALL_STATE(330)] = 12057,
  [SMALL_STATE(331)] = 12079,
  [SMALL_STATE(332)] = 12101,
  [SMALL_STATE(333)] = 12125,
  [SMALL_STATE(334)] = 12147,
  [SMALL_STATE(335)] = 12171,
  [SMALL_STATE(336)] = 12195,
  [SMALL_STATE(337)] = 12217,
  [SMALL_STATE(338)] = 12239,
  [SMALL_STATE(339)] = 12261,
  [SMALL_STATE(340)] = 12285,
  [SMALL_STATE(341)] = 12307,
  [SMALL_STATE(342)] = 12329,
  [SMALL_STATE(343)] = 12353,
  [SMALL_STATE(344)] = 12377,
  [SMALL_STATE(345)] = 12399,
  [SMALL_STATE(346)] = 12441,
  [SMALL_STATE(347)] = 12463,
  [SMALL_STATE(348)] = 12487,
  [SMALL_STATE(349)] = 12511,
  [SMALL_STATE(350)] = 12535,
  [SMALL_STATE(351)] = 12557,
  [SMALL_STATE(352)] = 12579,
  [SMALL_STATE(353)] = 12603,
  [SMALL_STATE(354)] = 12627,
  [SMALL_STATE(355)] = 12649,
  [SMALL_STATE(356)] = 12671,
  [SMALL_STATE(357)] = 12693,
  [SMALL_STATE(358)] = 12715,
  [SMALL_STATE(359)] = 12737,
  [SMALL_STATE(360)] = 12759,
  [SMALL_STATE(361)] = 12781,
  [SMALL_STATE(362)] = 12803,
  [SMALL_STATE(363)] = 12825,
  [SMALL_STATE(364)] = 12847,
  [SMALL_STATE(365)] = 12869,
  [SMALL_STATE(366)] = 12893,
  [SMALL_STATE(367)] = 12915,
  [SMALL_STATE(368)] = 12937,
  [SMALL_STATE(369)] = 12959,
  [SMALL_STATE(370)] = 13001,
  [SMALL_STATE(371)] = 13027,
  [SMALL_STATE(372)] = 13049,
  [SMALL_STATE(373)] = 13071,
  [SMALL_STATE(374)] = 13093,
  [SMALL_STATE(375)] = 13115,
  [SMALL_STATE(376)] = 13137,
  [SMALL_STATE(377)] = 13159,
  [SMALL_STATE(378)] = 13181,
  [SMALL_STATE(379)] = 13203,
  [SMALL_STATE(380)] = 13225,
  [SMALL_STATE(381)] = 13247,
  [SMALL_STATE(382)] = 13269,
  [SMALL_STATE(383)] = 13291,
  [SMALL_STATE(384)] = 13313,
  [SMALL_STATE(385)] = 13335,
  [SMALL_STATE(386)] = 13357,
  [SMALL_STATE(387)] = 13381,
  [SMALL_STATE(388)] = 13407,
  [SMALL_STATE(389)] = 13429,
  [SMALL_STATE(390)] = 13451,
  [SMALL_STATE(391)] = 13473,
  [SMALL_STATE(392)] = 13495,
  [SMALL_STATE(393)] = 13517,
  [SMALL_STATE(394)] = 13539,
  [SMALL_STATE(395)] = 13561,
  [SMALL_STATE(396)] = 13582,
  [SMALL_STATE(397)] = 13603,
  [SMALL_STATE(398)] = 13624,
  [SMALL_STATE(399)] = 13645,
  [SMALL_STATE(400)] = 13666,
  [SMALL_STATE(401)] = 13687,
  [SMALL_STATE(402)] = 13708,
  [SMALL_STATE(403)] = 13729,
  [SMALL_STATE(404)] = 13750,
  [SMALL_STATE(405)] = 13771,
  [SMALL_STATE(406)] = 13792,
  [SMALL_STATE(407)] = 13813,
  [SMALL_STATE(408)] = 13834,
  [SMALL_STATE(409)] = 13855,
  [SMALL_STATE(410)] = 13876,
  [SMALL_STATE(411)] = 13897,
  [SMALL_STATE(412)] = 13918,
  [SMALL_STATE(413)] = 13939,
  [SMALL_STATE(414)] = 13960,
  [SMALL_STATE(415)] = 13981,
  [SMALL_STATE(416)] = 14002,
  [SMALL_STATE(417)] = 14023,
  [SMALL_STATE(418)] = 14044,
  [SMALL_STATE(419)] = 14065,
  [SMALL_STATE(420)] = 14086,
  [SMALL_STATE(421)] = 14107,
  [SMALL_STATE(422)] = 14128,
  [SMALL_STATE(423)] = 14149,
  [SMALL_STATE(424)] = 14170,
  [SMALL_STATE(425)] = 14191,
  [SMALL_STATE(426)] = 14212,
  [SMALL_STATE(427)] = 14233,
  [SMALL_STATE(428)] = 14254,
  [SMALL_STATE(429)] = 14275,
  [SMALL_STATE(430)] = 14296,
  [SMALL_STATE(431)] = 14317,
  [SMALL_STATE(432)] = 14338,
  [SMALL_STATE(433)] = 14359,
  [SMALL_STATE(434)] = 14380,
  [SMALL_STATE(435)] = 14401,
  [SMALL_STATE(436)] = 14422,
  [SMALL_STATE(437)] = 14443,
  [SMALL_STATE(438)] = 14464,
  [SMALL_STATE(439)] = 14485,
  [SMALL_STATE(440)] = 14506,
  [SMALL_STATE(441)] = 14527,
  [SMALL_STATE(442)] = 14548,
  [SMALL_STATE(443)] = 14569,
  [SMALL_STATE(444)] = 14590,
  [SMALL_STATE(445)] = 14631,
  [SMALL_STATE(446)] = 14652,
  [SMALL_STATE(447)] = 14673,
  [SMALL_STATE(448)] = 14694,
  [SMALL_STATE(449)] = 14715,
  [SMALL_STATE(450)] = 14736,
  [SMALL_STATE(451)] = 14757,
  [SMALL_STATE(452)] = 14778,
  [SMALL_STATE(453)] = 14799,
  [SMALL_STATE(454)] = 14820,
  [SMALL_STATE(455)] = 14841,
  [SMALL_STATE(456)] = 14862,
  [SMALL_STATE(457)] = 14883,
  [SMALL_STATE(458)] = 14904,
  [SMALL_STATE(459)] = 14925,
  [SMALL_STATE(460)] = 14946,
  [SMALL_STATE(461)] = 14967,
  [SMALL_STATE(462)] = 14988,
  [SMALL_STATE(463)] = 15009,
  [SMALL_STATE(464)] = 15030,
  [SMALL_STATE(465)] = 15051,
  [SMALL_STATE(466)] = 15072,
  [SMALL_STATE(467)] = 15093,
  [SMALL_STATE(468)] = 15114,
  [SMALL_STATE(469)] = 15135,
  [SMALL_STATE(470)] = 15156,
  [SMALL_STATE(471)] = 15177,
  [SMALL_STATE(472)] = 15198,
  [SMALL_STATE(473)] = 15219,
  [SMALL_STATE(474)] = 15240,
  [SMALL_STATE(475)] = 15261,
  [SMALL_STATE(476)] = 15282,
  [SMALL_STATE(477)] = 15303,
  [SMALL_STATE(478)] = 15324,
  [SMALL_STATE(479)] = 15345,
  [SMALL_STATE(480)] = 15366,
  [SMALL_STATE(481)] = 15387,
  [SMALL_STATE(482)] = 15408,
  [SMALL_STATE(483)] = 15429,
  [SMALL_STATE(484)] = 15450,
  [SMALL_STATE(485)] = 15471,
  [SMALL_STATE(486)] = 15492,
  [SMALL_STATE(487)] = 15513,
  [SMALL_STATE(488)] = 15534,
  [SMALL_STATE(489)] = 15555,
  [SMALL_STATE(490)] = 15576,
  [SMALL_STATE(491)] = 15597,
  [SMALL_STATE(492)] = 15618,
  [SMALL_STATE(493)] = 15639,
  [SMALL_STATE(494)] = 15663,
  [SMALL_STATE(495)] = 15687,
  [SMALL_STATE(496)] = 15708,
  [SMALL_STATE(497)] = 15731,
  [SMALL_STATE(498)] = 15750,
  [SMALL_STATE(499)] = 15771,
  [SMALL_STATE(500)] = 15790,
  [SMALL_STATE(501)] = 15813,
  [SMALL_STATE(502)] = 15831,
  [SMALL_STATE(503)] = 15851,
  [SMALL_STATE(504)] = 15869,
  [SMALL_STATE(505)] = 15887,
  [SMALL_STATE(506)] = 15905,
  [SMALL_STATE(507)] = 15923,
  [SMALL_STATE(508)] = 15941,
  [SMALL_STATE(509)] = 15963,
  [SMALL_STATE(510)] = 15983,
  [SMALL_STATE(511)] = 16012,
  [SMALL_STATE(512)] = 16043,
  [SMALL_STATE(513)] = 16074,
  [SMALL_STATE(514)] = 16103,
  [SMALL_STATE(515)] = 16132,
  [SMALL_STATE(516)] = 16163,
  [SMALL_STATE(517)] = 16194,
  [SMALL_STATE(518)] = 16223,
  [SMALL_STATE(519)] = 16252,
  [SMALL_STATE(520)] = 16283,
  [SMALL_STATE(521)] = 16300,
  [SMALL_STATE(522)] = 16331,
  [SMALL_STATE(523)] = 16362,
  [SMALL_STATE(524)] = 16391,
  [SMALL_STATE(525)] = 16420,
  [SMALL_STATE(526)] = 16451,
  [SMALL_STATE(527)] = 16480,
  [SMALL_STATE(528)] = 16509,
  [SMALL_STATE(529)] = 16540,
  [SMALL_STATE(530)] = 16571,
  [SMALL_STATE(531)] = 16600,
  [SMALL_STATE(532)] = 16631,
  [SMALL_STATE(533)] = 16660,
  [SMALL_STATE(534)] = 16689,
  [SMALL_STATE(535)] = 16718,
  [SMALL_STATE(536)] = 16735,
  [SMALL_STATE(537)] = 16752,
  [SMALL_STATE(538)] = 16783,
  [SMALL_STATE(539)] = 16812,
  [SMALL_STATE(540)] = 16841,
  [SMALL_STATE(541)] = 16870,
  [SMALL_STATE(542)] = 16887,
  [SMALL_STATE(543)] = 16904,
  [SMALL_STATE(544)] = 16935,
  [SMALL_STATE(545)] = 16964,
  [SMALL_STATE(546)] = 16995,
  [SMALL_STATE(547)] = 17024,
  [SMALL_STATE(548)] = 17055,
  [SMALL_STATE(549)] = 17086,
  [SMALL_STATE(550)] = 17117,
  [SMALL_STATE(551)] = 17148,
  [SMALL_STATE(552)] = 17177,
  [SMALL_STATE(553)] = 17196,
  [SMALL_STATE(554)] = 17227,
  [SMALL_STATE(555)] = 17256,
  [SMALL_STATE(556)] = 17287,
  [SMALL_STATE(557)] = 17316,
  [SMALL_STATE(558)] = 17347,
  [SMALL_STATE(559)] = 17376,
  [SMALL_STATE(560)] = 17407,
  [SMALL_STATE(561)] = 17436,
  [SMALL_STATE(562)] = 17465,
  [SMALL_STATE(563)] = 17494,
  [SMALL_STATE(564)] = 17523,
  [SMALL_STATE(565)] = 17539,
  [SMALL_STATE(566)] = 17567,
  [SMALL_STATE(567)] = 17595,
  [SMALL_STATE(568)] = 17623,
  [SMALL_STATE(569)] = 17649,
  [SMALL_STATE(570)] = 17677,
  [SMALL_STATE(571)] = 17693,
  [SMALL_STATE(572)] = 17721,
  [SMALL_STATE(573)] = 17746,
  [SMALL_STATE(574)] = 17769,
  [SMALL_STATE(575)] = 17792,
  [SMALL_STATE(576)] = 17813,
  [SMALL_STATE(577)] = 17838,
  [SMALL_STATE(578)] = 17863,
  [SMALL_STATE(579)] = 17888,
  [SMALL_STATE(580)] = 17911,
  [SMALL_STATE(581)] = 17936,
  [SMALL_STATE(582)] = 17955,
  [SMALL_STATE(583)] = 17980,
  [SMALL_STATE(584)] = 18003,
  [SMALL_STATE(585)] = 18026,
  [SMALL_STATE(586)] = 18049,
  [SMALL_STATE(587)] = 18070,
  [SMALL_STATE(588)] = 18095,
  [SMALL_STATE(589)] = 18120,
  [SMALL_STATE(590)] = 18139,
  [SMALL_STATE(591)] = 18164,
  [SMALL_STATE(592)] = 18189,
  [SMALL_STATE(593)] = 18214,
  [SMALL_STATE(594)] = 18239,
  [SMALL_STATE(595)] = 18264,
  [SMALL_STATE(596)] = 18287,
  [SMALL_STATE(597)] = 18310,
  [SMALL_STATE(598)] = 18333,
  [SMALL_STATE(599)] = 18358,
  [SMALL_STATE(600)] = 18381,
  [SMALL_STATE(601)] = 18408,
  [SMALL_STATE(602)] = 18431,
  [SMALL_STATE(603)] = 18454,
  [SMALL_STATE(604)] = 18477,
  [SMALL_STATE(605)] = 18500,
  [SMALL_STATE(606)] = 18525,
  [SMALL_STATE(607)] = 18548,
  [SMALL_STATE(608)] = 18571,
  [SMALL_STATE(609)] = 18596,
  [SMALL_STATE(610)] = 18619,
  [SMALL_STATE(611)] = 18642,
  [SMALL_STATE(612)] = 18667,
  [SMALL_STATE(613)] = 18692,
  [SMALL_STATE(614)] = 18717,
  [SMALL_STATE(615)] = 18740,
  [SMALL_STATE(616)] = 18763,
  [SMALL_STATE(617)] = 18786,
  [SMALL_STATE(618)] = 18809,
  [SMALL_STATE(619)] = 18832,
  [SMALL_STATE(620)] = 18857,
  [SMALL_STATE(621)] = 18882,
  [SMALL_STATE(622)] = 18907,
  [SMALL_STATE(623)] = 18930,
  [SMALL_STATE(624)] = 18953,
  [SMALL_STATE(625)] = 18976,
  [SMALL_STATE(626)] = 18999,
  [SMALL_STATE(627)] = 19021,
  [SMALL_STATE(628)] = 19043,
  [SMALL_STATE(629)] = 19065,
  [SMALL_STATE(630)] = 19087,
  [SMALL_STATE(631)] = 19106,
  [SMALL_STATE(632)] = 19125,
  [SMALL_STATE(633)] = 19144,
  [SMALL_STATE(634)] = 19165,
  [SMALL_STATE(635)] = 19184,
  [SMALL_STATE(636)] = 19203,
  [SMALL_STATE(637)] = 19222,
  [SMALL_STATE(638)] = 19241,
  [SMALL_STATE(639)] = 19260,
  [SMALL_STATE(640)] = 19279,
  [SMALL_STATE(641)] = 19298,
  [SMALL_STATE(642)] = 19317,
  [SMALL_STATE(643)] = 19334,
  [SMALL_STATE(644)] = 19353,
  [SMALL_STATE(645)] = 19372,
  [SMALL_STATE(646)] = 19391,
  [SMALL_STATE(647)] = 19410,
  [SMALL_STATE(648)] = 19429,
  [SMALL_STATE(649)] = 19448,
  [SMALL_STATE(650)] = 19467,
  [SMALL_STATE(651)] = 19486,
  [SMALL_STATE(652)] = 19505,
  [SMALL_STATE(653)] = 19524,
  [SMALL_STATE(654)] = 19543,
  [SMALL_STATE(655)] = 19562,
  [SMALL_STATE(656)] = 19581,
  [SMALL_STATE(657)] = 19600,
  [SMALL_STATE(658)] = 19619,
  [SMALL_STATE(659)] = 19638,
  [SMALL_STATE(660)] = 19657,
  [SMALL_STATE(661)] = 19676,
  [SMALL_STATE(662)] = 19695,
  [SMALL_STATE(663)] = 19714,
  [SMALL_STATE(664)] = 19733,
  [SMALL_STATE(665)] = 19752,
  [SMALL_STATE(666)] = 19771,
  [SMALL_STATE(667)] = 19790,
  [SMALL_STATE(668)] = 19809,
  [SMALL_STATE(669)] = 19828,
  [SMALL_STATE(670)] = 19847,
  [SMALL_STATE(671)] = 19864,
  [SMALL_STATE(672)] = 19883,
  [SMALL_STATE(673)] = 19902,
  [SMALL_STATE(674)] = 19921,
  [SMALL_STATE(675)] = 19940,
  [SMALL_STATE(676)] = 19959,
  [SMALL_STATE(677)] = 19978,
  [SMALL_STATE(678)] = 19997,
  [SMALL_STATE(679)] = 20016,
  [SMALL_STATE(680)] = 20035,
  [SMALL_STATE(681)] = 20054,
  [SMALL_STATE(682)] = 20073,
  [SMALL_STATE(683)] = 20092,
  [SMALL_STATE(684)] = 20109,
  [SMALL_STATE(685)] = 20128,
  [SMALL_STATE(686)] = 20147,
  [SMALL_STATE(687)] = 20166,
  [SMALL_STATE(688)] = 20185,
  [SMALL_STATE(689)] = 20204,
  [SMALL_STATE(690)] = 20223,
  [SMALL_STATE(691)] = 20242,
  [SMALL_STATE(692)] = 20261,
  [SMALL_STATE(693)] = 20280,
  [SMALL_STATE(694)] = 20299,
  [SMALL_STATE(695)] = 20318,
  [SMALL_STATE(696)] = 20337,
  [SMALL_STATE(697)] = 20356,
  [SMALL_STATE(698)] = 20375,
  [SMALL_STATE(699)] = 20394,
  [SMALL_STATE(700)] = 20413,
  [SMALL_STATE(701)] = 20432,
  [SMALL_STATE(702)] = 20451,
  [SMALL_STATE(703)] = 20470,
  [SMALL_STATE(704)] = 20489,
  [SMALL_STATE(705)] = 20508,
  [SMALL_STATE(706)] = 20527,
  [SMALL_STATE(707)] = 20546,
  [SMALL_STATE(708)] = 20563,
  [SMALL_STATE(709)] = 20582,
  [SMALL_STATE(710)] = 20601,
  [SMALL_STATE(711)] = 20620,
  [SMALL_STATE(712)] = 20639,
  [SMALL_STATE(713)] = 20658,
  [SMALL_STATE(714)] = 20677,
  [SMALL_STATE(715)] = 20696,
  [SMALL_STATE(716)] = 20715,
  [SMALL_STATE(717)] = 20731,
  [SMALL_STATE(718)] = 20747,
  [SMALL_STATE(719)] = 20763,
  [SMALL_STATE(720)] = 20779,
  [SMALL_STATE(721)] = 20791,
  [SMALL_STATE(722)] = 20803,
  [SMALL_STATE(723)] = 20819,
  [SMALL_STATE(724)] = 20835,
  [SMALL_STATE(725)] = 20847,
  [SMALL_STATE(726)] = 20863,
  [SMALL_STATE(727)] = 20879,
  [SMALL_STATE(728)] = 20895,
  [SMALL_STATE(729)] = 20911,
  [SMALL_STATE(730)] = 20927,
  [SMALL_STATE(731)] = 20943,
  [SMALL_STATE(732)] = 20959,
  [SMALL_STATE(733)] = 20975,
  [SMALL_STATE(734)] = 20991,
  [SMALL_STATE(735)] = 21007,
  [SMALL_STATE(736)] = 21023,
  [SMALL_STATE(737)] = 21039,
  [SMALL_STATE(738)] = 21055,
  [SMALL_STATE(739)] = 21071,
  [SMALL_STATE(740)] = 21087,
  [SMALL_STATE(741)] = 21103,
  [SMALL_STATE(742)] = 21119,
  [SMALL_STATE(743)] = 21135,
  [SMALL_STATE(744)] = 21151,
  [SMALL_STATE(745)] = 21167,
  [SMALL_STATE(746)] = 21183,
  [SMALL_STATE(747)] = 21195,
  [SMALL_STATE(748)] = 21211,
  [SMALL_STATE(749)] = 21227,
  [SMALL_STATE(750)] = 21239,
  [SMALL_STATE(751)] = 21251,
  [SMALL_STATE(752)] = 21265,
  [SMALL_STATE(753)] = 21277,
  [SMALL_STATE(754)] = 21289,
  [SMALL_STATE(755)] = 21301,
  [SMALL_STATE(756)] = 21313,
  [SMALL_STATE(757)] = 21329,
  [SMALL_STATE(758)] = 21341,
  [SMALL_STATE(759)] = 21353,
  [SMALL_STATE(760)] = 21365,
  [SMALL_STATE(761)] = 21377,
  [SMALL_STATE(762)] = 21389,
  [SMALL_STATE(763)] = 21401,
  [SMALL_STATE(764)] = 21413,
  [SMALL_STATE(765)] = 21425,
  [SMALL_STATE(766)] = 21437,
  [SMALL_STATE(767)] = 21449,
  [SMALL_STATE(768)] = 21461,
  [SMALL_STATE(769)] = 21477,
  [SMALL_STATE(770)] = 21489,
  [SMALL_STATE(771)] = 21501,
  [SMALL_STATE(772)] = 21513,
  [SMALL_STATE(773)] = 21525,
  [SMALL_STATE(774)] = 21537,
  [SMALL_STATE(775)] = 21549,
  [SMALL_STATE(776)] = 21565,
  [SMALL_STATE(777)] = 21577,
  [SMALL_STATE(778)] = 21593,
  [SMALL_STATE(779)] = 21609,
  [SMALL_STATE(780)] = 21621,
  [SMALL_STATE(781)] = 21637,
  [SMALL_STATE(782)] = 21649,
  [SMALL_STATE(783)] = 21665,
  [SMALL_STATE(784)] = 21677,
  [SMALL_STATE(785)] = 21689,
  [SMALL_STATE(786)] = 21701,
  [SMALL_STATE(787)] = 21713,
  [SMALL_STATE(788)] = 21729,
  [SMALL_STATE(789)] = 21745,
  [SMALL_STATE(790)] = 21757,
  [SMALL_STATE(791)] = 21769,
  [SMALL_STATE(792)] = 21781,
  [SMALL_STATE(793)] = 21793,
  [SMALL_STATE(794)] = 21809,
  [SMALL_STATE(795)] = 21821,
  [SMALL_STATE(796)] = 21837,
  [SMALL_STATE(797)] = 21849,
  [SMALL_STATE(798)] = 21861,
  [SMALL_STATE(799)] = 21873,
  [SMALL_STATE(800)] = 21885,
  [SMALL_STATE(801)] = 21901,
  [SMALL_STATE(802)] = 21917,
  [SMALL_STATE(803)] = 21929,
  [SMALL_STATE(804)] = 21941,
  [SMALL_STATE(805)] = 21957,
  [SMALL_STATE(806)] = 21969,
  [SMALL_STATE(807)] = 21981,
  [SMALL_STATE(808)] = 21993,
  [SMALL_STATE(809)] = 22009,
  [SMALL_STATE(810)] = 22021,
  [SMALL_STATE(811)] = 22035,
  [SMALL_STATE(812)] = 22051,
  [SMALL_STATE(813)] = 22067,
  [SMALL_STATE(814)] = 22079,
  [SMALL_STATE(815)] = 22095,
  [SMALL_STATE(816)] = 22111,
  [SMALL_STATE(817)] = 22123,
  [SMALL_STATE(818)] = 22137,
  [SMALL_STATE(819)] = 22153,
  [SMALL_STATE(820)] = 22169,
  [SMALL_STATE(821)] = 22181,
  [SMALL_STATE(822)] = 22192,
  [SMALL_STATE(823)] = 22205,
  [SMALL_STATE(824)] = 22216,
  [SMALL_STATE(825)] = 22227,
  [SMALL_STATE(826)] = 22238,
  [SMALL_STATE(827)] = 22249,
  [SMALL_STATE(828)] = 22260,
  [SMALL_STATE(829)] = 22271,
  [SMALL_STATE(830)] = 22282,
  [SMALL_STATE(831)] = 22295,
  [SMALL_STATE(832)] = 22306,
  [SMALL_STATE(833)] = 22317,
  [SMALL_STATE(834)] = 22328,
  [SMALL_STATE(835)] = 22339,
  [SMALL_STATE(836)] = 22350,
  [SMALL_STATE(837)] = 22361,
  [SMALL_STATE(838)] = 22372,
  [SMALL_STATE(839)] = 22383,
  [SMALL_STATE(840)] = 22394,
  [SMALL_STATE(841)] = 22405,
  [SMALL_STATE(842)] = 22416,
  [SMALL_STATE(843)] = 22427,
  [SMALL_STATE(844)] = 22438,
  [SMALL_STATE(845)] = 22449,
  [SMALL_STATE(846)] = 22468,
  [SMALL_STATE(847)] = 22479,
  [SMALL_STATE(848)] = 22490,
  [SMALL_STATE(849)] = 22501,
  [SMALL_STATE(850)] = 22512,
  [SMALL_STATE(851)] = 22523,
  [SMALL_STATE(852)] = 22534,
  [SMALL_STATE(853)] = 22545,
  [SMALL_STATE(854)] = 22556,
  [SMALL_STATE(855)] = 22567,
  [SMALL_STATE(856)] = 22578,
  [SMALL_STATE(857)] = 22589,
  [SMALL_STATE(858)] = 22600,
  [SMALL_STATE(859)] = 22611,
  [SMALL_STATE(860)] = 22622,
  [SMALL_STATE(861)] = 22633,
  [SMALL_STATE(862)] = 22644,
  [SMALL_STATE(863)] = 22655,
  [SMALL_STATE(864)] = 22666,
  [SMALL_STATE(865)] = 22677,
  [SMALL_STATE(866)] = 22688,
  [SMALL_STATE(867)] = 22699,
  [SMALL_STATE(868)] = 22718,
  [SMALL_STATE(869)] = 22729,
  [SMALL_STATE(870)] = 22740,
  [SMALL_STATE(871)] = 22751,
  [SMALL_STATE(872)] = 22762,
  [SMALL_STATE(873)] = 22773,
  [SMALL_STATE(874)] = 22784,
  [SMALL_STATE(875)] = 22795,
  [SMALL_STATE(876)] = 22806,
  [SMALL_STATE(877)] = 22817,
  [SMALL_STATE(878)] = 22828,
  [SMALL_STATE(879)] = 22839,
  [SMALL_STATE(880)] = 22852,
  [SMALL_STATE(881)] = 22862,
  [SMALL_STATE(882)] = 22872,
  [SMALL_STATE(883)] = 22882,
  [SMALL_STATE(884)] = 22892,
  [SMALL_STATE(885)] = 22906,
  [SMALL_STATE(886)] = 22916,
  [SMALL_STATE(887)] = 22926,
  [SMALL_STATE(888)] = 22940,
  [SMALL_STATE(889)] = 22954,
  [SMALL_STATE(890)] = 22964,
  [SMALL_STATE(891)] = 22978,
  [SMALL_STATE(892)] = 22992,
  [SMALL_STATE(893)] = 23006,
  [SMALL_STATE(894)] = 23016,
  [SMALL_STATE(895)] = 23026,
  [SMALL_STATE(896)] = 23036,
  [SMALL_STATE(897)] = 23046,
  [SMALL_STATE(898)] = 23060,
  [SMALL_STATE(899)] = 23076,
  [SMALL_STATE(900)] = 23086,
  [SMALL_STATE(901)] = 23096,
  [SMALL_STATE(902)] = 23112,
  [SMALL_STATE(903)] = 23122,
  [SMALL_STATE(904)] = 23132,
  [SMALL_STATE(905)] = 23142,
  [SMALL_STATE(906)] = 23152,
  [SMALL_STATE(907)] = 23166,
  [SMALL_STATE(908)] = 23176,
  [SMALL_STATE(909)] = 23186,
  [SMALL_STATE(910)] = 23196,
  [SMALL_STATE(911)] = 23206,
  [SMALL_STATE(912)] = 23216,
  [SMALL_STATE(913)] = 23226,
  [SMALL_STATE(914)] = 23236,
  [SMALL_STATE(915)] = 23246,
  [SMALL_STATE(916)] = 23258,
  [SMALL_STATE(917)] = 23270,
  [SMALL_STATE(918)] = 23280,
  [SMALL_STATE(919)] = 23290,
  [SMALL_STATE(920)] = 23300,
  [SMALL_STATE(921)] = 23310,
  [SMALL_STATE(922)] = 23322,
  [SMALL_STATE(923)] = 23332,
  [SMALL_STATE(924)] = 23342,
  [SMALL_STATE(925)] = 23352,
  [SMALL_STATE(926)] = 23362,
  [SMALL_STATE(927)] = 23372,
  [SMALL_STATE(928)] = 23382,
  [SMALL_STATE(929)] = 23392,
  [SMALL_STATE(930)] = 23402,
  [SMALL_STATE(931)] = 23416,
  [SMALL_STATE(932)] = 23426,
  [SMALL_STATE(933)] = 23436,
  [SMALL_STATE(934)] = 23446,
  [SMALL_STATE(935)] = 23460,
  [SMALL_STATE(936)] = 23470,
  [SMALL_STATE(937)] = 23480,
  [SMALL_STATE(938)] = 23490,
  [SMALL_STATE(939)] = 23500,
  [SMALL_STATE(940)] = 23510,
  [SMALL_STATE(941)] = 23520,
  [SMALL_STATE(942)] = 23530,
  [SMALL_STATE(943)] = 23546,
  [SMALL_STATE(944)] = 23562,
  [SMALL_STATE(945)] = 23572,
  [SMALL_STATE(946)] = 23582,
  [SMALL_STATE(947)] = 23592,
  [SMALL_STATE(948)] = 23606,
  [SMALL_STATE(949)] = 23620,
  [SMALL_STATE(950)] = 23634,
  [SMALL_STATE(951)] = 23644,
  [SMALL_STATE(952)] = 23654,
  [SMALL_STATE(953)] = 23668,
  [SMALL_STATE(954)] = 23682,
  [SMALL_STATE(955)] = 23696,
  [SMALL_STATE(956)] = 23706,
  [SMALL_STATE(957)] = 23720,
  [SMALL_STATE(958)] = 23730,
  [SMALL_STATE(959)] = 23744,
  [SMALL_STATE(960)] = 23754,
  [SMALL_STATE(961)] = 23768,
  [SMALL_STATE(962)] = 23780,
  [SMALL_STATE(963)] = 23794,
  [SMALL_STATE(964)] = 23804,
  [SMALL_STATE(965)] = 23814,
  [SMALL_STATE(966)] = 23824,
  [SMALL_STATE(967)] = 23834,
  [SMALL_STATE(968)] = 23848,
  [SMALL_STATE(969)] = 23858,
  [SMALL_STATE(970)] = 23868,
  [SMALL_STATE(971)] = 23878,
  [SMALL_STATE(972)] = 23892,
  [SMALL_STATE(973)] = 23902,
  [SMALL_STATE(974)] = 23912,
  [SMALL_STATE(975)] = 23926,
  [SMALL_STATE(976)] = 23936,
  [SMALL_STATE(977)] = 23950,
  [SMALL_STATE(978)] = 23964,
  [SMALL_STATE(979)] = 23978,
  [SMALL_STATE(980)] = 23988,
  [SMALL_STATE(981)] = 24002,
  [SMALL_STATE(982)] = 24016,
  [SMALL_STATE(983)] = 24030,
  [SMALL_STATE(984)] = 24044,
  [SMALL_STATE(985)] = 24058,
  [SMALL_STATE(986)] = 24072,
  [SMALL_STATE(987)] = 24082,
  [SMALL_STATE(988)] = 24092,
  [SMALL_STATE(989)] = 24106,
  [SMALL_STATE(990)] = 24120,
  [SMALL_STATE(991)] = 24134,
  [SMALL_STATE(992)] = 24148,
  [SMALL_STATE(993)] = 24158,
  [SMALL_STATE(994)] = 24168,
  [SMALL_STATE(995)] = 24178,
  [SMALL_STATE(996)] = 24188,
  [SMALL_STATE(997)] = 24202,
  [SMALL_STATE(998)] = 24212,
  [SMALL_STATE(999)] = 24226,
  [SMALL_STATE(1000)] = 24240,
  [SMALL_STATE(1001)] = 24256,
  [SMALL_STATE(1002)] = 24270,
  [SMALL_STATE(1003)] = 24284,
  [SMALL_STATE(1004)] = 24300,
  [SMALL_STATE(1005)] = 24316,
  [SMALL_STATE(1006)] = 24332,
  [SMALL_STATE(1007)] = 24346,
  [SMALL_STATE(1008)] = 24360,
  [SMALL_STATE(1009)] = 24370,
  [SMALL_STATE(1010)] = 24380,
  [SMALL_STATE(1011)] = 24390,
  [SMALL_STATE(1012)] = 24400,
  [SMALL_STATE(1013)] = 24414,
  [SMALL_STATE(1014)] = 24428,
  [SMALL_STATE(1015)] = 24438,
  [SMALL_STATE(1016)] = 24452,
  [SMALL_STATE(1017)] = 24466,
  [SMALL_STATE(1018)] = 24476,
  [SMALL_STATE(1019)] = 24486,
  [SMALL_STATE(1020)] = 24496,
  [SMALL_STATE(1021)] = 24506,
  [SMALL_STATE(1022)] = 24516,
  [SMALL_STATE(1023)] = 24526,
  [SMALL_STATE(1024)] = 24536,
  [SMALL_STATE(1025)] = 24546,
  [SMALL_STATE(1026)] = 24556,
  [SMALL_STATE(1027)] = 24566,
  [SMALL_STATE(1028)] = 24576,
  [SMALL_STATE(1029)] = 24586,
  [SMALL_STATE(1030)] = 24596,
  [SMALL_STATE(1031)] = 24606,
  [SMALL_STATE(1032)] = 24616,
  [SMALL_STATE(1033)] = 24626,
  [SMALL_STATE(1034)] = 24636,
  [SMALL_STATE(1035)] = 24646,
  [SMALL_STATE(1036)] = 24656,
  [SMALL_STATE(1037)] = 24666,
  [SMALL_STATE(1038)] = 24680,
  [SMALL_STATE(1039)] = 24690,
  [SMALL_STATE(1040)] = 24700,
  [SMALL_STATE(1041)] = 24710,
  [SMALL_STATE(1042)] = 24720,
  [SMALL_STATE(1043)] = 24730,
  [SMALL_STATE(1044)] = 24740,
  [SMALL_STATE(1045)] = 24750,
  [SMALL_STATE(1046)] = 24760,
  [SMALL_STATE(1047)] = 24770,
  [SMALL_STATE(1048)] = 24784,
  [SMALL_STATE(1049)] = 24798,
  [SMALL_STATE(1050)] = 24810,
  [SMALL_STATE(1051)] = 24820,
  [SMALL_STATE(1052)] = 24830,
  [SMALL_STATE(1053)] = 24842,
  [SMALL_STATE(1054)] = 24852,
  [SMALL_STATE(1055)] = 24862,
  [SMALL_STATE(1056)] = 24872,
  [SMALL_STATE(1057)] = 24882,
  [SMALL_STATE(1058)] = 24892,
  [SMALL_STATE(1059)] = 24902,
  [SMALL_STATE(1060)] = 24912,
  [SMALL_STATE(1061)] = 24922,
  [SMALL_STATE(1062)] = 24932,
  [SMALL_STATE(1063)] = 24942,
  [SMALL_STATE(1064)] = 24952,
  [SMALL_STATE(1065)] = 24962,
  [SMALL_STATE(1066)] = 24972,
  [SMALL_STATE(1067)] = 24982,
  [SMALL_STATE(1068)] = 24992,
  [SMALL_STATE(1069)] = 25002,
  [SMALL_STATE(1070)] = 25012,
  [SMALL_STATE(1071)] = 25022,
  [SMALL_STATE(1072)] = 25032,
  [SMALL_STATE(1073)] = 25042,
  [SMALL_STATE(1074)] = 25052,
  [SMALL_STATE(1075)] = 25062,
  [SMALL_STATE(1076)] = 25072,
  [SMALL_STATE(1077)] = 25082,
  [SMALL_STATE(1078)] = 25092,
  [SMALL_STATE(1079)] = 25102,
  [SMALL_STATE(1080)] = 25112,
  [SMALL_STATE(1081)] = 25122,
  [SMALL_STATE(1082)] = 25132,
  [SMALL_STATE(1083)] = 25142,
  [SMALL_STATE(1084)] = 25152,
  [SMALL_STATE(1085)] = 25162,
  [SMALL_STATE(1086)] = 25172,
  [SMALL_STATE(1087)] = 25182,
  [SMALL_STATE(1088)] = 25192,
  [SMALL_STATE(1089)] = 25202,
  [SMALL_STATE(1090)] = 25212,
  [SMALL_STATE(1091)] = 25222,
  [SMALL_STATE(1092)] = 25232,
  [SMALL_STATE(1093)] = 25242,
  [SMALL_STATE(1094)] = 25252,
  [SMALL_STATE(1095)] = 25262,
  [SMALL_STATE(1096)] = 25272,
  [SMALL_STATE(1097)] = 25282,
  [SMALL_STATE(1098)] = 25292,
  [SMALL_STATE(1099)] = 25302,
  [SMALL_STATE(1100)] = 25314,
  [SMALL_STATE(1101)] = 25324,
  [SMALL_STATE(1102)] = 25334,
  [SMALL_STATE(1103)] = 25346,
  [SMALL_STATE(1104)] = 25356,
  [SMALL_STATE(1105)] = 25366,
  [SMALL_STATE(1106)] = 25376,
  [SMALL_STATE(1107)] = 25386,
  [SMALL_STATE(1108)] = 25396,
  [SMALL_STATE(1109)] = 25406,
  [SMALL_STATE(1110)] = 25416,
  [SMALL_STATE(1111)] = 25426,
  [SMALL_STATE(1112)] = 25440,
  [SMALL_STATE(1113)] = 25454,
  [SMALL_STATE(1114)] = 25468,
  [SMALL_STATE(1115)] = 25482,
  [SMALL_STATE(1116)] = 25492,
  [SMALL_STATE(1117)] = 25501,
  [SMALL_STATE(1118)] = 25510,
  [SMALL_STATE(1119)] = 25519,
  [SMALL_STATE(1120)] = 25528,
  [SMALL_STATE(1121)] = 25537,
  [SMALL_STATE(1122)] = 25546,
  [SMALL_STATE(1123)] = 25555,
  [SMALL_STATE(1124)] = 25564,
  [SMALL_STATE(1125)] = 25573,
  [SMALL_STATE(1126)] = 25582,
  [SMALL_STATE(1127)] = 25591,
  [SMALL_STATE(1128)] = 25600,
  [SMALL_STATE(1129)] = 25609,
  [SMALL_STATE(1130)] = 25618,
  [SMALL_STATE(1131)] = 25627,
  [SMALL_STATE(1132)] = 25636,
  [SMALL_STATE(1133)] = 25645,
  [SMALL_STATE(1134)] = 25654,
  [SMALL_STATE(1135)] = 25663,
  [SMALL_STATE(1136)] = 25676,
  [SMALL_STATE(1137)] = 25685,
  [SMALL_STATE(1138)] = 25694,
  [SMALL_STATE(1139)] = 25703,
  [SMALL_STATE(1140)] = 25712,
  [SMALL_STATE(1141)] = 25721,
  [SMALL_STATE(1142)] = 25734,
  [SMALL_STATE(1143)] = 25743,
  [SMALL_STATE(1144)] = 25752,
  [SMALL_STATE(1145)] = 25761,
  [SMALL_STATE(1146)] = 25770,
  [SMALL_STATE(1147)] = 25779,
  [SMALL_STATE(1148)] = 25788,
  [SMALL_STATE(1149)] = 25797,
  [SMALL_STATE(1150)] = 25810,
  [SMALL_STATE(1151)] = 25819,
  [SMALL_STATE(1152)] = 25828,
  [SMALL_STATE(1153)] = 25837,
  [SMALL_STATE(1154)] = 25846,
  [SMALL_STATE(1155)] = 25855,
  [SMALL_STATE(1156)] = 25864,
  [SMALL_STATE(1157)] = 25873,
  [SMALL_STATE(1158)] = 25882,
  [SMALL_STATE(1159)] = 25891,
  [SMALL_STATE(1160)] = 25900,
  [SMALL_STATE(1161)] = 25909,
  [SMALL_STATE(1162)] = 25918,
  [SMALL_STATE(1163)] = 25927,
  [SMALL_STATE(1164)] = 25936,
  [SMALL_STATE(1165)] = 25945,
  [SMALL_STATE(1166)] = 25954,
  [SMALL_STATE(1167)] = 25963,
  [SMALL_STATE(1168)] = 25972,
  [SMALL_STATE(1169)] = 25981,
  [SMALL_STATE(1170)] = 25990,
  [SMALL_STATE(1171)] = 25999,
  [SMALL_STATE(1172)] = 26008,
  [SMALL_STATE(1173)] = 26017,
  [SMALL_STATE(1174)] = 26026,
  [SMALL_STATE(1175)] = 26035,
  [SMALL_STATE(1176)] = 26044,
  [SMALL_STATE(1177)] = 26053,
  [SMALL_STATE(1178)] = 26062,
  [SMALL_STATE(1179)] = 26071,
  [SMALL_STATE(1180)] = 26080,
  [SMALL_STATE(1181)] = 26089,
  [SMALL_STATE(1182)] = 26098,
  [SMALL_STATE(1183)] = 26107,
  [SMALL_STATE(1184)] = 26116,
  [SMALL_STATE(1185)] = 26125,
  [SMALL_STATE(1186)] = 26134,
  [SMALL_STATE(1187)] = 26143,
  [SMALL_STATE(1188)] = 26152,
  [SMALL_STATE(1189)] = 26161,
  [SMALL_STATE(1190)] = 26170,
  [SMALL_STATE(1191)] = 26179,
  [SMALL_STATE(1192)] = 26192,
  [SMALL_STATE(1193)] = 26201,
  [SMALL_STATE(1194)] = 26210,
  [SMALL_STATE(1195)] = 26223,
  [SMALL_STATE(1196)] = 26232,
  [SMALL_STATE(1197)] = 26241,
  [SMALL_STATE(1198)] = 26250,
  [SMALL_STATE(1199)] = 26259,
  [SMALL_STATE(1200)] = 26268,
  [SMALL_STATE(1201)] = 26277,
  [SMALL_STATE(1202)] = 26286,
  [SMALL_STATE(1203)] = 26295,
  [SMALL_STATE(1204)] = 26304,
  [SMALL_STATE(1205)] = 26313,
  [SMALL_STATE(1206)] = 26326,
  [SMALL_STATE(1207)] = 26339,
  [SMALL_STATE(1208)] = 26348,
  [SMALL_STATE(1209)] = 26357,
  [SMALL_STATE(1210)] = 26366,
  [SMALL_STATE(1211)] = 26375,
  [SMALL_STATE(1212)] = 26384,
  [SMALL_STATE(1213)] = 26393,
  [SMALL_STATE(1214)] = 26402,
  [SMALL_STATE(1215)] = 26411,
  [SMALL_STATE(1216)] = 26420,
  [SMALL_STATE(1217)] = 26429,
  [SMALL_STATE(1218)] = 26438,
  [SMALL_STATE(1219)] = 26447,
  [SMALL_STATE(1220)] = 26460,
  [SMALL_STATE(1221)] = 26473,
  [SMALL_STATE(1222)] = 26482,
  [SMALL_STATE(1223)] = 26491,
  [SMALL_STATE(1224)] = 26500,
  [SMALL_STATE(1225)] = 26509,
  [SMALL_STATE(1226)] = 26518,
  [SMALL_STATE(1227)] = 26527,
  [SMALL_STATE(1228)] = 26540,
  [SMALL_STATE(1229)] = 26553,
  [SMALL_STATE(1230)] = 26562,
  [SMALL_STATE(1231)] = 26573,
  [SMALL_STATE(1232)] = 26584,
  [SMALL_STATE(1233)] = 26593,
  [SMALL_STATE(1234)] = 26602,
  [SMALL_STATE(1235)] = 26611,
  [SMALL_STATE(1236)] = 26620,
  [SMALL_STATE(1237)] = 26629,
  [SMALL_STATE(1238)] = 26640,
  [SMALL_STATE(1239)] = 26649,
  [SMALL_STATE(1240)] = 26658,
  [SMALL_STATE(1241)] = 26667,
  [SMALL_STATE(1242)] = 26680,
  [SMALL_STATE(1243)] = 26693,
  [SMALL_STATE(1244)] = 26702,
  [SMALL_STATE(1245)] = 26711,
  [SMALL_STATE(1246)] = 26720,
  [SMALL_STATE(1247)] = 26729,
  [SMALL_STATE(1248)] = 26737,
  [SMALL_STATE(1249)] = 26745,
  [SMALL_STATE(1250)] = 26753,
  [SMALL_STATE(1251)] = 26761,
  [SMALL_STATE(1252)] = 26769,
  [SMALL_STATE(1253)] = 26777,
  [SMALL_STATE(1254)] = 26785,
  [SMALL_STATE(1255)] = 26793,
  [SMALL_STATE(1256)] = 26801,
  [SMALL_STATE(1257)] = 26809,
  [SMALL_STATE(1258)] = 26819,
  [SMALL_STATE(1259)] = 26827,
  [SMALL_STATE(1260)] = 26835,
  [SMALL_STATE(1261)] = 26845,
  [SMALL_STATE(1262)] = 26853,
  [SMALL_STATE(1263)] = 26861,
  [SMALL_STATE(1264)] = 26869,
  [SMALL_STATE(1265)] = 26877,
  [SMALL_STATE(1266)] = 26885,
  [SMALL_STATE(1267)] = 26893,
  [SMALL_STATE(1268)] = 26901,
  [SMALL_STATE(1269)] = 26909,
  [SMALL_STATE(1270)] = 26917,
  [SMALL_STATE(1271)] = 26925,
  [SMALL_STATE(1272)] = 26933,
  [SMALL_STATE(1273)] = 26941,
  [SMALL_STATE(1274)] = 26949,
  [SMALL_STATE(1275)] = 26957,
  [SMALL_STATE(1276)] = 26965,
  [SMALL_STATE(1277)] = 26973,
  [SMALL_STATE(1278)] = 26981,
  [SMALL_STATE(1279)] = 26989,
  [SMALL_STATE(1280)] = 26997,
  [SMALL_STATE(1281)] = 27005,
  [SMALL_STATE(1282)] = 27013,
  [SMALL_STATE(1283)] = 27021,
  [SMALL_STATE(1284)] = 27029,
  [SMALL_STATE(1285)] = 27037,
  [SMALL_STATE(1286)] = 27045,
  [SMALL_STATE(1287)] = 27053,
  [SMALL_STATE(1288)] = 27061,
  [SMALL_STATE(1289)] = 27069,
  [SMALL_STATE(1290)] = 27077,
  [SMALL_STATE(1291)] = 27085,
  [SMALL_STATE(1292)] = 27093,
  [SMALL_STATE(1293)] = 27101,
  [SMALL_STATE(1294)] = 27109,
  [SMALL_STATE(1295)] = 27117,
  [SMALL_STATE(1296)] = 27125,
  [SMALL_STATE(1297)] = 27133,
  [SMALL_STATE(1298)] = 27141,
  [SMALL_STATE(1299)] = 27149,
  [SMALL_STATE(1300)] = 27157,
  [SMALL_STATE(1301)] = 27165,
  [SMALL_STATE(1302)] = 27173,
  [SMALL_STATE(1303)] = 27181,
  [SMALL_STATE(1304)] = 27189,
  [SMALL_STATE(1305)] = 27197,
  [SMALL_STATE(1306)] = 27205,
  [SMALL_STATE(1307)] = 27213,
  [SMALL_STATE(1308)] = 27221,
  [SMALL_STATE(1309)] = 27229,
  [SMALL_STATE(1310)] = 27237,
  [SMALL_STATE(1311)] = 27245,
  [SMALL_STATE(1312)] = 27253,
  [SMALL_STATE(1313)] = 27261,
  [SMALL_STATE(1314)] = 27269,
  [SMALL_STATE(1315)] = 27277,
  [SMALL_STATE(1316)] = 27285,
  [SMALL_STATE(1317)] = 27293,
  [SMALL_STATE(1318)] = 27301,
  [SMALL_STATE(1319)] = 27309,
  [SMALL_STATE(1320)] = 27317,
  [SMALL_STATE(1321)] = 27325,
  [SMALL_STATE(1322)] = 27333,
  [SMALL_STATE(1323)] = 27341,
  [SMALL_STATE(1324)] = 27349,
  [SMALL_STATE(1325)] = 27357,
  [SMALL_STATE(1326)] = 27365,
  [SMALL_STATE(1327)] = 27373,
  [SMALL_STATE(1328)] = 27381,
  [SMALL_STATE(1329)] = 27389,
  [SMALL_STATE(1330)] = 27397,
  [SMALL_STATE(1331)] = 27407,
  [SMALL_STATE(1332)] = 27415,
  [SMALL_STATE(1333)] = 27425,
  [SMALL_STATE(1334)] = 27433,
  [SMALL_STATE(1335)] = 27441,
  [SMALL_STATE(1336)] = 27449,
  [SMALL_STATE(1337)] = 27457,
  [SMALL_STATE(1338)] = 27465,
  [SMALL_STATE(1339)] = 27473,
  [SMALL_STATE(1340)] = 27481,
  [SMALL_STATE(1341)] = 27489,
  [SMALL_STATE(1342)] = 27497,
  [SMALL_STATE(1343)] = 27505,
  [SMALL_STATE(1344)] = 27513,
  [SMALL_STATE(1345)] = 27521,
  [SMALL_STATE(1346)] = 27529,
  [SMALL_STATE(1347)] = 27537,
  [SMALL_STATE(1348)] = 27545,
  [SMALL_STATE(1349)] = 27553,
  [SMALL_STATE(1350)] = 27561,
  [SMALL_STATE(1351)] = 27569,
  [SMALL_STATE(1352)] = 27579,
  [SMALL_STATE(1353)] = 27587,
  [SMALL_STATE(1354)] = 27595,
  [SMALL_STATE(1355)] = 27603,
  [SMALL_STATE(1356)] = 27611,
  [SMALL_STATE(1357)] = 27619,
  [SMALL_STATE(1358)] = 27627,
  [SMALL_STATE(1359)] = 27635,
  [SMALL_STATE(1360)] = 27643,
  [SMALL_STATE(1361)] = 27651,
  [SMALL_STATE(1362)] = 27659,
  [SMALL_STATE(1363)] = 27667,
  [SMALL_STATE(1364)] = 27675,
  [SMALL_STATE(1365)] = 27683,
  [SMALL_STATE(1366)] = 27691,
  [SMALL_STATE(1367)] = 27699,
  [SMALL_STATE(1368)] = 27707,
  [SMALL_STATE(1369)] = 27715,
  [SMALL_STATE(1370)] = 27723,
  [SMALL_STATE(1371)] = 27731,
  [SMALL_STATE(1372)] = 27739,
  [SMALL_STATE(1373)] = 27747,
  [SMALL_STATE(1374)] = 27755,
  [SMALL_STATE(1375)] = 27763,
  [SMALL_STATE(1376)] = 27771,
  [SMALL_STATE(1377)] = 27779,
  [SMALL_STATE(1378)] = 27787,
  [SMALL_STATE(1379)] = 27795,
  [SMALL_STATE(1380)] = 27803,
  [SMALL_STATE(1381)] = 27811,
  [SMALL_STATE(1382)] = 27819,
  [SMALL_STATE(1383)] = 27827,
  [SMALL_STATE(1384)] = 27835,
  [SMALL_STATE(1385)] = 27843,
  [SMALL_STATE(1386)] = 27851,
  [SMALL_STATE(1387)] = 27859,
  [SMALL_STATE(1388)] = 27867,
  [SMALL_STATE(1389)] = 27875,
  [SMALL_STATE(1390)] = 27883,
  [SMALL_STATE(1391)] = 27891,
  [SMALL_STATE(1392)] = 27899,
  [SMALL_STATE(1393)] = 27907,
  [SMALL_STATE(1394)] = 27915,
  [SMALL_STATE(1395)] = 27923,
  [SMALL_STATE(1396)] = 27931,
  [SMALL_STATE(1397)] = 27939,
  [SMALL_STATE(1398)] = 27947,
  [SMALL_STATE(1399)] = 27955,
  [SMALL_STATE(1400)] = 27963,
  [SMALL_STATE(1401)] = 27971,
  [SMALL_STATE(1402)] = 27979,
  [SMALL_STATE(1403)] = 27987,
  [SMALL_STATE(1404)] = 27995,
  [SMALL_STATE(1405)] = 28003,
  [SMALL_STATE(1406)] = 28011,
  [SMALL_STATE(1407)] = 28019,
  [SMALL_STATE(1408)] = 28027,
  [SMALL_STATE(1409)] = 28035,
  [SMALL_STATE(1410)] = 28043,
  [SMALL_STATE(1411)] = 28051,
  [SMALL_STATE(1412)] = 28059,
  [SMALL_STATE(1413)] = 28067,
  [SMALL_STATE(1414)] = 28075,
  [SMALL_STATE(1415)] = 28083,
  [SMALL_STATE(1416)] = 28091,
  [SMALL_STATE(1417)] = 28099,
  [SMALL_STATE(1418)] = 28107,
  [SMALL_STATE(1419)] = 28115,
  [SMALL_STATE(1420)] = 28123,
  [SMALL_STATE(1421)] = 28131,
  [SMALL_STATE(1422)] = 28139,
  [SMALL_STATE(1423)] = 28147,
  [SMALL_STATE(1424)] = 28155,
  [SMALL_STATE(1425)] = 28163,
  [SMALL_STATE(1426)] = 28171,
  [SMALL_STATE(1427)] = 28179,
  [SMALL_STATE(1428)] = 28187,
  [SMALL_STATE(1429)] = 28195,
  [SMALL_STATE(1430)] = 28203,
  [SMALL_STATE(1431)] = 28211,
  [SMALL_STATE(1432)] = 28219,
  [SMALL_STATE(1433)] = 28227,
  [SMALL_STATE(1434)] = 28235,
  [SMALL_STATE(1435)] = 28243,
  [SMALL_STATE(1436)] = 28251,
  [SMALL_STATE(1437)] = 28259,
  [SMALL_STATE(1438)] = 28267,
  [SMALL_STATE(1439)] = 28275,
  [SMALL_STATE(1440)] = 28283,
  [SMALL_STATE(1441)] = 28291,
  [SMALL_STATE(1442)] = 28299,
  [SMALL_STATE(1443)] = 28307,
  [SMALL_STATE(1444)] = 28315,
  [SMALL_STATE(1445)] = 28323,
  [SMALL_STATE(1446)] = 28331,
  [SMALL_STATE(1447)] = 28341,
  [SMALL_STATE(1448)] = 28349,
  [SMALL_STATE(1449)] = 28357,
  [SMALL_STATE(1450)] = 28367,
  [SMALL_STATE(1451)] = 28375,
  [SMALL_STATE(1452)] = 28383,
  [SMALL_STATE(1453)] = 28391,
  [SMALL_STATE(1454)] = 28399,
  [SMALL_STATE(1455)] = 28406,
  [SMALL_STATE(1456)] = 28413,
  [SMALL_STATE(1457)] = 28420,
  [SMALL_STATE(1458)] = 28427,
  [SMALL_STATE(1459)] = 28434,
  [SMALL_STATE(1460)] = 28441,
  [SMALL_STATE(1461)] = 28448,
  [SMALL_STATE(1462)] = 28455,
  [SMALL_STATE(1463)] = 28462,
  [SMALL_STATE(1464)] = 28469,
  [SMALL_STATE(1465)] = 28476,
  [SMALL_STATE(1466)] = 28483,
  [SMALL_STATE(1467)] = 28490,
  [SMALL_STATE(1468)] = 28497,
  [SMALL_STATE(1469)] = 28504,
  [SMALL_STATE(1470)] = 28511,
  [SMALL_STATE(1471)] = 28518,
  [SMALL_STATE(1472)] = 28525,
  [SMALL_STATE(1473)] = 28532,
  [SMALL_STATE(1474)] = 28539,
  [SMALL_STATE(1475)] = 28546,
  [SMALL_STATE(1476)] = 28553,
  [SMALL_STATE(1477)] = 28560,
  [SMALL_STATE(1478)] = 28567,
  [SMALL_STATE(1479)] = 28574,
  [SMALL_STATE(1480)] = 28581,
  [SMALL_STATE(1481)] = 28588,
  [SMALL_STATE(1482)] = 28595,
  [SMALL_STATE(1483)] = 28602,
  [SMALL_STATE(1484)] = 28609,
  [SMALL_STATE(1485)] = 28616,
  [SMALL_STATE(1486)] = 28623,
  [SMALL_STATE(1487)] = 28630,
  [SMALL_STATE(1488)] = 28637,
  [SMALL_STATE(1489)] = 28644,
  [SMALL_STATE(1490)] = 28651,
  [SMALL_STATE(1491)] = 28658,
  [SMALL_STATE(1492)] = 28665,
  [SMALL_STATE(1493)] = 28672,
  [SMALL_STATE(1494)] = 28679,
  [SMALL_STATE(1495)] = 28686,
  [SMALL_STATE(1496)] = 28693,
  [SMALL_STATE(1497)] = 28700,
  [SMALL_STATE(1498)] = 28707,
  [SMALL_STATE(1499)] = 28714,
  [SMALL_STATE(1500)] = 28721,
  [SMALL_STATE(1501)] = 28728,
  [SMALL_STATE(1502)] = 28735,
  [SMALL_STATE(1503)] = 28742,
  [SMALL_STATE(1504)] = 28749,
  [SMALL_STATE(1505)] = 28756,
  [SMALL_STATE(1506)] = 28763,
  [SMALL_STATE(1507)] = 28770,
  [SMALL_STATE(1508)] = 28777,
  [SMALL_STATE(1509)] = 28784,
  [SMALL_STATE(1510)] = 28791,
  [SMALL_STATE(1511)] = 28798,
  [SMALL_STATE(1512)] = 28805,
  [SMALL_STATE(1513)] = 28812,
  [SMALL_STATE(1514)] = 28819,
  [SMALL_STATE(1515)] = 28826,
  [SMALL_STATE(1516)] = 28833,
  [SMALL_STATE(1517)] = 28840,
  [SMALL_STATE(1518)] = 28847,
  [SMALL_STATE(1519)] = 28854,
  [SMALL_STATE(1520)] = 28861,
  [SMALL_STATE(1521)] = 28868,
  [SMALL_STATE(1522)] = 28875,
  [SMALL_STATE(1523)] = 28882,
  [SMALL_STATE(1524)] = 28889,
  [SMALL_STATE(1525)] = 28896,
  [SMALL_STATE(1526)] = 28903,
  [SMALL_STATE(1527)] = 28910,
  [SMALL_STATE(1528)] = 28917,
  [SMALL_STATE(1529)] = 28924,
  [SMALL_STATE(1530)] = 28931,
  [SMALL_STATE(1531)] = 28938,
  [SMALL_STATE(1532)] = 28945,
  [SMALL_STATE(1533)] = 28952,
  [SMALL_STATE(1534)] = 28959,
  [SMALL_STATE(1535)] = 28966,
  [SMALL_STATE(1536)] = 28973,
  [SMALL_STATE(1537)] = 28980,
  [SMALL_STATE(1538)] = 28987,
  [SMALL_STATE(1539)] = 28994,
  [SMALL_STATE(1540)] = 29001,
  [SMALL_STATE(1541)] = 29008,
  [SMALL_STATE(1542)] = 29015,
  [SMALL_STATE(1543)] = 29022,
  [SMALL_STATE(1544)] = 29029,
  [SMALL_STATE(1545)] = 29036,
  [SMALL_STATE(1546)] = 29043,
  [SMALL_STATE(1547)] = 29050,
  [SMALL_STATE(1548)] = 29057,
  [SMALL_STATE(1549)] = 29064,
  [SMALL_STATE(1550)] = 29071,
  [SMALL_STATE(1551)] = 29078,
  [SMALL_STATE(1552)] = 29085,
  [SMALL_STATE(1553)] = 29092,
  [SMALL_STATE(1554)] = 29099,
  [SMALL_STATE(1555)] = 29106,
  [SMALL_STATE(1556)] = 29113,
  [SMALL_STATE(1557)] = 29120,
  [SMALL_STATE(1558)] = 29127,
  [SMALL_STATE(1559)] = 29134,
  [SMALL_STATE(1560)] = 29141,
  [SMALL_STATE(1561)] = 29148,
  [SMALL_STATE(1562)] = 29155,
  [SMALL_STATE(1563)] = 29162,
  [SMALL_STATE(1564)] = 29169,
  [SMALL_STATE(1565)] = 29176,
  [SMALL_STATE(1566)] = 29183,
  [SMALL_STATE(1567)] = 29190,
  [SMALL_STATE(1568)] = 29197,
};

static const TSParseActionEntry ts_parse_actions[] = {
  [0] = {.entry = {.count = 0, .reusable = false}},
  [1] = {.entry = {.count = 1, .reusable = false}}, RECOVER(),
  [3] = {.entry = {.count = 1, .reusable = true}}, SHIFT_EXTRA(),
  [5] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1477),
  [7] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1454),
  [9] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1509),
  [11] = {.entry = {.count = 1, .reusable = true}}, SHIFT(738),
  [13] = {.entry = {.count = 1, .reusable = true}}, SHIFT(12),
  [15] = {.entry = {.count = 1, .reusable = true}}, SHIFT(252),
  [17] = {.entry = {.count = 1, .reusable = true}}, SHIFT(9),
  [19] = {.entry = {.count = 1, .reusable = true}}, SHIFT(10),
  [21] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5),
  [23] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6),
  [25] = {.entry = {.count = 1, .reusable = true}}, SHIFT(7),
  [27] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2),
  [29] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1135),
  [31] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1141),
  [33] = {.entry = {.count = 1, .reusable = true}}, SHIFT(27),
  [35] = {.entry = {.count = 1, .reusable = true}}, SHIFT(34),
  [37] = {.entry = {.count = 1, .reusable = true}}, SHIFT(18),
  [39] = {.entry = {.count = 1, .reusable = true}}, SHIFT(19),
  [41] = {.entry = {.count = 1, .reusable = true}}, SHIFT(511),
  [43] = {.entry = {.count = 1, .reusable = true}}, SHIFT(512),
  [45] = {.entry = {.count = 1, .reusable = true}}, SHIFT(572),
  [47] = {.entry = {.count = 1, .reusable = true}}, SHIFT(598),
  [49] = {.entry = {.count = 1, .reusable = true}}, SHIFT(284),
  [51] = {.entry = {.count = 1, .reusable = true}}, SHIFT(285),
  [53] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1147),
  [55] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1151),
  [57] = {.entry = {.count = 1, .reusable = true}}, SHIFT(97),
  [59] = {.entry = {.count = 1, .reusable = true}}, SHIFT(96),
  [61] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1456),
  [63] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1462),
  [65] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1463),
  [67] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1464),
  [69] = {.entry = {.count = 1, .reusable = true}}, SHIFT(8),
  [71] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_val_itm, 1),
  [73] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1242),
  [75] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1241),
  [77] = {.entry = {.count = 1, .reusable = true}}, SHIFT(29),
  [79] = {.entry = {.count = 1, .reusable = true}}, SHIFT(30),
  [81] = {.entry = {.count = 1, .reusable = true}}, SHIFT(21),
  [83] = {.entry = {.count = 1, .reusable = true}}, SHIFT(20),
  [85] = {.entry = {.count = 1, .reusable = true}}, SHIFT(515),
  [87] = {.entry = {.count = 1, .reusable = true}}, SHIFT(555),
  [89] = {.entry = {.count = 1, .reusable = true}}, SHIFT(620),
  [91] = {.entry = {.count = 1, .reusable = true}}, SHIFT(608),
  [93] = {.entry = {.count = 1, .reusable = true}}, SHIFT(354),
  [95] = {.entry = {.count = 1, .reusable = true}}, SHIFT(371),
  [97] = {.entry = {.count = 1, .reusable = true}}, SHIFT(405),
  [99] = {.entry = {.count = 1, .reusable = true}}, SHIFT(406),
  [101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(103),
  [103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(101),
  [105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1500),
  [107] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1490),
  [109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1481),
  [111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1497),
  [113] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_blk_val_itm, 1),
  [115] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_blk_key_itm, 1),
  [117] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_key_itm, 1),
  [119] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_key_itm, 1),
  [121] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_val_itm, 1),
  [123] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_blk_seq_itm, 1),
  [125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1205),
  [127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1206),
  [129] = {.entry = {.count = 1, .reusable = true}}, SHIFT(25),
  [131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(31),
  [133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(15),
  [135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(22),
  [137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(543),
  [139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(547),
  [141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(582),
  [143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(576),
  [145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(295),
  [147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(307),
  [149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1285),
  [151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1286),
  [153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(104),
  [155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(102),
  [157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1498),
  [159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1495),
  [161] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_seq_itm, 1),
  [163] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_seq_itm, 1),
  [165] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__blk_imp_itm_tal, 1),
  [167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(65),
  [169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(45),
  [171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(532),
  [173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(602),
  [175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(424),
  [177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(107),
  [179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1548),
  [181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1538),
  [183] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__exp_doc, 1),
  [185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(60),
  [187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(43),
  [189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(563),
  [191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(595),
  [193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1137),
  [195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(150),
  [197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1489),
  [199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1494),
  [201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(33),
  [203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(58),
  [205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(17),
  [207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(53),
  [209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1052),
  [211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1053),
  [213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(41),
  [215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(86),
  [217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(62),
  [219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(88),
  [221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(525),
  [223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(562),
  [225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(611),
  [227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(618),
  [229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(761),
  [231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(760),
  [233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(759),
  [235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(753),
  [237] = {.entry = {.count = 1, .reusable = true}}, SHIFT(187),
  [239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(184),
  [241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1472),
  [243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1482),
  [245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1458),
  [247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1469),
  [249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1102),
  [251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1103),
  [253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(296),
  [255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1442),
  [257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(830),
  [259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(831),
  [261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(810),
  [263] = {.entry = {.count = 1, .reusable = true}}, SHIFT(809),
  [265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(275),
  [267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1188),
  [269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1239),
  [271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(332),
  [273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(448),
  [275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(447),
  [277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1452),
  [279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1351),
  [281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1358),
  [283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(32),
  [285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(67),
  [287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1049),
  [289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1050),
  [291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(16),
  [293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(46),
  [295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(40),
  [297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(84),
  [299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(59),
  [301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(93),
  [303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(550),
  [305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(524),
  [307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(580),
  [309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(614),
  [311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(868),
  [313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(869),
  [315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(874),
  [317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(843),
  [319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(194),
  [321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(193),
  [323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1483),
  [325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1499),
  [327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1470),
  [329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1480),
  [331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(313),
  [333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1433),
  [335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1099),
  [337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1100),
  [339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(286),
  [341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1163),
  [343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1332),
  [345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1345),
  [347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(329),
  [349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(445),
  [351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(446),
  [353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1247),
  [355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(822),
  [357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(823),
  [359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(817),
  [361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(816),
  [363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1159),
  [365] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_prp_val, 1),
  [367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(209),
  [369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1460),
  [371] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_prp_val, 1),
  [373] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__doc_wo_bgn_w_end_seq, 1),
  [375] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__doc_w_bgn_w_end_seq, 1),
  [377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1515),
  [379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(28),
  [381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(64),
  [383] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__r_flw_exp_par, 1), REDUCE(sym__r_sgl_flw_exp_par, 1),
  [386] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_exp_par, 1),
  [388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(23),
  [390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(50),
  [392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(553),
  [394] = {.entry = {.count = 1, .reusable = true}}, SHIFT(513),
  [396] = {.entry = {.count = 1, .reusable = true}}, SHIFT(578),
  [398] = {.entry = {.count = 1, .reusable = true}}, SHIFT(623),
  [400] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1337),
  [402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1336),
  [404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1335),
  [406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1334),
  [408] = {.entry = {.count = 1, .reusable = true}}, SHIFT(229),
  [410] = {.entry = {.count = 1, .reusable = true}}, SHIFT(215),
  [412] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1533),
  [414] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1534),
  [416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1475),
  [418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1457),
  [420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(68),
  [422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(49),
  [424] = {.entry = {.count = 1, .reusable = true}}, SHIFT(85),
  [426] = {.entry = {.count = 1, .reusable = true}}, SHIFT(90),
  [428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(538),
  [430] = {.entry = {.count = 1, .reusable = true}}, SHIFT(599),
  [432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(185),
  [434] = {.entry = {.count = 1, .reusable = true}}, SHIFT(833),
  [436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1104),
  [438] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1360),
  [440] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1054),
  [442] = {.entry = {.count = 1, .reusable = true}}, SHIFT(807),
  [444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(56),
  [446] = {.entry = {.count = 1, .reusable = true}}, SHIFT(42),
  [448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(87),
  [450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(94),
  [452] = {.entry = {.count = 1, .reusable = true}}, SHIFT(556),
  [454] = {.entry = {.count = 1, .reusable = true}}, SHIFT(601),
  [456] = {.entry = {.count = 1, .reusable = true}}, SHIFT(192),
  [458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(813),
  [460] = {.entry = {.count = 1, .reusable = true}}, SHIFT(24),
  [462] = {.entry = {.count = 1, .reusable = true}}, SHIFT(63),
  [464] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__r_flw_njl_ann_par, 1), REDUCE(sym__r_sgl_flw_njl_ann_par, 1),
  [467] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_njl_ann_par, 1),
  [469] = {.entry = {.count = 1, .reusable = true}}, SHIFT(13),
  [471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(52),
  [473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(529),
  [475] = {.entry = {.count = 1, .reusable = true}}, SHIFT(523),
  [477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(605),
  [479] = {.entry = {.count = 1, .reusable = true}}, SHIFT(616),
  [481] = {.entry = {.count = 1, .reusable = true}}, SHIFT(925),
  [483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(880),
  [485] = {.entry = {.count = 1, .reusable = true}}, SHIFT(924),
  [487] = {.entry = {.count = 1, .reusable = true}}, SHIFT(922),
  [489] = {.entry = {.count = 1, .reusable = true}}, SHIFT(196),
  [491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(201),
  [493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1536),
  [495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1537),
  [497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1507),
  [499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1476),
  [501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(26),
  [503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(66),
  [505] = {.entry = {.count = 1, .reusable = true}}, SHIFT(14),
  [507] = {.entry = {.count = 1, .reusable = true}}, SHIFT(47),
  [509] = {.entry = {.count = 1, .reusable = true}}, SHIFT(537),
  [511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(546),
  [513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(594),
  [515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(584),
  [517] = {.entry = {.count = 1, .reusable = true}}, SHIFT(881),
  [519] = {.entry = {.count = 1, .reusable = true}}, SHIFT(907),
  [521] = {.entry = {.count = 1, .reusable = true}}, SHIFT(908),
  [523] = {.entry = {.count = 1, .reusable = true}}, SHIFT(909),
  [525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(198),
  [527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(203),
  [529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1542),
  [531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1543),
  [533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1535),
  [535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1508),
  [537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1051),
  [539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1349),
  [541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1101),
  [543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(829),
  [545] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_map_dat, 2),
  [547] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_map_dat, 3),
  [549] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_map_dat, 2),
  [551] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_map_dat, 3),
  [553] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_seq_dat, 3),
  [555] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_seq_dat, 3),
  [557] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_seq_dat, 2),
  [559] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_seq_dat, 2),
  [561] = {.entry = {.count = 1, .reusable = true}}, SHIFT(57),
  [563] = {.entry = {.count = 1, .reusable = true}}, SHIFT(54),
  [565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(540),
  [567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(604),
  [569] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1272),
  [571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(61),
  [573] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_exp_par, 1),
  [575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(51),
  [577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(551),
  [579] = {.entry = {.count = 1, .reusable = true}}, SHIFT(579),
  [581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(222),
  [583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(69),
  [585] = {.entry = {.count = 1, .reusable = true}}, SHIFT(44),
  [587] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_njl_ann_par, 1),
  [589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(544),
  [591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(583),
  [593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(195),
  [595] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_jsl_ann_par, 1),
  [597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(55),
  [599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(48),
  [601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(539),
  [603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(624),
  [605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(205),
  [607] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_jsl_ann_par, 1),
  [609] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_prp, 1),
  [611] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_prp, 1),
  [613] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__br_prp, 1), REDUCE(sym__br_sgl_prp, 1),
  [616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1461),
  [618] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__br_prp, 1), SHIFT(1471),
  [621] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_prp, 1),
  [623] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_prp, 1),
  [625] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__r_prp, 1), REDUCE(sym__r_sgl_prp, 1),
  [628] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__r_prp, 1), SHIFT(1471),
  [631] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_acr, 2),
  [633] = {.entry = {.count = 1, .reusable = true}}, SHIFT(106),
  [635] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__br_prp, 1), SHIFT(173),
  [638] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_acr, 2),
  [640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1505),
  [642] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__br_prp, 1), SHIFT(1506),
  [645] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__br_prp, 1), SHIFT(1474),
  [648] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__r_prp, 1), SHIFT(1506),
  [651] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__r_prp, 1), SHIFT(1474),
  [654] = {.entry = {.count = 1, .reusable = true}}, SHIFT(109),
  [656] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__r_prp, 1), SHIFT(174),
  [659] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_prp, 2),
  [661] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_prp, 2),
  [663] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__br_prp, 2), REDUCE(sym__br_sgl_prp, 2),
  [666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1562),
  [668] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_prp, 2),
  [670] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_prp, 2),
  [672] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__r_prp, 2), REDUCE(sym__r_sgl_prp, 2),
  [675] = {.entry = {.count = 1, .reusable = true}}, SHIFT(119),
  [677] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__r_prp, 1), SHIFT(177),
  [680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(118),
  [682] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__br_prp, 1), SHIFT(179),
  [685] = {.entry = {.count = 1, .reusable = true}}, SHIFT(117),
  [687] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__br_prp, 1), SHIFT(125),
  [690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(116),
  [692] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__r_prp, 1), SHIFT(124),
  [695] = {.entry = {.count = 1, .reusable = true}}, SHIFT(124),
  [697] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4),
  [699] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3),
  [701] = {.entry = {.count = 1, .reusable = true}}, SHIFT(138),
  [703] = {.entry = {.count = 1, .reusable = true}}, SHIFT(134),
  [705] = {.entry = {.count = 1, .reusable = true}}, SHIFT(949),
  [707] = {.entry = {.count = 1, .reusable = true}}, SHIFT(948),
  [709] = {.entry = {.count = 1, .reusable = true}}, SHIFT(389),
  [711] = {.entry = {.count = 1, .reusable = true}}, SHIFT(208),
  [713] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1485),
  [715] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1484),
  [717] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1128),
  [719] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1172),
  [721] = {.entry = {.count = 1, .reusable = true}}, SHIFT(148),
  [723] = {.entry = {.count = 1, .reusable = true}}, SHIFT(143),
  [725] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1518),
  [727] = {.entry = {.count = 1, .reusable = true}}, SHIFT(186),
  [729] = {.entry = {.count = 1, .reusable = true}}, SHIFT(190),
  [731] = {.entry = {.count = 1, .reusable = true}}, SHIFT(996),
  [733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(998),
  [735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1121),
  [737] = {.entry = {.count = 1, .reusable = true}}, SHIFT(493),
  [739] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1556),
  [741] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1544),
  [743] = {.entry = {.count = 1, .reusable = true}}, SHIFT(384),
  [745] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1146),
  [747] = {.entry = {.count = 1, .reusable = true}}, SHIFT(136),
  [749] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1145),
  [751] = {.entry = {.count = 1, .reusable = true}}, SHIFT(135),
  [753] = {.entry = {.count = 1, .reusable = true}}, SHIFT(181),
  [755] = {.entry = {.count = 1, .reusable = true}}, SHIFT(180),
  [757] = {.entry = {.count = 1, .reusable = true}}, SHIFT(891),
  [759] = {.entry = {.count = 1, .reusable = true}}, SHIFT(887),
  [761] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1238),
  [763] = {.entry = {.count = 1, .reusable = true}}, SHIFT(494),
  [765] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1554),
  [767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1541),
  [769] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1445),
  [771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(346),
  [773] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1368),
  [775] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1287),
  [777] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1517),
  [779] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1127),
  [781] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1192),
  [783] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1367),
  [785] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1404),
  [787] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1187),
  [789] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1175),
  [791] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1473),
  [793] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1363),
  [795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1346),
  [797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(482),
  [799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(388),
  [801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(392),
  [803] = {.entry = {.count = 1, .reusable = true}}, SHIFT(478),
  [805] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1359),
  [807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(481),
  [809] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_blk_map_repeat1, 2), SHIFT_REPEAT(4),
  [812] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_blk_map_repeat1, 2), SHIFT_REPEAT(3),
  [815] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_blk_map_repeat1, 2), SHIFT_REPEAT(138),
  [818] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_blk_map_repeat1, 2), SHIFT_REPEAT(134),
  [821] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_blk_map_repeat1, 2), SHIFT_REPEAT(949),
  [824] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_blk_map_repeat1, 2), SHIFT_REPEAT(948),
  [827] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_blk_map_repeat1, 2), SHIFT_REPEAT(389),
  [830] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_blk_map_repeat1, 2), SHIFT_REPEAT(208),
  [833] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_blk_map_repeat1, 2), SHIFT_REPEAT(1485),
  [836] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_blk_map_repeat1, 2), SHIFT_REPEAT(1484),
  [839] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__r_blk_map_repeat1, 2),
  [841] = {.entry = {.count = 1, .reusable = true}}, SHIFT(395),
  [843] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_col_dat, 3),
  [845] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_col_dat, 2),
  [847] = {.entry = {.count = 1, .reusable = true}}, SHIFT(174),
  [849] = {.entry = {.count = 1, .reusable = true}}, SHIFT(137),
  [851] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_njl_ann_par, 1),
  [853] = {.entry = {.count = 1, .reusable = true}}, SHIFT(140),
  [855] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1114),
  [857] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1113),
  [859] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1385),
  [861] = {.entry = {.count = 1, .reusable = true}}, SHIFT(496),
  [863] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1559),
  [865] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1547),
  [867] = {.entry = {.count = 1, .reusable = true}}, SHIFT(141),
  [869] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_exp_par, 1),
  [871] = {.entry = {.count = 1, .reusable = true}}, SHIFT(133),
  [873] = {.entry = {.count = 1, .reusable = true}}, SHIFT(983),
  [875] = {.entry = {.count = 1, .reusable = true}}, SHIFT(985),
  [877] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1531),
  [879] = {.entry = {.count = 1, .reusable = true}}, SHIFT(508),
  [881] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1563),
  [883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1512),
  [885] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_prp_val, 1),
  [887] = {.entry = {.count = 1, .reusable = true}}, SHIFT(156),
  [889] = {.entry = {.count = 1, .reusable = true}}, SHIFT(155),
  [891] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1037),
  [893] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1111),
  [895] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_prp_val, 1),
  [897] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1545),
  [899] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1546),
  [901] = {.entry = {.count = 1, .reusable = true}}, SHIFT(153),
  [903] = {.entry = {.count = 1, .reusable = true}}, SHIFT(158),
  [905] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_jsl_ann_par, 1),
  [907] = {.entry = {.count = 1, .reusable = true}}, SHIFT(897),
  [909] = {.entry = {.count = 1, .reusable = true}}, SHIFT(980),
  [911] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1376),
  [913] = {.entry = {.count = 1, .reusable = true}}, SHIFT(500),
  [915] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1561),
  [917] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1550),
  [919] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_prp_val, 1),
  [921] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1539),
  [923] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1540),
  [925] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1560),
  [927] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1555),
  [929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1558),
  [931] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1552),
  [933] = {.entry = {.count = 1, .reusable = true}}, SHIFT(233),
  [935] = {.entry = {.count = 1, .reusable = true}}, SHIFT(230),
  [937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(214),
  [939] = {.entry = {.count = 1, .reusable = true}}, SHIFT(238),
  [941] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_prp, 1),
  [943] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1551),
  [945] = {.entry = {.count = 1, .reusable = true}}, SHIFT(237),
  [947] = {.entry = {.count = 1, .reusable = true}}, SHIFT(239),
  [949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(259),
  [951] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1557),
  [953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1549),
  [955] = {.entry = {.count = 1, .reusable = true}}, SHIFT(241),
  [957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(236),
  [959] = {.entry = {.count = 1, .reusable = true}}, SHIFT(244),
  [961] = {.entry = {.count = 1, .reusable = true}}, SHIFT(256),
  [963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(254),
  [965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(243),
  [967] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_acr, 2),
  [969] = {.entry = {.count = 1, .reusable = true}}, SHIFT(260),
  [971] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_prp, 2),
  [973] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__imp_doc_end, 2),
  [975] = {.entry = {.count = 1, .reusable = true}}, SHIFT(268),
  [977] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bgn_imp_doc_end, 2),
  [979] = {.entry = {.count = 1, .reusable = true}}, SHIFT(266),
  [981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(267),
  [983] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__doc_end, 1),
  [985] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__exp_doc_end, 2),
  [987] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__drs_doc_end, 2),
  [989] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__imp_doc, 1),
  [991] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_imp_itm_key_only, 1, .production_id = 1),
  [993] = {.entry = {.count = 1, .reusable = true}}, SHIFT(11),
  [995] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__exp_doc_tal, 1),
  [997] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_imp_itm_key_only, 1, .production_id = 1),
  [999] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sqt_str, 2),
  [1001] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_sqt_str, 2),
  [1003] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sqt_str, 2),
  [1005] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_sqt_str, 2),
  [1007] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__flw_map_tal, 1),
  [1009] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__sgl_flw_map_tal, 1),
  [1011] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_als_val, 1),
  [1013] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sqt_str, 3),
  [1015] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_sqt_str, 3),
  [1017] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sqt_str, 3),
  [1019] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_sqt_str, 3),
  [1021] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_als_val, 1),
  [1023] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_dqt_str, 3),
  [1025] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_dqt_str, 3),
  [1027] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_dqt_str, 3),
  [1029] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_dqt_str, 3),
  [1031] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_dqt_str, 2),
  [1033] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_dqt_str, 2),
  [1035] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_dqt_str, 2),
  [1037] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_dqt_str, 2),
  [1039] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_pln_blk, 1),
  [1041] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_pln_blk, 1),
  [1043] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__flw_seq_tal, 1),
  [1045] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__sgl_flw_seq_tal, 1),
  [1047] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_als, 2),
  [1049] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_als, 2),
  [1051] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_pln_blk_val, 2),
  [1053] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_pln_blk_val, 1),
  [1055] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_pln_blk_val, 2),
  [1057] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_pln_blk_val, 1),
  [1059] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_pln_blk_val, 1),
  [1061] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_pln_blk_val, 1),
  [1063] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__blk_seq_itm_tal, 1),
  [1065] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__flw_map_tal, 1), REDUCE(sym__sgl_flw_map_tal, 1),
  [1068] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__blk_imp_itm_tal, 2, .production_id = 6),
  [1070] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_blk_imp_itm_key_only, 1, .production_id = 1),
  [1072] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__flw_seq_tal, 1), REDUCE(sym__sgl_flw_seq_tal, 1),
  [1075] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_map_val, 2),
  [1077] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_sqt_str, 3),
  [1079] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_dqt_str_val, 1),
  [1081] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_dqt_str_val, 1),
  [1083] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_exp_itm, 1, .production_id = 3),
  [1085] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_seq_val, 2),
  [1087] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_pln_blk_val, 1),
  [1089] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_flw_map_val, 1),
  [1091] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_map_val, 1),
  [1093] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_flw_seq_val, 1),
  [1095] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_seq_val, 1),
  [1097] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_sqt_str_val, 1),
  [1099] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_dqt_str_val, 2),
  [1101] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_flw_map_val, 2),
  [1103] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_flw_seq, 2),
  [1105] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_flw_seq_val, 2),
  [1107] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__sgl_flw_seq_tal, 2),
  [1109] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_dqt_str_val, 1),
  [1111] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_flw_map_val, 1),
  [1113] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_dqt_str_val, 2),
  [1115] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_sqt_str_val, 1),
  [1117] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_dqt_str, 3),
  [1119] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_sqt_str_val, 1),
  [1121] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_flw_seq_val, 1),
  [1123] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__sgl_flw_map_tal, 2),
  [1125] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_als_val, 1),
  [1127] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_sqt_str_val, 2),
  [1129] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_blk_exp_itm, 1, .production_id = 3),
  [1131] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_pln_blk_val, 2),
  [1133] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_sqt_str_val, 2),
  [1135] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_dqt_str_val, 2),
  [1137] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_flw_map_val, 2),
  [1139] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_flw_seq_val, 2),
  [1141] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_pln_blk_val, 2),
  [1143] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_als, 2),
  [1145] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_flw_map, 2),
  [1147] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_sqt_str, 2),
  [1149] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_dqt_str, 2),
  [1151] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_sqt_str_val, 2),
  [1153] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_flw_map, 2),
  [1155] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_exp_itm, 1, .production_id = 3),
  [1157] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_map, 2),
  [1159] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_sgl_pln_blk, 1),
  [1161] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_pln_blk_val, 2),
  [1163] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_flw_seq, 2),
  [1165] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_seq, 2),
  [1167] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_map, 3),
  [1169] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_exp_itm, 1, .production_id = 4),
  [1171] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_imp_itm_key_only, 2, .production_id = 10),
  [1173] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_exp_itm, 2, .production_id = 11),
  [1175] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_exp_itm, 2, .production_id = 11),
  [1177] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_blk_map_itm, 1, .production_id = 2),
  [1179] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_blk_exp_itm, 1, .production_id = 4),
  [1181] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_mtl_pln_blk, 1),
  [1183] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_mtl_pln_blk, 1),
  [1185] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_seq_r_val, 1),
  [1187] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_seq_val, 1),
  [1189] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_map_r_val, 1),
  [1191] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_val_itm, 2, .production_id = 6),
  [1193] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_map_val, 1),
  [1195] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_str_val, 1),
  [1197] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_blk_key_itm, 2, .production_id = 5),
  [1199] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_blk_val_itm, 2, .production_id = 6),
  [1201] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_str_val, 1),
  [1203] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_seq_val, 1),
  [1205] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_seq_val, 1),
  [1207] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_map_val, 1),
  [1209] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_map_val, 1),
  [1211] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_map_itm, 1, .production_id = 2),
  [1213] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_val_itm, 2, .production_id = 6),
  [1215] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_key_itm, 2, .production_id = 5),
  [1217] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_blk_imp_itm_key_only, 2, .production_id = 10),
  [1219] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_dqt_str_val, 1),
  [1221] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_key_itm, 2, .production_id = 5),
  [1223] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_dqt_str_val, 1),
  [1225] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_map_itm, 1, .production_id = 2),
  [1227] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sqt_str_val, 1),
  [1229] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_blk_exp_itm, 2, .production_id = 11),
  [1231] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_seq_spc_val, 2),
  [1233] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_seq_spc_val, 2),
  [1235] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sqt_str_val, 1),
  [1237] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_exp_itm, 1, .production_id = 4),
  [1239] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_blk_seq_spc_val, 1),
  [1241] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_blk_seq_spc, 2),
  [1243] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_imp_itm_key_only, 2, .production_id = 10),
  [1245] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_str, 2),
  [1247] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_str, 2),
  [1249] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_seq, 2),
  [1251] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_seq, 2),
  [1253] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_map, 2),
  [1255] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_map, 2),
  [1257] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_seq_br_val, 2),
  [1259] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_map_br_val, 2),
  [1261] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_str_val, 2),
  [1263] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_seq_val, 2),
  [1265] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_map_val, 2),
  [1267] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_dqt_str_val, 2),
  [1269] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sqt_str_val, 2),
  [1271] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_seq_val, 2),
  [1273] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_map_val, 2),
  [1275] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_str_val, 2),
  [1277] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_seq_val, 2),
  [1279] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_map_val, 2),
  [1281] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_dqt_str_val, 2),
  [1283] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sqt_str_val, 2),
  [1285] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_dqt_str, 5),
  [1287] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_dqt_str, 5),
  [1289] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_seq, 2),
  [1291] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sqt_str, 4),
  [1293] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sqt_str, 4),
  [1295] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_seq, 2),
  [1297] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_map, 2),
  [1299] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_dqt_str, 4),
  [1301] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_dqt_str, 4),
  [1303] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_map, 2),
  [1305] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_map, 3),
  [1307] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_seq, 3),
  [1309] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_seq, 3),
  [1311] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_str, 3),
  [1313] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_str, 3),
  [1315] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__flw_seq_tal, 2),
  [1317] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__flw_map_tal, 2),
  [1319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1565),
  [1321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1564),
  [1323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(503),
  [1325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1566),
  [1327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(501),
  [1329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1567),
  [1331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(520),
  [1333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1568),
  [1335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(535),
  [1337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(589),
  [1339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(575),
  [1341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(640),
  [1343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(641),
  [1345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1026),
  [1347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(531),
  [1349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(705),
  [1351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(666),
  [1353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(282),
  [1355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1246),
  [1357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(528),
  [1359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(667),
  [1361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(669),
  [1363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(283),
  [1365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1117),
  [1367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(558),
  [1369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(696),
  [1371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(702),
  [1373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1373),
  [1375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(673),
  [1377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(676),
  [1379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1078),
  [1381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(521),
  [1383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(692),
  [1385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(703),
  [1387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(334),
  [1389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(449),
  [1391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(677),
  [1393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(680),
  [1395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1077),
  [1397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1076),
  [1399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(672),
  [1401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(671),
  [1403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1378),
  [1405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(632),
  [1407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(635),
  [1409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(862),
  [1411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(861),
  [1413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(710),
  [1415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(711),
  [1417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(348),
  [1419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(489),
  [1421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(708),
  [1423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(709),
  [1425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(349),
  [1427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(490),
  [1429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(514),
  [1431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(682),
  [1433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(681),
  [1435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1057),
  [1437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(534),
  [1439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(655),
  [1441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(654),
  [1443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(836),
  [1445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(548),
  [1447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(685),
  [1449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(686),
  [1451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(806),
  [1453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(805),
  [1455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(646),
  [1457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(647),
  [1459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1126),
  [1461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(713),
  [1463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(712),
  [1465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(770),
  [1467] = {.entry = {.count = 1, .reusable = true}}, SHIFT(643),
  [1469] = {.entry = {.count = 1, .reusable = true}}, SHIFT(644),
  [1471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(280),
  [1473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1125),
  [1475] = {.entry = {.count = 1, .reusable = true}}, SHIFT(516),
  [1477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(687),
  [1479] = {.entry = {.count = 1, .reusable = true}}, SHIFT(684),
  [1481] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1055),
  [1483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1056),
  [1485] = {.entry = {.count = 1, .reusable = true}}, SHIFT(693),
  [1487] = {.entry = {.count = 1, .reusable = true}}, SHIFT(715),
  [1489] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1341),
  [1491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(281),
  [1493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(560),
  [1495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(636),
  [1497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(637),
  [1499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(863),
  [1501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(559),
  [1503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(651),
  [1505] = {.entry = {.count = 1, .reusable = true}}, SHIFT(650),
  [1507] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1105),
  [1509] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1106),
  [1511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(554),
  [1513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(518),
  [1515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(517),
  [1517] = {.entry = {.count = 1, .reusable = true}}, SHIFT(652),
  [1519] = {.entry = {.count = 1, .reusable = true}}, SHIFT(653),
  [1521] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1451),
  [1523] = {.entry = {.count = 1, .reusable = true}}, SHIFT(557),
  [1525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(322),
  [1527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(510),
  [1529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1333),
  [1531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(561),
  [1533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(649),
  [1535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(648),
  [1537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1107),
  [1539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(549),
  [1541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(656),
  [1543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(661),
  [1545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(321),
  [1547] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1450),
  [1549] = {.entry = {.count = 1, .reusable = true}}, SHIFT(706),
  [1551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(697),
  [1553] = {.entry = {.count = 1, .reusable = true}}, SHIFT(771),
  [1555] = {.entry = {.count = 1, .reusable = true}}, SHIFT(772),
  [1557] = {.entry = {.count = 1, .reusable = true}}, SHIFT(675),
  [1559] = {.entry = {.count = 1, .reusable = true}}, SHIFT(674),
  [1561] = {.entry = {.count = 1, .reusable = true}}, SHIFT(299),
  [1563] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1377),
  [1565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(519),
  [1567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(659),
  [1569] = {.entry = {.count = 1, .reusable = true}}, SHIFT(657),
  [1571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(834),
  [1573] = {.entry = {.count = 1, .reusable = true}}, SHIFT(835),
  [1575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(530),
  [1577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(701),
  [1579] = {.entry = {.count = 1, .reusable = true}}, SHIFT(631),
  [1581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1372),
  [1583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(564),
  [1585] = {.entry = {.count = 1, .reusable = true}}, SHIFT(545),
  [1587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1371),
  [1589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(522),
  [1591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(714),
  [1593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(698),
  [1595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(335),
  [1597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(451),
  [1599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(533),
  [1601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(300),
  [1603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(691),
  [1605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(704),
  [1607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1297),
  [1609] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1025),
  [1611] = {.entry = {.count = 1, .reusable = true}}, SHIFT(638),
  [1613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(639),
  [1615] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1024),
  [1617] = {.entry = {.count = 1, .reusable = true}}, SHIFT(527),
  [1619] = {.entry = {.count = 1, .reusable = true}}, SHIFT(630),
  [1621] = {.entry = {.count = 1, .reusable = true}}, SHIFT(688),
  [1623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(803),
  [1625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(526),
  [1627] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__doc_wo_bgn_wo_end_seq, 1),
  [1629] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__doc_w_bgn_wo_end_seq, 1),
  [1631] = {.entry = {.count = 1, .reusable = true}}, SHIFT(262),
  [1633] = {.entry = {.count = 1, .reusable = true}}, SHIFT(263),
  [1635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(249),
  [1637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(612),
  [1639] = {.entry = {.count = 1, .reusable = true}}, SHIFT(670),
  [1641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(273),
  [1643] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1245),
  [1645] = {.entry = {.count = 1, .reusable = true}}, SHIFT(683),
  [1647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(866),
  [1649] = {.entry = {.count = 1, .reusable = true}}, SHIFT(766),
  [1651] = {.entry = {.count = 1, .reusable = true}}, SHIFT(586),
  [1653] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_mtl_dqt_ctn, 1),
  [1655] = {.entry = {.count = 1, .reusable = true}}, SHIFT(858),
  [1657] = {.entry = {.count = 1, .reusable = true}}, SHIFT(592),
  [1659] = {.entry = {.count = 1, .reusable = true}}, SHIFT(319),
  [1661] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1447),
  [1663] = {.entry = {.count = 1, .reusable = true}}, SHIFT(767),
  [1665] = {.entry = {.count = 1, .reusable = true}}, SHIFT(593),
  [1667] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1379),
  [1669] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1381),
  [1671] = {.entry = {.count = 1, .reusable = true}}, SHIFT(609),
  [1673] = {.entry = {.count = 1, .reusable = true}}, SHIFT(619),
  [1675] = {.entry = {.count = 1, .reusable = true}}, SHIFT(837),
  [1677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(838),
  [1679] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__br_mtl_dqt_ctn_repeat1, 2), SHIFT_REPEAT(581),
  [1682] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__br_mtl_dqt_ctn_repeat1, 2),
  [1684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(588),
  [1686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(320),
  [1688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1448),
  [1690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(596),
  [1692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1109),
  [1694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(597),
  [1696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1110),
  [1698] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__drs_doc_repeat1, 2), SHIFT_REPEAT(1454),
  [1701] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__drs_doc_repeat1, 2), SHIFT_REPEAT(1509),
  [1704] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__drs_doc_repeat1, 2), SHIFT_REPEAT(738),
  [1707] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__drs_doc_repeat1, 2),
  [1709] = {.entry = {.count = 1, .reusable = true}}, SHIFT(581),
  [1711] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_mtl_dqt_ctn, 2),
  [1713] = {.entry = {.count = 1, .reusable = true}}, SHIFT(828),
  [1715] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1023),
  [1717] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1022),
  [1719] = {.entry = {.count = 1, .reusable = true}}, SHIFT(298),
  [1721] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1375),
  [1723] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_sgl_dqt_ctn, 2), SHIFT_REPEAT(589),
  [1726] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__r_sgl_dqt_ctn, 2),
  [1728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(353),
  [1730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(492),
  [1732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(352),
  [1734] = {.entry = {.count = 1, .reusable = true}}, SHIFT(491),
  [1736] = {.entry = {.count = 1, .reusable = true}}, SHIFT(297),
  [1738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1374),
  [1740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1251),
  [1742] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1252),
  [1744] = {.entry = {.count = 1, .reusable = true}}, SHIFT(587),
  [1746] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1108),
  [1748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(607),
  [1750] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1021),
  [1752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(613),
  [1754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(274),
  [1756] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1244),
  [1758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(574),
  [1760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(799),
  [1762] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__r_flw_map_itm, 1), REDUCE(sym__r_sgl_flw_col_itm, 1),
  [1765] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_map_itm, 1),
  [1767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(603),
  [1769] = {.entry = {.count = 1, .reusable = true}}, SHIFT(615),
  [1771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(453),
  [1773] = {.entry = {.count = 1, .reusable = true}}, SHIFT(865),
  [1775] = {.entry = {.count = 1, .reusable = true}}, SHIFT(622),
  [1777] = {.entry = {.count = 1, .reusable = true}}, SHIFT(621),
  [1779] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1058),
  [1781] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1059),
  [1783] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1254),
  [1785] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1118),
  [1787] = {.entry = {.count = 1, .reusable = true}}, SHIFT(590),
  [1789] = {.entry = {.count = 1, .reusable = true}}, SHIFT(339),
  [1791] = {.entry = {.count = 1, .reusable = true}}, SHIFT(454),
  [1793] = {.entry = {.count = 1, .reusable = true}}, SHIFT(765),
  [1795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(577),
  [1797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(802),
  [1799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(278),
  [1801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(277),
  [1803] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1119),
  [1805] = {.entry = {.count = 1, .reusable = true}}, SHIFT(573),
  [1807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(839),
  [1809] = {.entry = {.count = 1, .reusable = true}}, SHIFT(625),
  [1811] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1060),
  [1813] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1080),
  [1815] = {.entry = {.count = 1, .reusable = true}}, SHIFT(610),
  [1817] = {.entry = {.count = 1, .reusable = true}}, SHIFT(796),
  [1819] = {.entry = {.count = 1, .reusable = true}}, SHIFT(864),
  [1821] = {.entry = {.count = 1, .reusable = true}}, SHIFT(591),
  [1823] = {.entry = {.count = 1, .reusable = true}}, SHIFT(343),
  [1825] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1079),
  [1827] = {.entry = {.count = 1, .reusable = true}}, SHIFT(606),
  [1829] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1391),
  [1831] = {.entry = {.count = 1, .reusable = true}}, SHIFT(617),
  [1833] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1081),
  [1835] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_map_itm, 1),
  [1837] = {.entry = {.count = 1, .reusable = true}}, SHIFT(95),
  [1839] = {.entry = {.count = 1, .reusable = true}}, SHIFT(89),
  [1841] = {.entry = {.count = 1, .reusable = true}}, SHIFT(645),
  [1843] = {.entry = {.count = 1, .reusable = true}}, SHIFT(660),
  [1845] = {.entry = {.count = 1, .reusable = true}}, SHIFT(870),
  [1847] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__r_flw_seq_itm, 1), REDUCE(sym__r_sgl_flw_col_itm, 1),
  [1850] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_seq_itm, 1),
  [1852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1014),
  [1854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(664),
  [1856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(871),
  [1858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(663),
  [1860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1019),
  [1862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(634),
  [1864] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1020),
  [1866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(707),
  [1868] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_mtl_sqt_ctn, 2),
  [1870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(662),
  [1872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1131),
  [1874] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_dqt_str_repeat1, 2), SHIFT_REPEAT(575),
  [1877] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_dqt_str_repeat1, 2), SHIFT_REPEAT(645),
  [1880] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__r_dqt_str_repeat1, 2),
  [1882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(658),
  [1884] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1130),
  [1886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1134),
  [1888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(875),
  [1890] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1136),
  [1892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1010),
  [1894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(876),
  [1896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1087),
  [1898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1086),
  [1900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(642),
  [1902] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_mtl_sqt_ctn, 1),
  [1904] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1364),
  [1906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(678),
  [1908] = {.entry = {.count = 1, .reusable = true}}, SHIFT(665),
  [1910] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1083),
  [1912] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1280),
  [1914] = {.entry = {.count = 1, .reusable = true}}, SHIFT(679),
  [1916] = {.entry = {.count = 1, .reusable = true}}, SHIFT(668),
  [1918] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1082),
  [1920] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1357),
  [1922] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1352),
  [1924] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_sgl_sqt_ctn, 2), SHIFT_REPEAT(683),
  [1927] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__r_sgl_sqt_ctn, 2),
  [1929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1248),
  [1931] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1279),
  [1933] = {.entry = {.count = 1, .reusable = true}}, SHIFT(689),
  [1935] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1261),
  [1937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(690),
  [1939] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1259),
  [1941] = {.entry = {.count = 1, .reusable = true}}, SHIFT(754),
  [1943] = {.entry = {.count = 1, .reusable = true}}, SHIFT(752),
  [1945] = {.entry = {.count = 1, .reusable = true}}, SHIFT(764),
  [1947] = {.entry = {.count = 1, .reusable = true}}, SHIFT(472),
  [1949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(473),
  [1951] = {.entry = {.count = 1, .reusable = true}}, SHIFT(694),
  [1953] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__br_mtl_sqt_ctn_repeat1, 2), SHIFT_REPEAT(707),
  [1956] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__br_mtl_sqt_ctn_repeat1, 2),
  [1958] = {.entry = {.count = 1, .reusable = true}}, SHIFT(699),
  [1960] = {.entry = {.count = 1, .reusable = true}}, SHIFT(479),
  [1962] = {.entry = {.count = 1, .reusable = true}}, SHIFT(700),
  [1964] = {.entry = {.count = 1, .reusable = true}}, SHIFT(480),
  [1966] = {.entry = {.count = 1, .reusable = true}}, SHIFT(763),
  [1968] = {.entry = {.count = 1, .reusable = true}}, SHIFT(695),
  [1970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1133),
  [1972] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1266),
  [1974] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_seq_dat, 1),
  [1976] = {.entry = {.count = 1, .reusable = true}}, SHIFT(77),
  [1978] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_seq_dat, 1),
  [1980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(76),
  [1982] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__s_dir_rsv_repeat1, 2),
  [1984] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__s_dir_rsv_repeat1, 2), SHIFT_REPEAT(722),
  [1987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(475),
  [1989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(476),
  [1991] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1132),
  [1993] = {.entry = {.count = 1, .reusable = true}}, SHIFT(762),
  [1995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(758),
  [1997] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_sqt_str_repeat1, 2), SHIFT_REPEAT(670),
  [2000] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__r_sqt_str_repeat1, 2),
  [2002] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_seq_itm, 1),
  [2004] = {.entry = {.count = 1, .reusable = true}}, SHIFT(92),
  [2006] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1017),
  [2008] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__s_dir_rsv, 2),
  [2010] = {.entry = {.count = 1, .reusable = true}}, SHIFT(722),
  [2012] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__s_dir_rsv, 1),
  [2014] = {.entry = {.count = 1, .reusable = true}}, SHIFT(737),
  [2016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1018),
  [2018] = {.entry = {.count = 1, .reusable = true}}, SHIFT(872),
  [2020] = {.entry = {.count = 1, .reusable = true}}, SHIFT(873),
  [2022] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_pln_flw_val, 1),
  [2024] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_pln_flw_val, 1),
  [2026] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__r_pln_flw_val, 1), REDUCE(sym__r_sgl_pln_flw_val, 1),
  [2029] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_mtl_pln_flw, 1),
  [2031] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_mtl_pln_flw, 1),
  [2033] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_sgl_pln_flw, 1),
  [2035] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_pln_flw, 1),
  [2037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1265),
  [2039] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_pln_flw_val, 2),
  [2041] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__r_flw_seq_dat_repeat1, 2),
  [2043] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_flw_seq_dat_repeat1, 2), SHIFT_REPEAT(79),
  [2046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(71),
  [2048] = {.entry = {.count = 1, .reusable = true}}, SHIFT(73),
  [2050] = {.entry = {.count = 1, .reusable = true}}, SHIFT(75),
  [2052] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_map_dat, 1),
  [2054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(72),
  [2056] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_map_dat, 1),
  [2058] = {.entry = {.count = 1, .reusable = true}}, SHIFT(70),
  [2060] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_pln_flw_val, 2),
  [2062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1361),
  [2064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1362),
  [2066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1084),
  [2068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(74),
  [2070] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__r_flw_map_dat_repeat1, 2),
  [2072] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_flw_map_dat_repeat1, 2), SHIFT_REPEAT(78),
  [2075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1085),
  [2077] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_mtl_dqt_ctn, 3),
  [2079] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_imp_br_par, 2, .production_id = 10),
  [2081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1370),
  [2083] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_imp_r_par, 2, .production_id = 10),
  [2085] = {.entry = {.count = 1, .reusable = true}}, SHIFT(934),
  [2087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1168),
  [2089] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1369),
  [2091] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_sgl_sqt_ctn, 2), SHIFT_REPEAT(890),
  [2094] = {.entry = {.count = 1, .reusable = true}}, SHIFT(930),
  [2096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1167),
  [2098] = {.entry = {.count = 1, .reusable = true}}, SHIFT(166),
  [2100] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_imp_r_par, 2, .production_id = 10),
  [2102] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__s_dir_yml, 2),
  [2104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(953),
  [2106] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1255),
  [2108] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_imp_br_par, 2, .production_id = 10),
  [2110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(890),
  [2112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1516),
  [2114] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_map_itm, 1, .production_id = 8),
  [2116] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_map_itm, 1, .production_id = 8),
  [2118] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_map_itm, 1, .production_id = 7),
  [2120] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_map_itm, 1, .production_id = 7),
  [2122] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_njl_ann_par, 2, .production_id = 6),
  [2124] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__flw_ann_par_tal, 1), REDUCE(sym__sgl_flw_ann_par_tal, 1),
  [2127] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__flw_ann_par_tal, 1),
  [2129] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__flw_map_dat_rpt, 2),
  [2131] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_njl_ann_par, 2, .production_id = 6),
  [2133] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_exp_par, 2, .production_id = 12),
  [2135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1013),
  [2137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1169),
  [2139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1171),
  [2141] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_exp_par, 2, .production_id = 12),
  [2143] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__s_dir_tag, 3),
  [2145] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_jsl_ann_par, 2, .production_id = 6),
  [2147] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__r_sgl_flw_col_dat_repeat1, 2),
  [2149] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_sgl_flw_col_dat_repeat1, 2), SHIFT_REPEAT(172),
  [2152] = {.entry = {.count = 1, .reusable = true}}, SHIFT(982),
  [2154] = {.entry = {.count = 1, .reusable = true}}, SHIFT(381),
  [2156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(984),
  [2158] = {.entry = {.count = 1, .reusable = true}}, SHIFT(382),
  [2160] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1271),
  [2162] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1270),
  [2164] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1406),
  [2166] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1407),
  [2168] = {.entry = {.count = 1, .reusable = true}}, SHIFT(474),
  [2170] = {.entry = {.count = 1, .reusable = true}}, SHIFT(477),
  [2172] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1123),
  [2174] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__flw_seq_dat_rpt, 2),
  [2176] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_jsl_ann_par, 2, .production_id = 6),
  [2178] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_sgl_flw_col_dat_repeat1, 2), SHIFT_REPEAT(175),
  [2181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(169),
  [2183] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_col_itm, 1),
  [2185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(188),
  [2187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1184),
  [2189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(439),
  [2191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(952),
  [2193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1258),
  [2195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1162),
  [2197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(324),
  [2199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(989),
  [2201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1455),
  [2203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(359),
  [2205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(906),
  [2207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1519),
  [2209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(360),
  [2211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1520),
  [2213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(361),
  [2215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1122),
  [2217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1047),
  [2219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1232),
  [2221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1048),
  [2223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1233),
  [2225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(484),
  [2227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(91),
  [2229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(483),
  [2231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(189),
  [2233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1312),
  [2235] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_seq_itm, 1, .production_id = 7),
  [2237] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1311),
  [2239] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_sgl_dqt_ctn, 2), SHIFT_REPEAT(1013),
  [2242] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_col_dat, 1),
  [2244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(167),
  [2246] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_blk_seq_repeat1, 2), SHIFT_REPEAT(8),
  [2249] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__r_blk_seq_repeat1, 2),
  [2251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(990),
  [2253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(393),
  [2255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1240),
  [2257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1243),
  [2259] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_seq_itm, 1, .production_id = 8),
  [2261] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_flw_seq_itm, 1, .production_id = 8),
  [2263] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_flw_seq_itm, 1, .production_id = 7),
  [2265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(988),
  [2267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(394),
  [2269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(170),
  [2271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1007),
  [2273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1310),
  [2275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1012),
  [2277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1309),
  [2279] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_pln_flw_val, 1),
  [2281] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_pln_flw_val, 2),
  [2283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1194),
  [2285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1196),
  [2287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1191),
  [2289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1193),
  [2291] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__r_blk_str_repeat1, 2), SHIFT_REPEAT(1149),
  [2294] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__r_blk_str_repeat1, 2),
  [2296] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bgn_imp_doc, 1),
  [2298] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__drs_doc, 2),
  [2300] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1149),
  [2302] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1148),
  [2304] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1152),
  [2306] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1227),
  [2308] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1411),
  [2310] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1228),
  [2312] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1412),
  [2314] = {.entry = {.count = 1, .reusable = true}}, SHIFT(486),
  [2316] = {.entry = {.count = 1, .reusable = true}}, SHIFT(485),
  [2318] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__exp_doc, 2),
  [2320] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1403),
  [2322] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1398),
  [2324] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__doc_wo_bgn_wo_end, 1, .production_id = 9),
  [2326] = {.entry = {.count = 1, .reusable = true}}, SHIFT(245),
  [2328] = {.entry = {.count = 1, .reusable = true}}, SHIFT(258),
  [2330] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__doc_w_bgn_wo_end, 1, .production_id = 9),
  [2332] = {.entry = {.count = 1, .reusable = true}}, SHIFT(253),
  [2334] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1219),
  [2336] = {.entry = {.count = 1, .reusable = true}}, SHIFT(443),
  [2338] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1220),
  [2340] = {.entry = {.count = 1, .reusable = true}}, SHIFT(442),
  [2342] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_col_itm, 1, .production_id = 7),
  [2344] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_jsl_ann_par, 2, .production_id = 6),
  [2346] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__sgl_flw_col_dat_rpt, 2),
  [2348] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1027),
  [2350] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1028),
  [2352] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_imp_par, 2, .production_id = 10),
  [2354] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1380),
  [2356] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1386),
  [2358] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_blk_seq_itm, 2),
  [2360] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__br_blk_seq_itm, 2),
  [2362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(487),
  [2364] = {.entry = {.count = 1, .reusable = true}}, SHIFT(488),
  [2366] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1129),
  [2368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1075),
  [2370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1074),
  [2372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(859),
  [2374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(860),
  [2376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1138),
  [2378] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__b_blk_seq_itm, 2),
  [2380] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_njl_ann_par, 2, .production_id = 6),
  [2382] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_exp_par, 2, .production_id = 12),
  [2384] = {.entry = {.count = 1, .reusable = true}}, SHIFT(774),
  [2386] = {.entry = {.count = 1, .reusable = true}}, SHIFT(773),
  [2388] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__r_sgl_flw_col_itm, 1, .production_id = 8),
  [2390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1402),
  [2392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1383),
  [2394] = {.entry = {.count = 1, .reusable = true}}, SHIFT(894),
  [2396] = {.entry = {.count = 1, .reusable = true}}, SHIFT(98),
  [2398] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1393),
  [2400] = {.entry = {.count = 1, .reusable = true}}, SHIFT(724),
  [2402] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__doc_wo_bgn_w_end_seq, 2),
  [2404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(223),
  [2406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(108),
  [2408] = {.entry = {.count = 1, .reusable = true}}, SHIFT(100),
  [2410] = {.entry = {.count = 1, .reusable = true}}, SHIFT(287),
  [2412] = {.entry = {.count = 1, .reusable = true}}, SHIFT(288),
  [2414] = {.entry = {.count = 1, .reusable = true}},  ACCEPT_INPUT(),
  [2416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(945),
  [2418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1296),
  [2420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1295),
  [2422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(791),
  [2424] = {.entry = {.count = 1, .reusable = true}}, SHIFT(855),
  [2426] = {.entry = {.count = 1, .reusable = true}}, SHIFT(176),
  [2428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(199),
  [2430] = {.entry = {.count = 1, .reusable = true}}, SHIFT(171),
  [2432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(178),
  [2434] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1249),
  [2436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1065),
  [2438] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_stream, 1),
  [2440] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1529),
  [2442] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1526),
  [2444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(846),
  [2446] = {.entry = {.count = 1, .reusable = true}}, SHIFT(379),
  [2448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(200),
  [2450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(206),
  [2452] = {.entry = {.count = 1, .reusable = true}}, SHIFT(378),
  [2454] = {.entry = {.count = 1, .reusable = true}}, SHIFT(231),
  [2456] = {.entry = {.count = 1, .reusable = true}}, SHIFT(366),
  [2458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(165),
  [2460] = {.entry = {.count = 1, .reusable = true}}, SHIFT(115),
  [2462] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1221),
  [2464] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1216),
  [2466] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1150),
  [2468] = {.entry = {.count = 1, .reusable = true}}, SHIFT(312),
  [2470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(341),
  [2472] = {.entry = {.count = 1, .reusable = true}}, SHIFT(308),
  [2474] = {.entry = {.count = 1, .reusable = true}}, SHIFT(212),
  [2476] = {.entry = {.count = 1, .reusable = true}}, SHIFT(110),
  [2478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1160),
  [2480] = {.entry = {.count = 1, .reusable = true}}, SHIFT(351),
  [2482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1164),
  [2484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1528),
  [2486] = {.entry = {.count = 1, .reusable = true}}, SHIFT(122),
  [2488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(126),
  [2490] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1115),
  [2492] = {.entry = {.count = 1, .reusable = true}}, SHIFT(944),
  [2494] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1466),
  [2496] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1389),
  [2498] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1356),
  [2500] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1487),
  [2502] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_stream, 2, .production_id = 9),
  [2504] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__doc_w_bgn_wo_end_seq, 2),
  [2506] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__doc_w_bgn_w_end_seq, 2),
  [2508] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_stream, 3, .production_id = 9),
  [2510] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__doc_wo_bgn_wo_end_seq, 2),
  [2512] = {.entry = {.count = 1, .reusable = true}}, SHIFT(257),
  [2514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(250),
  [2516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1008),
  [2518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(217),
  [2520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(216),
  [2522] = {.entry = {.count = 1, .reusable = true}}, SHIFT(400),
  [2524] = {.entry = {.count = 1, .reusable = true}}, SHIFT(242),
  [2526] = {.entry = {.count = 1, .reusable = true}}, SHIFT(240),
  [2528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1173),
  [2530] = {.entry = {.count = 1, .reusable = true}}, SHIFT(225),
  [2532] = {.entry = {.count = 1, .reusable = true}}, SHIFT(220),
  [2534] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1177),
  [2536] = {.entry = {.count = 1, .reusable = true}}, SHIFT(221),
  [2538] = {.entry = {.count = 1, .reusable = true}}, SHIFT(227),
  [2540] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1348),
  [2542] = {.entry = {.count = 1, .reusable = true}}, SHIFT(121),
  [2544] = {.entry = {.count = 1, .reusable = true}}, SHIFT(265),
  [2546] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1338),
  [2548] = {.entry = {.count = 1, .reusable = true}}, SHIFT(235),
  [2550] = {.entry = {.count = 1, .reusable = true}}, SHIFT(264),
  [2552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(497),
  [2554] = {.entry = {.count = 1, .reusable = true}}, SHIFT(255),
  [2556] = {.entry = {.count = 1, .reusable = true}}, SHIFT(499),
  [2558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(269),
  [2560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(247),
  [2562] = {.entry = {.count = 1, .reusable = true}}, SHIFT(504),
  [2564] = {.entry = {.count = 1, .reusable = true}}, SHIFT(248),
  [2566] = {.entry = {.count = 1, .reusable = true}}, SHIFT(505),
  [2568] = {.entry = {.count = 1, .reusable = true}}, SHIFT(123),
  [2570] = {.entry = {.count = 1, .reusable = true}}, SHIFT(536),
  [2572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(506),
  [2574] = {.entry = {.count = 1, .reusable = true}}, SHIFT(507),
  [2576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(541),
  [2578] = {.entry = {.count = 1, .reusable = true}}, SHIFT(542),
  [2580] = {.entry = {.count = 1, .reusable = true}}, SHIFT(570),
};

#ifdef __cplusplus
extern "C" {
#endif
void *tree_sitter_yaml_external_scanner_create(void);
void tree_sitter_yaml_external_scanner_destroy(void *);
bool tree_sitter_yaml_external_scanner_scan(void *, TSLexer *, const bool *);
unsigned tree_sitter_yaml_external_scanner_serialize(void *, char *);
void tree_sitter_yaml_external_scanner_deserialize(void *, const char *, unsigned);

#ifdef _WIN32
#define extern __declspec(dllexport)
#endif

extern const TSLanguage *tree_sitter_yaml(void) {
  static const TSLanguage language = {
    .version = LANGUAGE_VERSION,
    .symbol_count = SYMBOL_COUNT,
    .alias_count = ALIAS_COUNT,
    .token_count = TOKEN_COUNT,
    .external_token_count = EXTERNAL_TOKEN_COUNT,
    .state_count = STATE_COUNT,
    .large_state_count = LARGE_STATE_COUNT,
    .production_id_count = PRODUCTION_ID_COUNT,
    .field_count = FIELD_COUNT,
    .max_alias_sequence_length = MAX_ALIAS_SEQUENCE_LENGTH,
    .parse_table = &ts_parse_table[0][0],
    .small_parse_table = ts_small_parse_table,
    .small_parse_table_map = ts_small_parse_table_map,
    .parse_actions = ts_parse_actions,
    .symbol_names = ts_symbol_names,
    .field_names = ts_field_names,
    .field_map_slices = ts_field_map_slices,
    .field_map_entries = ts_field_map_entries,
    .symbol_metadata = ts_symbol_metadata,
    .public_symbol_map = ts_symbol_map,
    .alias_map = ts_non_terminal_alias_map,
    .alias_sequences = &ts_alias_sequences[0][0],
    .lex_modes = ts_lex_modes,
    .lex_fn = ts_lex,
    .external_scanner = {
      &ts_external_scanner_states[0][0],
      ts_external_scanner_symbol_map,
      tree_sitter_yaml_external_scanner_create,
      tree_sitter_yaml_external_scanner_destroy,
      tree_sitter_yaml_external_scanner_scan,
      tree_sitter_yaml_external_scanner_serialize,
      tree_sitter_yaml_external_scanner_deserialize,
    },
    .primary_state_ids = ts_primary_state_ids,
  };
  return &language;
}
#ifdef __cplusplus
}
#endif
